/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.epm.web.core.app.internal.ProcessInitializer"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.init.InitializationData"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants");
dojo.requireLocalization("com.ibm.epm.web.core.app.internal", "ProcessInitializerMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var getAuthenticatedUserId = com.ibm.team.repository.web.client.session.getAuthenticatedUserId;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var messages = dojo.i18n.getLocalization("com.ibm.epm.web.core.app.internal", "ProcessInitializerMessages");
var SERVICE_URI = "com.ibm.team.process.internal.common.service.IProcessRestService";
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ViewConstants = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants;

com.ibm.epm.web.core.app.internal.ProcessInitializer = {
	// summary:
	//		Adds process awareness to Jazz Ajax Foundation web UIs.  
	//			- Reads project areas from InitializationData and sets global vars.  
	//
	// description:
	//		1) Implement jazz.app._JazzApplication.
	//		2) Call 'init', passing the _JazzApplication context as the first parameter.
	//			eg. com.ibm.team.process.web.ui.ApplicationInitializer.init(this);
	// prereq: 
	//		Repository and InitializationData must be initialized. Call 'init' 
	//		on 'com.ibm.team.repository.web.ui.ApplicationInitializer'.
	
	init: function (application, adminPA) {
		// set the global far for the current project area based on the last
		// segment of the document's URI
		var projectArea = dojo.setObject("com.ibm.team.process.web.projectArea", {}); //$NON-NLS-1$
		var adminProjectArea = dojo.setObject("com.ibm.epm.web.core.client.projectarea", {}); //$NON-NLS-1$
		
		var pathname = window.location.pathname;
		var projectName = pathname.substring(pathname.lastIndexOf('/')+1); //$NON-NLS-1;
		var decodedName = decodeURIComponent(projectName);
		var hashName = window.location.hash;
		var actionString = "#action="; //$NON-NLS-1$
		var actionID;
		if (hashName.indexOf(actionString)!=-1) {
			// contains action string, get the action ID
			actionID = hashName.slice(actionString.length, hashName.indexOf('&')); //$NON-NLS-1;
		}

		var contextArgs;
		
		if (adminPA) {
			projectArea.itemId 		= adminPA.itemId;
			projectArea.name 		= adminPA.name;
			projectArea.archived 	= adminPA.archived;
			
			adminProjectArea.itemId 	= adminPA.itemId;
			adminProjectArea.name 		= adminPA.name;
			adminProjectArea.archived 	= adminPA.archived;
		} 
		
		if (decodedName === "epmWeb") {
			if ((actionID) && (actionID != ViewConstants.ACTION_VIEW_EPMVIEW)) {
				// Invalid action ID, redirect to '/epmWeb'
				var href = net.jazz.ajax._contextRoot + "/epmWeb";
				if (window.location.search) {
					href += window.location.search;
				}
				window.location.href = href;
			}
			else {
			
				var projectAreas = [projectArea];
				
				contextArgs = {
					contexts: projectAreas,
					onChange: this._selectContext,
					selectLabel: messages.selectProject,
					emptyLabel: messages.noProjects,
					contextProvider: dojo.hitch(this, this._getOtherProjectAreas)
				};
				contextArgs.currentContext = projectArea;
				
				// this displays the project areas on the page
				application.config.contextSwitcher = contextArgs;
			}
		}
		
		return contextArgs;
	},
	
	_getOtherProjectAreas: function (dfd) {
		var srh = new ServiceResponseHandler(dfd, "callback", "errback");
		var authContribId = getAuthenticatedUserId();
		var serviceRequest = new ServiceRequest(SERVICE_URI, "getAllProjectAreas", {hideArchivedProjects: true, userId: authContribId});
		TeamServerClient.invokeService(serviceRequest, srh);
	},
	
	_selectContext: function(contextObj) {
		var curUrl = new dojo._Url(location.href);
		
		if (curUrl.query) {  //check for debug mode
			var query = "?" + curUrl.query;	
		} else {
			var query = "";
		}

		var curPageId = PlatformUI.getWorkbench().currentPageId;
		if (curPageId) {
			var action = "#action=com.ibm.epm.web.core.viewEPMPage&id=" + curPageId;
		} else {
			var action = "";
		}

		var href = "";
		if (contextObj.admin) {
			href = net.jazz.ajax._contextRoot + "/admin" + query;
		} else {
			var encodedName = encodeURIComponent(contextObj.name);
			href = net.jazz.ajax._contextRoot + "/epmWeb/projects/" + encodedName + query + action;
		}
		return href;
	}
};
})();