/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){

    dojo.provide("com.ibm.epm.web.core.app.internal.RepositoryInitialize");
    
    dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
    dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
    dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
    dojo.require("com.ibm.team.repository.web.client.session.Session");
    dojo.require("com.ibm.epm.web.core.app.internal.common.ConfirmationDialog");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants");

	dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var checkNotEmpty = net.jazz.ajax.toolbox.lang.checkNotEmpty;
    var isGuest = com.ibm.team.repository.web.client.session.isGuest;
    var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
    var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var LicenseInfoWidget = com.ibm.team.repository.web.ui.internal.LicenseInfoWidget;
	var Session = com.ibm.team.repository.web.client.session;
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var AppConstants = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants;

	var messages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
    
    com.ibm.epm.web.core.app.internal.RepositoryInitialize = {
		init: function(application, successCallback, failureCallback) {
			var srh = new ServiceResponseHandler(this, "_fetchInitDataSuccess", "_fetchInitDataFailure");
            RepositoryClient.getInitializationData(srh);
	
			if (successCallback) this._onSuccess = dojo.hitch(application, successCallback);
			if (failureCallback) this._onFailure = dojo.hitch(application, failureCallback);
		
			this._application = application;
		},

		_fetchInitDataSuccess: function(initializationData) {
			try {
				checkNotNull(initializationData, "initializationData"); //$NON-NLS-1$
				var loadSuccess = this._loadInitializationData(initializationData);
	
				// store the configuration properties in global vars with corresponding
				// names
				var configProperties = InitializationData.get("com.ibm.team.repository.service.configPropertyInitializer"); //$NON-NLS-1$
				for (var propName in configProperties) {
					var propVal = configProperties[propName];
					dojo.setObject(propName, propVal);
				}
				//Process all removalPropertyIds found in the Page Extensions.
				//Look for each property in the Initialization Data. If a
				//property exists convert into a global property so that 
				//future net.jazz.ajax.ui.internal.PageRegistry.getPages
				//calls will remove the required pages
				dojo.forEach(net.jazz.ajax.ui.internal.PageRegistry.getPages(), function(page) {
					if (page.removalPropertyId != null) {
						// TODO: Remove this dependency on repo.web
						var property = InitializationData.get(page.removalPropertyId);
						if (property != null && property.value != null) {
							dojo.setObject(page.removalPropertyId, property.value);
						}
					}
				});
				
				if (this._application && this._application.config) {
					this._setupApplication();
				}
			} catch (e) {
				document.title = messages.ErrorLoadingUser;
				var content = document.createElement("div"); //$NON-NLS-1$
				content.style.paddingBottom = "1.8em"; //$NON-NLS-1$
				var mainContent = document.createElement("div"); //$NON-NLS-1$
				mainContent.appendChild(document.createTextNode(messages.DatabaseError));
				var adminContent = document.createElement("div"); //$NON-NLS-1$
				adminContent.style.paddingTop = "8px"; //$NON-NLS-1$
				var adminWebUILink = dojo.string.substitute("<a href=\"${0}\">${1}</a>", [ net.jazz.ajax._contextRoot + "/admin", messages.JazzAdminUI]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				adminContent.innerHTML = dojo.string.substitute(messages.AdminInvestigate, [ adminWebUILink ]);
				content.appendChild(mainContent);
				content.appendChild(adminContent);
				if (this._onFailure) {
					this._onFailure(e,content,false);
				}
				return;
			}
			
			if (this._onSuccess) {
				this._onSuccess();
			}
		},

		_setupApplication: function() {
			var licenseInfoWidget = new LicenseInfoWidget();
			this._application.ui.bannerRightText.appendChild(licenseInfoWidget.domNode);
			
			if (this.contributor) {
				this._application.config.user = {
					name: this.contributor.name, 
					userId: this.contributor.userId,
					href: Session.getUserProfileLink(),
					menuProvider: dojo.hitch(this, this._userMenuProvider)
				};	
			}
			this._application.config.suppressedPages = getSuppressedPages(this._application.getCurrentWorkbenchId());
	
	//		if(getHomeUri()) {
	//			this._application.ui.navbarLeft.appendChild(getHomeLinkElement());
	//		}
			if (com.ibm.team.repository.web.logouturi) {
				this._application.config.logout = { href: com.ibm.team.repository.web.logouturi };
			}
			
			this._application.config.buttons.push({
				label: messages.HELP,
				iconSrc: dojo.moduleUrl("jazz.app", "graphics/JazzUI/icons/help.png"),
				align: "right",
				menuProvider: dojo.hitch(this,this._helpMenuProvider)
			});
		},

		_fetchInitDataFailure: function(errorObject) {
			if (errorObject.status !== 401) {
				var message = dojo.string.substitute(messages.initDataError, [errorObject.message]);
				alert(message);
				throw new Error(message);
			}
			
			var guestAccessIsAllowed= ws.allow.guest.access === "true"; //$NON-NLS-1$
			
			// TODO factor http/https logic into a common place
			var isHttp= location.href.match(/^https/) === null;
			
			var message= messages.invalidUserNamePassword;
			
			if (isHttp) {
				// if login failed in HTTP mode, redirect to HTTPS mode to force
				// the user to log in
				alert(message);
				window.location = "https://" + window.location.hostname + ":" + com.ibm.team.repository.server.repourl.port.https + window.location.pathname + window.location.search + window.location.hash; //$NON-NLS-1$ //$NON-NLS-2$
			} else if (guestAccessIsAllowed) {
				// if login failed in HTTPS mode and guest access is allowed,
				// redirect to HTTP mode
				alert(dojo.string.substitute(messages.guestUser, [ message ]));
				window.location = "http://" + window.location.hostname + ":" + com.ibm.team.repository.server.repourl.port.http + window.location.pathname + window.location.search + window.location.hash; //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				// otherwise just alert the user that login failed
				alert(message);
			}
			if (this._onFailure) {
				this._onFailure(errorObject);	
			}
			
		},
		_loadInitializationData: function(initializationData) {
			InitializationData._load(initializationData);
			
			// set the global var for the logged-in contributor
			this.contributor = InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer").currentContributor;
			dojo.setObject(
				"com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", //$NON-NLS-1$
				this.contributor);
			
			// if the user is the guest user and we're in HTTPS mode, display the
			// appropriate error
			if(isGuest() && window.location.href.match(/^https/)) {
				alert(messages.UserIdMismatch);
				return false; // early return
			}
			
			return true; // load succeeded
		},

		_helpMenuProvider: function() {
			var menu = new dijit.Menu();
			dojo.addClass(menu.domNode,"jazz-app-Banner-helpmenu");
				menu.addChild(new dijit.MenuItem({
					label: messages.HELP_CONTENT,
					iconClass: 'helpIcon',
					onClick: dojo.hitch(this,this._openHelpContents)
				}));

			var	welcome = new dijit.MenuItem({
		        	label: messages.WELCOME,
					iconClass: 'welcomeIcon',
		         	onClick: dojo.hitch(this, this._openWelcomePage, AppConstants.WELCOME)
		     	});
	     	menu.addChild(welcome);

			var about = dojo.string.substitute(messages.HELP_ABOUT, [this._application.productName]);
			menu.addChild(new dijit.MenuItem({
				label: about,
				iconClass: 'productIconSmall',
				onClick: dojo.hitch(this,this._openAboutBox)
			}));
			return menu;
		},
		
		_userMenuProvider: function() {
			var menu = new dijit.Menu();
			dojo.addClass(menu.domNode, "jazz-app-loginLogoutArea-userMenu");
			var menuItem = new dijit.MenuItem({
				label: messages.editProfile, 
				iconClass: 'userIcon',
				onClick: function() {
					if (PlatformUI.getWorkbench().shouldVetoNavigation() === false) {
						location.href = Session.getUserProfileLink();
					}
				}}
			);
			menu.addChild(menuItem);
			return menu;
		},	

		_openWelcomePage: function(event) {
			window.open(AppConstants.WELCOME_HREF);
		},
				
		_openHelpContents: function(event) {
			window.open(AppConstants.HELP_HREF);
		},

		_openAboutBox: function(){
			var about = dojo.string.substitute(messages.HELP_ABOUT, [this._application.productName]);
			new com.ibm.team.repository.web.ui.internal.about.AboutJazzTeamServer({dialogTitle: about});
		}
	};
	function getHomeUri() {
		if(com.ibm.team.repository.web.homeuri) {
			return com.ibm.team.repository.web.homeuri;
		}
		return null;
	}
	
	function getHomeLinkElement() {
		checkNotNull(getHomeUri(), "getHomeUri()");
		var homeLinkElement = document.createElement("a");
		homeLinkElement.href = getHomeUri();
		homeLinkElement.className = "home";
		homeLinkElement.innerHTML = messages.home;
		return homeLinkElement;
	}
	function sortPages(a, b, orderedPageIds) {
		for (var i=0; i<orderedPageIds.length; i++) {
			if(orderedPageIds[i] === a.id){
				return -1;
			}
			if(orderedPageIds[i] === b.id){
				return 1;
			}
		}
		return -1;
	}
	function getSuppressedPages (workbenchId) {
		if(!com.ibm.team.repository.web.suppressedPages) {
			return [];
		}
		
		var suppressedPageSets = dojo.fromJson(com.ibm.team.repository.web.suppressedPages);
		
		if(!dojo.isObject(suppressedPageSets)) {
			console.warn(dojo.string.substitute(messages.malformedValue, [ "com.ibm.team.repository.web.suppressedPages" ]) + ": " +
					dojo.string.substitute(messages.jsonError, [ com.ibm.team.repository.web.suppressedPages ]));
			return [];
		}
		var suppressedPageList = suppressedPageSets[workbenchId];
		if (!suppressedPageList) {
			return [];
		}
		var badArray = false;
		if (!dojo.isArray(suppressedPageList)) {
			badArray = true;
		} else {
			for(var i=0; i<suppressedPageList.length; i++) {
				if(!dojo.isString(suppressedPageList[i])) {
					badArray = true;
					break;
				}
			}	
		}
		if(badArray) {
			console.warn(dojo.string.substitute(messages.malformedValue, [ "com.ibm.team.repository.web.suppressedPages" ]) + ": " +
					dojo.string.substitute(messages.badSuppressedPagesArray, [ workbenchId, com.ibm.team.repository.web.suppressedPages ]));
			return [];
		}
		return suppressedPageList;
	}    
})();
