/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.timesheet.web.client.internal.TimesheetClient");
    
    dojo.require("com.ibm.epm.web.core.transport.EPMServerClient");
    dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
    
    var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
    var EpmServerClient = com.ibm.epm.web.core.transport.EPMServerClient;
    
    com.ibm.epm.timesheet.web.client.internal.TimesheetClient = {
    
        // The base URIs for each element.
        
        _USER_URI: "epm/resources/user",
        
        _taskAssignment: "/taskAssignment",
        
        _SCHEDULE_URI: "/schedule",
        
        _RESOURCE_URI: "epm/resources/",
        
        _TIMESHEET_URI: "timesheet",
        
        _TIMESHEET_VALUES_URI: "/timesheetvalues",
        
        _QUERY_TYPE_STRING: "type",
        
        _QUERY_TYPE_VALUE: "taskAssignment&execute=true",
		
		_QUERY_PROJ_ASSIGN_TYPE_VALUE: "projectAssignment&execute=true",
		
		_QUERY_JSON_EXPR:"&jsonExpression=",
        
        _QUERY_URI: "epm/query/",
		
		_getProjectAssignments: "epm/services/assignment/getProjectAssignments?expand=true&",
        
		getWeekEndInMilliSecs: function(week) {
			
			var weekEndDate = new Date(week.getTime());
            
            weekEndDate.setDate(week.getDate() + 6); // go to end of week
            
			weekEndDate.setDate(week.getDate() + 21); // provide additional 3 weeks of visibility
            
            return weekEndDate.getTime();	
					
		},
        
		getWeekBeginEveInMilliSecs: function(week) {
			
            var weekBeginEve = new Date(week.getTime());
            
            weekBeginEve.setDate(week.getDate() - 1); // One day before the week start day
			
			weekBeginEve.setDate(week.getDate() - 21); // provide additional 3 weeks of visibility
			
            return weekBeginEve.getTime();
						
		},
        
        /*
         * @params:
         * 		1. user: The userID of the user whose timesheet is required
         * 		2. week: It denotes the week for which the timesheet is required.
         *
         * The function takes these 2 arguments and prepare a JSON expression
         * which can be queried at the server side.
         */
        getJSONExpression: function(user, week){
        
            /*
             * Display if:
             if (user == <USER_VAL> AND
             (
             (if ActualStartDate == EMPTY AND ActualFinishDate == EMPTY) OR
             (if ActualStartDate <= <SELECTED_WEEK_END_DAY> AND ActualFinishDate == EMPTY) OR
             (if ActualStartDate <= <SELECTED_WEEK_END_DAY> AND ActualFinishDate >= <SELECTED_WEEK_FIRST_DAY>)
             )
             )
             
             */
            var weekEndInMilliSecs = this.getWeekEndInMilliSecs(week);
            
            var weekBeginEveInMilliSecs = this.getWeekBeginEveInMilliSecs(week);
            
            // XXX - Instead of Jan 1, 1970, add one day to make sure
            // timezone differences are accounted for
            //
            // Real fix would be to store only dates in RTC repository
            // and not date/times.
            var EPOCH = 1000 * 60 * 60 * 24;
			
			var paramUser = this.getParamUser(user);
            
            var jsonExpressionVal =
              new TermExpression(
                  new AttributeExpression("selfURI", "contains", paramUser),
                  new TermExpression([], // attributeExpressions
                    [new TermExpression(
                        [new AttributeExpression("actualSchedule/startDate",
                                                 "before", EPOCH),
                         new AttributeExpression("actualSchedule/finishDate",
                                                 "before", EPOCH)]),
                    new TermExpression(
                        [new AttributeExpression("actualSchedule/startDate",
                                                 "before", weekEndInMilliSecs),
                         new AttributeExpression("actualSchedule/finishDate",
                                                 "before", EPOCH)]),
                    new TermExpression(
                        [new AttributeExpression("actualSchedule/startDate",
                                                 "before", weekEndInMilliSecs),
                         new AttributeExpression("actualSchedule/finishDate",
                                                 "after", weekBeginEveInMilliSecs)])],
                   [], // similarityExpressions
                   "OR"));
            
            //        // for debugging purposes, dump string
            //        alert(jsonExpressionVal.toString());
            
            return jsonExpressionVal.toString();
        },
		
		getParamUser: function(user) {
			
			return user + this._taskAssignment;
			
		},
		
        getJSONExpressionForProjMgr: function(user, week, projectList){
        
            /*
             * Display if:
             if (user == <USER_VAL> AND task.containing project is [proj1 or proj2] AND
             (
             (if ActualStartDate == EMPTY AND ActualFinishDate == EMPTY) OR
             (if ActualStartDate <= <SELECTED_WEEK_END_DAY> AND ActualFinishDate == EMPTY) OR
             (if ActualStartDate <= <SELECTED_WEEK_END_DAY> AND ActualFinishDate >= <SELECTED_WEEK_FIRST_DAY>)
             )
             )
             
             */
            var weekEndInMilliSecs = this.getWeekEndInMilliSecs(week);
            
            var weekBeginEveInMilliSecs = this.getWeekBeginEveInMilliSecs(week);

            // XXX - Instead of Jan 1, 1970, add one day to make sure
            // timezone differerences are accounted for
            //
            // Real fix would be to store only dates in RTC repository
            // and not date/times.
            var EPOCH = 1000 * 60 * 60 * 24;
            
            //      projectList = "\"IPv6 VPN Client\"";
			
			var paramUser = this.getParamUser(user);
            
            var jsonExpressionVal =
              new TermExpression(
                  [new AttributeExpression("selfURI", "contains", paramUser),
                   new AttributeExpression(
               "link:com.ibm.epm.assignment.taskLink:source/projectArea/itemId",
               "is",
               projectList)],
                  new TermExpression(
                      [], // attributeExpressions
                      [new TermExpression(
                          [new AttributeExpression("actualSchedule/startDate",
                                                   "before", EPOCH),
                           new AttributeExpression("actualSchedule/finishDate",
                                                     "before", EPOCH)]),
                    new TermExpression(
                        [new AttributeExpression("actualSchedule/startDate",
                                                 "before", weekEndInMilliSecs),
                         new AttributeExpression("actualSchedule/finishDate",
                                                 "before", EPOCH)]),
                    new TermExpression(
                        [new AttributeExpression("actualSchedule/startDate",
                                                 "before", weekEndInMilliSecs),
                         new AttributeExpression("actualSchedule/finishDate",
                                                 "after", weekBeginEveInMilliSecs)])],
                     [], // similarityExpressions
                     "OR"));
            
            //        // for debugging purposes, dump string
            //        alert(jsonExpressionVal.toString());
            
            return jsonExpressionVal.toString();
        },
        
        /*
         * Get the list of all the projects under the specified project area.
         * TODO: presently project area name is hard coded.
         */
        getAllProjects: function(responseHandler, method){
        
            var serviceRequest = new ServiceRequest("epm/resources/projects", method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /*
         * Get the project assignments corresponding to the project href passed
         */
        getProjectAssignment: function(responseHandler, href, method){
            var serviceRequest = new ServiceRequest("epm/resources/" + href + "/projectAssignment", method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /*
         * Get the project assignment details corresponding to the project href passed
         */
        getProjectAssignmentDetail: function(responseHandler, href, method){
            var serviceRequest = new ServiceRequest("epm/resources/" + href, method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /*
         * It returns all the task assignment of a particular user.
         */
        getAllTaskAssignments: function(responseHandler, user, /*Date*/ week, projectList, isProjectManager, method){
        
            var jsonExpression = "";
            if (isProjectManager) {
                jsonExpression = this.getJSONExpressionForProjMgr(user, week, projectList);
            }
            else {
                jsonExpression = this.getJSONExpression(user, week);
            }
            
            
            var getURI = this._QUERY_URI;
            
            getURI += "?" + this._QUERY_TYPE_STRING + "=" + this._QUERY_TYPE_VALUE + this._QUERY_JSON_EXPR; //"?queryType=com.ibm.epm.resource.epmAssignmentType&execute=true&jsonExpression=";
            getURI += jsonExpression;
            
            // TODO: CLEAN this up
            getURI += '&';
            
            //		var serviceRequest= new ServiceRequest(
            //					this._USER_URI + "/" + user + this._taskAssignment,
            //					method);
            
            var serviceRequest = new ServiceRequest(getURI, method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getTaskAssignmentDetails: function(responseHandler, taskAssignmentURI, method){
        
            var serviceRequest = new ServiceRequest(this._RESOURCE_URI + taskAssignmentURI, method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getTaskDetail: function(responseHandler, taskURI, method){
        
            var serviceRequest = new ServiceRequest(this._RESOURCE_URI + taskURI, method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getTimesheetDetail: function(responseHandler, timesheetURI, method){
        
            var serviceRequest = new ServiceRequest(this._RESOURCE_URI + timesheetURI, method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getTimesheetValues: function(responseHandler, timesheetURI, method){
        
            var serviceRequest = new ServiceRequest(this._RESOURCE_URI + timesheetURI, method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        
        saveTimesheet: function(responseHandler, timesheetURI, dataToSave, method){
        
            var serviceRequest = new ServiceRequest(timesheetURI, method, dataToSave);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        saveTimesheetValues: function(responseHandler, timesheetURI, dataToSave, method){
        
            var serviceRequest = new ServiceRequest(timesheetURI, method, dataToSave);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getParentDetails: function(responseHandler, parentURI, method){
        
            var serviceRequest = new ServiceRequest(this._RESOURCE_URI + parentURI, method);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
		
		getUserQueryURI: function(user, week){
			
			var jsonExpression = this.getJSONExpression(user, week);
            
			var queryURI = this._QUERY_URI;
            
            queryURI += "?" + this._QUERY_TYPE_STRING + "=" + this._QUERY_TYPE_VALUE + this._QUERY_JSON_EXPR; //"?queryType=com.ibm.epm.resource.epmAssignmentType&execute=true&jsonExpression=";
            queryURI += jsonExpression;
            
            // TODO: CLEAN this up
            queryURI += '&';
			
            return queryURI;
		},
		
		getApproverQueryURI: function(user, week, projectList){
			
			var jsonExpression = this.getJSONExpressionForProjMgr(user, week, projectList);
            
			var queryURI = this._QUERY_URI;
            
            queryURI += "?" + this._QUERY_TYPE_STRING + "=" + this._QUERY_TYPE_VALUE + this._QUERY_JSON_EXPR; //"?queryType=com.ibm.epm.resource.epmAssignmentType&execute=true&jsonExpression=";
            queryURI += jsonExpression;
            
            // TODO: CLEAN this up
            queryURI += '&';
			
            return queryURI;
		},
		
		getUserQueryURIForPost: function(user, week){
			
			var jsonExpression = this.getJSONExpression(user, week);
            
			var queryURI = this._QUERY_URI;
            
            queryURI += "?" + this._QUERY_TYPE_STRING + "=" + this._QUERY_TYPE_VALUE ; //?queryType=com.ibm.epm.resource.epmAssignmentType&execute=true
            
			queryURI += '&';
			
            return queryURI;
		},
		
		getApproverQueryURIForPost: function(user, week, projectList){
			
			var jsonExpression = this.getJSONExpressionForProjMgr(user, week, projectList);
            
			var queryURI = this._QUERY_URI;
            
            queryURI += "?" + this._QUERY_TYPE_STRING + "=" + this._QUERY_TYPE_VALUE ; 
			
			queryURI += '&';
			
            return queryURI;
		},
		
		// TODO: checking in now, delete the next time this file is changed as this may be required for future reference
//		getProjectAssignmentQueryURIForPost: function(){
//			
//			var queryURI = this._QUERY_URI;
//            
//            queryURI += "?" + this._QUERY_TYPE_STRING + "=" + this._QUERY_PROJ_ASSIGN_TYPE_VALUE; 
//			
//			queryURI += '&';
//			
//            return queryURI;
//		},
//		
//        getJSONExpressionForProjAssign: function(projectList){
//        
//            /*
//             * {"attributeExpressions": [{"attributeId": 
//             * "link:com.ibm.epm.assignment.projectResourceLink:target", 
//             * "operator":"link.exists","values":[]}, 
//             * {"attributeId": "link:com.ibm.epm.assignment.projectLink:source/projectArea/itemId", 
//             * "operator": "is", "values": ["_N3JN4qgOEd6tKaDzKefeoQ"], "variables": []}], 
//             * "termExpressions": [], "similarityExpressions": [], "operator": "AND"}
//             */
//			
//            //      projectList = "\"IPv6 VPN Client\"";
//			
//            var jsonExpressionVal =
//		       new TermExpression(
//		        [new AttributeExpression(
//		        "link:com.ibm.epm.assignment.projectResourceLink:target",
//		        "link.exists"),
//		        new AttributeExpression(
//		        "link:com.ibm.epm.assignment.projectLink:source/projectArea/itemId",
//		        "is",
//		        projectList)]);
//			               
//            return jsonExpressionVal.toString();
//        },
				
        /*
         * Get the project assignments corresponding to the project list
         */
        getProjectAssignmentUsingQuery: function(responseHandler, projectList, method){
			
			var queryURI = this._getProjectAssignments;
			
			var args = {};
			
			args.projectAreaItemIds = [];
			
			for (var i = 0; i < projectList.length; i++) {
		
				args.projectAreaItemIds.push(projectList[i]);
		
			}
			
            var serviceRequest = new ServiceRequest(queryURI, method, args);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        }
        
    };
    
    /**
     * Contstruct attribute query expression JSON object representing some
     * mathematical or logical sub-expression against a ECore model
     * EAttribute.<p>
     *
     * Overridden toString() method will stringify the object into a
     * form that can be passed over the wire to the server.
     *
     * @param {Object} attributeId EAttribute XPath expression
     * @param {Object} operator logical or mathematical operator
     * @param {Object} values optional constant operands
     * @param {Object} variables optional EAttribute operands
     */
    function AttributeExpression(attributeId, operator, values, variables){
        /**
         * Return array representation of val
         *
         * @param {Object} val object to be turned into an Array object of
         * its contents
         */
        var arrayify = function(val){
            if (typeof(val) == "undefined") {
                return [];
            }
            else 
                if (!(val instanceof Array)) {
                    return [val];
                }
            
            return val;
        };
        
        this.attributeId = attributeId;
        this.operator = operator;
        this.values = arrayify(values);
        this.variables = arrayify(variables);
        
        /**
         * Override the built-in Object.toString() method to generate
         * stringified representation of JSON object for sending over
         * the wire to server.
         *
         * @returns string representation of JSON query object
         */
        this.toString = function(){
            /**
             * Stringify the contents of an array object
             *
             * @param {Object} array the array
             */
            function dumpArray(array){
                var str = "";
                
                for (var i = 0; i < array.length; i++) {
                
                    if (str.length > 0) {
                        str += ", ";
                    }
                    str += '"' + array[i].toString() + '"';
                }
                
                return str;
            }
            
            return '{"attributeId": "' + this.attributeId + '",' +
                   ' "operator": "' + this.operator + '",' +
                   ' "values": [' + dumpArray(this.values) + '],' +
                   ' "variables": [' + dumpArray(this.variables) + ']}';
        };
    }
    
    /**
     * Contstruct a logical query expression JSON object representing some
     * one or more sub-expressions that can be either themselves
     * TermExpression's or AttributeExpression's.<p>
     *
     * Overridden toString() method will stringify the object into a
     * form that can be passed over the wire to the server.
     *
     * @param {Object} attributeExpressions optional array of attribute
     * query sub-expressions
     * @param {Object} termExpressions optional array of logical query
     * sub-expressions
     * @param {Object} similarityExpressions optional array of
     * similarity (???) sub-expressions
     * @param {Object} operator operator that links all the sub-expressions;
     * defaults to AND
     */
    function TermExpression(attributeExpressions, termExpressions, similarityExpressions, operator){
        /**
         * Return array representation of val
         *
         * @param {Object} val object to be turned into an Array object of
         * its contents
         */
        var arrayify = function(val){
            if (typeof(val) == "undefined") {
                return [];
            }
            else 
                if (!(val instanceof Array)) {
                    return [val];
                }
            
            return val;
        };
        
        this.attributeExpressions = arrayify(attributeExpressions);
        this.termExpressions = arrayify(termExpressions);
        this.similarityExpressions = arrayify(similarityExpressions);
        this.operator = (typeof(operator) == "undefined") ? "AND" : operator;
        
        /**
         * Override the built-in Object.toString() method to generate
         * stringified representation of JSON object for sending over
         * the wire to server.
         *
         * @returns string representation of JSON query object
         */
        this.toString = function(){
            /**
             * Stringify the contents of an array object
             *
             * @param {Object} array the array
             */
            function dumpArray(array){
                var str = "";
                
                for (var i = 0; i < array.length; i++) {
                
                    if (str.length > 0) {
                        str += ", ";
                    }
                    
                    var item = array[i];
                    
                    if (typeof(item) == "object") {
                        str += item.toString();
                    }
                    else {
                        str += '"' + item.toString() + '"';
                    }
                }
                
                return str;
            }
            
            return '' +
  '{"attributeExpressions": [' + dumpArray(this.attributeExpressions) + '],' +
  ' "termExpressions": [' + dumpArray(this.termExpressions) + '],' +
  ' "similarityExpressions": [' + dumpArray(this.similarityExpressions) + '],' +
  ' "operator": "' + this.operator + '"}';
        };
    }
    
})();
