/********************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.admin.web.client.internal.calendar.CalendarClient");
    
    dojo.require("com.ibm.epm.web.core.transport.EPMServerClient");
    dojo.require("com.ibm.epm.web.core.transport.ServiceRequest");
    
    var ServiceRequest = com.ibm.epm.web.core.transport.ServiceRequest;
    var EpmServerClient = com.ibm.epm.web.core.transport.EPMServerClient;
    
    com.ibm.epm.admin.web.client.internal.calendar.CalendarClient = {

        _BASE_URI: "epm/resources/",
        _CALENDAR_URI: "epm/resources/calendars",
        _QUERY_PREDICATE_1: "epm/query/?type=",
        _QUERY_PREDICATE_2: "&execute=true&jsonExpression=",
        
        /**
         * Performs GET operation on given URI
         */
        getURI: function(responseHandler, uri){
            var serviceRequest = new ServiceRequest(this._BASE_URI + uri, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
       
        /**
         * Performs POST operation on given URI
         */
        createURI: function(uri, responseHandler, args){
            var serviceRequest = new ServiceRequest(this._BASE_URI + uri, "post", args);
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /**
         * Performs PUT(POST) operation on given URI
         */
        updateURI: function(uri, responseHandler, args){
            var serviceRequest = new ServiceRequest(this._BASE_URI + uri, "post", args);
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /**
         * Performs DELETE operation on given URI
         */
        deleteURI: function(responseHandler, uri){
            var serviceRequest = new ServiceRequest(this._BASE_URI + uri, "delete");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /**
         * Executes a given query on a given type of resource
         */
        executeQuery: function(responseHandler, type, jsonExpression){
            var serviceRequest = new ServiceRequest(this._QUERY_PREDICATE_1 + type 
            	+ this._QUERY_PREDICATE_2 + jsonExpression + '&', "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /**
         * Gets all admin calendars
         */
        getAllCalendars: function(responseHandler){
            var serviceRequest = new ServiceRequest(this._CALENDAR_URI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        }
    };
})();
