/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.AbstractGridController");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.DojoData");
    
    dojo.require("dojo.data.ItemFileWriteStore");
    
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
	
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
	
	//localization
	dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages");
    
	var CoreMessages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages");
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    
	var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;

    /**
     * The model must have functions:
     * getEmptyRow(rowID, collectionItemHandle),
     * updateRowData(row, loadedElement),
     * getRowGetURI(collectionItemHandle, rowID),
     *
     * The Viewer must have:
     * createView(this._gridData),
     * rowUpdated(row, rowID, rowIndex),
     * newRowInserted(row, rowID, rowIndex),
     * removeRows(removedRows)
     */
    dojo.declare("com.ibm.epm.web.core.grid.controller.AbstractGridController", [dijit._Widget], {
        _isGridLoading: false,
        _viewers: null,
        _gridModel: null,
        _linksToBeLoaded: null,
        _gridData: null,
        newElementPrefix: "_unsaved",
        isDeleted: false,
        lastUniqueRowID: 1,
		dialog: null,
		sortableFields :{},
        
        initializeController: function(dummy, viewer, model, linksToBeLoaded){
            if (viewer) {
                if (this._viewers == null) {
                    this._viewers = new Array(1);
                }
                this._viewers.push(viewer);
            }
            else {
                throw "Cannot create the controller without the default Viewer.";
            }
            if (model) {
                this._gridModel = model;
            }
            this._linksToBeLoaded = linksToBeLoaded;
                        
            // Initialize the number of save requests
            this.saveRequests = 0;
        },
        
        isInitialized: function(){
            var initialized = false;
            if (this._viewers != null &&
            this._gridModel != null            /*&& this._gridData != null*/
            ) {
                initialized = true;
            }
            return initialized;
        },
        
        destroy: function(){
            console.log("destroying the controller with id : " + this.id);
            if (this._gridData && this._gridData.destroy) {
                this._gridData.destroy();
                delete this._gridData;
            }
            dojo.unsubscribe(this.replaceNodeEventHandler);
            this.inherited(arguments);
        },
        
        postCreate: function(){
            this.replaceNodeEventHandler = dojo.subscribe(GridEvents.REPLACE_NODE, this, "replaceNode");
        },
        
        /**
         * Updates the Grid model by adding all the item handle to the model.
         * @param {boolean} removeNodes			: if true will make sure that the model only has the rows that are in the collectionItemHandles.
         * 								else does not remove any rows.
         * @param {Array} collectionItemHandles : The Array starting from index 1. Item handle consisting of the minimum details 
         * 								to add an empty row i.e. the row id. 
         */
		updateModel: function(removeNodes, collectionItemHandles){
            if (collectionItemHandles) {
                this._addAll(collectionItemHandles);
                if (removeNodes == true) {
                    this._removeAdditionalRows(collectionItemHandles);
                }
            }
        },
        
        _removeAdditionalRows: function(collectionItemHandles){
            if (this._gridData == null || (collectionItemHandles != null && collectionItemHandles.length == 0)) {
                return;
            }
            else {
                var gridRows = new Array(1);
                for (var i = 0; i < this._gridData.getRowCount(); i++) {
                    if (this._gridData.getRow) {
                        if (this._gridData.getRow(i)) {
                            gridRows.push(this._gridData.getRow(i));
                        }
                    }
                    
                }
                var removedRows = new Array(1);
				var grid = this._getViewGrid();
				if(grid && grid.beginUpdate){
					grid.beginUpdate();
				}
                for (var k in gridRows) {
                    var gridRowID = gridRows[k].id;
                    var removeRow = true;
                    for (var j in collectionItemHandles) {
                        var collectionItemHandle = collectionItemHandles[j];
                        var collectionRowID = this._getRowID(collectionItemHandle);
                        if (collectionRowID == gridRowID) {
                            removeRow = false;
                            break;
                        }
                    }
                    if (removeRow == true) {
                        this._removeGridRow(gridRowID);
                        removedRows.push(gridRowID);
                        //		            	this._hideGridRow(gridRowID);
                    }
                }
				if(grid && grid.endUpdate){
					grid.endUpdate();
				}
                this._propogateRowsRemoved(removedRows);
            }
        },
        
        _getRowID: function(collectionItemHandle){
            var rowID = null;
            if (collectionItemHandle.relativeHref) {//flat grid.
                rowID = collectionItemHandle.relativeHref;
            }
            else 
                if (collectionItemHandle.identifier) {// tree grid.
                    rowID = collectionItemHandle.identifier;
                }
            //console.log(collectionItemHandle);
            return rowID;
        },

        /*
         * When creating a new row, call this to get a row ID that is
         * guaranteed to be unique. Looks in existing grid data (saved and
         * unsaved) to be sure the ID is unique.
         */
        getUniqueRowID: function(prefix) {
            var rowID = prefix;
            while (this._gridData._rowIdentities[rowID]) {
                rowID = prefix + this.lastUniqueRowID;
                this.lastUniqueRowID++;
            }
            return rowID;
        },
        
        _addAll: function(collectionItemHandles){
            if (!collectionItemHandles || collectionItemHandles.length == 0) {
                return;
            }
			if(this._isGridLoading == true){
				console.log("controller is buzy building the model");
				return;
			}
			
			this._isGridLoading = true;
            var addRow = true;
            if (!this._gridData) {
                addRow = false;
                var collectionItemHandle = collectionItemHandles[1];
                if (collectionItemHandle) {
                    var rowID = this._getRowID(collectionItemHandle);
                    var row = this._getEmptyRow(rowID, collectionItemHandles[1]);
                    this._createModel(row);
                    var self = this;
                    var cnt = 0;
                    var intervalID = setInterval(function(){
                        console.log("inside set interval " + self.id);
                        if (cnt == 0) {// Wait till the model is built.
                            try{
								self._propogateCreateModel();
							} catch(exception){
								//Clearing the interval to avoid infinite loop
								//if any exception occurs
								clearInterval(intervalID);
							}
                        }
                        else 
                            if (cnt > 1) {//Add delay to get the rendering done.
                                clearInterval(intervalID);
                                self._propogateInsertElement(row, rowID, 0);
                                self._getAndUpdateRow(rowID, rowID, null);
                                collectionItemHandles[1] = null;
                                self._insertElements(collectionItemHandles, addRow);
                            }
                        cnt = cnt + 1;
                    }, 25);
                }
            }
            else {
                this._insertElements(collectionItemHandles, addRow);
            }
			this._isGridLoading = false;
        },
        
        _insertElements: function(collectionItemHandles, addRow){
			var grid = this._getViewGrid();
			if(grid && grid.beginUpdate){
				grid.beginUpdate();
			}
            for (var j in collectionItemHandles) {
                var collectionItemHandle = collectionItemHandles[j];
                var index = parseInt(j, 10);
                if (collectionItemHandle != null) {
                    this._insertElement(collectionItemHandle, index, addRow);
                }
            }
			if(grid && grid.endUpdate){
				grid.endUpdate();
				var publishObj = {
                	controllerID: null,
                	uniqueIdentifier: null
				} 
				if (this.epmUniqueID) {
	                publishObj.uniqueIdentifier = this.epmUniqueID;
	            }
	            if (this.id) {
	                publishObj.controllerID = this.id;
	            }
				dojo.publish(GridEvents.LOADING_COMPLETED, [publishObj]);
			}
        },
        
        _insertElement: function(collectionItemHandle, index, addRow){
            var rowID = this._getRowID(collectionItemHandle);
			if (this._gridData && this._gridData._rowIdentities) {
				if (!this._gridData._rowIdentities[rowID]) {
					var row = this._getEmptyRow(rowID, collectionItemHandle);
					try {
						if (addRow == true) {
							this._addRow(row, index - 1);
						}
						else {
							this._gridData.store.newItem(row);
						}
						this._gridData.store.save();
						var self = this;
						//					var intervalID = setInterval(function(){
						if (self._gridData._rowIdentities[rowID]) {
							//							clearInterval(intervalID);
							self._propogateInsertElement(row, rowID, index - 1);
							var rowURI = self._getRowGetURI(collectionItemHandle, rowID);
							self._getAndUpdateRow(rowURI, rowID, null);
						}
					//					}, 100);
					} 
					catch (e) {
						//TODO have to handle the new element insertion neatly.
						console.log("Error adding new Item to the grid data for index:" + (index - 1));
						console.log(e);
						if (!dojo.isIE) {
							console.trace();
						}
					}
				}
			}
        },
        
		/**
		 * Method to refresh a set of rows. <br>
		 * @param {Object} collectionItemHandles
		 */
        refreshSetOfRows: function(collectionItemHandles){
			for (var i in collectionItemHandles) {
				var rowID = this._getRowID(collectionItemHandles[i]);
				var rowURI = this._getRowGetURI(collectionItemHandles[i], rowID);
				if (rowURI && rowID) {
					rowURI = dojo.isArray(rowURI)?rowURI[0]:rowURI;
					this._getAndUpdateRow(rowURI, rowID, null);
				}
			}
		},
        /*
         * Delete Row Funtion
         */
        _deleteElement: function(rowIndex){
            var self = this;
            if (rowIndex < 0) {
                return;
            }
            var item = self._gridData.getRow(rowIndex);
			
            var publishObj = {
                controllerID: null,
                uniqueIdentifier: null,
                rowid: item.id
            };
            if (this.epmUniqueID) {
                publishObj.uniqueIdentifier = this.epmUniqueID;
            }
            if (this.id) {
                publishObj.controllerID = this.id;
            }                
            var self = this;
            if (item.id == null) {
                return;
            }
            var serviceObj = {
                success: function(data){
                    // Remove the Row after Deletion
                    self._removeGridRow(item.id);
					dojo.publish(GridEvents.ROW_DELETED, [publishObj]);
                },
                
                failure: function(errorObj){
                    //self._displayErrorMessage("Error while Deleting Skill - " + errorObj.message);
					self._propogateErrorMessage(errorObj);					
                	if(item.markedForDeletion) {
                		item.markedForDeletion = false;
                	}               
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            
            // Get the Delete call URI
            // Delete URI wud be same as relatedHref
            var deleteURI = item.id;
            if (deleteURI.indexOf("unsaved") != -1) {
                // Remove unsaved row
                item.markedForDeletion = true;
                self._removeGridRow(item.id);
				dojo.publish(GridEvents.ROW_DELETED, [publishObj]);
            }
            else {
                gridModelClient.deleteURI(srh, deleteURI);
            }           
        },
        
        /*
         * Delete of row
         * Incoming : row object
         */
        _deleteElementByRow: function(row) {
			// Get the Delete call URI
            // Delete URI wud be same as relatedHref
            var deleteURI = dojo.isArray(row.id)? row.id[0]:row.id;
            var publishObj = {
                controllerID: null,
                uniqueIdentifier: null,
                rowid: deleteURI
            };
            if (this.epmUniqueID) {
                publishObj.uniqueIdentifier = this.epmUniqueID;
            }
            if (this.id) {
                publishObj.controllerID = this.id;
            }                
            var self = this;
            if (row.id == null) {
                return;
            }
            var serviceObj = {
                success: function(data){
                    // Remove the Row after Deletion
                    self._removeGridRow(deleteURI);
					dojo.publish(GridEvents.ROW_DELETED, [publishObj]);
                },
                
                failure: function(errorObj){
                    //self._displayErrorMessage("Error while Deleting Skill - " + errorObj.message);
					self._propogateErrorMessage(errorObj);					
                	if(row.markedForDeletion) {
                		row.markedForDeletion = false;
                	}               
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            
            
            if (deleteURI.indexOf("unsaved") != -1) {
                // Remove unsaved row
                row.markedForDeletion = true;
                self._removeGridRow(deleteURI);
				dojo.publish(GridEvents.ROW_DELETED, [publishObj]);
            }
            else {
                gridModelClient.deleteURI(srh, deleteURI);
            }           
		},
		
        _getEmptyRow: function(rowID, collectionItemHandle){
            if (this._gridModel.getEmptyRow) {
                var row = this._gridModel.getEmptyRow(rowID, collectionItemHandle);
                // Give the views an option to add data to the empty row (if any)                
                this._propagateMassageEmptyRow(collectionItemHandle, row);
                return row;
            }
            else {
                throw "invalid model: function _getEmptyRow (rowID, collectionItemHandle) is undefined";
            }
        },
        
        _getRowGetURI: function(collectionItemHandle, rowID){
            if (this._gridModel.getRowGetURI) {
                return this._gridModel.getRowGetURI(collectionItemHandle, rowID);
            }
            else {
                throw "invalid controller  getRowGetURI(collectionItemHandle, rowID) method is undefined";
            }
        },
		
		_getRowGetUri_Link:function(loadedElement, links, linkName){
			if (this._gridModel.getRowGetUri_Link) {
                return this._gridModel.getRowGetUri_Link(loadedElement, links, linkName);
            }else if (links){
				var link = links;
                if (dojo.isArray(links)) {
                    link = links[0];
                }
                return link.relativeHref; 
            }else{
				return null;
			}
		},
		
		refreshGrid: function(){
            for (var i = 0; i < this._gridData.getRowCount(); i++) {
                if (this._gridData.getRow) {
                    var row = this._gridData.getRow(i);
                    var rowID = row.id;
                    if (row && rowID) {
                        this._getAndUpdateRow(rowID, rowID, null);
                    }
                }
            }
        },
		
		/**
		 * removeUnsavedRows: remove the items published by tree. <br>
		 * Only if the row is removed propogate the change 
		 */
		removeUnsavedRows: function(collectionElementHandles){
			for(var i in collectionElementHandles){
				var rowID = this._getRowID(collectionElementHandles[i]);
				if(this._removeGridRow(rowID))
				  {
					this._propogateRowsRemoved([rowID]);
				  }
			}
		},
		
		/**
		 * removeUnsavedRows: remove the unsaved row from grid.
		 */
		removeAllUnsavedRows: function(){
	        var _gridDataStore = this._gridData.store;
			var self = this;
	        if (_gridDataStore) {
				function completed(rows, findResult){
				  var rowsToBeDeleted = new Array();
				  for(var i = 0; i < rows.length; i++){
					var rowID = dojo.isArray(rows[i].id)?rows[i].id[0]:rows[i].id;
					if((rowID.indexOf(self.newElementPrefix)) > -1){
						rowsToBeDeleted.push(rows[i]);						
					}
				  }
				  var grid = self._getViewGrid();
				  if(grid && grid.beginUpdate){
						grid.beginUpdate();
				  }
				  for(var i = 0; i < rowsToBeDeleted.length; i++){
						_gridDataStore.deleteItem(rowsToBeDeleted[i]);					
				  }
				  if(grid && grid.endUpdate){
					grid.endUpdate();
				  }
				_gridDataStore.save();
				}
				function error(errData, request){
				  console.log("Failed in fetching the data.");
				}
				//Invoke the fetch.
				_gridDataStore.fetch({onComplete: completed, onError: error});
	        }
		},

        /**
         * Saves the unsaved data in the grid.
         * @param {boolean} synchronizedSave Optioanal: flag to inform controller weather to make the save calls 
         * one after another or in parallel. If not sent the save will be in parallel.
         * @param {boolean} lockPage  - if true, locks the page during save
         */
        persistGrid: function(synchronizedSave, lockPage){
			lockPage = (lockPage==false) ? lockPage : true;
			var sequential = false;
			if(synchronizedSave){
				sequential = synchronizedSave;
			}
			var rowsToBeSaved = this.getModifiedRows();
			var modifiedRowsCount = rowsToBeSaved.length; 
			if (modifiedRowsCount > 0) {
				// Initialize the number of potential save requests
				if (sequential) {
					// For sequential saves, we are estimating how many
					// POST requests will be sent (the actual number may
					// vary if some of the conditions fail);
					// Hence initialize to our best guess now.
				    this.saveRequests = modifiedRowsCount - 1;
				} else {
					// For parallel saves, we will know the exact number
					// of POST requests sent from each call to persistRow();
					// Hence initialize to zero for now.
					this.saveRequests = 0;
				}    

				if ( (lockPage == true) && (modifiedRowsCount > 1)) {
					Util._lockPage(CoreMessages.SAVING);
				}
				sequential? this._persistSequentially(rowsToBeSaved, 1, modifiedRowsCount):
							this._persistInParallel(rowsToBeSaved, modifiedRowsCount);    			
			}
        },
		
		/**
         * Get the modified rows.
         */
		getModifiedRows: function() {
			var rowsToBeSaved = new Array(1);
            for (var i = 0; i < this._gridData.getRowCount(); i++) {
				if (this._gridData.getRow) {
					var row = this._gridData.getRow(i);
					if (row) {
						var saveObject = this._gridModel.getSaveObject(row);
						if (saveObject != null) {
							var rowToBeSaved = {
								saveObject: saveObject,
								row: row,
								rowIndex: (i + 1)
							};
							rowsToBeSaved.push(rowToBeSaved);
						}
						// The main resource might not be modified but the link might be
						// modified hence saving the links. The links save will not be sequential.
						this._saveLinks(null, row);
					}
				}
			}
			return rowsToBeSaved;
		},
		
		/**
		 * Makes the persist row call to all the elements in the rows to be saved array.
		 * @param {Array} rowsToBeSaved: the array of objects. Each object will have:
		 * 					saveObject, row, rowIndex.
		 * @param {int} modifiedRowsCount: the number of items in the rowsToBeSaved.
		 */
		_persistInParallel: function(rowsToBeSaved, modifiedRowsCount){
			for(var j in rowsToBeSaved){
				var rowToBeSaved = rowsToBeSaved[j];
				this._persistRow(rowToBeSaved.saveObject, rowToBeSaved.row, rowToBeSaved.rowIndex, null);
				if (j == modifiedRowsCount -1) {
					Util._releasePage();
				}
			}
		},
		
		/**
		 * Recursive function. recursion end condition is either the  currentRowIndex > modifiedRowsCount
		 * or rowsToBeSaved[currentRowIndex] is undefined.
		 * Will call itself on the sucess or failure of the save.
		 * Will not abort the save of other element if one is failed.
		 * @param {Array} rowsToBeSaved: the array of objects. Each object will have:
		 * 					saveObject, row, rowIndex.
		 * @param {int} currentRowIndex: the current index in the rowsToBeSaved array that needs to be persisted.
		 * @param {int} modifiedRowsCount: the number of items in the rowsToBeSaved.
		 */
		_persistSequentially: function(rowsToBeSaved, currentRowIndex, modifiedRowsCount){
			if(currentRowIndex > modifiedRowsCount || !rowsToBeSaved[currentRowIndex]){
				// Check if this condition was true in the ultimate (or initial) call
				if (this.saveRequests == (modifiedRowsCount - 1)) {
					// Indicate to all viewers that save is complete
				    this._propagateSaveComplete();	
				}
				Util._releasePage();
				return;				
			}
			var rowToBeSaved = rowsToBeSaved[currentRowIndex];
			var saveObject = rowToBeSaved.saveObject;
			var row = rowToBeSaved.row;
			var rowIndex = rowToBeSaved.rowIndex;
		    var rowID = row.id;
            var self = this;
            var reloadURI = saveObject.rowURI;
			
			this._updateSaveProgressTitle(row, currentRowIndex, modifiedRowsCount);
			
            var persistRowFunctions = {
                _persistRowSuccess: function(savedElement){					
					self._handleSavedElement(savedElement, saveObject, row, rowIndex, reloadURI, null);
					currentRowIndex = currentRowIndex + 1;
					self._persistSequentially(rowsToBeSaved, currentRowIndex, modifiedRowsCount);					
                },
                
                _persistRowFailure: function(errorMsg){					
					self._handleSaveError(errorMsg, saveObject, row, rowIndex, null, reloadURI);
					currentRowIndex = currentRowIndex + 1;
					self._persistSequentially(rowsToBeSaved, currentRowIndex, modifiedRowsCount);	                   
                }
            };
            
            var srh = new ServiceResponseHandler(persistRowFunctions, "_persistRowSuccess", "_persistRowFailure");
            if (saveObject && saveObject.rowURI && saveObject.params) {
            	
            	// Check if this is the first POST we are going to make
            	// If yes, then propagate the viewers to take appropriate
            	// action (for e.g., disable their SAVE button)
            	if (this.saveRequests == (modifiedRowsCount -1)) {
            	   this._propagateSaveInProgress();
            	}
            	
                gridModelClient.postURI(srh, saveObject.rowURI, saveObject.params);
            } else {
            	// Since we are not making the POST call, reduce the "guestimate" count
            	this.saveRequests--;
            }
        },
     
		/**
		 * Just makes the post call to the server on the url specified by the save object.
		 * @param {Object} saveObject: The object which has the uri and post params.
		 * @param {Object} row : The row that will be saved.
		 * @param {int} rowIndex : The index of the Row.
		 * @param {String} linkName : The link that will be saved. If null the base resource will be saved.
		 */
        _persistRow: function(saveObject, row, rowIndex, linkName){
            var rowID = row.id;
            var self = this;
            var reloadURI = saveObject.rowURI;
            
            var persistRowFunctions = {
                _persistRowSuccess: function(savedElement){
					
					self._handleSavedElement(savedElement, saveObject, row, rowIndex, reloadURI, linkName);
					
                },
                
                _persistRowFailure: function(errorMsg){
					self._handleSaveError(errorMsg, saveObject, row, rowIndex, linkName, reloadURI);
                    
                }
            };
            
            var srh = new ServiceResponseHandler(persistRowFunctions, "_persistRowSuccess", "_persistRowFailure");
            if (saveObject && saveObject.rowURI && saveObject.params) {
            	
            	// Increment the number of POST calls we are making
            	this.saveRequests++;
            	
            	// Check if this is the first POST we are going to make
                // If yes, then propagate the viewers to take appropriate
                // action (for e.g., disable their SAVE button)
            	if (this.saveRequests == 1) {
            	   this._propagateSaveInProgress();
            	}   
                
                gridModelClient.postURI(srh, saveObject.rowURI, saveObject.params);
            }
        },
		
		/**
		 * If the saved element is a new element then it calls the replace node to replace with the new id.
		 * for new or existing element updates the row by fetching the fresh details from the server after save.
		 * @param {Object} savedElement: The response after the save.
		 * @param {Object} saveObject: The object which has the uri and post params.
		 * @param {Object} row: : The row that will be saved.
		 * @param {Object} rowIndex : The index of the Row.
		 * @param {Object} reloadURI : The save URI.
		 * @param {Object} linkName : The link that will be saved. If null the base resource will be saved.
		 */
		_handleSavedElement: function(savedElement, saveObject, row, rowIndex, reloadURI, linkName){
			var rowID = row.id;
            if (savedElement) {
                if (saveObject.isNew == true) {
                    var publishObj = this.getReplaceNodePublishObject(row, rowID, rowIndex, savedElement);
					if (saveObject.isParentCollapsed) {
						// Flag to indicate if the new node is saved due to collapse
						publishObj.isParentCollapsed = saveObject.isParentCollapsed;
					}
                    dojo.publish(GridEvents.REPLACE_NODE, [publishObj]);
                }
                else {
                    this._saveLinks(null, row);
                }
                if (savedElement.relativeHref) {
                    reloadURI = savedElement.relativeHref;
                }
            }
            this._getAndUpdateRow(reloadURI, rowID, linkName);
                                    
            // Reduce the count of responses received
            this.saveRequests--;
            
            // Check if all responses have been received
            if (this.saveRequests == 0) {
                this._propagateSaveComplete();             
            }
		},
		
		/**
		 * If the row element is not new it resets the data in the row with the data from the server.
		 * Informs the viewers about the error message.
		 * @param {Object} errorMsg: The error response after the save.
		 * @param {Object} saveObject: The object which has the uri and post params.
		 * @param {Object} row: : The row that will be saved.
		 * @param {Object} rowIndex : The index of the Row.
		 * @param {Object} reloadURI : The save URI.
		 * @param {Object} linkName : The link that will be saved. If null the base resource will be saved.
		 */
		_handleSaveError: function(errorMsg, saveObject, row, rowIndex, linkName, reloadURI){
			var rowID = row.id;
            //Update the row with the correct data.
            //Defect 13226 : Disable the get call for all failures
            //if (saveObject.isNew == false) {
                //this._getAndUpdateRow(reloadURI, rowID, linkName);
            //}
            
            // Reduce the count of responses received
            this.saveRequests--;
            
            // Check if all responses have been received
            if (this.saveRequests == 0) {
                this._propagateSaveComplete();              
            }
            
            this._propogateErrorMessage(errorMsg, row, rowID);
		},
        
        checkForReplaceNode: function(){
            throw "checkForReplaceNode: function() must be overridden";
        },
		
		_getElementHandleFromRow: function(row, savedElement){
            var elementHandle = {
                identifier: savedElement.relativeHref,
                relativeHref: savedElement.relativeHref,
                title: savedElement.title,
                resourcetype: row.resourcetype
            };
			
			// Pass the parent id of the element saved, its used later for rollup
			if (row.parentId) {
				elementHandle.parentIdentifier = row.parentId;
			}
			if (row.rpcSequenceValue && savedElement.rpcSequenceValue){
				elementHandle.rpcSequenceValue = savedElement.rpcSequenceValue;
			}
            return elementHandle;
        },
        
        getReplaceNodePublishObject: function(row, rowID, rowIndex, savedElement){
            var newRowID = savedElement.relativeHref;
            var rowElementHandle = this._getElementHandleFromRow(row, savedElement);
            rowElementHandle.id = newRowID;
            var publishObj = {
                oldRowID: rowID,
                newObject: rowElementHandle,
                rowIndex: rowIndex
            };
            publishObj.uniqueIdentifier = {};
            this.addUniqueIdentifier(publishObj.uniqueIdentifier);
            return publishObj;
        },
        
        replaceNode: function(publishedObj){
            console.log("caught the replace node in " + this.id);
			
			// ignore the replace node event if its due to collapse of the parent node
			if (publishedObj.isParentCollapsed) {
				return;
			}
			
            var nodeToBeReplaced = publishedObj;
            if (dojo.isArray(publishedObj) && publishedObj[0]) {
                nodeToBeReplaced = publishedObj[0];
            }
            if (this.checkForReplaceNode(nodeToBeReplaced)) {
                var gridRow = this._gridData.getRow(parseInt(nodeToBeReplaced.rowIndex, 10) - 1);
                this._saveLinks(nodeToBeReplaced.newObject.identifier, gridRow);
                this._removeGridRow(nodeToBeReplaced.oldRowID);
                this._propogateRowsRemoved([nodeToBeReplaced.oldRowID]);
                this._insertElement(nodeToBeReplaced.newObject, nodeToBeReplaced.rowIndex, true);
            }
        },
        
        _saveLinks: function(newElementID, gridRow){
            if (this._linksToBeLoaded && dojo.isArray(this._linksToBeLoaded)) {
                for (var i in this._linksToBeLoaded) {
                    var linkName = this._linksToBeLoaded[i];
                    if (gridRow) {
                        var saveObject = this._gridModel.getSaveObject_link(gridRow, linkName, newElementID);
                        if (saveObject != null) {
                            if (newElementID) {
                                gridRow.id = newElementID;
                            }
                            this._persistRow(saveObject, gridRow, -1, linkName);
                        }
                    }
                }
            }
        },
          
		  /**
		   * Get the Row and delete it from the model. 
		   * If we do not find the row or cannot delete , return false.
		   * @param {Object} gridRowID
		   */
        _removeGridRow: function(gridRowID){
			
			var bflag = false;
			if (this._gridData) {
				var rowIndex = this._gridData.getRowByIdentity(gridRowID);
				if (rowIndex >= 0) {
					//				if (this._gridData.data[rowIndex].__dojo_data_item){
					this._gridData.remove([rowIndex]);
					this._gridData.rebuildRowIdentities(gridRowID);
					this._gridData.store.save();
					bflag = true;
				//				}
				}
				else {
					console.log("rowIndex: " + rowIndex);
					
				}
			}
			return bflag;
        },
        
        _addRow: function(row, index){
        	if (this._gridData) {
				this._gridData.insert(row, index);
				//Defect 6362 : Donot modify the row when adding, this is called when user edits the row
				//this._gridData.beginModifyRow(index);
				this._gridData.rebuildRowIdentities();
			}
        },		
		
        _createModel: function(row){
            var item = new Array(1);
            item[0] = row;
            var gridData = {
                identifier: 'id',
                items: item
            };
            var _gridDataStore = new dojo.data.ItemFileWriteStore({
                data: gridData
            });
            _gridDataStore.save();
            this._gridData = new com.ibm.epm.web.core.ui.internal.DojoData(null, _gridDataStore);
            this._gridData.rebuildRowIdentities();
			this._gridData.canSort= dojo.hitch(this, this.canSort);
			this._gridData.sort = dojo.hitch(this, this.sortGridNew);
            var epmtyUniqueIdentifier = {};
            this.addUniqueIdentifier(epmtyUniqueIdentifier);
            this.epmUniqueID = epmtyUniqueIdentifier;
            dojo.connect(_gridDataStore, "onSet", this, this._onStoreFieldChange);
        },
		
		//stores the name of column clicked, need for sorting later
        _colClicked:"",
		
		/**
		 * 
		 * @param {Object} inSortInfo Sort index of the clicked column
		 */
		canSort: function(inSortInfo)
			{
				var colIndex =  Math.abs(inSortInfo) - 1
				this._colClicked = this._gridData.fields.values[colIndex].name;
				if (this.sortableFields) {
					if(this.sortableFields[this._colClicked])
					{
						return true;
					}
					else
					{
						return false;
					}
				}
				else 
				{
					return false;
				}
			},
			
		//stores the index of column, if negative its ascending, otherwise descending order	
		_sortOrder:0,
		
		/**
		 * 
		 * @param {Object} inSortInfo Sort index of the clicked column
		 */
		
		sortGridNew : function(inSortInfo)
		{			
		  	this._sortOrder = inSortInfo;
			var callbackFunc = dojo.hitch(this, this.sortFunc);			
			this._gridData.data.sort(callbackFunc);						
			this._gridData.store.save();			
			this._gridData.rebuildRowIdentities();
		},
		
		/**
		 * Generic Sort function compares two values, overriding it here so as to get the right coumn field from
		 * data store/model
		 * @param {Object} lval
		 * @param {Object} rval
		 */
		sortFunc : function(lval,rval)
		{
			var fieldName = this.sortableFields[this._colClicked];
			
			if(lval[fieldName] == rval[fieldName]){
				return 0;
			}
			if (this._sortOrder<0) {
				return (lval[fieldName] < rval[fieldName]) ? -1 : 1;
			}
			else
			{
				return (lval[fieldName] > rval[fieldName]) ? -1 : 1;
			}			
		},
		
        _getAndUpdateRow: function(rowURI, rowID, linkName){
            var self = this;
			var updateWBSGeneralGridFunctions = {
                success: function(loadedElement){
                    if (loadedElement) {
                        self._updateGridRow(loadedElement, rowID, linkName);
                        self._getLinkedResources(loadedElement, rowID);
                    }
                },
                failure: function(errorMsg){
                    console.log(errorMsg);
                    //sending null as the row is not fetched here.
                    self._propogateErrorMessage(errorMsg, null, rowID);
                }
            };
            
            var srh = new ServiceResponseHandler(updateWBSGeneralGridFunctions, "success", "failure");
            var getURI = this._gridModel._getRowURI(rowURI);
            if ((getURI != null) && (getURI.indexOf(this.newElementPrefix) == -1)) {
                gridModelClient.getURI(srh, rowURI);
            }
        },
        
        _updateGridRow: function(loadedElement, rowID, linkName){
			//Need a null check if user is switching the row when the row is getting updated.
			if (this._gridData && this._gridData._rowIdentities) {
				if (this._gridData._rowIdentities[rowID]) {
					var rowIndex = this.getRowIndex(rowID);
					var row = this.getRowByIndex(rowIndex);
					
					// Give the views an option to massage some data
					// before it is displayed in the UI
					this._propagateMassageData(loadedElement);
					
					this._updateRowData(row, loadedElement, linkName);
					rowIndex = this.getRowIndex(rowID);
					this.setRowData(row, rowID, rowIndex);
					
					this._gridData.store.save();
				}
				
				else {
					console.log("Row not found with rowID ");
					console.log(rowID);
				}
			}
        },
        
        /*
         * Copy of the getLink that is in ControllerUtil, to break cyclic dependency.
         * See work item 11231.
         */
        _getLink: function(data, linkName){
            if (data && data[linkName]) {
                var links = data[linkName];
                return links;
            }
            return null;
        },
        
        _getLinkedResources: function(loadedElement, rowID){
            if (this._linksToBeLoaded && dojo.isArray(this._linksToBeLoaded)) {
                for (var i in this._linksToBeLoaded) {
					var linkName = this._linksToBeLoaded[i];
                    var links = this._getLink(loadedElement, linkName);
					
                    var linkURI = this._getRowGetUri_Link(loadedElement, links, linkName);
                    if (links != null || linkURI != null) {
						if(linkURI && linkURI != null){
							this._getAndUpdateRow(linkURI, rowID, this._linksToBeLoaded[i]);
						}
                    }
                }
            }
        },
        
        _onStoreFieldChange: function(item, attribute, oldValue, newValue){
            if (oldValue != newValue) {
                var publishObj = {
                    controllerID: null,
                    uniqueIdentifier: null,
                    row: item,
                    fieldName: attribute,
                    oldValue: oldValue,
                    newValue: newValue
                };
                if (this.epmUniqueID) {
                    publishObj.uniqueIdentifier = this.epmUniqueID;
                }
                if (this.id) {
                    publishObj.controllerID = this.id;
                }
                dojo.publish(GridEvents.FIELD_CHANGED, [publishObj]);
            }
        },
        
        addUniqueIdentifier: function(publishObj){
            console.log("function addUniqueIdentifier(publishObj) needs to be over written");
        },

		/**
		 * Fetch the modified rows, update the rows with the server data.
		 */
		discardUnsavedGridChanges: function(){
            var rowsToBeDiscarded = this.getModifiedRows();
			var length = rowsToBeDiscarded.length;
			for (var i = 1; i < length; i++) {
                    var row = rowsToBeDiscarded[i].row;
                    var rowID = row.id;
                    if (row && rowID) {
                        this._getAndUpdateRow(rowID, rowID, null);
                    }
            }
        },

        /**
         *  checkForUnsavedRows : Method to check if the grid store has any unsaved elements
         *  Incoming : none
         *  Outgoing : none
         */        
		checkForUnsavedRows: function(deletedRowId){
            var rowsToBeSaved = this.getModifiedRows();
			var length = rowsToBeSaved.length;
			if (length > 1) {
				if (length == 2) {
					var modifiedRowId = rowsToBeSaved[1].row.id;
					// Only one row contains changes, check if its the same row as the one requested to be deleted
					if ((deletedRowId) && (modifiedRowId == deletedRowId)) {
						return false;
					}
				}
				return true;
			}

			return false;
        },
		
        /**
         * util functions
         */
        setRowData: function(row, rowID, rowIndex){
            this._gridData.setRow(row, rowIndex);
            this._propogateUpdateElement(row, rowID, rowIndex);
        },
        
        getRowIndex: function(rowID){
            var rowIndex = this._gridData.getRowByIdentity(rowID);
            return rowIndex;
        },
        
        getRowByIndex: function(rowIndex){
        
            if (rowIndex == -1 || !(this._gridData)) {
                return null;
            }
            return this._gridData.getRow(rowIndex);
        },
        
        getDataModel: function(){
            if (this._gridData) {
                return this._gridData;
            }
        },
        
        addViewer: function(viewer){
            var viewerPresent = false;
            for (var i in this._viewers) {
                if (this._viewers[i] === viewer) {
                    viewerPresent = true;
                }
            }
            if (viewerPresent == false) {
                this._viewers.push(viewer);
            }
        },
        /**
         * The functions that call the default Viewer
         */
        _updateRowData: function(row, loadedElement, linkName){
			//Remove the errorObject from the row. Row is updated on a save success and a get
			if( row && row.errorObject) {
				delete row.errorObject;
			}
            if (this._gridModel.updateRowData) {
                this._gridModel.updateRowData(row, loadedElement, linkName);
            }
            else {
                throw "invalid model: function _updateRowData(row, loadedElement) is undefined";
            }
        },
		
		_getViewGrid: function(){
			for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.getViewGrid) {
                    return view.getViewGrid();
                }
            }
		},
        
        /**
         * The functions that informs all Viewers.
         */
        _propogateUpdateElement: function(row, rowID, rowIndex){
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.rowUpdated) {
                    view.rowUpdated(row, rowID, rowIndex);
                }
            }
        },
        _propogateInsertElement: function(row, rowID, rowIndex){
        
            var insertedRow = this.getRowByIndex(rowIndex);
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.newRowInserted) {
                    view.newRowInserted(insertedRow, rowID, rowIndex);
                }
            }
        },
        
        _propogateCreateModel: function(){
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.createView) {
                    view.createView(this._gridData);
                }
            }
        },
        
        _propogateRowsRemoved: function(removedRows){
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.removeRows) {
                    view.removeRows(removedRows);
                }
            }
        },
        
        _propogateErrorMessage: function(errorMessage, row, rowID){
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.handleError) {
                    view.handleError(errorMessage, row, rowID);
                }
            }
        },
        
        _propagateSaveInProgress: function(){
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.saveInProgress) {
                    view.saveInProgress();
                }
            }
        },
        
        _propagateSaveComplete: function(){
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.saveComplete) {
                    view.saveComplete();
                }
            }
        },
        
        _propagateMassageData: function(loadedElement) {
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.massageData) {
                    view.massageData(loadedElement);
                }
            }
        },
        
        _propagateMassageEmptyRow: function(collectionItemHandle, emptyRow) {
            for (var i in this._viewers) {
                var view = this._viewers[i];
                if (view.massageEmptyRow) {
                    view.massageEmptyRow(collectionItemHandle, emptyRow);
                }
            }
        },

        /**
         *  _updateSaveProgressTitle : Update the progress of save ( it will show Saving "Task1" (1/15) ... )
         *  Incoming : row, modifiedRowCount
         *  Outgoing : none
         */        
		_updateSaveProgressTitle: function(row, currentRowIndex, modifiedRowsCount){
			//modifiedRows always contains a blank entry and hence the count needs to be reduced by 1.
			modifiedRowsCount -= 1;
			var rowName = "";
			if ((row.name) && (dojo.string.trim(row.name) != "")) {
				rowName = (" \"" + row.name + "\" ");
			}
			var updatedMessage = CoreMessages.SAVING + 
								 rowName + 
								 "( " + currentRowIndex + " / " +
								 modifiedRowsCount + " ) ";
			Util.showMessage(updatedMessage);
		}
		
    });
})();
