/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/**
 * @author suryakant
 */
(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMAssignmentGridController");
    dojo.require("com.ibm.epm.web.core.grid.controller.EPMGridController");
    dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    dojo.require("com.ibm.epm.web.core.util.internal.Constants");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    
    var EPMGridController = com.ibm.epm.web.core.grid.controller.EPMGridController;
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var Constants = com.ibm.epm.web.core.util.internal.Constants;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    dojo.declare("com.ibm.epm.web.core.grid.controller.EPMAssignmentGridController", EPMGridController, {
    
        wbsNodeType: Constants.EPM_PROJECT,
        blankRowID: Constants.UNSAVED,
        ProjectType: Constants.EPM_PROJECT,
        TaskType: Constants.EPM_TASK,
        DefectType: Constants.EPM_DEFECT,
        IssueType: Constants.EPM_ISSUE,
        RiskType: Constants.EPM_RISK,
        SummaryTaskType: Constants.EPM_SUMMARYTASK,
        ChangeRequestType: Constants.EPM_CHANGEREQUEST,
        MileStoneType: Constants.EPM_MILESTONE,
        _projectID: null, //Stores the containing project Id for project Children
        initializeController: function(allElementsURI, viewer, model, linksToBeLoaded){
            // Calling super function
            this.inherited(arguments);
        },
        
        postCreate: function(){
            this.inherited(arguments);
        },
        
        buildGridModel: function(){
            this._get_n_updateModel();
        },
        
        setWBSNodeType: function(nodeType){
        
            this.wbsNodeType = nodeType;
        },
        
        _get_n_updateModel: function(){
            var self = this;
            
            var getAllElementsHandlers = {
                _updateAllElements: function(collectionElements){
                    if (!collectionElements) {
                        console.log("No data returned from the server");
                        self._createBlankRow();
                        return;
                    }
                    
                    if (!dojo.isArray(collectionElements)) { // for single elements it will not be a array.
                        collectionElements = [collectionElements];
                    }
                    // If the Reuslt doesnt contain any information then create blank grid
                    if (collectionElements.length == 0) {
                        self._createBlankRow();
                    }
                    else {
						var acctiveAssignmentArray = new Array();
                        if (self.wbsNodeType == self.ProjectType) {
                            var massagedLinks = this.massageLinks(collectionElements);
                            self.updateModel(false, massagedLinks);
                        }
                        else 
                            if (self.wbsNodeType.indexOf(Constants.WORKITEM) != -1 || self.wbsNodeType.indexOf(Constants.EPM_RISK) != -1) {
                                var taskAssignment = collectionElements[0][Constants.TASK_ASSIGNMENT];
                                self._projectID = collectionElements[0].containingProject;
                                if (taskAssignment) {
                                    if (!dojo.isArray(taskAssignment)) {
                                        taskAssignment = [taskAssignment];
                                    }
                                    var massagedLinks = this.massageLinks(taskAssignment);
                                    self.updateModel(false, massagedLinks);
                                }
                                else { // If the Task doest have any assignment links
                                    self._createBlankRow();
                                    
                                }
                            }
                    }
                },
                
                /*
                 *  converts an array into object literals
                 * @param {Object} arrayOrArgs
                 */
                objectconverter: function(arrayOrArgs){
                    var o = {};
                    var a = (dojo.isArray(arrayOrArgs)) ? arrayOrArgs : arguments;
                    for (var i = 0; i < a.length; i++) {
                        o[a[i]] = '';
                    }
                    return o;
                },
                
                _getAllElementsFailure: function(errorMsg){
                	self._propogateErrorMessage(errorMsg);
                    self._createBlankRow();
                    
                },
                massageLinks: function(collectionElementsLinks){
                    if (!collectionElementsLinks || collectionElementsLinks.length == 0) {
                        return null;
                    }
                    var massagedLinks = new Array(1);
                    for (var i in collectionElementsLinks) {
                        massagedLinks.push(collectionElementsLinks[i]);
                    }
                    return massagedLinks;
                }
                
            };
            var srh = new ServiceResponseHandler(getAllElementsHandlers, "_updateAllElements", "_getAllElementsFailure");
            
            //Fetch the data from the server
            // Display blank grid if the data is not available
            if (this._allElementsURI != null && this._allElementsURI != "" && this._allElementsURI.indexOf(this.newElementPrefix) == -1) {
                gridModelClient.getURI(srh, this._allElementsURI);
            }
            else {
            
                self._createBlankRow();
                
            }
        },
        
		/**
		 * If the saved element is a new element then it calls the replace node to replace with the new id.
		 * for new or existing element updates the row by fetching the fresh details from the server after save.
		 * @param {Object} savedElement: The response after the save.
		 * @param {Object} saveObject: The object which has the uri and post params.
		 * @param {Object} row: : The row that will be saved.
		 * @param {Object} rowIndex : The index of the Row.
		 * @param {Object} reloadURI : The save URI.
		 * @param {Object} linkName : The link that will be saved. If null the base resource will be saved.
		 */
		_handleSavedElement: function(savedElement, saveObject, row, rowIndex, reloadURI, linkName){
			var rowID = row.id;
            if (savedElement) {
                if (saveObject.isNew == true) {
                    var publishObj = this.getReplaceNodePublishObject(row, rowID, rowIndex, savedElement);
                    dojo.publish(GridEvents.REPLACE_NODE, [publishObj]);
                }
                else {
                    this._saveLinks(null, row);
                }
                if (savedElement.relativeHref) {
                    reloadURI = savedElement.relativeHref;
                }
            }
			if(row.resourceRelativeHref.toUpperCase().indexOf("profile".toUpperCase()) != -1 && saveObject.params.User && saveObject.params.User != ""){
				 var publishObj = {
				 	id: rowID
				 };
				 this._removeGridRow(rowID);
				 dojo.publish(Constants.UPDATE_REPLACED_ROW, [publishObj]);
			}
			else{
            	this._getAndUpdateRow(reloadURI, rowID, linkName);
			}
                                    
            // Reduce the count of responses received
            this.saveRequests--;
            
            // Check if all responses have been received
            if (this.saveRequests == 0) {
                this._propagateSaveComplete();             
            }
		},
		
		_getAndUpdateRow: function(rowURI, rowID, linkName){
            var self = this;
            var updateWBSGeneralGridFunctions = {
                success: function(loadedElement){
                    if (loadedElement) {
						var plannedScheduleIndex = rowID.indexOf(Constants.SUFFIX_FOR_DISCRETE_ASSIGNMENT);
							if(plannedScheduleIndex != -1){
								rowID = rowID.substring(0,plannedScheduleIndex);
							}
                        self._updateGridRow(loadedElement, rowID, linkName);
                        self._getLinkedResources(loadedElement, rowID);
                    }
                },
                failure: function(errorMsg){
                    console.log(errorMsg);
                    self._propogateErrorMessage(errorMsg);
                }
            };
            
            var srh = new ServiceResponseHandler(updateWBSGeneralGridFunctions, "success", "failure");
            var getURI = this._gridModel._getRowURI(rowURI);
            if ((getURI != null) && (getURI.indexOf(this.newElementPrefix) == -1)) {
				if(rowURI.indexOf(Constants.SUFFIX_FOR_DISCRETE_ASSIGNMENT)!= -1){
					rowURI = rowURI.substr(0,rowURI.indexOf(Constants.SUFFIX_FOR_DISCRETE_ASSIGNMENT));
				}
                gridModelClient.getURI(srh, rowURI);
            }
        },
        _createBlankRow: function(){
        
            var self = this;
            // Implementing Blank Grid Display
            // Create Blank Row
            var blankRow = {
                "relativeHref": self.blankRowID,
                "href": self.blankRowID,
                "name": " ",
                "role": "",
                "proposedstart": "",
                "proposedduration": "",
                "proposedfinish": "",
                "contourType": "",
                "value": "0",
                "assignmentID": "",
                "resourceRelativeHref": "",
                "costRate": "0",
                "sellingRate": "0"
            
            };
            var massagedLinks = new Array(1);
            massagedLinks.push(blankRow);
            // Insert the Blank Row
            self.updateModel(false, massagedLinks);
            
            
            //Wait till the row is inserted and then delete the row
            var intervalID = setInterval(function(){
                // Wait till the model is built.
                var grid = self._gridData;
                if (grid) {
                
                    var rowInfo = self._gridData._rowIdentities[self.blankRowID];
                    
                    if (rowInfo) {
                        self._removeGridRow(self.blankRowID);
                        clearInterval(intervalID);
                    }
                }
                
            }, 50);
        }
    });
})();
