/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMFinancialGridController");
    dojo.require("com.ibm.epm.web.core.grid.controller.EPMGridController");
    dojo.require("com.ibm.epm.web.core.util.internal.Constants");
	dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    dojo.requireLocalization("com.ibm.epm.financial.web.ui", "Messages"); 
	dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
	dojo.require("com.ibm.epm.web.core.ui.internal.FinancialConstants"); 
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
	 
	var controllerFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
	var FinancialConstants = com.ibm.epm.web.core.ui.internal.FinancialConstants;
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var abstractEPMGridController = com.ibm.epm.web.core.grid.controller.EPMGridController;
    var Constants = com.ibm.epm.web.core.util.internal.Constants;
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.financial.web.ui", "Messages");
	var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
	
    dojo.declare("com.ibm.epm.web.core.grid.controller.EPMFinancialGridController", abstractEPMGridController, {
        loadedElement: null,
		budgetURI: null,
		budgetSummary: false,
		modified: false,
        /* 
         * initializeController calling the super function
         */
        initializeController: function(allElementsURI, viewer, model, linksToBeLoaded){
            // Calling super function
            this.inherited(arguments);
        },
        
        /* 
         * postCreate calling the super function
         */
        postCreate: function(){
            this.inherited(arguments);
        },
        
        /* 
         * buildGridModel - This function creates the grid model
         */
        buildGridModel: function(){
            this._updateFinancialModel();
        },
        
        /* 
         * massageCodeLinks - This function will wrap the links into an array. Before
         * adding to array it checks whether it is removed or not.
         */
        massageCodeLinks: function(massagedLinks, collectionElementsLinks){
            for (var i in collectionElementsLinks) {
                massagedLinks.push(collectionElementsLinks[i]);
            }
        },
        
        getLink: function(data, linkName){
            if (data && data[linkName]) {
                var links = data[linkName];
                return links;
            }
            return null;
        },
        
        /* 
         * isCodeRemoved - This function get and build the model. It calls the super class method to create the model
         *
         */
        _updateFinancialModel: function(){
            var self = this;
            var codeLinks = null;
            var codeAssociatedURIs = "";
            
            if (self.loadedElement == null) {
                self._createBlankRow();
                return;
            }
            
            if (this._allElementsURI == "timecode") {
                codeLinks = this.getLink(self.loadedElement, Constants.TIMECODE);
                
            }
            if (this._allElementsURI == "chargecode") {
                codeLinks = this.getLink(self.loadedElement, Constants.CHARGECODE);
            }
            
			if (this._allElementsURI == "Financials") {
                var references = self.loadedElement.financialDerived;
				codeLinks = new Array(1);
				for (var k in references) {	
					var itemHandle = {
						title: (references[k].name==Constants.LABOUR_REVENUE_FIELD)?Messages.LABOUR_REVENUE:Messages.LABOUR_COST,
						relativeHref: references[k].id
					};
					if( (this.budgetSummary) && (references[k].name==Constants.LABOUR_REVENUE_FIELD)){
						codeLinks.push(itemHandle);
					}
					else if ( (!this.budgetSummary) && (references[k].name!=Constants.LABOUR_REVENUE_FIELD)){
						codeLinks.push(itemHandle);
					}					
				}
				
				if(!this.budgetSummary){
					references = null;
					references = self.loadedElement.financialDetails;
					for (var j in references) {										
						var itemHandle = {
							title: references[j].AccountHead ? references[j].AccountHead.relativeHref : "",
							relativeHref: references[j].id
						};
						codeLinks.push(itemHandle);
					}
				}
				
            	var itemCost = {
					title: Messages.TOTAL_COST,
					relativeHref: Constants.TOTAL_COST_FIELD
				};
				codeLinks.push(itemCost);
				
				if(this.budgetSummary){
					var itemMargin = {
						title: Messages.MARGIN,
						relativeHref: Constants.MARGIN_FIELD
					};
					codeLinks.push(itemMargin);
				}
            }
            //massage the code from the links
            if (codeLinks) {
				
                var massagedLinks = new Array(1);
                this.massageCodeLinks(massagedLinks, codeLinks);
                self.updateModel(false, massagedLinks);
            }
            else {
                self._createBlankRow();
            }
        },
		
		/**
		 * This method will be called from gridController when a new element is inserted
		 * overriding the method so that references are handled separatly
		 * @param {Object} rowURI
		 * @param {Object} rowID
		 * @param {Object} linkName
		 */
		_getAndUpdateRow: function(rowURI, rowID, linkName){
			if (this._allElementsURI != "Financials") {
				this.inherited(arguments);
			}
			else 
			{
				if (this.loadedElement !== null) {
					var found = false;
					var row = {
						name : Messages.ADD_FINANCIAL_TITLE,
						description:"",
		                forecast: 0.00,
		                estimate: 0.00,
				        actual: 0.00,						
						resourceType: Constants.FINANCIAL_DETAILS,
						isForecastCalculate: this.loadedElement.forecastCalculate,
						action: " "
						};
						
					if (rowURI == Constants.TOTAL_COST_FIELD) {
						row.name = Messages.TOTAL_COST;
						if(!this.budgetSummary){
							this._calculateTotal(row);
						}
						else{
							this._calculateSummaryTotal(row);
						}
					} else 	if (rowURI == Constants.MARGIN_FIELD) {
						row.name = Messages.MARGIN;
						if(this.budgetSummary){
							this._calculateMargin(row);
						}
					}
					else {
						var references = this.loadedElement.financialDerived;
						for (var k in references) {
							if ((references[k].id) == rowURI) {
								row.name = (references[k].name==Constants.LABOUR_REVENUE_FIELD)?Messages.LABOUR_REVENUE:Messages.LABOUR_COST;
								row.forecast =  row.isForecastCalculate ? references[k].forecastCalculated : references[k].forecast;
								row.estimate = references[k].estimate;
								row.actual = references[k].actual;
								row.resourceType = Constants.FINANCIAL_DERIVED;
								found = true;
								break;
							}
						}
						if (!found) {
							var references = this.loadedElement.financialDetails;
							for (var j in references) {
								if ((references[j].id) == rowURI) {
									row.name = references[j].AccountHead ? references[j].AccountHead.title : "";
									row.chargeCodeUri = references[j].AccountHead ? references[j].AccountHead.relativeHref : "";
									row.forecast = references[j].forecast;
									row.estimate = references[j].estimate;
									row.actual = references[j].actual;
									row.description=references[j].description;
									row.resourceType = Constants.FINANCIAL_DETAILS;
									break;
								}
							}
						}
					}
					this._updateGridRow(row, rowID, linkName);
					this._getLinkedResources(row, rowID);				
				}					
			}
        },
		
		persistGrid: function(){
			if (this._allElementsURI != "Financials") {
				this.inherited(arguments);
			}
			else {
				if(!this.modified){
					return;
				}
				//Locking the page
				Util._lockPage(Messages.SAVING_BUDGET);
				try{
						var financialDetailsArray = new Array();
						var financialDerivedArray = new Array();
						
						var todeleteRowIDs = this._prepareSave(financialDetailsArray, financialDerivedArray);
						if(todeleteRowIDs.errorMessage){
							this._release();
							this._propogateErrorMessage(todeleteRowIDs);
							return;
						}
						var params = {
							financialDerived: [],
							financialDetails: []
						};
						if((financialDerivedArray) && (financialDerivedArray.length > 0)){
							params.financialDerived = financialDerivedArray;
						}
						
						if((financialDetailsArray) && (financialDetailsArray.length > 0)){
							params.financialDetails = financialDetailsArray;
						}
						
						var self = this;
						var persistRowFunctions = {
							_persistRowSuccess: function(savedElement){
								self._refreshBudgets(todeleteRowIDs);
							},
							
							_persistRowFailure: function(errorMsg){
								self._release();
								self._propogateErrorMessage(errorMsg);
							}
						};
						
						var srh = new ServiceResponseHandler(persistRowFunctions, "_persistRowSuccess", "_persistRowFailure");
						if (this.budgetURI != null) {
							gridModelClient.postURI(srh, this.budgetURI, params);
						}
					}
					catch(exception){
						this._release();
						var errorObject ={
							errorMessage:Messages.ERROR_WHILE_SAVING_BUDGET
						};
							
						this._propogateErrorMessage(errorObject);
					}
			}                     
        },
		
		_calculateSummaryTotal:function(row){
			var references = this.loadedElement.financialDerived;
			codeLinks = new Array(1);
			for (var k in references) {	
				if(references[k].name!=Constants.LABOUR_REVENUE_FIELD){
					row.forecast =  row.isForecastCalculate ? references[k].forecastCalculated : references[k].forecast;
					row.estimate = references[k].estimate;
					row.actual = references[k].actual;	
				}
			}
			var references = this.loadedElement.financialDetails;
			for (var j in references) {
				row.forecast += references[j].forecast;
				row.estimate += references[j].estimate;
				row.actual += references[j].actual;
			}
		},

		_calculateTotal: function(row){
			var forecast = 0.00;
			var estimate = 0.00;
			var actual = 0.00;
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
				if (this._gridData.getRow) {
					var financial = this._gridData.getRow(i);
					if ((financial != null) && (financial.name != Messages.LABOUR_REVENUE)) {
						forecast += financial.forecast;
						estimate += financial.estimate;
						actual   += financial.actual;
					}
				}
			}
			row.forecast = forecast;
			row.estimate = estimate;
			row.actual = actual;			
		},
		
		_calculateMargin: function(row){
			var forecastRevenue = 0.00;
			var estimateRevenue = 0.00;
			var actualRevenue = 0.00;
			
			var forecastCost = 0.00;
			var estimateCost = 0.00;
			var actualCost = 0.00;
			
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
				if (this._gridData.getRow) {
					var financial = this._gridData.getRow(i);
					if ((financial != null) && (financial.name == Messages.LABOUR_REVENUE)) {
						forecastRevenue = financial.forecast;
						estimateRevenue = financial.estimate;
						actualRevenue   = financial.actual;
					}
					if ((financial != null) && (financial.name == Messages.TOTAL_COST)) {
						forecastCost = financial.forecast;
						estimateCost = financial.estimate;
						actualCost   = financial.actual;
					}
				}
			}
			row.forecast = forecastRevenue - forecastCost;
			row.estimate = estimateRevenue - estimateCost;
			row.actual = actualRevenue - actualCost;
		},
		
		_prepareSave:function(financialDetailsArray, financialDerivedArray){
			var allRowIds = new Array();
			var allNewRowIds = new Array();
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
                if (this._gridData.getRow) {
					var row = this._gridData.getRow(i);
					if ((row != null) && (row.id != Constants.TOTAL_COST_FIELD) && (row.id != Constants.MARGIN_FIELD)) {
						allRowIds.push(row.id);
						var rowparams = "forecast=" + row.forecast + 
						   ", estimate=" + row.estimate +
						   ", actual=" + row.actual+
						   ", description=" + row.description;					   
						
						if((typeof(row.loadedelement[0].chargeCodeUri) != Constants.UNDEFINED) && 
						   (row.loadedelement[0].chargeCodeUri != "") &&						   	
					       (row.chargeCodeUri != row.loadedelement[0].chargeCodeUri))
					    {
					   		var param = row.loadedelement[0].chargeCodeUri + Constants.COLON + Constants.DEL_OPERATION;
							rowparams = rowparams + ", AccountHead=" + param;
							rowparams = rowparams + ", AccountHead=" + row.chargeCodeUri;
					    }
						else if (((typeof(row.loadedelement[0].chargeCodeUri) == Constants.UNDEFINED) || (row.loadedelement[0].chargeCodeUri == "") ) &&
						(typeof(row.chargeCodeUri) != Constants.UNDEFINED) &&
						(row.chargeCodeUri != "")) {
							rowparams = rowparams + ", AccountHead=" + row.chargeCodeUri;
						}	
						  
						if (row.resourceType == Constants.FINANCIAL_DETAILS) {
							//Making Category as Mandatory
							if((typeof(row.chargeCodeUri) == Constants.UNDEFINED) || 
							   (row.chargeCodeUri == "")){
							   	var errObject = {
									errorMessage: Messages.CATEGORY_MANDATORY
								};	
								return errObject;
							}	
							if (row.id.match("_unsaved")) {
								allNewRowIds.push(row.id);
								financialDetailsArray.push(rowparams);
							}
							else {
								financialDetailsArray.push((rowparams + ", id=" + row.id));
							}
						}
						else if (row.resourceType == Constants.FINANCIAL_DERIVED) {
							var rowparams = "forecast=" + row.forecast;
							financialDerivedArray.push((rowparams + ", id=" + row.id));
							if(!self.budgetSummary){
								//Getting value from Summary Grid 
								var financialSummaryController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE);
								for (var k = 0; k < financialSummaryController._gridData.getRowCount(); k++) {
									if (financialSummaryController._gridData.getRow) {
										var financial = financialSummaryController._gridData.getRow(k);
										if ((financial != null) && (financial.name == Messages.LABOUR_REVENUE)) {
											var rowparams = "forecast=" + financial.forecast;
											financialDerivedArray.push((rowparams + ", id=" + financial.id));
										}
									}
								}
							}
						}
					}
				}					
            }
			
			var references = this.loadedElement.financialDetails;
			for (var j in references) {
				var refId = references[j].id;
				if (!this._containId(allRowIds, refId)) {
					var params = "id=" + refId + Constants.COLON + Constants.DEL_OPERATION ;   		///Send the deleted reference params
					financialDetailsArray.push(params);
				}
			}
			//Delete total so that it will recalculate
			allNewRowIds.push(Constants.TOTAL_COST_FIELD);
			allNewRowIds.push(Constants.MARGIN_FIELD);
			return allNewRowIds;			
		},
		
		/**
		 * _updateFinancialDetails - method will set the charge code uri in the loadedelement
		 * so that second time pressing on save after changing the charge code it throws error
		 */
		_updateFinancialDetails: function(){
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
                if (this._gridData.getRow) {
					var row = this._gridData.getRow(i);
					if ((row != null) && (row.id != Constants.TOTAL_COST_FIELD) && 
					    (row.id != Constants.MARGIN_FIELD) && 
						(row.resourceType == Constants.FINANCIAL_DETAILS)) {
							row.loadedelement[0].chargeCodeUri = row.chargeCodeUri;
					}
				}					
            }
		},
		
		_containId:function(arrayObject, key){
			var found = false;
			if((arrayObject != null) && (arrayObject.length > 0)){
				var num = arrayObject.length;
	            for (var i = 0; i < num; i++) {	            
	                if (arrayObject[i] === key) {
	                    found = true;
	                    break;
	                }
	            }	
			}
			return found;
		},
		
		/**
		 * _refreshBudgets - method will be called soon after saving the budget resource
		 * it will refresh the grid with the latest values
		 * @param {Object} todeleteRowIDs
		 */
		_refreshBudgets: function(todeleteRowIDs){
            this._updateFinancialDetails();
			var self = this;
            var generalFunctions = {
                success: function(loadedElement){
                    if (loadedElement) {
						self.loadedElement = loadedElement;	
						for (var k in todeleteRowIDs){
							self._removeGridRow(todeleteRowIDs[k]);
						}				
						self._updateFinancialModel();
						if(!self.budgetSummary){
							var financialSummaryController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE);
							financialSummaryController.loadedElement = loadedElement;
							//Delete total so that it will recalculate
							financialSummaryController._removeGridRow(Constants.TOTAL_COST_FIELD);
							financialSummaryController._removeGridRow(Constants.MARGIN_FIELD);
							financialSummaryController._updateFinancialModel();	
						}
						self._release();
                    }
                },
                failure: function(errorMsg){
					self._release();
                    console.log(errorMsg);
					self._propogateErrorMessage(errorMsg);
                }
            };			            
            var srh = new ServiceResponseHandler(generalFunctions, "success", "failure");
            if (this.budgetURI != null) {
				gridModelClient.getURI(srh, this.budgetURI);
			}            
        },
		
		/**
		 * The method will releasing page lock and also
		 * reset the modifed flag to false
		 */
		_release:function(){
			this.modified = false;
			//Releasing page after refreshing the data
			Util._releasePage();
		},

		_insertFinancials: function(itemHandle){	
			var financials = new Array();		
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
				if (this._gridData.getRow) {
					var financial = this._gridData.getRow(i);
					if (financial != null) {						
						var item = {
							title: financial.name,
							relativeHref: financial.id
						};
						financials.push(item);
						if (financial.name == Messages.TOTAL_COST) {
							financials.push(itemHandle);
						}
					}
				}
			}
			this.updateModel(true, financials);			
		}
    });
})();
