/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMGanttController");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.TreeBasedGridController");
    
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");

	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
	
	dojo.require("com.ibm.epm.web.core.client.internal.EPMTreeClient");
	dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");

	//localization
	dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages");
    
    var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
	var CoreMessages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages");	    
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var EPMTreeClient = com.ibm.epm.web.core.client.internal.EPMTreeClient;
    var TreeBasedGridController = com.ibm.epm.web.core.grid.controller.TreeBasedGridController;
	
	var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
	    
	var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
	var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
	
	var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler; 
	
    var _CRITICAL_PATH_URI = "/criticalPath";
    
    dojo.declare("com.ibm.epm.web.core.grid.controller.EPMGanttController", TreeBasedGridController, {
        _updateGridModel: false,
		_updateParent: false,
		_calculateInProgress: false,

		/**
         *  calculateCriticalPathForProject : function call to calculate the CriticalPath for the specified project.
         *
         *  Incoming : Selected project tree node and project identifier
         *  Outgoing : none
         *
         */        
		calculateCriticalPathForProject: function(ctxtNode, projectID){
			if (projectID) {
				this.ctxtNode = ctxtNode;
				
				var rowIndex = this.getRowIndex(projectID);
				if (rowIndex) {
					var row = this.getRowByIndex(rowIndex);
					if ((row) && (row.name)) {
						Util.showMessage(CoreMessages.CALCULATE_CRITICAL_PATH + row.name);
						this._calculateInProgress = true;
					}
				}				
				this.calculateCriticalPath(projectID);
			}
		},

		/**
         *  calculateCriticalPath : function call to calculate the CriticalPath for a specific project.
         *							uri for calculate critical path: project/id/criticalPath/id
         *
         *  Incoming : uri to calculate critical path for a project
         *  Outgoing : none
         *
         */     			
        calculateCriticalPath: function(uri){
            var self = this;
            
            var getAllElementsHandlers = {
                _updateAllElements: function(collectionElements){
                	self._updateProjectNodeOnCalculationCompletion();
                },

                _getAllElementsFailure: function(errorObj){
                	self._displayAndClearErrorMessage(errorObj.errorMessage);
					Util.hideMessageArea();
                }
            };
            var srh = new ServiceResponseHandler(getAllElementsHandlers, "_updateAllElements", "_getAllElementsFailure");
            
            // Fetch the data from the server
            gridModelClient.getURI(srh, uri + _CRITICAL_PATH_URI);
        },

		/**
         *  _updateProjectNodeOnCalculationCompletion : method triggers the updation of the project node
         *
         *  Incoming : none
         *  Outgoing : none
         *
         */  
		_updateProjectNodeOnCalculationCompletion: function() {
			this._calculateInProgress = false;
			if (this.ctxtNode && this._tree) {
				this._tree.nodeRefresh(this.ctxtNode.item, true);
			}
		},
        
		/**
		 * Make a post call to critical path service .
		 * The critical path is freshly computed and then 
		 * the tasks which are inconsistent are updated with
		 * the earlystart and earlyfinish dates.
		 * The node refresh in called on refresh.
		 * @param {Object} projectId
		 */
		_makeCalltoResolveDependency: function(ctxNode, projectId, force)
		{
			var self = this;
			force = (force==true) ? force : false;
			//Show the message resolving ... 
			Util.showMessage(CoreMessages.RESOLVING);
			var serviceObj = {
                success: function(data){
					if (ctxNode && ctxNode.item && self._tree ) {
						//call the node refresh
						self._tree.nodeRefresh(ctxNode.item, true);
					}
				// hide the message
				Util.hideMessageArea();
					
                },
                failure: function(errorObj){
					if (errorObj.errorMessage) {
						var workitemIndex = errorObj.errorMessage.indexOf(WBSConstants.WORKITEM_TYPE);
						if (workitemIndex && workitemIndex > -1) {
							var workitemID = errorObj.errorMessage.substring(workitemIndex);
							var rowIndex = self.getRowIndex(workitemID);
							if (rowIndex && rowIndex > -1) {
								var row = self.getRowByIndex(rowIndex);
								self._propogateErrorMessage(errorObj, row, workitemID);
							}
							self._displayAndClearErrorMessage(errorObj.errorMessage.substring(0, workitemIndex));
						}
						else{
							if(errorObj.errorCode == "CRIPM2241E")
							{
								//display a dialog to confirm force action
								self._okToResolveDependency(ctxNode, projectId);
								
								//discard if cancel selected or call this method (recursive) with force=true option
							}
							else 
								self._displayAndClearErrorMessage(errorObj.errorMessage);
						}
					}
				 // hide the message
				Util.hideMessageArea();
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            
            
            var inputparams = {"resolveDependency":true};
            
           	if(force == true) {
           		inputparams = {"resolveDependency":true, "forceProjectAssignmentExtension":true};
           	}
            
			gridModelClient.postURI(srh,  projectId + _CRITICAL_PATH_URI, inputparams);	
			
		},


		/**
		 * Resolve Project Dependency
		 * Invoke a warning message 
		 * On Yes , make the call to the server 
		 * 
		 */
		_okToResolveDependency: function(ctxNode, projectId){
			var buttonlist = [{
	            label: CoreMessages.YES,
                action: dojo.hitch(this, this._YesButtonHandler,ctxNode, projectId)
            }, {
                label: CoreMessages.NO,
                action: dojo.hitch(this, this._NoButtonHandler)
            }];

			var labelList = [{
				label: CoreMessages.WARNING_BEFORE_RESOLVING_DEPENDENCY_IF_ASSIGNMENT_EXISTS
            }];
                             
			
			this.resolveDependencyDialog = new EPMConfirmDialog({
                                            labelList: labelList,
                                            title: CoreMessages.RESOLVE_DEPENDENCY,
                                            buttonlist: buttonlist
                                        });
            this.resolveDependencyDialog.show();
		},

		
		/**
		 * Make the call if user clicks on yes 
		 */
		_YesButtonHandler: function(ctxtNode, parentID){
			if (this.resolveDependencyDialog != null) {
				this.resolveDependencyDialog.destroy();
				this._makeCalltoResolveDependency(ctxtNode, parentID, true);
			}
		},
		/**
		 * close and destroy if user clicks of No 
		 */
		_NoButtonHandler: function(){
            if(this.resolveDependencyDialog != null)
			{
				this.resolveDependencyDialog.destroy();
			}
        },
		
		/**
		 * Call for Auto calculation 
		 * A call is made to the server with new dates and 
		 * since in gantt we can only change start and finish dates 
		 * we always have to calcuate the Duration and effort .
		 * 
		 * On success the duration and the new effort is updated in row 
		 * The start and finish will be updated in Gantt view [onAfterUpdateGanttBar ]  
		 * @param {Object} params
		 * @param {Object} item
		 */
		_getProposedScheduleFromCalendar: function(params,item,newData,selectedRow,ganttBar_index){
			var self = this;
			var serviceObj= {
				success: function(response)
				 {
					if (params.bProposedDuration == true && params.bProposedFinishDate == false && response != null && response.duration !=null && response.duration >= 0)
					{
						self._gridData.store._setValueOrValues(item, WBSConstants.PROPOSED_DURATION_FIELD_NAME, response.duration, true);
						self._gridData.store._setValueOrValues(item, WBSConstants.PLANNED_DURATION_FIELD_NAME, response.duration, true);

						 if(response.effort != null && response.effort >= 0)
						 {
							self._gridData.store._setValueOrValues(item, WBSConstants.PROPOSED_EFFORT_FIELD_NAME, response.effort, true);
							self._gridData.store._setValueOrValues(item, WBSConstants.PLANNED_EFFORT_FIELD_NAME, response.effort, true);
						 }
						 else
						 {
						 	self._gridData.store._setValueOrValues(item, WBSConstants.PROPOSED_EFFORT_FIELD_NAME, response.duration * 8, true);
							self._gridData.store._setValueOrValues(item, WBSConstants.PLANNED_EFFORT_FIELD_NAME, response.duration * 8, true);
						 }
					}
						 self._gridData.store._setValueOrValues(item, WBSConstants.PROPOSED_START_FIELD_NAME, dateUtil.parseDate(response.startDate), true);
						 self._gridData.store._setValueOrValues(item, WBSConstants.PLANNED_START_FIELD_NAME, dateUtil.parseDate(response.startDate), true);
						 self._gridData.store._setValueOrValues(item, WBSConstants.PROPOSED_FINISH_FIELD_NAME, dateUtil.parseDate(response.finishDate), true);
						 self._gridData.store._setValueOrValues(item, WBSConstants.PLANNED_FINISH_FIELD_NAME, dateUtil.parseDate(response.finishDate), true);
						 
						 // update the listener's ( Like Gantt ) after update of the row data in grid
						 if(selectedRow && ganttBar_index)
						 {
						 	self._propogateUpdateElement(selectedRow, selectedRow.id, ganttBar_index)
						 }
				},
				
				failure: function(errorObj) {
					//If in case there is any failure 
					// Show the basic aithematic difference between the dates 
					
					// Default constant effort per day
		            var constEffortPerDay = 8;
		            // One day in milliseconds
		            var dayInMilli = 1000 * 60 * 60 * 24;
					//let the default value be loaded 
					
					if (params.bProposedDuration == true && params.bProposedFinishDate == false)
					{
						 var duration = Math.ceil((newData.endDate.getTime() - newData.startDate.getTime()) / (dayInMilli)) + 1;
					  	self._gridData.store._setValueOrValues(item, WBSConstants.PROPOSED_DURATION_FIELD_NAME,duration , true);
						self._gridData.store._setValueOrValues(item, WBSConstants.PLANNED_DURATION_FIELD_NAME, duration, true);
		
						 
						self._gridData.store._setValueOrValues(item, WBSConstants.PROPOSED_EFFORT_FIELD_NAME, duration * constEffortPerDay, true);
						self._gridData.store._setValueOrValues(item, WBSConstants.PLANNED_EFFORT_FIELD_NAME, duration * constEffortPerDay, true);
				
						 
					}
					// update the listener's ( Like Gantt ) after update of the row data in grid
						 if(selectedRow && ganttBar_index)
						 {
						 	self._propogateUpdateElement(selectedRow, selectedRow.id, ganttBar_index)
						 }
				}
			};
			
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");			
			EPMTreeClient.getProposedFromCalendar(srh,params);			
		},
				
		/**
		 * _getAndUpdateRow: inherits from the abstract class to update the parent in case the flag for updating parent is set. It ignores
		 * the parent updation if linkName is not null.
		 * 
		 * @param {Object} rowURI
		 * @param {Object} rowID
		 * @param {Object} linkName
		 */	        
        _getAndUpdateRow: function(rowURI, rowID, linkName){			
            this.inherited(arguments);
			
			if ((linkName === null) ||
				(linkName == WBSConstants.TASK_ASSIGNMENT)) {
	            var rowIndex = this.getRowIndex(rowID);
	            var row = this.getRowByIndex(rowIndex);
	            // Get parent element and update
	            if ((row) && (row.parentId) && (this._updateParent)) {
	                var parentRowIndex = this.getRowIndex(row.parentId);
	                var parentRow = this.getRowByIndex(parentRowIndex);
	                if (parentRow) {
	                    this._getAndUpdateRow(parentRow.href, parentRow.id, linkName);
	                } else {
	                	// reset the flag after reaching the root of the hierarchy
						this._updateParent = false;
					}
	            }
			}
        },

		/**
		 * persistGrid: inherits from the abstract class to set the flag to update the parent
         * @param {boolean} synchronizedSave Optioanal: flag to inform controller weather to make the save calls 
         * one after another or in parallel. If not sent the save will be in parallel.
		 */		
		 persistGrid: function(synchronizedSave){
			this.inherited(arguments);
			
			// Set the flag to ensure the changes are rolled up to the parent
			this._updateParent = true;
		}
    });
})();
