/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMGridController");
    dojo.require("com.ibm.epm.web.core.grid.controller.AbstractGridController");
    
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    
    var abstractGridController = com.ibm.epm.web.core.grid.controller.AbstractGridController;
    
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    
    dojo.declare("com.ibm.epm.web.core.grid.controller.EPMGridController", abstractGridController, {
        _allElementsURI: null,
        blankRowID: "Blank_Row_unsaved",
        
        initializeController: function(allElementsURI, viewer, model, linksToBeLoaded){
            // Calling super function
            this.inherited(arguments);
            // Initializing allElementsURI
            if (allElementsURI != null && allElementsURI != "") {
                this._allElementsURI = allElementsURI;
            }
        },
        
		setAllElementsURI:function(newURI){
			
			this._allElementsURI = newURI;
			
		},
        isInitialized: function(){
            var initialized = false;
            if (this.inherited(arguments) &&
            this._allElementsURI != null) {
                initialized = true;
            }
            return initialized;
        },
        
        destroy: function(){
            this.inherited(arguments);
        },
        
        postCreate: function(){
            this.inherited(arguments);
        },
        
        buildGridModel: function(){
            this._get_n_updateModel();
        },
        
        _get_n_updateModel: function(){
            var self = this;
            
            var getAllElementsHandlers = {
                _updateAllElements: function(collectionElements){
                    if (collectionElements) {
                        if (!dojo.isArray(collectionElements)) { // for single elements it will not be a array.
                            collectionElements = [collectionElements];
                        }
                        // If the Reuslt doesnt contain any information then create blank grid
                        if (collectionElements.length == 0) {
                            self._createBlankRow();
                        }
                        else {
                            var massagedLinks = this.massageLinks(collectionElements);
                            self.updateModel(false, massagedLinks);
                        }
                    }
                    else {
                        self._createBlankRow();
                    }
                },
                _getAllElementsFailure: function(errorMsg){
					self._propogateErrorMessage(errorMsg);
                    self._createBlankRow();
                    
                },
                massageLinks: function(collectionElementsLinks){
                    if (!collectionElementsLinks || collectionElementsLinks.length == 0) {
                        return null;
                    }
                    var massagedLinks = new Array(1);
                    for (var i in collectionElementsLinks) {
                        massagedLinks.push(collectionElementsLinks[i]);
                    }
                    return massagedLinks;
                }
                
            };
            var srh = new ServiceResponseHandler(getAllElementsHandlers, "_updateAllElements", "_getAllElementsFailure");
            
            //Fetch the data from the server
            // Display blank grid if the data is not available
            if (this._allElementsURI != null && this._allElementsURI != "" && this._allElementsURI.indexOf(this.newElementPrefix) == -1) {
                gridModelClient.getURI(srh, this._allElementsURI);
            }
            else {
            
                self._createBlankRow();
                
            }
        },
        
        _createBlankRow: function(){
        
            var self = this;
            // Implementing Blank Grid Display
            // Create Blank Row
            var blankRow = {
                "relativeHref": self.blankRowID
            
            };
            
            var massagedLinks = new Array(1);
            massagedLinks.push(blankRow);
            // Insert the Blank Row
            self.updateModel(false, massagedLinks);
            //Wait till the row is inserted and then delete the row
            var intervalID = setInterval(function(){
                // Wait till the model is built.
                var grid = self._gridData;
                if (grid) {
                
                    var rowInfo = self._gridData._rowIdentities[self.blankRowID];
                    
                    if (rowInfo) {
                        self._removeGridRow(self.blankRowID);
                        clearInterval(intervalID);
                    }
                }
                
            }, 50);
        },
        
        checkForReplaceNode: function(nodeToBeReplaced){
            var uniqueId = nodeToBeReplaced.uniqueIdentifier;
            if (uniqueId && uniqueId.allElementsURI && uniqueId.allElementsURI == this._allElementsURI && this._gridData) {
                return true;
            }
            return false;
        },
        
        addUniqueIdentifier: function(publishObj){
            publishObj.allElementsURI = this._allElementsURI;
        }
        
    });
})();
