/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMRiskGridQueryController");
    
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.risk.web.client.internal.RiskClient");
    dojo.require("com.ibm.epm.web.core.grid.controller.EPMGridController");
    
    var abstractEPMGridController = com.ibm.epm.web.core.grid.controller.EPMGridController;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var riskClient = com.ibm.epm.risk.web.client.internal.RiskClient;
    dojo.declare("com.ibm.epm.web.core.grid.controller.EPMRiskGridQueryController", abstractEPMGridController, {
    
        /* 
         * initializeController calling the super function
         */
        initializeController: function(allElementsURI, viewer, model, linksToBeLoaded){
            // Calling super function			
            this.inherited(arguments);
            this.EPMCollection = allElementsURI;
        },
        
		/*
		 * First time the grid loads, it adds a blank row and remove it. This enables the delete button
		 * Here overriding the following method to unselect the rows after removing the blank row. 
		 * 
		 */
          _removeGridRow: function(gridRowID){
		  	this.inherited(arguments);
		  	if (gridRowID === this.blankRowID && this._gridModel._containingGrid && this._gridModel._containingGrid._generalGrid) {
		  		this._gridModel._containingGrid._generalGrid.selection.unselectAll();
		  	}
		  },
        
        /* 
         * postCreate calling the super function
         */
        postCreate: function(){
            this.inherited(arguments);
        },
        
        
        buildGridModel: function(){
            this._get_n_updateModel();
        },
        
        _get_n_updateModel: function(){
            var self = this;
            
            var getAllElementsHandlers = {
                _updateAllElements: function(collectionElements){
					
					 if (!collectionElements) {
                        self._createBlankRow();
                        return;
                    }
                	if (!dojo.isArray(collectionElements)) { // for single elements it will not be a array.
                        collectionElements = [collectionElements];
                    }
					if (collectionElements.length == 0) {
						self._createBlankRow();
					}
					else {
						 var massagedLinks = this.massageLinks(collectionElements);
                         self.updateModel(false, massagedLinks);
					}
                },
                _getAllElementsFailure: function(errorMsg){
					self._propogateErrorMessage(errorMsg);
                    self._createBlankRow();
                    
                },
                massageLinks: function(collectionElementsLinks){
                    if (!collectionElementsLinks || collectionElementsLinks.length === 0) {
                        return null;
                    }
                    var massagedLinks = new Array(1);
                    for (var i in collectionElementsLinks) {
                        massagedLinks.push(collectionElementsLinks[i]);
                    }
                    return massagedLinks;
                }
                
            };
            var srh = new ServiceResponseHandler(getAllElementsHandlers, "_updateAllElements", "_getAllElementsFailure");
            
            //Fetch the data from the server
            // Display blank grid if the data is not available
            if (this._allElementsURI && this._allElementsURI !== "" && this._allElementsURI.indexOf(this.newElementPrefix) === -1) {
            
                riskClient.getRiskURI(srh, this._allElementsURI);
            }
            else {
            
                self._createBlankRow();
                
            }
        }
        
    });
})();
