/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMRiskStrategiesGridController");
    dojo.require("com.ibm.epm.web.core.grid.controller.EPMGridController");
	
	var epmGridController = com.ibm.epm.web.core.grid.controller.EPMGridController;
    
	dojo.declare("com.ibm.epm.web.core.grid.controller.EPMRiskStrategiesGridController", epmGridController, {
    	        
        strategies : null,
        
		/**
         *  buildGridModel : This method overrides the parent.
         *  This is being done since the data for this grid need not be 
         *  fetched from the uri calls.
         */
        buildGridModel: function(){
            if (this.strategies)
            {
               	var references = this.strategies;
				var data = [];
				for (var k in references)
				{										
					var itemHandle = {
						relativeHref: references[k].id
					};
					data.push(itemHandle);
				}		
				if (data.length > 0)
	           	{
	               	this.updateModel(true, data);
					return;
	           	}		
            }
            this._createBlankRow();
        },
		
		/**
         *  _getAndUpdateRow : This method overrides the parent.
         *  This is being done since the row has to be built from 
         *  the data available in 'strategies'
         */
		_getAndUpdateRow: function(rowURI, rowID, linkName){
			if (rowURI && this.strategies) {
				var row = this._getEmptyRow(rowID);
				var references = this.strategies;
				for (var j in references) {
					if ((references[j].id) == rowURI) {
						row.summary = references[j].summary;
						row.description = references[j].description;
						row.name = references[j].summary;
						row.actions = references[j].actions;
						break;
					}
				}
				this._updateGridRow(row, rowID, linkName);
			}
		},
		
		/**
         * _deleteElement: This method deletes the row from the Strategies grid. <br>
         * This is called from the click event handler for the delete toolbar button <br>
         * rowIndex: Index of the selected row. <br>
         */
		_deleteElement: function(rowIndex){
			if (rowIndex >= 0) {
				var item = this._gridData.getRow(rowIndex);
				
				if (item.id) {
					this._removeGridRow(item.id);
				}
			}
		},
		
		/*
		 * First time the grid loads, it adds a blank row and remove it. This enables the delete button
		 * Here overriding the following method to unselect the rows after removing the blank row. 
		 * 
		 */
          _removeGridRow: function(gridRowID){
		  	this.inherited(arguments);
		  	if (gridRowID === this.blankRowID && this._gridModel._containingGrid && this._gridModel._containingGrid._generalGrid) {
		  		this._gridModel._containingGrid._generalGrid.selection.unselectAll();
		  	}
		  }
    });
})();
