/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008,2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.web.core.grid.model.AbstractGridModel");
    
    dojo.declare("com.ibm.epm.web.core.grid.model.AbstractGridModel", null, {
        _controller: null,
        getEmptyRow: function(rowID, treeElementHandle){
            throw "function getEmptyRow(rowID, treeElementHandle) needs to be over written";
        },
        updateRowData: function(row, loadedElement, linkName){
            throw "function updateRowData (row, loadedElement) needs to be over written";
        },
        getRowGetURI: function(treeElementHandle, rowID){
            throw "function getRowGetURI (treeElementHandle, rowID) needs to be over written";
        },
        
        prepareSaveParams: function(row){
            throw "function prepareSaveParams (row) needs to be over written";
        },
        
        _getNewElementURI: function(row){
            throw "function _getNewElementURI(row) needs to be over written";
        },
        
        getLoadedelement_link: function(gridRow){
            // over ride this method to return the loaded backup element from the row.
            // If null is returned the save call is made weather it has been changed or not.
            return null;
        },
        
        getRowPostURI_link: function(row, linkName, newElementID){
            throw "function getRowPostURI_link(row, linkName, newElementID) needs to be over written if you want to save a linked resourece with name " + linkName;
        },
        
        prepareSaveParams_link: function(row, linkName){
            throw "function prepareSaveParams_link(row, linkName) needs to be over written if you want to save a linked resourece with name " + linkName;
        },
        
        getSaveObject: function(row){
            var saveParams = this.prepareSaveParams(row);
            var saveObj = this.getRowPostURI(row);
            var loadedelement = this.getLoadedelement(row);
			//Row doesn't have a loaded element if updateRowData is not executed/the row is new.
            if (loadedelement != null) {
                var modifiedParams = this._isRowModified(loadedelement, saveParams, row, null);
                if (modifiedParams != null) {
                    saveObj.params = modifiedParams;
                    return saveObj;
                }
                else {
                    return null;
                }
            }else if(saveObj.isNew){
				if (saveParams != null) {
	                saveObj.params = saveParams;
	            }
				return saveObj;
			}else{
				return null;
			}
        },
        
        getLoadedelement: function(row){
            // override this method if the row Object that was loaded in the model is not loadedelement.
            if (row.loadedelement && row.loadedelement != "") {
                return row.loadedelement[0];
            }
            else {
                return null;
            }
        },
        
        _isRowModified: function(loadedelement, saveParams, row, linkName){
            var tempRow = this.getEmptyRow(row.id, loadedelement);
            this.updateRowData(tempRow, loadedelement, linkName);
            if (linkName != null) {
                var tempSaveParams = this.prepareSaveParams_link(tempRow, linkName);
            }
            else {
                var tempSaveParams = this.prepareSaveParams(tempRow);
            }
            
            var modifiedParams = {};
            var modified = false;
            
            for (var field in saveParams) {
                if (this._isFieldModified(saveParams, tempSaveParams, field, row, loadedelement) == true) {
                    modified = true;
                    modifiedParams[field] = saveParams[field];
                }
            }
            
            if (modified == true) {
                return modifiedParams;
            }
            else {
                return null;
            }
        },
        
        getSaveObject_link: function(gridRow, linkName, newElementID){
        
            var saveParams = this.prepareSaveParams_link(gridRow, linkName);
            var saveObj = this.getRowPostURI_link(gridRow, linkName, newElementID);
            if (saveObj != null) {
                var loadedelement = this.getLoadedelement_link(gridRow, linkName);
                
                if (loadedelement != null) {
                    var modifiedParams = this._isRowModified(loadedelement, saveParams, gridRow, linkName);
                    if (modifiedParams != null) {
                        saveObj.params = modifiedParams;
                        return saveObj;
                    }
                    else {
                        return null;
                    }
                }
            }
            return null;
        },
        
        _isFieldModified: function(saveParams, tempSaveParams, field, row, loadedelement){
			fieldValue = saveParams[field];
			fieldValue_orig = tempSaveParams[field];
			return this.compareFields(fieldValue, fieldValue_orig, field);
        },
		
		/**
		 * checks if both the fields are same.
		 * @param {Object} fieldValue : modified field value
		 * @param {Object} fieldValue_orig
		 * @param {String} {optional} field: the field that is compared
		 * @return :true if both are not same.
		 * 			false if both are same. 
		 */
		compareFields: function(fieldValue, fieldValue_orig, field){
			if (fieldValue == null || fieldValue == "") {
				return false;
			}
            if (fieldValue != fieldValue_orig) {
                return true;
            } else {
                return false;
            }
		},
        
        getRowGetURI: function(treeElementHandle, rowID){
            var rowURI = this._getRowURI(treeElementHandle.relativeHref);
            return rowURI;
        },
        
		/**
		 * The function to get the uri to make the post and get call.
		 * @param {String} rowHref: the Href or the id of the row.
		 * @return {String} the uri to which the get and post calls can be made.
		 */
        _getRowURI: function(rowHref){
			var isNewRow = this.isNewRow(rowHref);
            if ((rowHref) && !isNewRow) {
                return rowHref;
            }else {
                return null;
            }
        },
		
		/**
		 * checks if the row contains the new element id prefix.
		 * @param {String} rowHref: the Href or the id of the row.
		 * @return {boolean} true if the row is new row else false
		 */
		isNewRow: function(rowHref){
			if (rowHref) {
				return (rowHref.lastIndexOf(this.newElementPrefix) != -1);
			}else {
				return false;
			}
		},
        
        getRowPostURI: function(row){
            var rowURI = this._getRowURI(row.href);
            var rowURIObj = {
                rowURI: rowURI,
                isNew: false
            };
            if (rowURI == null) {
                rowURIObj.rowURI = this._getNewElementURI(row);
                rowURIObj.isNew = true;
            }
            return rowURIObj;
        }
    });
})();
