/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.web.core.ui.internal.EPMGrid");

dojo.require("dojox.grid.Grid");
dojo.require("dojox.grid.compat._data.model");
dojo.require("dojox.grid.compat._data.editors");
dojo.require("dojo.parser");
dojo.require("dojox.grid.compat._data.dijitEditors");
dojo.require("com.ibm.epm.web.core.util.internal.Util");

// Events published by EPMGrid
dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");

(function(){

    var DojoGrid = dojox.Grid;
    var Util = com.ibm.epm.web.core.util.internal.Util;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    
    dojo.declare("com.ibm.epm.web.core.ui.internal.EPMGrid", DojoGrid, {
    
        _gridID: null,
        
        _selected: null,
        
		overRideCellClasses: true,
		
        postCreate: function(){
            
            this.cellFieldName = "";
            this.isEditable = true;
            // override to add null check of inRowNodes[0].parentNode
            this.views.normalizeRowNodeHeights = function(inRowNodes){
                var h = 0;
                var i = 0;
                var n = 0;
                var o = 0;
                for (i = 0; (n = inRowNodes[i]); i++) {
                    h = Math.max(h, (n.firstChild.clientHeight) || (n.firstChild.offsetHeight));
                }
                h = (h >= 0 ? h : 0);
                
                var hpx = h + 'px';
                for (i = 0; (n = inRowNodes[i]); i++) {
                    if (n.firstChild.clientHeight != h) {
                        n.firstChild.style.height = hpx;
                    }
                }
                // querying the height here seems to help scroller measure the page on IE
                if (inRowNodes && inRowNodes[0] && inRowNodes[0].parentNode) {
                    inRowNodes[0].parentNode.offsetHeight;
                }
            };
            
            if (this.model) {
            
                var m = this.model;
                if (dojo.isString(m)) {
                
                    m = dojo.getObject(m);
                }
                this.model = (dojo.isFunction(m)) ? new m() : m;
                
                this._setModel(this.model);
                
            }
            
            this.inherited("postCreate", arguments);
            
            // Context menu disappears quickly when right clicking on a selected record of resultset
            this.focus.styleRow = function(){
            };
        },
        
        // Cannot navigate through rows using keyboard, there is a defect to Community Source
        // https://cs.opensource.ibm.com/tracker/index.php?func=detail&aid=76353&group_id=2078&atid=5124
        onKeyDown: function(e){
            if (e.metaKey) {
                return;
            }
            
            if (e.ctrlKey) {
                this.ctrlKeyPressed = true;
            }
            
            if (e.altKey) {
                this.altKeyPressed = true;
            }
            
            
            if (this.ctrlKeyPressed) {
                e.ctrlKeyPressed = true;
                this.ctrlKeyPressed = false;
            }
            
            if (this.altKeyPressed) {
                e.altKeyPressed = true;
                this.altKeyPressed = false;
            }
            
            var keys = dojo.keys;
            var edit = this.edit;
            var focus = this.focus;
			
            e.gridID = this._gridID;
			e.cell = focus.cell;
			e.rowIndex = focus.rowIndex
            dojo.publish(EPMGridPubSub.ON_KEY_DOWN, [e]);
			
            switch (e.keyCode) {
                case keys.ESCAPE:
                    edit.cancel();
                    break;
                case keys.ENTER:
					if (this.selection.selectedIndex !== focus.rowIndex) {
						var publishObj = {
							gridID: this._gridID,
							rowIndex: focus.rowIndex
						};
						dojo.publish(EPMGridPubSub.ON_ROW_CLICK, [publishObj]);
						
					}
					this.selection.select(focus.rowIndex);
					if (dojo.isFunction(this.isCellEditable)) {
                        e.cell = this.selection.grid.focus.cell;
                        if (!this.isCellEditable(e)) {
                            return;
                        }
                    }
                    if (!this.isEditable) {
                        return;
                    }
                    if (!e.shiftKey) {
                        var isEditing = edit.isEditing();
                        edit.apply();
                        if (!isEditing) {
                            edit.setEditCell(focus.cell, focus.rowIndex);
                        }
                    }
                    break;
                case keys.TAB:
                
                    // Check to see if the grid is editing
                    // If yes, then save the edit first
                    var isEditing = edit.isEditing();
                    if (isEditing) {
                        edit.apply();
                    }
                        
                    focus[e.shiftKey ? 'previousKey' : 'nextKey'](e);
                    if (dojo.isFunction(this.isCellEditable)) {
                        e.cell = this.selection.grid.focus.cell;
                        if (!this.isCellEditable(e)) {
                            return;
                        }
                    }
                    break;
                case keys.LEFT_ARROW:
                    if (!edit.isEditing()) {
                        dojo.stopEvent(e);
                        focus.move(0, -1);
                    }
                    break;
                case keys.RIGHT_ARROW:
                    if (!edit.isEditing()) {
                        focus.move(0, 1);
                    }
                    break;
                case keys.UP_ARROW:
                    if (!edit.isEditing() && focus.rowIndex !== 0) {
                        dojo.stopEvent(e);
                        focus.move(-1, 0);
                    }
                    else 
                        if (focus.rowIndex === 0) {
                            dojo.stopEvent(e);
                        }
                    break;
                case keys.DOWN_ARROW:
                    if (!edit.isEditing() && focus.rowIndex + 1 != this.model.count) {
                        dojo.stopEvent(e);
                        focus.move(1, 0);
                    }
                    break;
            }
        },
        // eventhandler to handle out of focus events like clicking on 
        // clicking on toolbars from Admin view etc.
        onBlur: function(e){
            if (this.edit.isEditing()) {
                this.edit.apply();
            }
        },
        // Override dojox.VirtualGrid#doApplyCellEdit so that it notifies triggers if cell values changes by edit
        // (doApplyCellEdit is called even if values are not changed and onclick)
        doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
            var oldVal = this.model.getDatum(inRowIndex, inFieldIndex);
            //inValue check is added to skip the call to parent grid in case of dialog editors
            if (typeof(inValue) != "undefined") {
                this.inherited("doApplyCellEdit", arguments);
            }
            if (inValue != oldVal) {
                this.onEditChanged();
            }
        },
        
        // a stub function that can be connected
        onEditChanged: function(){
        },
        
        getColumnWidth: function(colIndex){
            var cell = this.getCell(colIndex);
            if (!cell) {
                return -1;
            }
            var node = cell.getNode(0);
            if (!node) {
                return -1;
            }
            var w = dojo.contentBox(node).w;
            return w;
        },
        
        setStructure: function(inStructure){
            var model = this.model;
			var self = this;
            var convertDateTime = function(inRowIndex){
                var index = this.fieldIndex; // Column has no value when columns are customized
                if (index == -1) {
                    return "";
                }
                return Util.convertDateTime(model.getDatum(inRowIndex, index));
            };
			
			/* Before creating new layout, disconnecting the events connected to the inline editor.
			 * Each time new Inline editors will get created when you create new layout.
			 */ 
			
            if(this.layout) {
				dojo.forEach(this.layout.cells, function(cell){              
					if(cell && cell.editor && cell.editor.disconnectEvents){
							cell.editor.disconnectEvents();
					}
            	});
			}
            dojo.forEach(inStructure, function(view){
				if(view.cells){
					dojo.forEach(view.cells, function(cell){
	                    dojo.forEach(cell, function(item){
	                        // when property view preference is changed from hidden -> show, script error occurs because 'item' is undefined
	                        if (item && item.dataType == "DateTime" && !item.get) {
	                            item.get = convertDateTime;
	                        }
							if (dojo.isIE) {
								if (!item.formatter) {
									item.formatter = Util.addSpanforWhiteSpace;
								}
							}
	                    });
	                });
					if (self.overRideCellClasses) {
						if (view.defaultCell) {
							if (!view.defaultCell.editor) {
								dojo.forEach(view.cells, function(item){
									Util.addCellClass(item);
								});
							}
						}
						else {
							dojo.forEach(view.cells, function(item){
								Util.addCellClass(item);
							});
						}
					}
				}else{
					if (self.overRideCellClasses) {
						if (view.defaultCell) {
							if (!view.defaultCell.editor) {
								dojo.forEach(view.rows, function(item){
									Util.addCellClass(item);
								});
							}
						}
						else {
							dojo.forEach(view.rows, function(item){
								Util.addCellClass(item);
							});
						}
					}
				}
                
				
            });
		if (dojo.isIE) {
			dojo.forEach(inStructure, function(view){
				dojo.forEach(view.rows, function(cell){
					dojo.forEach(cell, function(item){
						if (!item.formatter) {
							item.formatter = Util.addSpanforWhiteSpace;
						}
					});
				});
			});
		}
            this.inherited("setStructure", arguments);
        },
        
        // model
        _setModel: function(inModel){
            // if(!inModel){ return; }
            this.model = inModel;
            if (this.model) {
                this.model.observer(this);
                this.model.measure();
                this.indexCellFields();
            }
        },
        
        refresh: function(){
            // summary:
            //		Re-render the grid, getting new data from the model
            this.edit.cancel();
            this.model.measure();
        },
        
        /**
         *  onCellMouseOver : Event handler for cell mouse over event of EPMGrid;
         *				 Overrides the dojox onCellMouseOver to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_CELL_MOUSE_OVER
         *					- Set cellNode title
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onCellMouseOver: function(event){
            var publishObj = event;
            if (publishObj) {
				//This works in IE.
				if(publishObj.toElement){
					if(dojo.trim(publishObj.toElement.innerText) != ""){
						publishObj.cellNode.title = publishObj.cell.name + " : " + publishObj.toElement.innerText;
					}else {
						publishObj.cellNode.title = publishObj.cell.name;
					}
				}
		         else if (publishObj.cellNode) {
				 	if (dojo.trim(publishObj.cellNode.textContent) != "") {
						publishObj.cellNode.title = publishObj.cell.name + " : " + publishObj.cellNode.textContent;
					}else {
						publishObj.cellNode.title = publishObj.cell.name;
					}
				}
				
                publishObj.gridID = this._gridID;
                dojo.publish(EPMGridPubSub.ON_CELL_MOUSE_OVER, [publishObj]);
            }
            
            this.inherited(arguments);
        },
        
        /**
         *  onCellMouseOut : Event handler for cell mouse out event of EPMGrid;
         *				 Overrides the dojox onCellMouseOut to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_CELL_MOUSE_OUT
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onCellMouseOut: function(event){
            var publishObj = event;
            if (publishObj) {
                publishObj.gridID = this._gridID;
                dojo.publish(EPMGridPubSub.ON_CELL_MOUSE_OUT, [publishObj]);
            }
            
            this.inherited(arguments);
        },
        
        /**
         *  onRowClick : Event handler for row click event of EPMGrid;
         *				 Overrides the dojox onRowClick to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_ROW_CLICK
         *					- Set the selected row in this._selected
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onRowClick: function(event){
            var publishObj = event;
            
            if (publishObj) {
                var rowIndex = event.rowIndex;
                this._selected = rowIndex;
                
                publishObj.gridID = this._gridID;
                publishObj.rowIndex = event.rowIndex;
                dojo.publish(EPMGridPubSub.ON_ROW_CLICK, [publishObj]);
            }
            this.inherited(arguments);
        },
        
        /**
         *  onHeaderCellClick : Event handler for header click event of EPMGrid;
         *				 Overrides the dojox onHeaderCellClick to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_HEADER_CLICK
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onHeaderCellClick: function(event){
			this.inherited(arguments);
            var publishObj = event;
            if (publishObj) {
                publishObj.gridID = this._gridID;
                dojo.publish(EPMGridPubSub.ON_HEADER_CLICK, [publishObj]);
            }
        },
        
        /**
         *  onMouseOver : Event handler for mouse hover event of EPMGrid;
         *				 Overrides the dojox onMouseOver to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_MOUSE_OVER
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onMouseOver: function(event){
            var publishObj = event;
            if (publishObj) {
                publishObj.gridID = this._gridID;
                publishObj.rowIndex = event.rowIndex;
                dojo.publish(EPMGridPubSub.ON_MOUSE_OVER, [publishObj]);
            }
            this.inherited(arguments);
        },
        
        /**
         *  onMouseOut : Event handler for mouse out event of EPMGrid;
         *				 Overrides the dojox onMouseOut to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_MOUSE_OUT
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onMouseOut: function(event){
            var publishObj = event;
            if (publishObj) {
                publishObj.gridID = this._gridID;
                publishObj.rowIndex = event.rowIndex;
                dojo.publish(EPMGridPubSub.ON_MOUSE_OUT, [publishObj]);
            }
            this.inherited(arguments);
        },
        
        /*
         * This function always returns true by default.
         * If any object which has instantiated the EPMGrid should use hitch to over-write this function.
         * This will be required if any cell needs to be non-editable.
         * The function which over-writes this function should return true if the cel has to be editable
         * else should return false.
         */
        isCellEditable: function(event){
            return true;
        },
        
        /**
         *  onCellDblClick : Event handler for cell double click event of EPMGrid;
         *				 Overrides the dojox onCellDblClick to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_CELL_DBL_CLICK
         *					- Handle enable/disable cell edit clicks
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onCellDblClick: function(event){
            if (dojo.isFunction(this.isCellEditable)) {
                if (!this.isCellEditable(event)) {
                    return;
                }
            }
            var publishObj = event;
            if (publishObj) {
                publishObj.gridID = this._gridID;
                dojo.publish(EPMGridPubSub.ON_CELL_DBL_CLICK, [publishObj]);
            }
            
            if (!this.isEditable) {
                return;
            }
            if (this.cellFieldName == event.cell.field) {
                return;
            }
            this.inherited(arguments);
        },
        
        /**
         *  onCellClick : Event handler for cell click event of EPMGrid;
         *				 Overrides the dojox onCellClick to handle additional funtionality such as
         *					- Publish EPMGridPubSub.ON_CELL_CLICK
         *  Incoming :
         *  	event - event object
         *  Outgoing : none
         *
         */
        onCellClick: function(event){
			var cellEditable = false;
			if (dojo.isFunction(this.isCellEditable)) {
				cellEditable = this.isCellEditable(event);
			}
			
			if(!cellEditable && this.edit.isEditing()) {
				this.edit.apply();
			}
			//Defect 14833: Set the focus if the cell is not focussed
			if (!this.focus.isFocusCell(event.cell, event.rowIndex)) {
				this.focus.setFocusIndex(event.rowIndex, event.cellIndex);
			}

            this.selection.grid.focus.cell = event.cell; 
			if (dojo.isFunction(this.isCellEditable)) {
                if (!cellEditable) {
					this.onRowClick(event);
                    return;
                }
            }
            var publishObj = event;
            if (publishObj) {
                publishObj.gridID = this._gridID;
                
                dojo.publish(EPMGridPubSub.ON_CELL_CLICK, [publishObj]);
            }
            
            this.inherited(arguments);
        },
        
        // TODO port once we move to new grid
        onSelected: function(inRowIndex){
		    var publishObj = {
			    gridID: this._gridID,
			    rowIndex: inRowIndex
			};
            dojo.publish(EPMGridPubSub.ON_ROW_SELECTED, [publishObj]);
            
            this.inherited(arguments);
        },
        
        // TODO port once we move to new grid
	    onDeselected: function(inRowIndex){
		    var publishObj = {
			    gridID: this._gridID,
			    rowIndex: inRowIndex
			};
            dojo.publish(EPMGridPubSub.ON_ROW_DESELECTED, [publishObj]);
            
            this.inherited(arguments);
	    },

        // TODO port once we move to new grid
	    onSelectionChanged: function(){
		    var publishObj = {
			    gridID: this._gridID
			};
            dojo.publish(EPMGridPubSub.ON_ROW_SELECTION_CHANGED, [publishObj]);
            
            this.inherited(arguments);
	    },
          
        // TODO port once we move to new grid
        onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		    var publishObj = {
			    gridID: this._gridID,
			    value: inValue,
			    rowIndex: inRowIndex,
			    fieldIndex: inFieldIndex
			};
            dojo.publish(EPMGridPubSub.ON_APPLY_CELL_EDIT, [publishObj]);
            
            this.inherited(arguments);
        },
        
        setEditableCellState: function(cellField, state){
            if (!state) {
                this.cellFieldName = cellField;
            }
            else {
                this.cellFieldName = "";
            }
        },
        setEditableState: function(state){
            this.isEditable = state;
        },
        
        /**
         * 	setGridRowSelection: 	To select the default row after populating the grid.
         * 	@Params: rowIndex - the defeult row index.
         */
        setGridRowSelection: function(rowIndex){
            this.selection.select(rowIndex);
            this._selected = rowIndex;
        },
        
		/**
		 * generic resize grid.
		 * @param {Object} a
		 */
        resizeGrid: function(a){
            this.autoHeight = false;
			this.updateGrid(a);
        },
		
		/**
		 * resize the grid with auto-growing height. We need this for tree based grids.
		 * @param {Object} a
		 */
		resizeGridWithAutoHeight: function(a){
            this.autoHeight = true;
            this.updateGrid(a);
		},
		
		/**
		 * code common to resizeGrid and resizeGridWithAutoHeight.
		 * @param {Object} a: Height and width params passed by the view.
		 */
		updateGrid: function(a){
            this.autoWidth = false;
            var h = 300;
            
            if (a.h === 0) {
            
                dojo.contentBox(this.domNode, {
                    w: a.w,
                    h: h
                });
                
            }
            else {
                dojo.contentBox(this.domNode, {
                    w: a.w,
                    h: a.h
                });
                
            }
            this.update();
		},
		
		onHeaderCellMouseOver: function(event){
			// summary:
			//		Event fired when mouse moves over a header cell.
			// e: Event
			// 		Decorated event object which contains reference to grid, cell, and rowIndex
			var publishObj = event;
            if (publishObj) {
                publishObj.gridID = this._gridID;
                dojo.publish(EPMGridPubSub.ON_HEADER_CELL_MOUSE_OVER, [publishObj]);
            }
			this.inherited(arguments);
		},
        //Surya - Making Grid comaptible for IE
        hasLayout: function(){
            return this.layout.cells.length;
        },
        
        //Surya - Making Grid compatible for IE
        resize: function(){
            if (!this.domNode || !this.domNode.parentNode || this.domNode.parentNode.nodeType != 1 || !this.hasLayout()) {
                return;
            }
            this.inherited(arguments);
            
        },
        destroy: function(){
            this.inherited('destroy', arguments);
        }
        
    });
	
	var oldFunc = dijit._onFocusNode;
	dijit._onFocusNode = function(node) {
		if(node.nodeName && node.nodeName.toLowerCase() == "body"){
			// Ignore focus events on main document <body>.  This is specifically here
			// so that clicking the up/down arrows of a spinner (which don't get focus)
			// won't cause that widget to blur.  (IIRC, IE issue)
			return;
		}
       oldFunc(node);
	};
})();
