/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.ui.internal.EPMMenu");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    
    // EPM Core Constants
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    
    
    var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	
    dojo.declare("com.ibm.epm.web.core.ui.internal.EPMMenu", [dijit._Widget], {
    
        _menu: null,
        menuItems: null,
        
        _subMenu: new Array(),
        
        /**
         * handleMenuPopUp: Method to create a menu with the menuItems.<br>
         * @param {Object} e: onClick event passed by EPMTree.<br>
         */
        handleMenuPopUp: function(e){
            if (this._isMenuOpen) {
                this._closeMenu();
            }
            if (!this._menu) {
                //Create the menu if its not already created for the same widget.
                this._menu = new dijit.Menu({
                    onExecute: dojo.hitch(this, this._closeMenu)
                });
                this._addSubMenuAndItems(this.menuItems);
            }
			
			// Handle enabling of menuitem option for iterations
			var element = e.target;
            var treeWidget = dijit.getEnclosingWidget(element);
			if (treeWidget) {
				this._disableEnableCurrentIterationMenuItem(treeWidget);
			}

            var coords = dojo.coords(e.target, true);
            dijit.popup.open({
                popup: this._menu,
                parent: this,
                x: coords.x + 10,
                y: coords.y + 10,
                onExecute: dojo.hitch(this, this._closeMenu),
                orient: 'BL'
            });
            //register event handler that causes menu to close
            this._menuCloseOnBody = this.connect(this._menu, "_onBlur", this._closeMenu);
			this._menuCloseOnEsckey = this.connect(this._menu.domNode, "onkeydown", this._keydown);
            this._isMenuOpen = true;
            this._menu.focus();
        },

        /**
         * _disableEnableCurrentIterationMenuItem: Method to enable or disabled the menu item for 'Set as Current' <br>
         * treeWidget: Tree widget <br>
         */
        _disableEnableCurrentIterationMenuItem: function(treeWidget){
			// Handle enabling of menuitem option for iterations
			if ((treeWidget) && (treeWidget._wbsTree)) {
				var ctxtNode = treeWidget._wbsTree.getNodeHover();
				if (ctxtNode) {
					var item = ctxtNode.item;					
					if (item) {
						var resourceType = dojo.isArray(item.resourcetype)?item.resourcetype[0]:item.resourcetype;
						if (resourceType == EPMConstants.ITERATION_ID) {
							var menuItems = this._menu.getChildren();
							if (menuItems) {
								for (var i = 0; i < menuItems.length; i++) {
									if (menuItems[i].iconClass == IconClassConstants.CURRENT_ITERATION_MENU_ICON_CLASS) {
										var setCurIterationMI = menuItems[i];
										if (setCurIterationMI) {
											if (item.currentIteration[0] != 0) {
												setCurIterationMI.attr('disabled', true);
											} else {
												setCurIterationMI.attr('disabled', false);
											}
										}
									}
								}									
							}
						}
					}
				}
			}
		},
		
        /**
         * _addSubMenuAndItems: Method to add subMenu and the menuItems. <br>
         * items: collection of menus and items. <br>
         */
        _addSubMenuAndItems: function(items){
            if (!items || items.length === 0) {
                return;
            }
            
            this.popupMenuItems = new Array();
            
            for (var i = 0; i < items.length; i++) {
                if (items[i].type == EPMConstants.POPUP_TYPE) {
                    this._addPopUpMenuItem(items[i]);
                }
                else {
                    this._addMenuItem(items[i]);
                }
            }
        },
        
        /**
         * _addPopUpMenuItem: Method to add a popup in the main menu.
         * item: item to be added as a popup.
         */
        _addPopUpMenuItem: function(item){
            var newMenu = new dijit.Menu();
            var popUpMenuItem = new dijit.PopupMenuItem({
                label: item.label,
                iconClass: item.iconClass,
                popup: newMenu
            });
            popUpMenuItem.startup();
            
            //Add the item under the parentID.										
            this._addItemUnderSubMenu(popUpMenuItem, item.parentID, item.addSeparator);
            
            //add the id, menu pair to the subMenu which can be searched later to add items under it.
            var keyValue = {
                "id": item.id,
                "popup": newMenu
            };
            this._subMenu.push(keyValue);
        },
        
        /**
         * _addMenuItem: Form a menuItem object and add that under the menu/subMenu.
         * item: menuItem to be added to the menu.
         * item.iconUrl:Non-null value for the items being fetched from the process spec.
         * item.iconClass: Non-null for the items which have the image refrence in a css class.
         */
        _addMenuItem: function(item){
			if (item.iconUrl) {
				var menuItem = new dijit.MenuItem({
					label: item.label,
					_onClick: item._onClick,
					disabled: item.disabled
				});
				var img = new Image();
				img.src = item.iconUrl;
				menuItem.iconNode.appendChild(img);
			}else if(item.iconClass){
				var menuItem = new dijit.MenuItem({
					label: item.label,
					iconClass:item.iconClass,
					_onClick: item._onClick,
					disabled: item.disabled
				});
			}
            
			this._addItemUnderSubMenu(menuItem, item.parentID, item.addSeparator);
        },
        
        /**
         * _addItemUnderSubMenu: add popup/menu Item under the parent with given parentID. <br>
         * item: item to be added.<br>
         * parentID: id of the parent under which the item has to be added.<br>
         */
        _addItemUnderSubMenu: function(item, parentID, addSeparator){
            //if parentID is null, add the popup/item directly to menu. 
            if (parentID == null) {
                if (addSeparator == true) {
                    var separator = new dijit.MenuSeparator();
                    this._menu.addChild(separator);
                }
                this._menu.addChild(item);
            }
            //Else add that to respective subMenu.
            for (var i = 0; i < this._subMenu.length; i++) {
                var currentSubMenu = this._subMenu[i];
                var currentMenu = currentSubMenu.popup;
                
                if (currentSubMenu.id == parentID) {
                    if (addSeparator == true) {
                        var separator1 = new dijit.MenuSeparator();
                        currentMenu.addChild(separator1);
                    }
                    currentMenu.addChild(item);
                }
            }
        },
        /**
         * _closeMenu: Menu to be closed.
         * e: captured event.
         *
         */
        _closeMenu: function(e){
            if (e) {
                dojo.stopEvent(e);
            }
            this.disconnect(this._menuCloseOnBody);
			this.disconnect(this._menuCloseOnEsckey);
            dijit.popup.close(this._menu);
            this._isMenuOpen = false;
        },
		
		_keydown: function(e){
			var key = e.keyCode || e.charCode;
            var k = dojo.keys;            
            if (key == k.ESCAPE) {
				this._closeMenu(e);
			}
		}
        
    });
    
})();
