/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.web.core.ui.internal.EPMTree");

// Dijit Tree Base class
dojo.require("dijit.Tree");
dojo.require("dijit.InlineEditBox");

// Context Menu widget
dojo.require("com.ibm.epm.web.core.ui.internal.EPMMenu");

//EPMTreeClient
dojo.require("com.ibm.epm.web.core.client.internal.EPMTreeClient");

// Icon class constants
dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");

// EPM Core Constants
dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
dojo.require("com.ibm.epm.web.core.util.internal.Constants");
// Events published by EPMTree
dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");

//Service Response Handler
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

//Error Handling
//dojo.require("com.ibm.epm.web.core.ui.internal.common.ErrorDialog");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");

dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");

dojo.require("com.ibm.epm.web.core.ui.internal.WorkItems.WorkItemConfigurationsHandler");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMSequenceValue"); //$NON-NLS-1$
// Localization
dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages");

(function(){

	// Context Menu widget
	var EPMMenu = com.ibm.epm.web.core.ui.internal.EPMMenu;
	var EPMTreeClient = com.ibm.epm.web.core.client.internal.EPMTreeClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
	var ControllersFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
	
	//IMP : To make the tree code generic pls provide the Icon class name as 
	// Resource+ "Icon" in the CSS file like for task resource the name taskIcon
	var IconClass = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	
	var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
	var WBS_Constants = com.ibm.epm.web.core.util.internal.Constants;
	
	var EPMTreeEvents = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
	var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
	var CoreMessages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages");
    var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
	var SequenceValue = com.ibm.epm.web.core.ui.internal.EPMSequenceValue;
    
	var INITIAL_OFFSET = 32;
	
    dojo.declare("com.ibm.epm.web.core.ui.internal.EPMTree", dijit.Tree, {
    
        templateString: "<div class=\"dijitTreeContainer\" style=\"\" waiRole=\"tree\"\r\n\t" +
        "dojoAttachEvent=\"onclick:_onClick,onmouseover:_onMouseOver,onmouseout:_onMouseOut," +
        "onkeypress:_onKeyPress,onkeydown:_onKeyDown\">\r\n",
        
        _ctxtItems: null,
        _ctxMenu: null,
        _ctxNode: null,
        /**
         * The widget that created the tree
         */
        _containingWidget: null,
        
        // Defining the tree store
        _treeStore: null,
        
        
        //Will store the information for fetching first level element for the Tree and 
        //then using the LinkType info to fetch the children nodes
        _topLevelElementsURI: null,
        _linkTypeForChildren: null,
        _treeID: null,
        
		// flag to indicate the tree has an unsaved element, currently the tree based grids donot support
		// row level save, if that is supported then this flag needs to be an array representing each row
		hasNewNode: false,
		
        // Selected Node
        _selectedNode: null,
        _previousSelectedNode: null,
        
        /*
         * TreeNodes for which we dont want the menu; set to READONLY to make all nodes
         * be without menus.
         */
        _nodesWithoutMenu: [],

        // Constant to pass to _nodesWithoutMenu to cause entire tree to be without menus and readonly
        READONLY: "readonly",
        
        // variable to track if the tree inline editing is in progress
        editing: false,
		_hoverNode: null,
        
        // The information that needs to be sent by anyone using EPMTree
        // ID : id of the tree to be created.
        // URI to get top level elements
        // LinkType for the childrens
        // Pass the array of nodes for which we dont want Tree Menu to appear like 
        // in WBS We dont menu for epmtask.
        
        // The call will create the tree store and will create the base tree.
        // Pass READONLY for nodes arg to remove menus from all nodes.
        initialize: function(topLevelURI, linkType, ID, nodes, menuImage){
            // store other information
            this._topLevelElementsURI = topLevelURI;
            this._linkTypeForChildren = linkType;
            this._treeID = ID;
            this._treeStore = this.model.store;
            this._openedItemIds = [];
            this._nodesWithoutMenu = nodes;
			this.menuImage = menuImage;
            this.refresh();
            this._expandAllTopLevelNodes();
			this.connectToMenuImageEvents();
			
			//Updated the image source with the relative path.
			if (this.menuImage) {
				this.menuImage.src = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/dd-arrow.gif");
			}
        },

        /**
         *  _onKeyPress : Override the onKeyPress event to ignore all keys while editing
         *  Incoming : event e
         *  Outgoing : none
         */
        _onKeyPress: function(/*Event*/ e) {
            if (this.tree.editing)  
                    return; // we are editing so we should ignore all keys
            this.inherited(arguments);
        },
        
        _onKeyDown: function(/* Event */ e) {
            if (this.tree.editing)  
                return; // we are editing so we should ignore all keys
            
            // Publish an event for a key press (incase someone is interested in doing
            // something special)
            var element = e.target;
            var nodeWidget = dijit.getEnclosingWidget(element);
            var nodeWidgetItem = null;
            if (nodeWidget) {
                if (nodeWidget.item) {
                    nodeWidgetItem = nodeWidget.item;
                } else {
                    if (nodeWidget.tree) {
                        if (nodeWidget.tree._selectedNode) {
                            if (nodeWidget.tree._selectedNode.item) {
                                nodeWidgetItem = nodeWidget.tree._selectedNode.item;
                            }
                        }
                    }            
                }
            }
            
            if (nodeWidgetItem) {
                
                if (!nodeWidgetItem.treeID) {
                    // Add a tree ID to this item if its missing one
                    // This problem was seen when a new item was created
                    // using the keyboard, its name was changed and a
                    // TAB was hit on it. In AbstractGridView._validateTreeId()
                    // (called by AbstractGridView._validateAndGetRow()), the
                    // check to validate the tree ID failed and as a result
                    // the TAB operation did not work.
                    nodeWidgetItem.treeID = this._treeID;
                }
                            
                var publishObj = {
                    nodeWidgetItem: nodeWidgetItem,
                    treeID: this._treeID,
                    event: e
                };
                    
                // Publish the set of keys pressed                
                publishObj.keySet = { 
                                      keyCode: e.keyCode,
                                      charCode: e.charCode,
                                      altKey: e.altKey,
                                      metaKey: e.metaKey,
                                      ctrlKey: e.ctrlKey,
                                      shiftKey: e.shiftKey
                                    };                     
                dojo.publish(EPMTreeEvents.ON_KEY_DOWN, [publishObj]);
				/* Ctrl-down arrow pops up the context menu */
				if(e.altKey && e.keyCode === 40){					
					if (this.menuImage) {
						if(document.createEvent){
							/* firefox event creation */
							var evt = document.createEvent("MouseEvents");
           					evt.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
						}
						else if(document.createEventObject)
							var evt = document.createEventObject(); /* IE event creation */
						else 
							return;

						if(this.menuImage.dispatchEvent)
							this.menuImage.dispatchEvent(evt); /* firefox event dispatch */
						else if	(this.menuImage.fireEvent){
							this.menuImage.fireEvent("onclick",evt); /* IE event dispatch */
						}												
            		}
					dojo.stopEvent(evt);							
				}
				/* Enter key or space bar for selecting tree node */
				if(e.keyCode === 32 || e.keyCode === 13){					
					this._onClick(e);							
				}
			
            } 
        },

        /**
         *  _buildHandlesArrayOfAllChildNodes : Method to build an array of item handles for the entire sub tree from the specified node
         *  Incoming : TreeNode - nodeWidget
         *  Outgoing : none
         */
		_buildHandlesArrayOfAllChildNodes: function(node, collectionItemHandles){
			var self = this;
			var children = node.getChildren();
			//recursively add the children as well.
            dojo.forEach(children, function(child){
                handle = self.convertToItemHandle(child.item);
				collectionItemHandles.push(handle);
				self._buildHandlesArrayOfAllChildNodes(child, collectionItemHandles);
            });
			
			return collectionItemHandles;
		},

        /**
         *  _deleteSubTree : Method to delete the entire sub tree from the specified node
         *  Incoming : TreeNode - nodeWidget
         *  Outgoing : none
         */		
		_deleteSubTree: function(nodeWidget) {
			var item = nodeWidget.item;
			var handle = this.convertToItemHandle(item);
			var collectionItemHandles = [];
			
			collectionItemHandles.push(handle);

			collectionItemHandles = this._buildHandlesArrayOfAllChildNodes(nodeWidget, collectionItemHandles);

			this._deselectTreeNode(item, true);						
			this._deleteItemNode(item);

			var publishedObject = {
				collectionItemHandle: collectionItemHandles,
				treeID: this._treeID
			};
			dojo.publish(EPMTreeEvents.ON_NODE_DELETE, [publishedObject]);

			if (!this._isNodeNew(item)) {
				var parentId = dojo.isArray(item.parentIdentifier) ? item.parentIdentifier[0] : item.parentIdentifier;
				if (parentId) {
					var parentNodeItem = this.getItemNodeByIdentity(parentId);
					if (parentNodeItem) {
						this.nodeRefreshParentInGrid(parentNodeItem);
					}
				}
			}
		},

        /**
         *  _getRootNode : Method to fetch the root node in the tree hierarchy the specified node is part of
         *  Incoming : TreeNode - nodeWidget
         *  Outgoing : none
         */
		_getRootNode: function (nodeWidget) {
			var item = nodeWidget.item;
			var parentNode = this._itemNodeMap[item.parentIdentifier[0]];
			if (parentNode) {
				var parentItem = parentNode.item;
				if (parentItem.parentIdentifier[0]) {
					parentNode = this._getRootNode(parentNode);
				}
			}
			return parentNode;			
		},

		/**
		 * Method to show a confirmation dialog to user before removing the node.
		 */
		confirmAndRemoveNode: function(/*TreeNode*/ nodeWidget, depSelected){
			if (!nodeWidget) {
                this._displayError(CoreMessages.REMOVE_WBS_SELECT_ELEMENT);
                return;
            }

			var item = nodeWidget.item;
			var name = dojo.isArray(item.name)?item.name[0]:item.name;
			var resourceType = dojo.isArray(item.resourcetype)?item.resourcetype[0]:item.resourcetype;
			var workItemType = dojo.isArray(item.workItemType)?item.workItemType[0]:item.workItemType;

			if (!this._isNodeNew(item)) {
				if ((resourceType == WBS_Constants.EPM_PROJECT) ||
				(resourceType == WBS_Constants.EPM_PROGRAM) ||
				(resourceType == WBS_Constants.ITERATION_TYPE)) {
					this._displayError(CoreMessages.REMOVE_WBS_NOT_SUPPORTED);
					return;
				}
			} else {
				workItemType = resourceType;
			}
			
			this._nodeToBeRemoved = nodeWidget;
            var buttonlist = [{
                label: CoreMessages.YES,
                action: dojo.hitch(this, this._removeNode)
            }, {
                label: CoreMessages.NO,
                action: dojo.hitch(this, this._cancelRemoveOperation)
            }];
            
            var labelList = [{
                label: dojo.string.substitute(CoreMessages.CONFIRM_REMOVE_MSG, [workItemType, name])
            }];

			if ((this._isNodeNew(item)) && (depSelected)) {
				var length = labelList.length;
				// Add a empty line
				labelList[length] = {
					label: "\n"
				};
				// Append the message for selected dependency
				labelList[length + 1] = {
					label: dojo.string.substitute(CoreMessages.DEPENDENCY_IS_SELECTED, [workItemType, name])
            	};
			}

	        this.confirmRemoveDialog = new EPMConfirmDialog({
	            labelList: labelList,
	            title: CoreMessages.CONFIRM_REMOVE_TITLE,
	            buttonlist: buttonlist
	        });
	        this.confirmRemoveDialog.show();
		},

		/**
		 * Method called when user presses No on the dialog.
		 * Destroy the dialog.
		 */
		_cancelRemoveOperation: function(){
			this.confirmRemoveDialog.destroy();
		},
						
		/**
         *  _removeNode : Method to remove the specified node from the tree
         *  Incoming : TreeNode - nodeWidget
         *  Outgoing : none
         */
        _removeNode: function(){
			this.confirmRemoveDialog.destroy();

			var nodeWidget = this._nodeToBeRemoved;
            if (nodeWidget) {
				// reset the value
				this._nodeToBeRemoved = null;

                var item = nodeWidget.item;
                if (!item) {
                    return;
                } else {
					// Check if node is unsaved
					if (this._isNodeNew(item)) {
						this._deleteSubTree(nodeWidget);
						// check if there are any unsaved elements, if no set the hasNewNode flag to false
						this.checkForUnsavedElements();
						return;
					}
                }
            } else {
                this._displayError(CoreMessages.REMOVE_WBS_SELECT_ELEMENT);
                return;
            }
            
            var uri = item.href;
            var self = this;
            
            //Show the message resolving ... 
            Util.showMessage(CoreMessages.REMOVING + "\"" + item.name + "\"");
            
            var serviceObj = {
                success: function(){
                    self._deleteSubTree(nodeWidget);
                    // hide the message
                    Util.hideMessageArea();
                },
                
                failure: function(errorObj){
                    //Show the error
                    var errMsg = errorObj.errorMessage;
                    self._displayError(errMsg);
    
                    // hide the message
                    Util.hideMessageArea();
                }
            };

			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			EPMTreeClient.deleteURI(srh, uri);
		},

        /**
         *  _makePostCall : Method to make any post call directly in the tree widget
         *  Incoming : uri
         *  Incoming: params
         *  Outgoing : none
         */
        _makePostCall: function(uri, params) {
            var self = this;
            
            var serviceObj = {
                success: function(data){
					var node = self.getNodeByItemIdentity(uri);
					self.nodeRefresh(node.item, false);
					console.debug("_makePostCall Success");
                },
                
                failure: function(errorObj){
                    //Show the error
                    var errMsg = errorObj.errorMessage;
                    self._displayError(errMsg);
                }
            };

			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			EPMTreeClient.postURI(srh, uri, params);
		},
		
        /**
         *  _renameNode : Method to create an inline editor for renaming the tree node name
         *  Incoming : TreeNode - nodeWidget
         *  Outgoing : none
         */
        _renameNode: function(/*TreeNode*/ nodeWidget){
            // Don't rename if widget is in readonly mode
            if (this._nodesWithoutMenu == this.READONLY)
                return;

            // If the tree node is not valid return
            if(!nodeWidget ||
			   !nodeWidget.isTreeNode ||
			   (nodeWidget.item.identifier == WBS_Constants.ROOT)){
                    return;
            }

            // turn off keys for the tree
            this.editing = true;
            
            // Create a span in to attach the editor to.
            var labelNode = nodeWidget.labelNode;
			if (labelNode) {
				var treeEditorSpan = document.createElement('span');
				
				// Multiple clicks on rename editor adds unwanted html tags in the editor
				if ( (labelNode.innerHTML.indexOf("<span") != -1) || (labelNode.innerHTML.indexOf("<SPAN") != -1) ) {
					treeEditorSpan.innerHTML = labelNode.textContent;
				} else {
					//Creating textContent attribute if not exist already
					labelNode.textContent = labelNode.innerHTML;
					treeEditorSpan.innerHTML = labelNode.innerHTML;
				}

				labelNode.innerHTML = "";
				labelNode.appendChild(treeEditorSpan);
				var treeEditor = new dijit.InlineEditBox({
					value: labelNode.textContent,
					node: nodeWidget,
					tree: this,
					model: this.model,
					autoSave: true,
					onChange: this.onNameChange,
					onCancel: this.onRenameCancel,
					width: "100px"
				}, treeEditorSpan);
				
				if (treeEditor) {
					dojo.addClass(treeEditor, "dijitTextBox");
					treeEditor.startup();
					treeEditor.edit();
				}
			}	
        },

        /**
         *  onRenameCancel : Event handler when no change occured in the inline Editor
         */
		onRenameCancel: function() {
            // Select the tree node and bring focus on it
            this.tree._selectTreeNode(this.node);
            this.tree.focusNode(this.node);
            
            // This is a dirty hack for IE only due to a bug in
            // InlineEditBox._showText() where it makes the node
            // lose focus after the user is done editing the label. 
            // This timeout operation is delayed by
            // 0.25 secs from a similar focus operation
            // (dijit.focus(dijit.getFocus()) that happens in
            // InlineEditBox._showText(). This hack allows
            // the tree node to grab focus once the edit box
            // is done. For details refer WI 11024.
		    var _this = this;
            if(dojo.isIE){
                setTimeout(function() {
                    if (_this.node == _this.tree.lastFocused) {
                        _this.tree.focusNode(_this.node);
                    }    
                }, 1250);
            }
                       
			// Turn off the editing
            this.tree.editing = false;
			this.node.labelNode.innerHTML = this.value;
			this.destroy();
		},
		
        /**
         *  onNameChange : Event handler once the inline editing is complete
         *  Incoming : newValue - newly edited value for the tree node
         *  Outgoing : none
         */
		onNameChange: function(newValue){
			var oldValue = this.model.store.getValue(this.node.item, 'name');
			
            // Select the tree node and bring focus on it
            this.tree._selectTreeNode(this.node);
            this.tree.focusNode(this.node);
            
			if((this.tree._containingWidget) 
			&& (this.tree._containingWidget.onNameChangeValidate) 
			&& (!this.tree._containingWidget.onNameChangeValidate(oldValue, newValue))){
				this.model.store.setValue(this.node.item, 'name', oldValue);
				return;
			}
			// Check if there is any change in the previous and new value
			if (oldValue != newValue) {
				// update the value in the tree store
	            this.model.store.setValue(this.node.item, 'name', newValue);	

				// prepare the publish object for the grid to handle the Save
                var publishObj = {
                    treeID: this.tree._treeID,
					item: this.node.item,
                    oldValue: oldValue,
                    newValue: newValue
                };
                dojo.publish(EPMTreeEvents.ON_RENAME_NODE, [publishObj]);
            }
            // Turn off the editing
            this.tree.editing = false;
            
            // This is a dirty hack for IE only due to a bug in
            // InlineEditBox._showText() where it makes the node
            // lose focus after the user is done editing the label. 
            // This timeout operation is delayed by
            // 0.25 secs from a similar focus operation
            // (dijit.focus(dijit.getFocus()) that happens in
            // InlineEditBox._showText(). This hack allows
            // the tree node to grab focus once the edit box
            // is done. For details refer WI 11024.
            var _this = this;
            if(dojo.isIE){
                setTimeout(function() {
                    if (_this.node == _this.tree.lastFocused) {
                        _this.tree.focusNode(_this.node);
                    }    
                }, 1250);
            }
        },		
        
        /**
         *  _addNode : Adds a new node to the tree
         *  Incoming :
         *		parentNode
         *		node - new node
         *		index - child index
         *  Outgoing : none
         *
         */
        _addNode: function(parentNode, node, index){
            if (parentNode === null) {
                if (index == 0) {
                    var item = this.getItemNodeByIdentity("Root");
                    if (item) {
                        this._deleteItemNode(item);
                    }
                }
                this._treeStore.newItem(node);
				if(!this.firstRowOffsetHeight){
					//Fetch the offset for the first row to calculate menuImage offset.
					var treeNode = this.getNodeByItemIdentity(node.identifier);
					this.firstRowOffsetHeight = treeNode.domNode.offsetTop;
				}
            }
            else {
                this._treeStore.newItem(node, {
                    parent: parentNode,
                    attribute: "children"
                });
            }
            
            node.topLevel = true;
            node.index = index;
			
			var treeNode = this.getNodeByItemIdentity(node.identifier);
			if(node.iconUrl){		        
    	        if (treeNode) {
					treeNode.iconNode.src = node.iconUrl;
				}
			}			
			if(node.label){		        
    	        if (treeNode) {
					treeNode.iconNode.alt = node.label;
				}
			}
            //Add a dropdown after adding a node.
            this._addDropDownIcon(node);
            
            // Select the newly added node
            if (this._isNodeNew(node)) {
				// enable the flag to indicate there is a new unsaved node in the tree
				this.hasNewNode = true;
				
                // Since new element is added, Expand the current Tree Node to show the
                // newly added child
                if (parentNode) {
                    this._expandNodeUsingID(parentNode.identifier);
                }
                this._selectNodeById(node.identifier);
				
				if (treeNode) {
					this._renameNode(treeNode);
				}
            }
			
			this._updateNodeForLargeContainer(parentNode);
        },
        
        //Adds Temp child to get the expand icon 	
        _addTempChildNode: function(parentNode){
            //TODO: remove the temp child add method to get the expand node
            //add loading child to get the expand icon
            // set a temp child to show the expand button.. 
            //when the node is expanded delete the temp node
            // making the detail level false for the temp child 
            
            var temp_Child = {
            
                "href": EPMConstants.TEMPCHILDHREF,
                "name": EPMConstants.LOADING,
                "identifier": parentNode.href + EPMConstants.TEMPCHILD,
                "detailLevel": false,
                "resourcetype": "loading",
                "iconClass": IconClass.LOADING_ICON_CLASS,
				"childNodeAddedInStore": false,
				menu: null
            };
            var identity = parentNode.identifier;
            
            var node = this.getItemNodeByIdentity(identity);
            
            // For parent node with more than 100 children the "Loading..." node
            // is expanded to include the number of children.
         	if (this.isLargeContainer(node)) {
				temp_Child.name = dojo.string.substitute(CoreMessages.LOADING_LARGE_CONTAINER, node.childrenCount);
			}

            // add the temp child at index 0
            this._addNode(node, temp_Child, 0);
        },
        
        
        
        // Updates the Tree Node
        
        _updateNode: function(item){
        
            // when updating for refresh call - make the detailLevel value as false
            // Existing element 
            var node = this.getItemNodeByIdentity(item.identifier);
            
            if (node) {
                this._treeStore.setValue(node, 'name', [item.name]);
                this._treeStore.setValue(node, 'resourcetype', [item.resourcetype]);
                this._treeStore.setValue(node, 'href', [item.href]);
				
				if (item.resourcetype == WBS_Constants.ITERATION_TYPE) {
	                this._treeStore.setValue(node, 'currentIteration', [item.currentIteration]);
				}
				//Not updating the detail level with the one fetched from the server.
                //this._treeStore.setValue(node, 'detailLevel', [item.detailLevel]);
            }
            
        },
        
        /**
         * @param treeNode: The Node which will be renamed
         * @param newName: The new name of the node
         * Changes the name of the node by setting the value in the store.
         * It also adds the context menu dialog.
         */
        updateNodeName: function(treeNode, newName){
            if (treeNode) {
                this._treeStore.setValue(treeNode, 'name', [newName]);
            }
        },
        
        //Deletes the tree node from the store
        _deleteItemNode: function(item){
            this._treeStore.deleteItem(item);
        },
        
		// Returns true if tree node has more than 100 children
		isLargeContainer: function(item) {
			if ((item) &&
			    (item.childrenCount) &&
			    (item.childrenCount > 100)) {
				return true;
			} else {
				return false;
			}
		},
		
        //Returns true if tree node has children
        mayHaveChildren: function(item){
            //Call the super class funtion to handle the prcessing		
            var hasChildren = this.model.mayHaveChildren(item);
            
            // We add a temp node for loading, however this should not be considered as a child node
            // FIXME find a neater way to delete nodes
            if (hasChildren) {
                if ((item.children.length == 1) && (item.children[0].name == EPMConstants.LOADING)) {
                    hasChildren = false;
                }
            }
            
            return hasChildren;
        },
        
        // Refreshes the tree form top nodes to all the child nodes if open
        refresh: function(){
            //store the openItemIds to restore their position
            //this._openNodes = this._openItem
            
            //Get the top level tree elements
            this.getRootNodes();
        },
        
		/**
		 * Method to refresh the tree.
		 * 
		 */
		refreshTree: function(){
			// Reset the stored value for current iteration
			this.currentIterationHref = "";
			
			//remove the unaved data.
			this.removeUnsavedTreeNodes();
	        var _treeNodeStore = this._treeStore;
			var self = this;

			//Refresh the top level nodes and recursively call the child nodes for refresh. 			
	        if (_treeNodeStore) {
				function completed(nodes, findResult){
				  for(var i = 0; i < nodes.length; i++){
					self.nodeRefresh(nodes[i], true); 
				  }
				_treeNodeStore.save();
				}
				function error(errData, request){
				  console.log("Failed in fetching the data.");
				}
				//Invoke the fetch for the top level elements.
				_treeNodeStore.fetch({onComplete: completed, onError: error});
	        }
		},

		/**
		 * nodeRefreshParentInGrid: This function is used to refresh the node and all its parent in the node's parent hierarchy in the grid
		 */
		nodeRefreshParentInGrid: function(item){
			if(item == null){
				return;
			}

			var publishObj = {
				collectionItemHandle: [item],
				treeID: this._treeID
			};
			dojo.publish(EPMTreeEvents.ON_NODE_REFRESH, [publishObj]);
			
			var parentId = dojo.isArray(item.parentIdentifier) ? item.parentIdentifier[0] : item.parentIdentifier;
			if (parentId) {
				var parentNodeItem = this.getItemNodeByIdentity(parentId);
				if (parentNodeItem) {
					this.nodeRefreshParentInGrid(parentNodeItem);
				}
				parentId = dojo.isArray(parentNodeItem.parentIdentifier) ? parentNodeItem.parentIdentifier[0] : parentNodeItem.parentIdentifier;
			}
		},
		
		/**
		 * nodeRefresh: This function should do the following:
		 * Add the nodes which are on server but not on client.
		 * remove the unsaved nodes.
		 * update the nodes with the latest info.
		 * trigger refresh on the rows.
		 */
		nodeRefresh: function(item, bTriggerGridUpdate){
			if(item == null){
				return;
			}
			var uri = item.href;
			var self = this;

			var serviceObj = {
			success: function(data){
                if ((!data)) {
                    console.log("No response");
                    return;
                }
				//Build an item with the data fetched from the server.
				var parentId = dojo.isArray(item.parentIdentifier) ? item.parentIdentifier[0] : item.parentIdentifier;
				var nodeModel = self._getTreeNodeModel(data, parentId);
				
				if (self._treeStore && self.getItemNodeByIdentity(nodeModel.identifier)) {
					//The same item should be there in the tree store, update that information.
					self._updateNode(nodeModel);
					
					//Fetch the detail level value from store.
					var itemInStore = self.getItemNodeByIdentity(nodeModel.identifier);
					var detailLevelValue = self._treeStore.getValue(itemInStore, "detailLevel");
					var updateCurrentNode = false;
					
					//If the node is already expanded only then refresh the children.
					if(detailLevelValue){
						if (!parentId) {
							updateCurrentNode = true;
						}
						var parentIdentifier = uri;
						self._refreshChildrenByLinks(data, parentIdentifier, bTriggerGridUpdate);
					}else{
						updateCurrentNode = true;
					}
					
					if (nodeModel && updateCurrentNode) {
						var publishObj = {
							collectionItemHandle: [nodeModel],
							treeID: self._treeID
						};
						dojo.publish(EPMTreeEvents.ON_NODE_REFRESH, [publishObj]);
					}
				}
				else {
					self._addNode(null, nodeModel, i);
					self._addTempChildNode(nodeModel);
				}
			},
			failure: function(errorObj){
				var errMsg = errorObj.errorMessage || "Cannot Retrieve Information";
				self._displayError(errMsg);				
			}
			};
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            EPMTreeClient.getAllResources(srh, uri);
		},
		
		/**
		 * _refreshChildrenByLinks: Method to refresh the children.
		 * @param {Object} data
		 * @param {Object} parentIdentifier
		 * @param {Object} bTriggerGridUpdate
		 */
        _refreshChildrenByLinks: function(data, parentIdentifier, bTriggerGridUpdate){
            var parentNode = this.getItemNodeByIdentity(parentIdentifier);
            var links = this._linkTypeForChildren;
            if (this._linkTypeForChildren && !dojo.isArray(this._linkTypeForChildren)) {
                links = [this._linkTypeForChildren];
            }
			
            for (var i in links) {
                this._refreshChildrenByLink(data, links[i], parentNode, parentIdentifier, bTriggerGridUpdate);
            }
			
        },
        
		/**
		 * Method to refresh the child node.
		 * @param {Object} data
		 * @param {Object} link
		 * @param {Object} parentNode
		 * @param {Object} parentIdentifier
		 */
        _refreshChildrenByLink: function(data, link, parentNode, parentIdentifier, bTriggerGridUpdate){
            var childrenLinks = data[link];
			var collectionItemHandle = null;
            if (!childrenLinks) {
                return;
            }
            if (!dojo.isArray(childrenLinks)) {
                childrenLinks = [childrenLinks];
            }
			var collectionItemHandles = new Array();
            
			for (var i in childrenLinks) {
                var childLink = childrenLinks[i];
                if (childLink != null) {
                    try {
                        var nodeModel = this._getTreeNodeModel(childLink, parentIdentifier);
                        nodeModel.topLevel = true;
                        nodeModel.parentId = parentIdentifier;
                        
                        if (this._treeStore && this.getItemNodeByIdentity(nodeModel.identifier)) {
                            this._updateNode(nodeModel);
							//Call the function recursively for the child elements.
							this.nodeRefresh(nodeModel, bTriggerGridUpdate);				
                        }
                        else {
                            this._addNode(parentNode, nodeModel, i);
                            this._addTempChildNode(nodeModel);
                        }
						var handle = this.convertToItemHandle(nodeModel);
						if (handle) {
							collectionItemHandles.push(handle);
						}
                    } 
                    catch (e) {
                        console.log(e);
                    }
                }
            }

			//Trigger grid update
			if (bTriggerGridUpdate && collectionItemHandles.length > 0) {
				var publishObj = {
					collectionItemHandle: collectionItemHandles,
					treeID : this._treeID
				};
				dojo.publish(EPMTreeEvents.ON_NODE_REFRESH, [publishObj]);
			}
        },
		
        /**
         *  _expandNodeUsingID : Expands node using the identifier
         *  Incoming :
         *		parent node identifier
         *  Outgoing : none
         *
         */
        _expandNodeUsingID: function(parentID){
            var parentTreeNode = this._itemNodeMap[parentID];
            if ((parentTreeNode) &&
            (parentTreeNode.isExpandable === true) &&
            (parentTreeNode.isExpanded === false)) {
                this._expandNode(parentTreeNode);
            }
        },
        
        /*
         * Displays the Message
         */
        _displayErrorMessage: function(errorMsg){
            //            var errorDialog = new com.ibm.epm.web.core.ui.internal.common.ErrorDialog();
            //            errorDialog.displayError(errorMsg);
            var labelList = [{
                label: errorMsg
            }];
            var buttonlist = [{
                label: WBS_Constants.OK_BUTTON_LABEL
            }];
            var confirmDialog = new EPMConfirmDialog({
                labelList: labelList,
                title: WBS_Constants.ERROR_TITLE,
                buttonlist: buttonlist
            });
            confirmDialog.show();
        },
        
        
        //Method to extract the resource type from the href
        _getResourceType: function(relativeHref){
        
            var index = relativeHref.lastIndexOf("/");
            
            if (index > 0) {
                relativeHref = relativeHref.substring(0, index);
                index = relativeHref.lastIndexOf("/");
                return relativeHref.substring(index + 1);
            }
        },
        
        //Gets the icon class for the resource
        _getIconClassForResourceType: function(resourcetype){
        	return resourcetype + "Icon";
            
        },
        
        //TODO : IMP - Have to remove this funtion to get data for the tree to separate class so that
        // different data fetching mechanisms can be implemented
        
        // Get all the Root level tree nodes for the Tree
        getRootNodes: function(){
        
            var self = this;
            
            var serviceObj = {
            
                success: function(data){
                    if ((!data)) {
                        console.log("No response");
                        return;
                    }
                    var item = null;
                    var isEmptyCollection = true;
                    for (var i in data) {
                        // add all the required information to the variable
                        
                        var currentElementLink = data[i];
                        item = self._getTreeNodeModel(currentElementLink, null);
                        if (self._treeStore && self.getItemNodeByIdentity(item.identifier)) {
                            self._updateNode(item);
                        }
                        else {
                            self._addNode(null, item, i);
                            self._addTempChildNode(item);
                        }
                        isEmptyCollection = false;
                        
                        if (self._openedItemIds) {
                            if (self._openedItemIds[item.identifier]) {
                                //Expand Node
                                var itemNode = self.getNodeByItemIdentity(item.identifier);
                                if (itemNode) {
                                    self._expandNode(itemNode);
                                }
                            }
                        }
                    }
					var publishObj = {
					 	treeID: self._treeID
					 };
                	dojo.publish(EPMTreeEvents.ROOT_NODES_LOADED, [publishObj]);
                    if (isEmptyCollection && self._containingWidget && self._containingWidget.handleEmptyCollection) {
                        self._containingWidget.handleEmptyCollection();
                    }
                },
                
                failure: function(errorObj){
                    //Show the error
                    var errMsg = errorObj.errorMessage || "Cannot Retrieve Information";
                    self._displayErrorMessage(errMsg);
                    if (self._containingWidget && self._containingWidget.handleRootElementFailure) {
                        self._containingWidget.handleRootElementFailure(errorObj);
                    }
                    
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            if (dojo.isArray(this._topLevelElementsURI)) {
                for (var k in this._topLevelElementsURI) {
                    EPMTreeClient.getAllResources(srh, this._topLevelElementsURI[k]);
                }
            }
            else {
                EPMTreeClient.getAllResources(srh, this._topLevelElementsURI);
            }
        },
        
		/**
		 * Method called to initialize the node item. <br>
		 * @param {Object} childLink: Link of the element to be added to the WBS Tree. <br>
		 * @param {Object} parentIdentifier: relative HREF of the parent node. <br>
		 */
        _getTreeNodeModel: function(childLink, parentIdentifier){
			
            var childURI = childLink.relativeHref;
 
			var resourcetype = this._getResourceType(childURI);
			//For types which are defined by Tara.
			var resourcetypeiconclass = resourcetype + "Icon";
			var iconUrl = null;
			var label = null;
			
			//For types which are defined by Tara.
			label = this.getIconText(resourcetype);
			
			if(resourcetype.indexOf(WBS_Constants.CHARGE_CODE_RESOURCE_TYPE) !== -1){
				resourcetype = "epm" + resourcetype;
				resourcetypeiconclass = IconClass.CHARGECODES_ICON_CLASS;
				label = CoreMessages.CHARGE_CODE_TREE_LABEL;
			}else if(resourcetype.indexOf(WBS_Constants.TIME_CODE_RESOURCE_TYPE) !== -1){
				resourcetype = "epm" + resourcetype;
				resourcetypeiconclass = IconClass.TIMECODES_ICON_CLASS;
				label = CoreMessages.TIME_CODE_TREE_LABEL;
			}//Once we get the project instead of projects from the URI, we'll remove this condition.
			else if(resourcetype.indexOf(WBS_Constants.EPM_PROJECT) !== -1){
				resourcetype = WBS_Constants.EPM_PROJECT;
				//Setting the icon for project if the query from process spec isn't yet executed.
				resourcetypeiconclass = IconClass.PROJECT_CLOSED_ICON_CLASS;
				label = CoreMessages.PROJECT;
			}else if(resourcetype.indexOf(WBS_Constants.WORKITEM) !== -1){
				resourcetype = WBS_Constants.WORKITEM;
			}

			//For items fetched from process spec, this map will have the iconUrl.
			/*if (this.resourceTypeIconMap) {
				iconUrl = (resourcetype == WBS_Constants.EPM_PROJECT)? this.resourceTypeIconMap[WBS_Constants.EPM_PROJECT]:this.resourceTypeIconMap[resourcetype] ;
			}*/
			
			
            var item = {
                "href": childURI,
                "name": childLink.title,
                "identifier": childURI,
                "parentIdentifier": parentIdentifier,
                "resourcetype": resourcetype,
                "detailLevel": false,
				"iconClass":resourcetypeiconclass,
				"iconUrl":iconUrl,
				"label" : label ,
				"childNodeAddedInStore":false,
				"rpcSequenceValue": null,
				menu: null
            };
			
			this._updateTitle(item, childLink);
			if (resourcetype.indexOf(WBS_Constants.EPM_PROJECT) !== -1) {
				//loading the wi config for project with URI  + item.identifier + with name  + item.name
				ControllersFactory.getWIConfigHandler().fetchConfig(item.identifier, item.name);
			}else if(resourcetype == WBS_Constants.WORKITEM && item.workItemType){
				
				var containingProjectID = this._getContainingProjectID(item);
				if(containingProjectID == null){
					return;
				}
				var wiConfig = ControllersFactory.getWIConfigHandler().getWIConfig(containingProjectID, item.workItemType);
				item.iconUrl = wiConfig.iconUrl;
				item.label = wiConfig.label;
			}
			
            return item;
        },
		
		_updateTitle: function(item, childLink){
			var title = childLink.title;
			item.name = title;
			if (this._treeID == WBS_Constants.WBS_TREE_ID) {
				var title = null;
				//There should be a better way to update the title, Please comment on this change.
				if(childLink.title){
					title = childLink.title;
				}else if(childLink.name){
					title = childLink.name;
				}else if(childLink.summary){
					title = childLink.summary;
				}
				var titleObj = dojo.fromJson(title);
				item.name = titleObj.summary || title;
				var wiType = titleObj.workItemType;
				if(wiType){
					item.workItemType = wiType;
					//Initialize iconUrl for work item here.
					if (this.resourceTypeIconMap && this.resourceTypeIconMap[wiType]) {
						item.iconUrl = this.resourceTypeIconMap[wiType];
					}
					//Initialize label for work item here.
					if (this.resourceTypeLabelMap && this.resourceTypeLabelMap[wiType]) {
						item.label = this.resourceTypeLabelMap[wiType];
					}
				}
				if (item.resourcetype == WBS_Constants.ITERATION_TYPE) {
					if (titleObj.summary) {
						item.currentIteration = titleObj.currentIteration;
					} else {
						item.currentIteration = childLink.currentIteration;
					}
					if (item.currentIteration == 1) {
						// Store the current iteration if there is no new value or if the parent of the item is the current iteration
						if ((!this.currentIterationHref) || (this.currentIterationHref == item.parentIdentifier)){
							this.currentIterationHref = item.identifier;
						}
					}
				}
				
				// Update the childrenCount for the item from the title json
				var childrenCount = titleObj.childrenCount;
				if (childrenCount) {
					item.childrenCount = childrenCount;
				}
				
				// Update the childrenCount for the item from the title json
				var rpcSequenceValue = titleObj.rpcSequenceValue;
				if (rpcSequenceValue) {
					item.rpcSequenceValue = rpcSequenceValue;
				}
			}
		},
        
        //Get all the children for the current Tree Node
        // Method to get the detailed information of the resource
        getChildNodes: function(uri){
            var self = this;
            var parentIdentifier = null;
            
			if ((typeof(uri) == "undefined")  || (uri == null)){
				return;
			}
			
            if (dojo.isArray(uri)) {
                parentIdentifier = uri[0];
            }
            else {
                parentIdentifier = uri;
            }
            
            var serviceObj = {
                success: function(data){
                    if (!data) {
                        console.log("Data is null");
                        return;
                    }
                    var node;
                    self._addChiildrenByLinks(data, parentIdentifier);
                },
                
                failure: function(errorObj){
                    //Show the error
                    var parentNode = self.getNodeByItemIdentity(parentIdentifier);
                    self._removeTempChild(parentNode.item);
                    //					var errMsg = errorObj.errorMessage || "Cannot Retrieve Information";
                    //                    self._displayErrorMessage(errMsg);
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            EPMTreeClient.getURI(srh, uri);
        },
        
        _addChiildrenByLinks: function(data, parentIdentifier){
        
            var parentNode = this.getItemNodeByIdentity(parentIdentifier);
            var links = this._linkTypeForChildren;
            if (this._linkTypeForChildren && !dojo.isArray(this._linkTypeForChildren)) {
                links = [this._linkTypeForChildren];
            }
            
 
            for (var i in links) {
                this._addChiildrenByLink(data, links[i], parentNode, parentIdentifier);
            }
			
            var publishObj = parentNode;
            if (publishObj) {
                publishObj.treeID = this._treeID;
                dojo.publish(EPMTreeEvents.ON_EXPAND, [publishObj]);
            }
			
			// Add a short delay to sync the load between the tree and grid
			// otherwise the tree starts rendering slightly earlier than the grid
			var delay = 0;
			var item = parentNode.item;
			if (this.isLargeContainer(item)) {
				delay = 10;
			}
			setTimeout(dojo.hitch(this, this._removeTempChild, parentNode), delay);
        },
        
        _addChiildrenByLink: function(data, link, parentNode, parentIdentifier){
            var childrenLinks = data[link];
            if (!childrenLinks) {
                return;
            }
            if (!dojo.isArray(childrenLinks)) {
                childrenLinks = [childrenLinks];
            }
            for (var i in childrenLinks) {
                var childLink = childrenLinks[i];
                 if (childLink != null) {
                    try {
                        var nodeModel = this._getTreeNodeModel(childLink, parentIdentifier);
                        
                        nodeModel.topLevel = true;
                        nodeModel.parentId = parentIdentifier;
                        
                        if (this._treeStore && this.getItemNodeByIdentity(nodeModel.identifier)) {
                            this._updateNode(nodeModel);
                        }
                        else {
                            this._addNode(parentNode, nodeModel, i);
                            this._addTempChildNode(nodeModel);
                        }
                        // For restoring the earlier state of the tree after refresh opertaion.
                        // chk whether the node was already opened if yes - call the expand funtion for that node
                        // reset the previous state of the tree														
                        
                        if (this._openedItemIds && this._openedItemIds[nodeModel.identifier]) {
                            var itemNode = this.getNodeByItemIdentity(nodeModel.identifier);
                            if (itemNode) { //Expand the node
                                this._expandNode(itemNode);
                            }
                        }
                    } 
                    catch (e) {
                        console.log(e);
                    }
                }
            }

            if (this._treeStore && this._treeStore.isItem(data)) {
                this._treeStore.setValue(data, "detailLevel", true);
            } else {
                data.detailLevel = true;
            }
                  
            if (parentNode) {
                // Icon needs to be changed to closed if parent doesnt have any child nodes other than the temp node created
                var nodeWidget = this._itemNodeMap[parentNode.identifier];
                var iconClass = this.getIconClass(nodeWidget.item, true);
                nodeWidget.iconNode.className = "dijitInline dijitTreeIcon " + iconClass;
				
				//As we have loaded the child nodes, we will set the childNodeAddedInStore flag to true.
				var itemInStore = this.getItemNodeByIdentity(parentNode.identifier);
	            if (this._treeStore && this._treeStore.isItem(itemInStore)) {
					this._treeStore.setValue(itemInStore, "childNodeAddedInStore", true);
	            }

            }
        },
        
        _removeTempChild: function(parentNode){
            var tempNode = this.getItemNodeByIdentity(parentNode.href + "_tempChild");
            if (tempNode) {
                this._deleteItemNode(tempNode);
            }
        },
        
        //Creates the menu for Tree Nodes
        createMenu: function(ctxtItems){
            this._ctxMenu = new EPMMenu({
                menuItems: ctxtItems
            });
        },
        
        /**
         * closeMenu: Method to close the menu.<br>
         * If there are multiple menus, as in case of WBS Tree, we will loop through the menu and will check, if there is any menu open.<br>
         */
        closeMenu: function(){
            if (this._ctxMenu && this._ctxMenu._isMenuOpen) {
                this._ctxMenu._closeMenu();
            }
        },
        
        /**
         *  getNodeContext : Function used to context node which is needed to popup the menu for the specific node
         *  Incoming : none
         *  Outgoing : context Node
         *
         */
        getNodeContext: function(){
            return this._ctxNode;
        },
        
		/**
         *  getNodeContext : Function used to hover node which is needed to popup the tooitip for the specific node
         *  Incoming : none
         *  Outgoing : hover Node
         *
         */
        getNodeHover: function(){
            return this._hoverNode;
        },
		
        // Get the item from the Node
        getItemNodeByIdentity: function(identifier){
            var node = this.getNodeByItemIdentity(identifier);
            
            if (node) {
                return node.item;
            }
            else {
                return null;
            }
        },
        /*
         * Get the tree node by its item's identity
         */
        getNodeByItemIdentity: function(identity){
            if (identity === null) {
                return null;
            }
            return this._itemNodeMap[identity];
        },
        
        /*
         * Overrides dijit tree function, used to define the iconclass to be used with the TreeNode
         */
        getIconClass: function(/* dojo.data.Item */item, /*Boolean*/ opened){
            var iconclass = "";
            var nodeSelected;
			
            if ((item) && (item.resourcetype)) {
                // FIXME: Check how this can be done in the derived class
                switch (item.resourcetype[0]) {
                    case WBS_Constants.EPM_PROJECT:
                        if (this.mayHaveChildren(item)) {
							nodeSelected = this._itemNodeMap[item.identifier[0]];
							if ((nodeSelected) && (nodeSelected.item)) {
								if (opened) {
									nodeSelected.iconNode.src = dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/projectopen_obj.gif");
									nodeSelected.item.iconClass = IconClass.PROJECT_OPENED_ICON_CLASS;
									return IconClass.PROJECT_OPENED_ICON_CLASS;
								}
								else {
									nodeSelected.item.iconClass = IconClass.PROJECT_CLOSED_ICON_CLASS;
									nodeSelected.iconNode.src = dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/projectclosed_obj.gif");
								}
							}                            
                        }
                        break;
                    
					 case WBS_Constants.ITERATION_TYPE:
					 	if (item.currentIteration) {
							if (item.currentIteration[0] == 1) {
								item.iconClass = IconClass.CURRENT_ITERATION_ICON_CLASS;
								return IconClass.CURRENT_ITERATION_ICON_CLASS;
							} else if ((item.currentIteration[0] == 0) || (item.currentIteration[0] == -1)) {
								item.iconClass = IconClass.ITERATION_ICON_CLASS;
								return IconClass.ITERATION_ICON_CLASS;
							}
						}	
					 	break;

                    default:
                        break;
                }
            }
            
            if (item) {
                iconclass = item.iconClass;
				if ((item.identifier) && (item.identifier[0])) {
					nodeSelected = this._itemNodeMap[item.identifier[0]];
					this._updateNodeForLargeContainer(nodeSelected);
				}
            }
            return iconclass;
        },
        
        /**
         *  _updateNodeForLargeContainer : Function used to flag elements with more than 100 children by...
         *      a) Bolding font of parent name in wbs (if it has more than 100 children).
         *      b) Providing tooltip with the actual number of children contained.
         */
		_updateNodeForLargeContainer: function(/*TreeNode*/ node) {
			if (node) {
				var item = node.item;
				if (this.isLargeContainer(item)) {
					dojo.addClass(node.contentNode, "dijitTreeContentBold");
					node.contentNode.title = CoreMessages.LOADING_LARGE_CONTAINER_TOOLTIP;
				}
			}
		},
		
        /**
         *  _highlightSpecifiedNode : Function used to set the highlight/undo on the node specified;
         *					Used for highlighting the node when grid mouse hover/out events take place
         *  Incoming :
         *  	nodeId - TreeNode Item Identifier
         *  	bSelected - Flag to indicate if its a mousehover event or mouseOut event
         *  Outgoing : none
         *
         */
        _highlightSpecifiedNode: function(nodeId, bSelected){
            var nodeSelected = this._itemNodeMap[nodeId];
            if (bSelected) {
                dojo.addClass(nodeSelected.rowNode, "dijitTreeContentFocused");
            }
            else {
                if (nodeSelected != this._selectedNode) {
                    dojo.removeClass(nodeSelected.rowNode, "dijitTreeContentFocused");
                }
            }
        },
        
        _selectNodeById: function(nodeId){
            var nodeSelected = this._itemNodeMap[nodeId];
            if (nodeSelected) {
                this._selectTreeNode(nodeSelected);
            }
        },
        
        /**
         *  _selectTreeNode : Function used to set focus on the currently selected node, internally stores this in this._selectedNode
         *  Incoming : selectNode
         *
         */
        _selectTreeNode: function(selectNode){
			var alreadySelected = false;
            if (selectNode) {
				alreadySelected = this._compareWithSelectedNode(selectNode);				
				if (!alreadySelected) {
					dojo.addClass(this.menuImage, "selectedMenuImage");
					////this.menuImage.src = "/jazz/web/com.ibm.epm.web.core/ui/internal/images/DropdownMenuSelected.gif";
					// Deselect the previously selected node.
					if ((this._selectedNode) && (this._selectedNode.rowNode)) {
						dojo.removeClass(this._selectedNode.rowNode, "dijitTreeContentFocused");
						if (dojo.hasClass(this._selectedNode.labelNode, "editableNode")) {
							dojo.removeClass(this._selectedNode.labelNode, "editableNode");
						}
					}
					if (selectNode.rowNode) {
						var resourceType = dojo.isArray(selectNode.item.resourcetype) ? selectNode.item.resourcetype[0] : selectNode.item.resourcetype;
						dojo.addClass(selectNode.rowNode, "dijitTreeContentFocused");
						
						if (resourceType !== WBS_Constants.EPM_PROJECT) {
							dojo.addClass(selectNode.labelNode, "editableNode");
						}
					}
					// Set the selected node
					this._selectedNode = selectNode;
					
					// set the context node 
					this._ctxNode = selectNode;
					
					// Publish the selected node
	        	    var publishObj = selectNode.item;
	            	if (publishObj) {
	                	publishObj.treeID = this._treeID;
		                dojo.publish(EPMTreeEvents.ON_SELECT, [publishObj]);
	    	        }
				}
            }
			
			return alreadySelected;
        },
        
        /*
         * _onClick: translates click events into commands for process. Handles onclick on
         * Tree node row. First click to select the node and second click for edit text focus
         */
        _onClick: function(/*Event*/e){
        
            //Call the super class funtion to handle the prcessing
            this.inherited(arguments);
            
            //Publish EPMTree specific event
            var element = e.target;
            var nodeWidget = dijit.getEnclosingWidget(element);
            
            // compare if this is a previously selected node, if yes
            // then single click enables the rename editor, else the node is selected
            if (this._selectTreeNode(nodeWidget)) {
				// Rename should be called only for the label node clicks
				if (dojo.hasClass(element, "dijitTreeLabel")) {
					//Rename not supported on popup Dialogs, which used to select codes for
					//association to project and assignment
					if((this._containingWidget) 
					&& (this._containingWidget.isRenameAllowed) 
					&& (!this._containingWidget.isRenameAllowed())){
						return;
					}
					this._renameNode(nodeWidget);
				}    			
    		}
        },

       /**
         *  _compareWithSelectedNode : Compares the specified node with the currently selected node
         *  Incoming : node - Treenode
         *  Outgoing : true, if the specified node is already selected, false otherwise
         */        
		_compareWithSelectedNode: function(/*TreeNode*/ node) {
			if ((this._selectedNode) && (node) && (node.item) && (this._selectedNode.item)) {
				if (this._selectedNode.item.identifier[0] == node.item.identifier[0]) {
					return true;
				}
			}
			return false;
		},
		
        /*
         * Used to handle any processing on mouse over event
         */
        _onMouseOver: function(e){
            var domElement = e.target;
            
            // find node
            var nodeWidget = dijit.getEnclosingWidget(domElement);
            if (!nodeWidget || !nodeWidget.isTreeNode) {
                return;
            }

            /*
             * Previously in this section, tooltips were enabled for the tree.
             * Per comment from UX team, it has been taken out. If in future
             * they need to be enabled again, use the following mechanisms:
             * Show:
             * dijit.showTooltip(nodeWidget.item.name, nodeWidget.contentNode, "after");
             * NOTE: Rememeber to set a timer to hide it after sometime like this:
             * setTimeout(dojo.hitch(this, function(){
             *      dijit.hideTooltip(nodeWidget.contentNode);
             * }), 1000);
             * 
             * Hide:
             * dijit.hideTooltip(nodeWidget.contentNode);
             * 
             * IMPORTANT: Tooltips on the tree node interfere during drag-n-drop. You've
             * been warned !!! If you ever enable them again, take caution to disable
             * them during drag-n-drop using the following code block to check if the
             * node is being dragged: 
             * 
             * var m = dojo.dnd.manager();
             * var dragging = false;
             * if (m) {
             *   if (m.source && m.source.isDragging) {
             *       dragging = true;
             *   }
             * }
             * if (dragging) {
             *      // Hide tooltip
             * } else {
             *      // Show tooltip
             * }     
             */            
			

			//Unbind the previous bindings
			if(this._hoverNode){
				var menu = this._hoverNode.item.menu;
				var menuValue = dojo.isArray(menu)?menu[0]:menu;
				if (menuValue) {
					menuValue.unBindDomNode(this.menuImage);
				}	
				dojo.disconnect(this.menuImageClickHandler);
			}
			
			//set the hoverNode
			this._hoverNode = nodeWidget;

			//Mark the menu visible and change background colour for the selected node.
			this.menuImage.style.visibility = "";
			isHoverNodeSelected = this._compareWithSelectedNode(nodeWidget);
			if(isHoverNodeSelected){
				dojo.addClass(this.menuImage, "selectedMenuImage");
			}else{
				if(dojo.hasClass(this.menuImage, "selectedMenuImage")){
					dojo.removeClass(this.menuImage, "selectedMenuImage");
				}
			}

			//Re-position the menu image. 			
			var baseOffset = nodeWidget.domNode.offsetTop;
			
			//In IE offsetTop is calculated with respect to immediate parent.
			if (dojo.isIE) {
				var parentID = nodeWidget.item.parentIdentifier;
				var parentIDValue = dojo.isArray(parentID) ? parentID[0] : parentID;
				//Recursively add the parent offset value.
				while (parentIDValue) {
					var parentNode = this.getNodeByItemIdentity(parentIDValue);
					baseOffset = parentNode.domNode.offsetTop + baseOffset;
					parentID = parentNode.item.parentIdentifier;
					var parentIDValue = dojo.isArray(parentID) ? parentID[0] : parentID;
				}
				dojo.style(this.menuImage, "top", (baseOffset + INITIAL_OFFSET) + "px");
			}else{
				dojo.style(this.menuImage, "top", (baseOffset - this.firstRowOffsetHeight + INITIAL_OFFSET) + "px");	
			}

			//Bind the menu image to the node.
			var menu = nodeWidget.item.menu;
			var menuValue = dojo.isArray(menu)?menu[0]:menu;
			if (menuValue) {
				menuValue.bindDomNode(this.menuImage);
			}	
			
			//Select the node when the menu Image is clicked.
			this.menuImageClickHandler = dojo.connect(this.menuImage, "onclick", this, dojo.hitch(this,  this._selectTreeNode, nodeWidget));
            var publishObj = nodeWidget.item;
            if (publishObj) {
                publishObj.treeID = this._treeID;			
                
                // Modifies the item node in the tree store, if this is needed for the subscriber then the published object
                // needs to be copied and passed without modifying the store 
                //publishObj.pageX = e.pageX;
                //publishObj.pageY = e.pageY;
                //publishObj.domNode = nodeWidget.domNode;
                
                // Publish mouseover event
                dojo.publish(EPMTreeEvents.ON_MOUSE_OVER, [publishObj]);
            }          
            
        },
        
        /*
         * Used to handle any processing on mouse out event
         */
        _onMouseOut: function(e){
            var domElement = e.target;
            
            // find node
            var nodeWidget = dijit.getEnclosingWidget(domElement);
            if (!nodeWidget || !nodeWidget.isTreeNode) {
                return;
            }
            
            var publishObj = nodeWidget.item;
            if (publishObj) {
                publishObj.treeID = this._treeID;
                
                // Modifies the item node in the tree store, if this is needed for the subscriber then the published object
                // needs to be copied and passed without modifying the store 
                //publishObj.domNode = nodeWidget.domNode;
                
                // Publish mouseout event
                dojo.publish(EPMTreeEvents.ON_MOUSE_OUT, [publishObj]);
            }
			this.menuImage.style.visibility = "hidden";
        },
        
		/**
		 * connect to menu image events.
		 */
		connectToMenuImageEvents: function(){
			this.menuImageMouseOver = dojo.connect(this.menuImage, "onmouseover", this, "_onMenuImageMouseOver");
			this.menuImageMouseOut = dojo.connect(this.menuImage, "onmouseout", this, "_onMenuImageMouseOut");
		},

		/**
		 * Hide the menu image on mouse out
		 * @param {Object} e
		 */
		_onMenuImageMouseOut: function(e){
			dojo.style(this.menuImage, "visibility", "hidden"); 
		},
		
		/**
		 * Hide the menu image on mouse out
		 * @param {Object} e
		 */
		_onMenuImageMouseOver: function(e){
			dojo.style(this.menuImage, "visibility", ""); 
		},

        /**
         * _isNodeNew: returns true if node is unsaved. <br>
         * node: Node to be checked. <br>
         */
        _isNodeNew: function(node){
            var nodeIdentifier = dojo.isArray(node.identifier) ? node.identifier[0] : node.identifier;
            if (nodeIdentifier && (nodeIdentifier.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1)) {
                return true;
            }
            else {
                return false;
            }
        },
        
        /**
         *  _addDropDownIcon : Function used to add the drop down icon to the tree node and attach mouse click event
         *  Conditions: Don't add the icon for the new node (except for WBSTree) &/ the _nodesWithoutMenu,
         *  or if _nodesWithoutMenu was set to this.READONLY.
         *  Incoming : Label Node
         *  Outgoing : none
         *
         */
        _addDropDownIcon: function(node){
            // Don't add icon if no nodes have any menus
        	if (this._nodesWithoutMenu == this.READONLY)
        	    return;
        	
        	// Check for individual nodes that don't have a menu
            for (var i = 0; i < this._nodesWithoutMenu.length; i++) {
                if (node.resourcetype == (this._nodesWithoutMenu[i]) ||
                node.resourcetype == this._nodesWithoutMenu[i][0]) {
                    return;
                }
            }

            var treeNode = this.getNodeByItemIdentity(node.identifier);
            if (treeNode) {
				var menu = new dijit.Menu({leftClickToOpen:true});
				if (menu) {
					var itemInStore = this.getItemNodeByIdentity(node.identifier);
                    if (this._treeStore && itemInStore) {
						//In case of type change from details view, the menu needs to be updated.
						var prevMenu = this._treeStore.getValue(itemInStore, "menu");
						var menuValue = dojo.isArray(prevMenu)?prevMenu[0]:prevMenu;
						if (menuValue && menuValue._bindings) {
							menuValue.unBindDomNode(this.menuImage);
						}	
                        this._treeStore.setValue(itemInStore, "menu", menu);
                    }
					if (this._treeID == WBS_Constants.WBS_TREE_ID) {
						this._addWBSMenuItems(node, menu);
                	}
					if (this._ctxMenu) {
						//Code added to set context menu for the timecode and chargecode.
						if((this._containingWidget) 
						&& (this._containingWidget.setctxMenu)){
							this._containingWidget.setctxMenu(node);
						}
                    	dojo.connect(menu, "_openMyself", this._ctxMenu, this._ctxMenu.handleMenuPopUp);
                    }
				}
                //Rename not supported on popup Dialogs, which used to select codes for
				//association to project and assignment
				if((this._containingWidget) 
				&& (this._containingWidget.isRenameAllowed) 
				&& (!this._containingWidget.isRenameAllowed())){
					return;
				}
				else{
					dojo.connect(treeNode.labelNode, "onclick", this, this._onClick);
				}
            }
        },
		
		_addWBSMenuItems: function(node, menu){
			var resourceType = this._getResourceTypeForMenu(node);
            if (resourceType == null) {
				return;
			}
			if (resourceType.indexOf(WBS_Constants.EPM_PROGRAM) !== -1) {
				//loading the wi config for project with URI  + item.identifier + with name  + item.name
				this.updateCreateMenu(resourceType, menu, null, node);
			}
			var containingProjectID = this._getContainingProjectID(node);
			if(containingProjectID == null){
				if (resourceType.indexOf(EPMConstants.NEWElEMENTSUFFIX) !== -1) {
					// check if its an unsaved program
					var resourcetype = dojo.isArray(node.resourcetype) ? node.resourcetype[0] : node.resourcetype;
					if (resourcetype.indexOf(WBS_Constants.EPM_PROGRAM) !== -1) {
						this.updateCreateMenu(resourceType, menu, null, node);
					}
				}
				return;
			}
			
			var allWIConfig = ControllersFactory.getWIConfigHandler().getAllWIConfig(containingProjectID);
			if (allWIConfig == null && resourceType.indexOf(WBS_Constants.EPM_PROJECT) !== -1) {
				//loading the wi config for project with URI  + item.identifier + with name  + item.name
				ControllersFactory.getWIConfigHandler().fetchConfig(node.identifier, node.name);
			}
			var tempThis = this;
			var loadConfigInterval = setInterval(function(){
				if(allWIConfig == EPMConstants.LOADING_CONFIG){
					allWIConfig = ControllersFactory.getWIConfigHandler().getAllWIConfig(containingProjectID);
				}else{
					clearInterval(loadConfigInterval);
					tempThis.updateCreateMenu(resourceType, menu, allWIConfig, node);
				}
			},100);
		},
		
		updateCreateMenu: function(resourceType, menu, allWIConfig, node){
			if(resourceType.indexOf(WBS_Constants.EPM_PROJECT) !== -1 && !node.childrenCount){
				childrenCnt = ControllersFactory.getWIConfigHandler().getChildCount(node.identifier);
				var treeNode = this.getNodeByItemIdentity(node.identifier);
				if(childrenCnt > 0){
					this._treeStore.setValue(treeNode.item, "childrenCount", childrenCnt);
					this._updateNodeForLargeContainer(treeNode);
				}
			}
			if((this._containingWidget) 
				&& (this._containingWidget.getContextMenuItems)){
				var contextMenuItems = 	this._containingWidget.getContextMenuItems(resourceType
							, allWIConfig, node);
				var wbsMenu = new EPMMenu({
				                    menuItems:contextMenuItems
				                });
				dojo.connect(menu, "_openMyself", wbsMenu, wbsMenu.handleMenuPopUp);
			}
		},
        
        /**
         * _getResourceTypeForMenu: Method to return the resource type of the selected WBS node.
         * @param {Object} nodeItem: node item.
         */
        _getResourceTypeForMenu: function(nodeItem){
            var resourceType = null;
			if (this._isNodeNew(nodeItem)) {
				return EPMConstants.NEWElEMENTSUFFIX;
			}
            resourceType = dojo.isArray(nodeItem.resourcetype) ? nodeItem.resourcetype[0] : nodeItem.resourcetype;
            return resourceType;
        },
		
		_getContainingProjectID: function(node){
			if(node == null){
				return null;
			}
			var resourcetype = dojo.isArray(node.resourcetype) ? node.resourcetype[0] : node.resourcetype;
			var id = dojo.isArray(node.identifier) ? node.identifier[0] : node.identifier;
			if (resourcetype.indexOf(WBS_Constants.EPM_PROJECT) !== -1) {
				return id;
			}else{
				var parentID = node.parentIdentifier;
				if(parentID == null){
					return;
				}
				var parentNode = this.getItemNodeByIdentity(node.parentIdentifier);
				if(parentNode){
					return this._getContainingProjectID(parentNode);
				}else{
					return null;
				}
			}
			
		},
        
	   _onItemChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList){
			//summary: processes notification of a change to an item's children
			var model = this.model,
				identity = model.getIdentity(parent),
				parentNode = this._itemNodeMap[identity];
	
			if(parentNode){
				this.setChildItems(parentNode, newChildrenList);
			}
		},
		setChildItems: function(parentNode, /* Object[] */ items){
				// summary:
				//		Sets the child items of this node, removing/adding nodes
				//		from current children to match specified items[] array.
			
				var tree = parentNode.tree,
					model = tree.model;
		
				// Orphan all my existing children.
				// If items contains some of the same items as before then we will reattach them.
				// Don't call this.removeChild() because that will collapse the tree etc.
				//this.getChildren().forEach(function(child){
				//	dijit._Container.prototype.removeChild.call(this, child);
				//}, this);
		
				
				if(parentNode.getChildren().length + 1 != items.length) {					
					parentNode.setChildItems(items);
					return;
				}	
					
				//Check if child needs to be appended
				var childNodes = parentNode.getChildren();
				for(var i = 0; i< childNodes.length; i++) {
					var childNode = childNodes[i].label;
					var item = items[i].name[0];
					if(item != childNode){					
						parentNode.setChildItems(items);
						return;
					}	
				}
				
				parentNode.state = "LOADED";							
		
				if(items && items.length > 0) {
					if (parentNode.getChildren().length + 1 == items.length) {
						parentNode.isExpandable = true;
			
						// Create _TreeNode widget for each specified tree node, unless one already
						// exists and isn't being used (presumably it's from a DnD move and was recently
						// released
						var item = items[items.length-1];
						var id = model.getIdentity(item),
							existingNode = tree._itemNodeMap[id],
							node = 
								( existingNode && !existingNode.getParent() ) ?
								existingNode :
								parentNode.tree._createTreeNode({
									item: item,
									tree: tree,
									isExpandable: model.mayHaveChildren(item),
									label: tree.getLabel(item)
								});
						parentNode.addChild(node);
						// note: this won't work if there are two nodes for one item (multi-parented items); will be fixed later
						tree._itemNodeMap[id] = node;
						if(parentNode.tree.persist){
							if(tree._openedItemIds[id]){
								tree._expandNode(node);
							}
						}
					
			
						// note that updateLayout() needs to be called on each child after
						// _all_ the children exist
						dojo.forEach(parentNode.getChildren(), function(child, idx){
							child._updateLayout();
						});
					}
				}else{
					parentNode.isExpandable=false;
				}
		
				if(parentNode._setExpando){
					// change expando to/from dot or + icon, as appropriate
					parentNode._setExpando(false);
				}
		
				// On initial tree show, put focus on either the root node of the tree,
				// or the first child, if the root node is hidden
				if(parentNode == tree.rootNode){
					var fc = parentNode.tree.showRoot ? parentNode : parentNode.getChildren()[0],
						tabnode = fc ? fc.labelNode : parentNode.domNode;
					tabnode.setAttribute("tabIndex", "0");
					tree.lastFocused = fc;
				}
			},
		
        /*
         * override dijit.Tree implementation to change node style after expand
         */
        _expandNode: function(/*_TreeNode*/node){
			
            // summary: called when the user has requested to expand the node
            //set the icon class else it will be replaced by folder icon
            if (node.item) {
                node.item._openedItemIds = this._openedItemIds;
                var iconClass = this.getIconClass(node.item, true);
                node.iconNode.className = "dijitInline dijitTreeIcon " + iconClass;
                var eventPublished = false;
                var isExpanded = null;
                var data = node.item;
                
                if (dojo.isArray(data.detailLevel)) {
                    isExpanded = data.detailLevel[0];
                }
                else {
                    isExpanded = data.detailLevel;
                }
                if (!isExpanded) {
                    // If we havent got the data from the server we make a uri call, this happens the first time 
                    // we expand the node
                    this.getChildNodes(data.href);
                    // once expanded set the flag to true
                    if (this._treeStore && this._treeStore.isItem(data)) {
                        this._treeStore.setValue(data, "detailLevel", true);
                    } else {
                        data.detailLevel = true;
                    }
                }
                this.startup();
            }
            
            this.inherited(arguments);
 
        },
        
        /**
         * expandSpecifiedNode: recursive function called for node to be expanded.<br>
         * Incoming: node to be expanded. <br>
         * Outgoing: none.
         */
        expandSpecifiedNode: function(node){
            var self = this;
            var data = node.item;
            
            var counter = 0;
            var children = null;
            
            var expandSpecifiedIntervalID = setInterval(function(){
                counter++;
                if (data == null || data.children == null) {
                    clearInterval(expandSpecifiedIntervalID);
                    return;
                }
                self._expandNode(node);
                
                //childNodeAddedInStore flag is associated as an attribute to all the tree nodes.
				//It is set to true whenever the node is expanded even once.
				var itemInStore = self.getItemNodeByIdentity(node.item.identifier);
				var bChildAddedToStore = self._treeStore.getValue(itemInStore, 'childNodeAddedInStore');
                if (bChildAddedToStore === true) {
                    clearInterval(expandSpecifiedIntervalID);
                    children = node.getChildren();
                    //recursively expand the children as well.
                    dojo.forEach(children, function(child){
                        if (child.hasChildren()) {
                            self.expandSpecifiedNode(child);
                        }
                    });
                }
                
                //Exit from the setInterval to avoid infinite looping.
                if (counter > 5) {
                    clearInterval(expandSpecifiedIntervalID);
                }
            }, 150);
        },
        
        /*
         * override dijit.Tree implementation to change node style after collapse
         */
        _collapseNode: function(/*_TreeNode*/node){
            // summary: called when the user has requested to collapse the node
            
            this.inherited(arguments);
            var iconClass = this.getIconClass(node.item, false);
            node.iconNode.className = "dijitInline dijitTreeIcon " + iconClass;
            
            var publishObj = node.item;
            if (publishObj) {
                publishObj.treeID = this._treeID;
                var cnt = 0;
                var collapseIntervalID = setInterval(function(){
                    if (cnt > 1) {
                        clearInterval(collapseIntervalID);
                        dojo.publish(EPMTreeEvents.ON_COLLAPSE, [publishObj]);
                    }
                    cnt = cnt + 1;
                }, 100);
            }
            if (node.containerNode) {
                node.containerNode.style.overflow = "";
            }
            
            if (this._treeStore && this._treeStore.isItem(node.item)) {
                this._treeStore.setValue(node.item, "detailLevel", false);
            } else {
                node.item.detailLevel = false;
            }   
        },
        
        /**
         * _collapseSpecifiedNode: called when the user clicks on the 'collapseAll'.<br>
         * It collpases all the child nodes and later on the parentNode recursvely. <br>
         * @param {Object} node: node to be collapsed. <br>
         */
        _collapseSpecifiedNode: function(node){
            var children = node.getChildren();
            var self = this;
            
            //collapse children first.
            dojo.forEach(children, function(child){
                if (self.mayHaveChildren(child.item)) {
                    self._collapseSpecifiedNode(child);
                }
                else {
                    self._collapseNode(child);
					self._publishCollapseEvent(child, true);
				}
			});
            //collapse the parentNode now.
            this._collapseNode(node);
			this._publishCollapseEvent(node, true);
			
        },

        /**
         * _publishCollapseEvent: publish the collapse event.
         * Incoming: Collapsing Tree node.
         * Outgoing: none.
         *
         */
		_publishCollapseEvent: function(node, collapseAll) {
			var publishObj = node.item;
            if (publishObj) {
                publishObj.treeID = this._treeID;
				publishObj.collapseAll = collapseAll;
                dojo.publish(EPMTreeEvents.ON_COLLAPSE, [publishObj]);
            }
		},
		        
        /**
         * getRootLevelNames: Returns the root level node names.
         * Incoming: none.
         * Outgoing: none.
         *
         */
		getRootLevelNames: function()
		{
            var rootLevelNames = new Array();
            var topLevelItems = this._treeStore._getItemsArray();
            
            for (var item in topLevelItems) {
                var topLevelNode = this._itemNodeMap[topLevelItems[item].identifier[0]];
                rootLevelNames.push(topLevelNode.label);
            }
            return rootLevelNames;
        },
        
        getAllVisibleItems: function(){
            var allVisibleItems = new Array(1);
            var topLevelItems = this._treeStore._getItemsArray();
            
            for (var item in topLevelItems) {
                var topLevelNode = this._itemNodeMap[topLevelItems[item].identifier[0]];
                var treeItemHandle = this.convertToItemHandle(topLevelNode.item);
                if (treeItemHandle) {
                    allVisibleItems.push(treeItemHandle);
                    this._addVisibleChildren(allVisibleItems, topLevelNode);
                }
            }
            return allVisibleItems;
        },
        
        convertToItemHandle: function(topNodeItem){
            //If the topNode is root then return null
            if (!topNodeItem.href) {
                return null;
            }
            var itemHandle = {
                title: dojo.isArray(topNodeItem.name)?topNodeItem.name[0]: topNodeItem.name,
                relativeHref: dojo.isArray(topNodeItem.href)?topNodeItem.href[0]:topNodeItem.href,
                resourcetype: topNodeItem.resourcetype
            };
            if (topNodeItem.parentIdentifier) {
                itemHandle.parentIdentifier = topNodeItem.parentIdentifier[0];
            }
            if (topNodeItem.workItemType) {
                itemHandle.workItemType = topNodeItem.workItemType;
            }
			if (topNodeItem.rpcSequenceValue) {
                itemHandle.rpcSequenceValue = topNodeItem.rpcSequenceValue;
            }
            return itemHandle;
        },
        
        
        _addVisibleChildren: function(/*Array */allVisibleItems, /*TreeNode */ currentNode){
            var self = this;
            if (currentNode.hasChildren()) {
                var children = currentNode.getChildren();
                dojo.forEach(children, function(child){
                    var childVisible = self._checkVisibility(child);
                    if (childVisible == true) {
                        var childItemHandle = self.convertToItemHandle(child.item);
                        allVisibleItems.push(childItemHandle);
                        self._addVisibleChildren(allVisibleItems, child);
                    }
                });
            }
            else {
                return;
            }
        },
        
        _checkVisibility: function(currentNode){
            var nodeVisible = false;
            var nodeID = currentNode.item.identifier[0];
            
            if (nodeID.lastIndexOf("tempChild") == -1) {
                var currentItemNode = this._itemNodeMap[nodeID];
                if (currentItemNode != null) {
                    var nodeVisiblityStyle = dojo.style(currentItemNode.domNode.parentNode, "display");
                    if (nodeVisiblityStyle != "none") {
                        nodeVisible = true;
                    }
                }
            }
            return nodeVisible;
        },
		
		/**
		 * removeUnsavedTreeNodes: Method to remove the unsaved data from the tree. <br>
		 * Fetch All the nodes, Compare id with the unsavedID and if found, delete the node. <br>
		 * Maintain an array of the deleted elements, publish an event to remove the rows from the grid. <br> 
		 */
 		removeUnsavedTreeNodes: function(){
	        var _treeNodeStore = this._treeStore;
			this.hasNewNode = false;
			var self = this;

	        if (_treeNodeStore) {

				function completed(nodes, findResult){
				  	var nodesToBeDeleted = new Array();
					var collectionItemHandles = new Array();
				  for(var i = 0; i < nodes.length; i++){
				  	if (self._isNodeNew(nodes[i])) {
						nodesToBeDeleted.push(nodes[i]);						
						self._deselectTreeNode(nodes[i], true);							
						_treeNodeStore.deleteItem(nodes[i]);					
					}
				  }
				 if (nodesToBeDeleted.length > 0) {
				 	for (var i in nodesToBeDeleted) {
				 		var handle = self.convertToItemHandle(nodesToBeDeleted[i]);
				 		collectionItemHandles.push(handle);
				 	}
				 	var publishedObject = {
				 		collectionItemHandle: collectionItemHandles,
				 		treeID: self._treeID
				 	};
				 	dojo.publish(EPMTreeEvents.ON_NODE_DELETE, [publishedObject]);
				 }	
				_treeNodeStore.save();
				}
				function error(errData, request){
				  console.log("Failed in fetching the data.");
				}
				//Invoke the fetch.
				_treeNodeStore.fetch({onComplete: completed, onError: error, queryOptions: {deep:true}});
	        }
	    },

		/**
		 * _deselectTreeNode: Method to deselect the specified node if selected, if selectParent is true then select the parent node. 
		 */
		_deselectTreeNode: function(deselectNode, selectParent) {
			 var nodeId = dojo.isArray(deselectNode.identifier) ? deselectNode.identifier[0] : deselectNode.identifier;
			 var nodeToBeDeSelected = this._itemNodeMap[nodeId];
			 
			 // the node is currently selected, deselect it
			 if (nodeToBeDeSelected == this._selectedNode) {
			 	dojo.removeClass(nodeToBeDeSelected.rowNode, "dijitTreeContentFocused");
				
				var parentId = dojo.isArray(deselectNode.parentIdentifier) ? deselectNode.parentIdentifier[0] : deselectNode.parentIdentifier;
				if ((selectParent) && (parentId)) {
					// select the parent
					this._selectNodeById(parentId);
				} else {
					this._selectedNode = null;
					
					// Publish the deselection event
	        	    var publishObj = nodeToBeDeSelected.item;
	            	if (publishObj) {
	                	publishObj.treeID = this._treeID;
		                dojo.publish(EPMTreeEvents.ON_DESELECT, [publishObj]);
	    	        }
				}
			 }
		},
		
        _expandAllTopLevelNodes: function(){
            // Ensure we have a root node
            if (!this.rootNode) {
                return;
            }
            // Get all of the top level nodes
            var treeNodeList = this.rootNode.getChildren();
            // Ensure we have an array
            if ((treeNodeList instanceof Array) === false) {
                return;
            }
            // Expand each top level node
            for (var index = 0; index < treeNodeList.length; index++) {
                var treeNode = treeNodeList[index];
                // Ensure the node is a folder
                if (!treeNode) {
                    continue;
                }
                else 
                    if (!treeNode.isExpandable) {
                        continue;
                    }
                // Track which nodes have been expanded already
				if (this._folderFirstExpandedMap){
					this._folderFirstExpandedMap[treeNode.label] = true;
				}
                // Expand the node
                this._expandNode(treeNode);
            }
        },
		
        _displayError: function(errorMessage){
            var self = this;
            EPMErrorHandler.setError(errorMessage);
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);
        },

        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
        
        /**
         * Handler for updating the parent-child links changed due to drag-n-drop in the tree
         * 
         * treeID: ID of the tree initiating the event
         * sourceItem: Item reference of the tree node moved
         * targetItem: Item reference of the new parent tree node
         * controller: Controller reference from the tree
         */
        updateParentChildLinks: function(treeID, sourceItem, targetItem, controller) {
            // Get an access to the old parent item first
            var oldParentItem = this.getItemNodeByIdentity(sourceItem.parentIdentifier[0]);
            
            // Get an access to the new parent item first in case target node is leaf node
            var newParentItem = this.getItemNodeByIdentity(targetItem.parentIdentifier[0]);
			
            var self = this;
			
			//The source item is moved to a new container
			if(!this.tree.dragNdropAsPeer){ 
			
				var parentItem = (this.tree._containingWidget._isALeafType(targetItem))? newParentItem : targetItem;
	            // First check to see if visually the sourceItem is the child of the targetItem
	            this.model.getChildren(parentItem, function (children) {
	                var found = false;
	                if (children) {
	                    for (var i in children) {
	                        var child = children[i];
	                        if (child && sourceItem && (child.identifier[0] == sourceItem.identifier[0])) {
	                            found = true;
	                            break;
	                        }
	                    }
	                }
	                
	                // If we did not find the source item as a child of the target item
	                // then move it under the target item
	                if (!found) {
	                    self.model.pasteItem(sourceItem, oldParentItem, parentItem, false);
	                }
					
					//Move the newly copied item as first Node
					if(!(self.tree._containingWidget._isALeafType(targetItem))){
						var values = self.model.store.getValues(parentItem, "children");
						if(values && values.length > 1){
							var oldValues = values.slice(0, values.length);
							var itemMoved = false;
							var tempValues = new Array();
							//The tree is in Expanded state so the newly moved item will be at the 
							//end of the parent node. So move the item above to all the workitems 
							if(sourceItem.identifier[0] != oldValues[0].identifier[0]){
								for(var cnt in oldValues){
									if((values.length - 1) > cnt){
										if((!itemMoved) && (oldValues[cnt].resourcetype[0] != "iteration")){
											tempValues.push(oldValues[values.length - 1]);
											itemMoved = true;
										}
										tempValues.push(oldValues[cnt]);
									}
								}
								if(!itemMoved){
									tempValues.push(oldValues[values.length - 1]);
								}
							}
							else if(sourceItem.identifier[0] == oldValues[0].identifier[0]){
								//Newly copied item is the first element in the container
								tempValues = null;
							}
							//The tree is in collapsed state so the newly moved item will be at the 
							//begining of the parent node. So move the item below to all the iterations 
							else if(oldValues[1].resourcetype[0] == "iteration"){
								for(var cnt in oldValues){
									if(cnt == 0){
										continue;
									}
									if((values.length) > cnt){
										if((!itemMoved) && (oldValues[cnt].resourcetype[0] != "iteration")){
											tempValues.push(oldValues[0]);
											itemMoved = true;
										}
										tempValues.push(oldValues[cnt]);
									}
								}
								if(!itemMoved){
									tempValues.push(oldValues[0]);
								}
							}	
							if(tempValues != null){						
								self.model.store._setValueOrValues(parentItem, "children", tempValues, false);
								var parentNode = self._itemNodeMap[parentItem.identifier[0]];
								self.setChildItems(parentNode, tempValues);
							}
						}
					}
	            }); 
	        }
			
			//The souce item is moved as a peer to the target item
			//Set the postion of the item in the tree nodelist
			if(this.tree.dragNdropAsPeer)
			{
				 var values = self.model.store.getValues(newParentItem, "children");
				 if(values && values.length > 0){
					 var oldValues = values.slice(0, values.length);
					 var sourceItemValue = null; 
					 var tempValues = new Array();
					 //Remove the source node
					 for(var cnt in oldValues){
					 	if(sourceItem.identifier[0] == oldValues[cnt].identifier[0]){
							sourceItemValue = oldValues[cnt];
							continue;
						}
					 	tempValues.push(oldValues[cnt]);
					 }
					 //Add the source node
					 var newValues = new Array();
					 for(var cnt1 in tempValues){					 	
					 	newValues.push(tempValues[cnt1]);
						if(targetItem.identifier[0] == tempValues[cnt1].identifier[0]){
							newValues.push(sourceItemValue);							
						}
					 }					 
				 	 self.model.store._setValueOrValues(newParentItem, "children", newValues, false);
					 var parentNode = this._itemNodeMap[targetItem.parentIdentifier[0]];
					 this.setChildItems(parentNode, newValues);
				 }
			} 
            // Second, request the server to save the parent-child change
            this._saveOnParentChildChange(treeID, sourceItem, targetItem, oldParentItem, 
                controller, true /* Do a POST to server */); 
        },
        
        /**
         * Handler for updating the parent-child links changed due to drag-n-drop in the tree
         * 
         * treeID: ID of the tree initiating the event
         * sourceItem: Item reference of the tree node moved
         * targetItem: Item reference of the new parent tree node
         * oldParentItem: Item reference to the old parent tree node
         * controller: Controller reference from the tree
         * postToServer: Boolean to indicate if a POST to the server is required
         *               true: POST to the server
         *               false: Do *not* POST to the server
         */
        _saveOnParentChildChange: function(treeID, sourceItem, targetItem, oldParentItem, controller, postToServer) {
           
            // Construct an object to the published
            var publishObj = {
                sourceItem: sourceItem,
                targetItem: targetItem,
                oldParentItem: oldParentItem,
				parentIdentifierID: (this.tree.dragNdropAsPeer)? targetItem.parentIdentifier[0] : targetItem.identifier[0],
				rpcSequenceValue: this._getSequencevalueForSourceItem(sourceItem, targetItem),
                treeID: treeID,
                postToServer: postToServer
            };
                        
            // Save the structure change (and any additional change made 
            // to the tree node, such as a name change prior to it being moved)
            if (controller && controller._onParentChildChange) {
                controller._onParentChildChange(publishObj);
            }    
        },
		
		/**
		* The method will generate a sequence value for the new element
		* created under the parent Node. The new value will be generated 
		* with respect to the last element in the container
		*/
		_getSequencevalueForNewNode: function(parentNode){
			var predecessorSeqRawValue = null;
			if(parentNode != null){
				var values = this.model.store.getValues(parentNode.item, "children");
				if(values && values.length > 0){
					var children = values.slice(0, values.length);
					for(var cnt = 0; cnt < values.length; cnt++){
						if((children[cnt].resourcetype[0] == "iteration") || (children[cnt].resourcetype[0] == "loading")) {
							continue;
						}
						predecessorSeqRawValue = children[cnt].rpcSequenceValue[0];
					}
				}
			}
			console.log("predecessorSeqRawValue and successorSeqRawValue");
			console.log(predecessorSeqRawValue);
			console.log(null);
			var predecessorSeq = (predecessorSeqRawValue != null)? new SequenceValue(predecessorSeqRawValue) : null;
			return (this.calculateSeq(predecessorSeq, null)).getRawValue();		
		},
        
        /**
		* The method will generate a sequence value for the sourceItem
		* which is drap and drop on the targetItem
		*/
		_getSequencevalueForSourceItem: function(sourceItem, targetItem){
			var predecessorSeqRawValue = null;
			var successorSeqRawValue = null;
			
            // Get an access to the new parent item first in case target node is leaf node
            var targetParentItem = this.getItemNodeByIdentity(targetItem.parentIdentifier[0]);
			var parentItem = (this.tree._containingWidget._isALeafType(targetItem) || (this.tree.dragNdropAsPeer))? targetParentItem : targetItem;			
			var values = this.model.store.getValues(parentItem, "children");
			if(values && values.length > 0){
				var children = values.slice(0, values.length);
				for(var cnt = 0; cnt < values.length; cnt++){
					if(children[cnt].resourcetype[0] == "loading") {
						continue;
					}
					if(sourceItem.identifier[0] == children[cnt].identifier[0]){
						for(var successorCnt = cnt + 1; successorCnt < values.length; successorCnt++){
							if((children[successorCnt].resourcetype[0] == "iteration") || (children[successorCnt].resourcetype[0] == "loading")) {
								continue;
							}				
							successorSeqRawValue = (successorCnt < values.length)? children[successorCnt].rpcSequenceValue[0] : null;
							break;
						}
						break;
					}
					if(children[cnt].resourcetype[0] != "iteration") {					
						predecessorSeqRawValue = children[cnt].rpcSequenceValue[0];
					}
				}
			}
			console.log("predecessorSeqRawValue and successorSeqRawValue");
			console.log(predecessorSeqRawValue);
			console.log(successorSeqRawValue);
			var predecessorSeq = (predecessorSeqRawValue != null)? new SequenceValue(predecessorSeqRawValue) : null;
			var successorSeq = (successorSeqRawValue != null)? new SequenceValue(successorSeqRawValue) : null;
			return (this.calculateSeq(predecessorSeq, successorSeq)).getRawValue();		
		},
		
		/**
		* The method will generate a sequence value with respect to 
		* predecessorSeq, successorSeq SequenceObject
		*/	
		calculateSeq: function (predecessorSeq, successorSeq) {	
			if (predecessorSeq != null) {
				if (successorSeq != null) {
					return SequenceValue.FACTORY.between(predecessorSeq, successorSeq);
				} else {
					return SequenceValue.FACTORY.successor(predecessorSeq);
				}
			} else if (successorSeq != null) {
				return SequenceValue.FACTORY.predecessor(successorSeq);
			} else {
				return SequenceValue.INITIAL;
			}
		},	


        /**
         * Updates the parentId and parentIdentifier in the tree's model after a
         * successful parent-child change has been persisted on the server.
         * 
         * sourceItem: Item reference of the tree node moved
         * targetItem: Item reference of the new parent tree node
         */
        _updateIdentifiersOnParentChildChange: function(sourceItem, targetItem) {
            if (sourceItem && targetItem) {
				if(!this.tree.dragNdropAsPeer){ 
	                // Change references to the new parent
	                this.model.store.setValue(sourceItem, 'parentId', targetItem.identifier[0]);
	                this.model.store.setValue(sourceItem, 'parentIdentifier', targetItem.identifier[0]);
				}
				else{
					// Change references to the new parent
	                this.model.store.setValue(sourceItem, 'parentId', targetItem.parentIdentifier[0]);
	                this.model.store.setValue(sourceItem, 'parentIdentifier', targetItem.parentIdentifier[0]);
				}
				var savedNode = this.getNodeByItemIdentity(sourceItem.identifier[0]);           
            }
        },
        
        /**
         * Synchronizes the grid to match the structure in the tree after a drag-n-drop
         * has happened. It is worth noting that this method is called in both success
         * and failure case of persisting the parent-child link change. The latter is the
         * case since the node revert to its original parent on failure cause the tree to become
         * out of sync with the grid and so its needed to refetch the rows.
         * 
         * treeID: ID of the tree initiating the event
         * sourceItem: Item reference of the tree node moved
         * targetItem: Item reference of the new parent tree node
         * oldParentItem: Item reference to the old parent tree node
         */
        _syncGridOnParentChildChange: function(treeID, sourceItem, targetItem, oldParentItem) {
            if (treeID && sourceItem && targetItem && oldParentItem) {
                var collectionItemHandles = new Array();
                var sourceHandle = this.convertToItemHandle(sourceItem);
                var targetHandle = this.convertToItemHandle(targetItem);
                var oldParentHandle = this.convertToItemHandle(oldParentItem);
				
                // The rows that are affected by DND are:
                // Source, Target, Old Parent, Child Items of the source
                collectionItemHandles.push(sourceHandle);
                collectionItemHandles.push(targetHandle);
				//If the drag and drop happens within the same project , oldparent and target are one and the same 
				
				if(targetItem.identifier && oldParentItem.identifier && targetItem.identifier != oldParentItem.identifier )
				{
                collectionItemHandles.push(oldParentHandle);
				}
				var newParentItem = this.getItemNodeByIdentity(targetItem.parentIdentifier[0]);
				if(newParentItem != null){
					if(oldParentItem.identifier[0] != newParentItem.identifier[0]){
						var newParentHandle = this.convertToItemHandle(newParentItem);
						collectionItemHandles.push(newParentHandle);
					}
				}
                var childHandles = this._getChildHandles(sourceItem);
                for (var i in childHandles) {
                    collectionItemHandles.push(childHandles[i]);
                }
                
				var childHandlesTarget = this._getChildHandles(targetItem);
                for (var i in childHandlesTarget) {
                    collectionItemHandles.push(childHandlesTarget[i]);
                }
				
                // If the old parent is an iteration, then add its children too
                var oldParentType = dojo.isArray(oldParentItem.resourcetype)?oldParentItem.resourcetype[0]:oldParentItem.resourcetype;
                if (oldParentType == WBS_Constants.ITERATION_TYPE) {
					if(targetItem.identifier && oldParentItem.identifier && targetItem.identifier != oldParentItem.identifier ){
	                    var oldParentChildHandles = this._getChildHandles(oldParentItem);
	                    for (var i in oldParentChildHandles) {
	                        collectionItemHandles.push(oldParentChildHandles[i]);
	                    }
					}
                }
                var publishObj = {
                    collectionItemHandle: collectionItemHandles,
                    treeID: treeID
                };
                
                // Publish an event for the grid to sync itself with the structure in the tree
                dojo.publish(EPMTreeEvents.ON_PARENT_CHILD_CHANGE_SYNC_GRID, [publishObj]);
                
                // Release the page
                Util._releasePage();
            }
        },
        
        /**
         * Recursively gets the child items of a given tree item
         * 
         * sourceItem: Item whose children (and grand-children) are needed
         */
        _getChildHandles: function(sourceItem) {
            var handles = new Array();
            if (sourceItem) {
                for (var i in sourceItem.children) {
                    var childItem = sourceItem.children[i];
                    if (childItem.name[0] !== EPMConstants.LOADING) {
                        var childHandle = this.convertToItemHandle(childItem);
                        handles.push(childHandle);
                        var childHandles = this._getChildHandles(childItem);
                        for (var j in childHandles) {
                            handles.push(childHandles[j]);
                        }
                    }
                }
            }
            
            return handles;
        },
        
        /**
         * Revert the tree to its original state as before the drag-n-drop happened
         * NOTE: In this instance, oldParentItem refers to the node in which the sourceItem
         * was attempted to be moved but was unsuccessful. targetItem now refers to the 
         * the original parent as before the DND. This switch has already been made by the caller
         * of this function.
         * 
         * treeID: ID of the tree initiating the event
         * sourceItem: Item reference of the tree node moved
         * targetItem: Item reference of the new parent tree node
         * oldParentItem: Item reference to the old parent tree node
         * copyOrMove: Boolean to indicate if the node needs to be copied or moved
         *             true: Copy node
         *             false: Move node
         * controller: Controller reference from the tree
         */
        moveItem: function(treeID, sourceItem, targetItem, oldParentItem, copyOrMove, controller) {
            
			// Update Parent Identifiers
			var parentIdentifier = (this.tree.dragNdropAsPeer)? oldParentItem.parentIdentifier[0] : oldParentItem.identifier[0];
            if(sourceItem.parentIdentifier[0] != parentIdentifier){
				// Revert the node in the tree to its original location
	            this.model.pasteItem(sourceItem, oldParentItem, targetItem, copyOrMove);
			}
            // Revert the parent as saved on the grid
            // NOTE: The following should not cause any POST request to the server
            // as we are just reverting what used to be the row's parentIdentifier before.
            this._saveOnParentChildChange(treeID, sourceItem, targetItem, oldParentItem, controller, false);
            
            // Sync the grid to match the tree structure
            this._syncGridOnParentChildChange(treeID, sourceItem, targetItem, oldParentItem);
        },

        /**
         *  checkForUnsavedElements : Method to check if the tree store has any unsaved elements
         *  Incoming : none
         *  Outgoing : none
         */		
		checkForUnsavedElements: function(){
			var self = this;

			function completed(nodes, findResult){
				var length = nodes.length;
				for (var i = 0; i < length; i++) {
					if (self._isNodeNew(nodes[i])) {
						self.hasNewNode = true;
						return;
					}
				}
				self.hasNewNode = false;
			}
			
			function error(errData, request){
			  console.debug("checkForUnsavedElements: Failed in fetching the data.");
			}

			this._treeStore.fetch({onComplete: completed, onError: error, queryOptions: {deep:true}});
		},

		getIconText: function(type){
			var IconName;
			if(type == WBS_Constants.EPM_PROGRAM)
				IconName = CoreMessages.PROGRAM;
			else if(type == WBS_Constants.ITERATION_TYPE)
				IconName = CoreMessages.ITERATION;
			else
				IconName = "";						
			return IconName;
			
		}
    });
})();
