/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridView");
    
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.web.core.ui.internal.FinancialConstants");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.requireLocalization("com.ibm.epm.financial.web.ui", "Messages");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
   
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.financial.web.ui", "Messages");
	var controllerFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var FinancialConstants = com.ibm.epm.web.core.ui.internal.FinancialConstants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
	var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
	var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    
    dojo.declare("com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridView", abstractView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.financial.web", "ui/internal/templates/FinancialDetailsGrid.html"),
		accountHeads: null,
		accountHeadUri: null,
		parent: null,
		overRideCellClasses: false,
		/**
         * postCreate: Method called after a widget's dom has been setup, all initialization and event subscriptions are done here <br>
         * @param none
         */
        postCreate: function(){
			this._rowHoverArray = [];
			this._pconnects = [];
            this._subscriptions = [
				// Tree event on rename of node
				dojo.subscribe(GridEvents.FIELD_CHANGED, this, "_fieldChanged")
            ];            
        },
        
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
        },
		
		/**
		 * The method will be called when Grid view is getting destroyed
		 */
		destroy: function(){
            this._unsubscribeToAllEvents();
			dojo.forEach(this._pconnects, dojo.disconnect);
            this._pconnects = [];
			this.inherited(arguments);
        },
		
		_rowSelected:function(publishedObj){
			if(!this._validatePublishedObject(publishedObj)){
				return;
			}
			this._rowSelect();			
		},
		
		_rowDeselected: function(publishedObj){
			if(!this._validatePublishedObject(publishedObj)){
				return;
			}
			this._rowSelect();
		},
		
		_rowSelect:function(){
			var selectedRows = this._generalGrid.selection.getSelected();
        	if (selectedRows.length !== 0) {
				for (var cnt = 0; cnt < selectedRows.length; cnt++) {
					var selectedRow = this._generalGrid.model.getRow(selectedRows[cnt]);
					if (selectedRow) {
						if ((selectedRow.name == Messages.TOTAL_COST) ||
						(selectedRow.name == Messages.LABOUR_REVENUE) ||
						(selectedRow.name == Messages.MARGIN) ||
						(selectedRow.name == Messages.LABOUR_COST)) {
							//Do not enable the remove button
						}
						else {
							return;
						}
					}
				}
			}
		},
		
		_fieldChanged:function(publishedObj){
			if(this._validatePublishedObject(publishedObj)){
				this.parent._toggleSaveButton(false);
				var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_GRID_TITLE);
				financialController.modified = true;
				var selectedRows = this._generalGrid.selection.getSelected();
	            if (selectedRows.length !== 0) {
					for (var cnt = 0; cnt < selectedRows.length; cnt++) {
						var selectedRow = this._generalGrid.model.getRow(selectedRows[cnt]);
						if (selectedRow) {
							selectedRow.forecastVariance = selectedRow.forecast - selectedRow.actual;
							selectedRow.estimateVariance = selectedRow.estimate - selectedRow.actual;
						}
					}
				}
			}
		},
		
		_validatePublishedObject: function(publishedItem){
			var publishedData = null;
            if (dojo.isArray(publishedItem)) {
                publishedData = publishedItem[0];
            }
            else {
                publishedData = publishedItem;
            }
			if((publishedData.gridID == FinancialConstants.FINANCIAL_GRID_TITLE) ||
			   (publishedData.controllerID == FinancialConstants.FINANCIAL_GRID_TITLE) ){
				return true;
			}
			return false;
		},
		
		onCellClick: function(event){
            if (event) {
                if ((event.cell.field == "action") && (event.target.name == "deleteImage")) {
					this._deleteFinancials();
                }            	
			}
        },
		
        /* 
         * buildChargecodesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with TimecodesGrid.
         */
        buildFinancialGrid: function(parentViewlet, uri, loadedElement, accounts, accountUris){
			this.parent = parentViewlet;
			if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridModel();
            }
			this.accountHeads = accounts;
			this.accountHeadUri = accountUris;
            var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_GRID_TITLE);
			financialController.loadedElement = loadedElement;
			financialController.budgetURI = uri;
            this.buildGrid(this, this._gridModel, financialController);
        },
        
        /*
         * Returns the Grid Title
         */
        getGridTitle: function(){
            return Messages.FINANCIAL_GRID_TITLE;
        },
        
         /*
         * Returns the Grid Internal ID
         */
        getGridInternalID: function(){
            return FinancialConstants.FINANCIAL_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.gridAttachPoint;
        },
		
		 _saveFinancials: function(){
		 	
			this.parent._toggleSaveButton(true);
			this._rowSelect();
            this.persistGrid();
        },
		
		/**
		 * The _deleteFinancials method will remove the selected rows from the grid
		 * and call the save by calling persistGrid. Also it closees the confirmation 
		 * Dialog 
		 */
		_deleteFinancials: function(){
			var selectedRows = this._generalGrid.selection.getSelected();
			if (selectedRows.length !== 0) {
				this._generalGrid.removeSelectedRows();
			}
			this.parent._toggleSaveButton(false);
			var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_GRID_TITLE);
			financialController.modified = true;
			if (this.confirmDialog) {
				this.confirmDialog.hide();
				this.confirmDialog.destroy();
			}
		},
		
		_addFinancials: function(){
			var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_GRID_TITLE);

            var rowID = financialController.getUniqueRowID("_unsaved");
			var itemHandle = {
                title: Messages.ADD_FINANCIAL_TITLE,
                relativeHref: rowID
            };
			financialController._insertFinancials(itemHandle);
			financialController.modified = true;
			this.scrollToSelectedRow();
			this.startFirstCellEdit();
		},
		
		/**
		 * The _removeFinancials method will remove the top selected row from the grid
		 *  and call save of the budgets. 
		 */
		_removeFinancials: function() {

			var selectedRows = this._generalGrid.selection.getSelected();
            if (selectedRows.length !== 0) {
				this._generalGrid.selection.unselectAll();
				for (var cnt = 0; cnt < selectedRows.length; cnt++) {
					var selectedRow = this._generalGrid.model.getRow(selectedRows[cnt]);
					if (selectedRow) {
						if ((selectedRow.name == Messages.TOTAL_COST) ||
						(selectedRow.name == Messages.LABOUR_REVENUE) ||
						(selectedRow.name == Messages.MARGIN) ||
						(selectedRow.name == Messages.LABOUR_COST)) {
							//DO NOT ADD TO SELECTION
						}
						else {
							this._generalGrid.selection.addToSelection(selectedRows[cnt]);
						}
					}
				}				
				// Get selected Rows and then call delete on them in a loop
				var selectedRowIndexes = this._generalGrid.selection.getSelected();
				if (selectedRowIndexes.length !== 0) {
					var self = this;
					var buttonlist = [{
						label: Messages.YES,
						action: dojo.hitch(this, this._deleteFinancials)
					}, {
						label: Messages.NO
					}];
					var dialogContent;
					if (selectedRowIndexes.length > 1) {
						dialogContent = Messages.DIALOG_CONTENT_MULTIPLE_FINANCIALS_SELECTED;
					}
					else {
						dialogContent = Messages.DIALOG_CONTENT_SINGLE_FINANCIALS_SELECTED;
					}
					var labelList = [{
						label: dialogContent
					}];
					
					this.confirmDialog = new com.ibm.epm.web.core.ui.internal.EPMConfirmDialog({
						labelList: labelList,
						title: Messages.DIALOG_TITLE,
						buttonlist: buttonlist
					});
					this.confirmDialog.show();
				}
			}
		},
		
		_getAccounts: function(){
			if (this.accountHeads != null) {
				return this.accountHeads;
			}
			return [];	
		},
		
		_getAccountUris: function(){
			if (this.accountHeadUri != null) {
				return this.accountHeadUri;
			}
			return [];	
		},
		
		/*
         * Override the function defined in EPMGrid.
         */
        isCellEditable: function(event){			
			var rowIndex = parseInt(event.rowIndex, 10);
			if (isNaN(rowIndex)) {
			    return true;
			}
			
			var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_GRID_TITLE);
			var row = financialController.getRowByIndex(rowIndex);
			if (row ) {
				if (row.name == Messages.LABOUR_COST){
						if ((event.cell.field == Messages.FORECAST_FIELD) &&
							(row.isForecastCalculate == false))
						{
						    return true;
						}
						return false;
				}
				if ((row.name == Messages.LABOUR_COST) ||
					(row.name == Messages.TOTAL_COST)){
						return false;
				}
			}
			return true;
        },
		
		/*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        
			_formatFinancials = function(inDatum, inRowIndex){
	            var row = null;
	            if (this.grid && this.grid.model) {
	                row = this.grid.model.getRow(inRowIndex);
	            }
	            
	            if (row ) {
					if (row.name == Messages.TOTAL_COST){
						var font = "font-weight: bold; color: black";
						this.customStyles.push(font);							
					}
					else if((row.name == Messages.LABOUR_COST) &&
							(this.name != Messages.ACCOUNT_HEAD)){
							
							if(this.name == Messages.FORECAST) {
								if(row.isForecastCalculate == true){
									var font = "color: #999999";
									this.customStyles.push(font);	
								}
							}
							else{	
								var font = "color: #999999";
								this.customStyles.push(font);								
							}
					}
					else{
						if ((row.name != Messages.LABOUR_COST) && 
						    (this.editor) && 
							(this.editor.declaredClass == "dojox.grid.editors.Select")) {
								var nod = this.editor.getNode(inRowIndex);
								if (nod) {
									if (typeof(nod.selectedIndex) != "undefined") {
										var icnt = nod.selectedIndex;
										row.chargeCodeUri = this.uris[icnt];
										row.name = this.editor.options[icnt];										
									}
								}
						}												
					}
	            }				
				return isNaN(inDatum) ? inDatum : parseFloat(inDatum).toFixed(2);
	        };
			
			_displayActions = function(inRowIndex){
				if (this.grid && this.grid.model) {
	                row = this.grid.model.getRow(inRowIndex);
	            }	            
	            if (row ) {
					if ((row.name == Messages.TOTAL_COST) || 
						(row.name == Messages.LABOUR_COST)){
						return '';
					}
	            }	
				return '<img name="deleteImage" style="margin-right: 5px; margin-left: 5px; cursor: pointer;" title="' + Messages.REMOVE +
                    '" alt="' + Messages.REMOVE + '" src =' +
                    dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/actions/action_unassign.gif") +
                    " />";
			};
            var gridLayout = null;
            gridLayout = [{
                noscroll: false,
                rows: [[{
                    name: Messages.ACCOUNT_HEAD,
                    width: "25%",
                    field: "name",
					editor: dojox.grid.editors.Select,
					options: this._getAccounts(),
					uris: this._getAccountUris(),
					styles: 'text-align: left;',
					formatter: _formatFinancials,
					cellClasses: "com-ibm-epm-financial-web-ui-internal-FinancialDetailsGrid_Category_Col"
                },{
                	name: Messages.DESCRIPTION,
                	width: "15%",
                	field: "description",
                	editor: dojox.grid.editors.Input,
                	styles: 'text-align: right;'
                }, {
                    name: Messages.FORECAST,
                    width: "10%",
                    field: "forecast",
					editor: dojox.grid.editors.Dijit,
                    editorClass: "dijit.form.CurrencyTextBox",
                    constraint: {
                        min: 0
                    },
					currency: Messages.CURRENCY ,
					styles: 'text-align: right;',
					formatter: _formatFinancials                 
                }, {
                    name:  Messages.ESTIMATE,
                    width: "8%",
                    field: "estimate",
					editor: dojox.grid.editors.Dijit,
                    editorClass: "dijit.form.CurrencyTextBox",
                    constraint: {
                        min: 0
                    },
                    currency: Messages.CURRENCY,
					styles: 'text-align: right;',
					formatter: _formatFinancials
                }, {
                    name: Messages.ACTUAL,
                    width: "8%",
                    field: "actual",
					editor: dojox.grid.editors.Dijit,
                    editorClass: "dijit.form.CurrencyTextBox",
                    constraint: {
                        min: 0
                    },
                    currency: Messages.CURRENCY,
					styles: 'text-align: right;',
					formatter: _formatFinancials
                }, {
                    name: Messages.FORECAST_VARIANCE,
                    width: "12%",
                    field: "forecastVariance",
					styles: 'text-align: right;',
					formatter: _formatFinancials
                }, {
                    name: Messages.ESTIMATE_VARIANCE,
                    width: "12%",
                    field: "estimateVariance",
					styles: 'text-align: right;',
					formatter: _formatFinancials
                }, {
	                name: Messages.ACTIONS,
	                width: "auto",
	                field: "action",
	                headerClasses: "com-ibm-epm-financial-web-ui-internal-FinancialDetailsGrid-unsortableColumn",
					get:_displayActions
            	}	]]
            }];
            return gridLayout;
        },
		/**
		 * The method will be called from the grid controller whenever there
		 * is an error occur while create/update/delate a resource
		 * @param {Object} errorObject
		 */
		handleError: function(errorObject){
            this.inherited("handleError", arguments);
            this._displayError(errorObject);
        },
		
		/**
         *
         * @param {Object} row
         * This function is called from the onStyleRow listener event.
         * Depending upon the state of the task or if the timesheet is
         * for the project manager view, approriate class is attached to
         * the row node or the cell node.
         */
		
        _styleRow: function(row){
			if(this._generalGrid){
				var financialGridController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_GRID_TITLE);
				var item = financialGridController.getRowByIndex(row.index);
	            if (item) {
	               if (item.name == Messages.TOTAL_COST){
	                	row.customClasses = row.customClasses + " " + "rowBorderTop";
					}
	            }
			}
        },
		
		/**
		 * createView - method will call the createView of the abstractView
		 * And also set the grid to grow auto.
		 */
		
		createView: function(){
			this.inherited('createView', arguments);		
			this._pconnects.push(dojo.connect(this._generalGrid, 'onStyleRow', this, this._styleRow));
			this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_DBL_CLICK, this, this.onCellClick));
            this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_CLICK, this, this.onCellClick));
			
			this._generalGrid.autoHeight = true;
			this._generalGrid.update();
		}
    });
})();

