/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.financial.web.ui.internal.FinancialSummaryGridView");
    
   dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.web.core.ui.internal.FinancialConstants");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.requireLocalization("com.ibm.epm.financial.web.ui", "Messages");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
   
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.financial.web.ui", "Messages");
	var controllerFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var FinancialConstants = com.ibm.epm.web.core.ui.internal.FinancialConstants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
	var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    
    dojo.declare("com.ibm.epm.financial.web.ui.internal.FinancialSummaryGridView", abstractView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.financial.web", "ui/internal/templates/FinancialSummaryGrid.html"),
		accountHeads: null,
		accountHeadUri: null,
		parent: null,
		overRideCellClasses: false,

		/**
         * postCreate: Method called after a widget's dom has been setup, all initialization and event subscriptions are done here <br>
         * @param none
         */
        postCreate: function(){
            this._subscriptions = [
				// Tree event on rename of node
				dojo.subscribe(GridEvents.FIELD_CHANGED, this, "_fieldChanged")
            ];            
        },
        
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
        },
		
		/**
		 * The method will be called when Grid view is getting destroyed
		 */
		destroy: function(){
            this._unsubscribeToAllEvents();
			this.inherited(arguments);
        },
		
		_rowSelected:function(publishedObj){
			if(!this._validatePublishedObject(publishedObj)){
				return;
			}
			this._rowSelect();			
		},
		
		_rowDeselected: function(publishedObj){
			if(!this._validatePublishedObject(publishedObj)){
				return;
			}
			this._rowSelect();
		},
		
		_rowSelect:function(){
			var selectedRows = this._generalGrid.selection.getSelected();
        	if (selectedRows.length !== 0) {
				for (var cnt = 0; cnt < selectedRows.length; cnt++) {
					var selectedRow = this._generalGrid.model.getRow(selectedRows[cnt]);
					if (selectedRow) {
						if ((selectedRow.name == Messages.TOTAL_COST) ||
						(selectedRow.name == Messages.LABOUR_REVENUE) ||
						(selectedRow.name == Messages.MARGIN) ||
						(selectedRow.name == Messages.LABOUR_COST)) {
							//Do not enable the remove button
						}
						else {
							return;
						}
					}
				}
			}
		},
		
		_fieldChanged:function(publishedObj){
			if(this._validatePublishedObject(publishedObj)){
				this.parent._toggleSaveButton(false);
				var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_GRID_TITLE);
				financialController.modified = true;
			}
		},
		
		_validatePublishedObject: function(publishedItem){
			var publishedData = null;
            if (dojo.isArray(publishedItem)) {
                publishedData = publishedItem[0];
            }
            else {
                publishedData = publishedItem;
            }
			if((publishedData.gridID == FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE) ||
			   (publishedData.controllerID == FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE) ){
				return true;
			}
			return false;
		},
		
        /* 
         * buildChargecodesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with TimecodesGrid.
         */
        buildFinancialGrid: function(parentViewlet, uri, loadedElement, accounts, accountUris){
			this.parent = parentViewlet;
			if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridModel();
            }
			this.accountHeads = accounts;
			this.accountHeadUri = accountUris;
            var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE);
			financialController.loadedElement = loadedElement;
			financialController.budgetURI = uri;
			financialController.budgetSummary = true;
            this.buildGrid(this, this._gridModel, financialController);
        },
        
        /*
         * Returns the Grid Title
         */
        getGridTitle: function(){
            return Messages.FINANCIAL_GRID_TITLE;
        },
        
         /*
         * Returns the Grid Internal ID
         */
        getGridInternalID: function(){
            return FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.gridSummaryAttachPoint;
        },
		
		 _saveFinancials: function(){
			this.parent._toggleSaveButton(true);
        },
		
		/**
		 * The _deleteFinancials method will remove the selected rows from the grid
		 * and call the save by calling persistGrid. Also it closees the confirmation 
		 * Dialog 
		 */
		_deleteFinancials: function(){
		},
		
		_addFinancials: function(){
		},
		
		/**
		 * The _removeFinancials method will remove the top selected row from the grid
		 *  and call save of the budgets. 
		 */
		_removeFinancials: function() {

			
		},
		
		/*
         * Override the function defined in EPMGrid.
         */
        isCellEditable: function(event){			
			var rowIndex = parseInt(event.rowIndex, 10);
			if (isNaN(rowIndex)) {
			    return true;
			}
			
			var financialController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE);
			var row = financialController.getRowByIndex(rowIndex);
			if (row ) {
				if (row.name == Messages.LABOUR_REVENUE){
					if ((event.cell.field == Messages.FORECAST_FIELD) &&
						(row.isForecastCalculate == false))
					{
					    return true;
					}
				}				
			}
			return false;
        },
		
		/*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        
			_formatFinancials = function(inDatum, inRowIndex){
	            var row = null;
	            if (this.grid && this.grid.model) {
	                row = this.grid.model.getRow(inRowIndex);
	            }
	            
	            if (row ) {
					if (row.name == Messages.MARGIN){
						var font = "font-weight: bold; color: black";
						this.customStyles.push(font);									
					}
					else if((row.name == Messages.LABOUR_REVENUE) &&
							(row.isForecastCalculate == true) &&
							(this.name == Messages.FORECAST)){
						var font = "color: #999999";
						this.customStyles.push(font);
					}
					else if((row.name == Messages.TOTAL_COST) && 
						    (this.name == Messages.FORECAST)){
						var font = "color: #999999";
						this.customStyles.push(font);
					}					
	            }				
				return isNaN(inDatum) ? inDatum : parseFloat(inDatum).toFixed(2);
	        };
		
            var gridLayout = null;
            gridLayout = [{
                noscroll: false,
                rows: [[{
                    name: Messages.ACCOUNT_HEAD,
                    width: "25%",
                    field: "name",
					styles: 'text-align: left;',
					formatter: _formatFinancials
                },{
                	name: Messages.DESCRIPTION,
                	width: "15%",
                	field: "description",
                	editor: dojox.grid.editors.Input,
                	styles: 'text-align: right;',
					cellClasses: "com-ibm-epm-financial-web-ui-internal-FinancialSummaryGrid_Others"
                }, {
                    name: Messages.FORECAST,
                    width: "10%",
                    field: "forecast",
					editor: dojox.grid.editors.Dijit,
                    editorClass: "dijit.form.CurrencyTextBox",
                    constraint: {
                        min: 0
                    },
					currency: Messages.CURRENCY ,
					styles: 'text-align: right;',
					formatter: _formatFinancials
                }, {
                    name:  Messages.ESTIMATE,
                    width: "8%",
                    field: "estimate",
					styles: 'text-align: right;',
					formatter: _formatFinancials,
					cellClasses: "com-ibm-epm-financial-web-ui-internal-FinancialSummaryGrid_Others"
                }, {
                    name: Messages.ACTUAL,
                    width: "8%",
                    field: "actual",
					styles: 'text-align: right;',
					formatter: _formatFinancials,
					cellClasses: "com-ibm-epm-financial-web-ui-internal-FinancialSummaryGrid_Others"
                }, {
                    name: Messages.FORECAST_VARIANCE,
                    width: "12%",
                    field: "forecastVariance",
					styles: 'text-align: right;',
					formatter: _formatFinancials,
					cellClasses: "com-ibm-epm-financial-web-ui-internal-FinancialSummaryGrid_Others"
                }, {
                    name: Messages.ESTIMATE_VARIANCE,
                    width: "12%",
                    field: "estimateVariance",
					styles: 'text-align: right;',
					formatter: _formatFinancials,
					cellClasses: "com-ibm-epm-financial-web-ui-internal-FinancialSummaryGrid_Others"
                }, {
	                name: Messages.ACTIONS,
	                width: "auto",
	                field: "action"
            	}	]]
            }];
            return gridLayout;
        },
		/**
		 * The method will be called from the grid controller whenever there
		 * is an error occur while create/update/delate a resource
		 * @param {Object} errorObject
		 */
		handleError: function(errorObject){
            this.inherited("handleError", arguments);
			this._displayError(errorObject);
        },
		
		/**
         *
         * @param {Object} row
         * This function is called from the onStyleRow listener event.
         * Depending upon the state of the task or if the timesheet is
         * for the project manager view, approriate class is attached to
         * the row node or the cell node.
         */
        _styleRow: function(row){
        	
			if(this._generalGrid){
				var financialGridController = controllerFactory.getEPMFinancialGridController(FinancialConstants.FINANCIAL_SUMMARY_GRID_TITLE);
				var item = financialGridController.getRowByIndex(row.index);
	            
	            if (item) {
	                if (item.name == Messages.MARGIN){
	                	row.customClasses = row.customClasses + " " + "rowBorderTop";
					}
	            }
			}
        },
		/**
		 * createView - method will call the createView of the abstractView
		 * And also set the grid to grow auto.
		 */
		createView: function(){
			this.inherited('createView', arguments);
			dojo.connect(this._generalGrid, 'onStyleRow', this, this._styleRow);
			this._generalGrid.autoHeight = true;
			this._generalGrid.update();
		}
    });
})();

