/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.financial.web.ui.internal.FinancialViewlet");
    
    // Viewlet
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    dojo.require("com.ibm.epm.web.core.ui.internal.FinancialConstants");
	dojo.require("com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridView");
	dojo.require("com.ibm.epm.financial.web.ui.internal.FinancialSummaryGridView");
	dojo.require("com.ibm.epm.financial.web.ui.internal.AssociateProjectCodes");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
	dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
	dojo.require("com.ibm.epm.web.core.util.internal.Constants");
	dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    dojo.requireLocalization("com.ibm.epm.financial.web.ui", "Messages");
    
	var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
	var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.financial.web.ui", "Messages");
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var Constants = com.ibm.epm.web.core.ui.internal.FinancialConstants;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var EPMConstants = com.ibm.epm.web.core.util.internal.Constants;
	var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
	var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
	
	var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	
    dojo.declare("com.ibm.epm.financial.web.ui.internal.FinancialViewlet", Viewlet, {
        templatePath: dojo.moduleUrl("com.ibm.epm.financial.web", "ui/internal/templates/FinancialViewlet.html"),
        contextHelpId: Constants.FINANCIAL_VIEWLET_CONTEXT_HELP_ID, //	The Context Sensitive Help ID for the WBS grid viewlet.
        noWBSTree: false,

        /**
         *  init : To initialize the widget
         *  Incoming : none
         *  Outgoing : none
         *
         */
        init: function(){
			this.inherited(arguments);

            this._createInformationLabel();
            
            // Disable viewlet if no WBS tree
            if (this._disableViewletIfNoWBSTree())
                return;
                
			this.financialTimeCodes = false;
			this.financialChargeCodes = false;
			this.financialBudgets = false;
			this.setCSHelpId(this.contextHelpId);
            this._financialDetailsGrid = null;
			this._financialSummaryGrid = null;
			this.budget = null;
			this.budgetUri = null;
			this.accountHeads = null;
			this.accountHeadUris = null;
			this.intialize = false;
            this._subscriptions = [];
			this._createToolbar();
            this.eventSubscribe();
            			
			this._updateViewletBasedOnSelectedNode();
        },
        
        /*
         * Create information label text. Do this just one time for the life of the
         * viewlet.
         */
        _createInformationLabel: function() {
            this.informationLabel = document.createElement("label");
            this.informationLabel.innerHTML = "";
			this.setCSHelpIdInToolbar(this.contextHelpId, this._informationLabel);
            this._informationLabel.appendChild(this.informationLabel);
        },
               
        /*
         * Disable viewlet and display an appropriate message if there is no WBS Tree
         * for the viewlet to make use of. Returns true if viewlet was disabled, signifying
         * to caller that further setup is not necessary.
         */
        _disableViewletIfNoWBSTree: function() {
            var tree = dijit.byId(Constants.WBS_TREE_ID);
            if (tree) {
                return false;
            }

            this.noWBSTree = true;
            this._displayInformationLabel(Messages.WBS_TREE_MISSING);
            return true;
        },
        
        /*
         * If any node is selected in the tree, a document can be attached, so enable the viewlet. 
         * Otherwise, disable the viewlet and display appropriate message. This assumes that the
         * viewlet is being displayed along side a WBS tree.
         */
        _updateViewletBasedOnSelectedNode: function() {
            if (this._isProjectNodeSelected())
                this._updateWBSNodeIdentifier(dijit.byId(Constants.WBS_TREE_ID)._selectedNode.item);
            else
                this._displayInformationLabel(Messages.NO_WBS_PROJECT_SELECTED);
        },
        
        // Return true only if a WBS tree project node is selected
        _isProjectNodeSelected: function() {
            var tree = dijit.byId(Constants.WBS_TREE_ID);
            if (tree && tree._selectedNode) {
                var item = tree._selectedNode.item;
                if (item && item.resourcetype == EPMConstants.EPM_PROJECT)
                    return true;
            }
            return false;
        },
        
        /*
         * Displays information label and hides the rest of the viewlet.
         */
        _displayInformationLabel: function(text) {
			//Set the dirty flag so that no message is thrown while switching
			//between tabs
			this._toggleSaveButton(true);
            dojo.addClass(this._content, "HiddenClass");
            this.informationLabel.innerHTML = text;
            dojo.removeClass(this._informationLabel, "HiddenClass");
        },
        
        /*
         * Hides information label and shows the rest of the viewlet.
         */
        _hideInformationLabel: function() {
            dojo.addClass(this._informationLabel, "HiddenClass");
            this.informationLabel.innerHTML = "";
            dojo.removeClass(this._content, "HiddenClass");
        },
        
		setActive: function(/* boolean */active){
            if (active) {
                this._updateViewBasedonTree();
            }
         },
		 
		/*
		* Get the current tree node to load the view
		* Incoming : None
		* Outgoing : None
		*/
		_updateViewBasedonTree: function(){
			var tree = dijit.byId(Constants.WBS_TREE_ID);
			if (tree && tree._selectedNode) {
				var item = tree._selectedNode.item;
				if (item && item.identifier != this.identifier) {
					var obj = {
						treeID: Constants.WBS_TREE_ID,
						identifier: item.identifier,
						name: item.name,
						resourcetype: item.resourcetype,
						href: item.href
					};
					this._updateWBSNodeIdentifier(obj);
				}
			}
		},
		
        /**
         * eventSubscribe: Method to subscribe to all events.
         */
        eventSubscribe: function(){
            this._subscriptions = [dojo.subscribe(EPMTreePubSub.ON_SELECT, this, "_updateWBSNodeIdentifier")];
        },
        
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events.
         */
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
        },
        
        /**
         *  _createToolbar : Creates the common toolbar which is populated by the Grid and the Gantt view
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createToolbar: function(){
	
			//Viewlet Tool bar		
			//Save button
			this.saveButton = document.createElement(Constants.BUTTON_ELEMENT);
            this.saveButton.innerHTML = Messages.SAVE;
            this.saveButton.title = Messages.SAVE;
			dojo.connect(this.saveButton, "onclick", this, this._saveFinancials);
			//Make disable by default
			this._toggleSaveButton(true);
			
			//Refresh button
			this.refreshButton = new dijit.form.Button({
                title: Messages.REFRESH,
				label : Messages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this.refreshFinancialViewlet)
            });
			
			var rightContentList = [this.refreshButton, this.saveButton];
            
            this.viewletToolbar = new EPMToolbar({
                rightContentList: rightContentList
            });
			
			this.setCSHelpIdInToolbar(this.contextHelpId, this.viewletToolbar);
			this.imageButtons.appendChild(this.viewletToolbar.domNode);
			
			//Budget Section
			//Add button
			this.addButtonBudget = new dijit.form.Button({
                title: Messages.ADD,
                label: Messages.ADD.bold(),				               
                onClick: dojo.hitch(this, this._addFinancials)
            });
			
			var rightContentListForBudget = [this.addButtonBudget];
            
            this.toolbarFinancialBudgets = new EPMToolbar({
				title: Messages.BUDGET,
                rightContentList: rightContentListForBudget				
            });
			this._sectionFinancialsBudget.appendChild(this.toolbarFinancialBudgets.domNode);
			
			//Charge Code Section
			//Add button
			this.addButtonChargeCode = new dijit.form.Button({
                title: Messages.ADD,
                label: Messages.ADD.bold(),				               
                onClick: dojo.hitch(this, this._addChargeCodes)				
            });
			
			var rightContentListForChargeCode = [this.addButtonChargeCode];
            
            this.toolbarFinancialChargeCodes = new EPMToolbar({
				title: Messages.CHARGE_CODES,
                rightContentList: rightContentListForChargeCode				
            });
			this._sectionFinancialsChargeCode.appendChild(this.toolbarFinancialChargeCodes.domNode);
			
			//Time Code Section
			//Add button
			this.addButtonTimeCode = new dijit.form.Button({
                title: Messages.ADD,
                label: Messages.ADD.bold(),				               
                onClick: dojo.hitch(this, this._addTimeCodes)
            });
			
			var rightContentListForTimeCode = [this.addButtonTimeCode];
            
            this.toolbarFinancialTimeCodes = new EPMToolbar({
				title: Messages.TIME_CODES,
                rightContentList: rightContentListForTimeCode				
            });
			this._sectionFinancialsTimeCode.appendChild(this.toolbarFinancialTimeCodes.domNode); 
        },
        
        // Destroy needs to be implemented by the viewlet as its invoked by the framework on cleanup
        destroy: function(){
            this._unsubscribeToAllEvents();
            this._destroyGrid();
            
            for (var i=0; i < this._informationLabel.childNodes.length; i++) {
				this._informationLabel.removeChild(this._informationLabel.childNodes[i]);
			}
        },       
        
        _destroyGrid: function() {
			if (this._financialDetailsGrid) {
                this._gridContent.removeChild(this._financialDetailsGrid.domNode);
                this._financialDetailsGrid.destroy();
                delete this._financialDetailsGrid;
                this._financialDetailsGrid = null;
            } 
			
			if(this._financialSummaryGrid) {
				this._gridSumContent.removeChild(this._financialSummaryGrid.domNode);
				this._financialSummaryGrid.destroy();
				delete this._financialSummaryGrid;
				this._financialSummaryGrid = null;
			}
			this._destroyCodesGrid();
        },
		
		_destroyCodesGrid:function(){
			if (this._financialChargeCodes) {
                this._gridChargeCodeContent.removeChild(this._financialChargeCodes.domNode);
				this.financialChargeCodes = false;
				this._financialChargeCodes.destroy();
				this._financialChargeCodes = null;
            }
			if (this._financialTimeCodes) {
                this._gridTimeCodeContent.removeChild(this._financialTimeCodes.domNode);
				this.financialTimeCodes = false;
				this._financialTimeCodes.destroy();
				this._financialTimeCodes = null;
            } 
		},
        
        /**
         *  refresh : This function will be called when the viewlet loads
         *  Incoming : none
         *  Outgoing : none
         *
         */
        refresh: function(){
			if(this.intialize){
				this._destroyGrid();
			
				// Only bother recreating grid if a project node is currently selected
				if (!this.noWBSTree) {
	                if (this._isProjectNodeSelected()) {
	                    this._createWidgets();
			            this._toggleButtons(true);
			        }
				}
			}			
        },
		
		_createCodesWidget:function(){
			if (this._financialChargeCodes == null) {
                this._financialChargeCodes = new com.ibm.epm.financial.web.ui.internal.AssociateProjectCodes();
                this._gridChargeCodeContent.appendChild(this._financialChargeCodes.domNode);
				this.financialChargeCodes = true;				
            }
			this._financialChargeCodes.load(this, Constants.CHARGE_CODE_TYPE, this.project);
			if (this._financialTimeCodes == null) {
                this._financialTimeCodes = new com.ibm.epm.financial.web.ui.internal.AssociateProjectCodes();
                this._gridTimeCodeContent.appendChild(this._financialTimeCodes.domNode);
				this.financialTimeCodes = true;				
            }
			this._financialTimeCodes.load(this, Constants.TIME_CODE_TYPE, this.project);
		},
		
		_createWidgets: function(){
            if (this._financialDetailsGrid == null) {
                this._financialDetailsGrid = new com.ibm.epm.financial.web.ui.internal.FinancialDetailsGridView({
                    _treeID: Constants.FINANCIAL_ID
                });
                this._gridContent.appendChild(this._financialDetailsGrid.domNode);
				this.financialBudgets = true;
            }
            this._financialDetailsGrid.buildFinancialGrid(this, this.budgetUri, this.budget, this.accountHeads, this.accountHeadUris);
			
			if (this._financialSummaryGrid == null) {
                this._financialSummaryGrid = new com.ibm.epm.financial.web.ui.internal.FinancialSummaryGridView({
                    _treeID: Constants.FINANCIAL_ID
                });
                this._gridSumContent.appendChild(this._financialSummaryGrid.domNode);
            }
            this._financialSummaryGrid.buildFinancialGrid(this, this.budgetUri, this.budget);
			
			this._createCodesWidget();
        },
		
		_clickedFinancialsBudget:function(e){
			if(e && e.target && e.target.nodeName != "DIV"){
				return;
			}
			if(this._financialDetailsGrid == null){
				return;
			}
			this._toggleGridWidgetClass(this._financialDetailsGrid, this.financialBudgets);
			this._toggleGridWidgetClass(this._financialSummaryGrid, this.financialBudgets);	
			this.financialBudgets = !this.financialBudgets;		
		},
		
		_keypressFinancialsBudget:function(e){
			if(e.keyCode === 32 || e.keyCode === 13){					
				this._clickedFinancialsBudget(e);							
			}
		},
		
		_clickedFinancialsChargeCode:function(e){
			if(e && e.target && e.target.nodeName != "DIV"){
				return;
			}
			if(this._financialChargeCodes == null){
				return;
			}
			this._toggleGridWidgetClass(this._financialChargeCodes, this.financialChargeCodes);
			this.financialChargeCodes = !this.financialChargeCodes;
		},
		
		_keypressFinancialsChargeCode:function(e){
			if(e.keyCode === 32 || e.keyCode === 13){					
				this._clickedFinancialsChargeCode(e);							
			}
		},
		
		_clickedFinancialsTimeCode:function(e){
			if(e && e.target && e.target.nodeName != "DIV"){
				return;
			}
			if(this._financialTimeCodes == null){
				return;
			}
			
			this._toggleGridWidgetClass(this._financialTimeCodes, this.financialTimeCodes);
			this.financialTimeCodes = !this.financialTimeCodes;			
		},
		
		/**
		 * The method will toggle the widget class depending upon the flag
		 * @param {Object} widget
		 * @param {Object} flag
		 */
		_toggleGridWidgetClass:function(widget, flag){
			if(widget == null){
				return;
			}
			if(flag){
				flag = false;
				dojo.removeClass(widget.domNode,"com-ibm-epm-web-core-ui-internal-templates-EPMGrid");
				dojo.addClass(widget.domNode ,"HiddenClass");
			}else{
				flag = true;
				dojo.removeClass(widget.domNode,"HiddenClass");
				dojo.addClass(widget.domNode ,"com-ibm-epm-web-core-ui-internal-templates-EPMGrid");
			}
		},
		
		_keypressFinancialsTimeCode:function(e){
			if(e.keyCode === 32 || e.keyCode === 13){					
				this._clickedFinancialsTimeCode(e);							
			}
		},
		
		_addChargeCodes: function(){
			if(this._financialChargeCodes){
				if(!this.financialChargeCodes){
					this._toggleGridWidgetClass(this._financialChargeCodes._codesGrid, this.financialChargeCodes);
					this.financialChargeCodes = !this.financialChargeCodes;
				}
				this._financialChargeCodes._addCodes(); 	
			}
		},
		
		_addTimeCodes: function(){
			if(this._financialTimeCodes){
				if(!this.financialTimeCodes){
					this._toggleGridWidgetClass(this._financialTimeCodes._codesGrid, this.financialTimeCodes);
					this.financialTimeCodes = !this.financialTimeCodes;
				}
				this._financialTimeCodes._addCodes();
			}
		},
		
		_toggleButtons: function(state){
			this._toggleSaveButton(state);
			this._toggleRefreshButton(state);
		},
		
		
		
		_toggleSaveButton: function(state){
			this.saveButton.disabled = state;
			if (state == true) {
				this.resetDirtyFlag();
            }
            else {
				this.setDirtyFlag();
            }
		},
		
		_toggleRefreshButton: function(state){
			this.refreshButton.attr('disabled',state);
		},
        
        /**
         *  refreshFinancialViewlet : This method is called when the refresh button is pressed
         *  Incoming : none
         *  Outgoing : none
         *
         */
        refreshFinancialViewlet: function() {
            this.budget = null;
			this.budgetUri = null;
			this.project = null;
			
			// Viewlet is only useful if a Project node is selected
			if (this.wbsNodeType == EPMConstants.EPM_PROJECT) {
                this._hideInformationLabel();
				EPMErrorHandler.clearError();
				Util.showMessage(Messages.LOADING_FINANCIALS);	
				//TODO insert
				this._getAccountHeads();
				this._getResource(this.identifier, Constants.HANDLE_PROJECT);
            }

            // Unsupported node selected; disable viewlet
            else {				
                this._displayInformationLabel(Messages.NO_WBS_PROJECT_SELECTED);
                
                // Destroy grid so it doesn't flash old data at user when a project is later selected
                this._destroyGrid();
            }
        },
        
        /*
         * Called when Tree Node Select Event is published
         * Incoming : WBS Tree Node
         * Outgoing : None
         */
        _updateWBSNodeIdentifier: function(wbsNodeDetail){
            // Process only if the Tree event has come from WBSTree
            if ((!this.getSite().isActive()) ||
            (this.needToHandleEvent(wbsNodeDetail) == false)) {
                //Event ignored due to ID mismatch
                return;
            }
            
            this.identifier = wbsNodeDetail.identifier;
            this.wbsNodeName = wbsNodeDetail.name;
            this.wbsNodeType = wbsNodeDetail.resourcetype;
			this.refreshFinancialViewlet();			
        },
        
        /*
         * Checks whether the event has to be handled or not
         * Incoming : Tree Id
         * Outgoing : Boolean value
         */
        needToHandleEvent: function(publishedData){
        
            var boolNeedToHandleEvent = false;
            if (publishedData.treeID == Constants.WBS_TREE_ID) {
                boolNeedToHandleEvent = true;
            }
            return boolNeedToHandleEvent;
        },
		
		_addFinancials: function(){
			if(this._financialDetailsGrid){
				if(!this.financialBudgets){
					this._toggleGridWidgetClass(this._financialDetailsGrid, this.financialBudgets);
					this._toggleGridWidgetClass(this._financialSummaryGrid, this.financialBudgets);	
					this.financialBudgets = !this.financialBudgets;	
				}
				//validation
				if(this.accountHeads == null){
					EPMErrorHandler.setError(Messages.NO_CATEGORY_EXISTS);
					// Set an interval and clear the error message
		            var intervalID = setInterval(function(){
		                EPMErrorHandler.clearError();
		                clearInterval(intervalID);
		            }, 10000);			
					return;
				}
				this._financialDetailsGrid._addFinancials();
				this._toggleSaveButton(false);	
			}		
		},
		
		_saveFinancials: function(){
			var saveParams = {};
			var modifiedTimeCodes = this._financialTimeCodes._getProjectCodes(saveParams);
			var modifiedChargeCodes = this._financialChargeCodes._getProjectCodes(saveParams);
			if(modifiedTimeCodes || modifiedChargeCodes){
				this._saveProject(saveParams);
			}
			else{
				//save the financial only after saving the project assocation codes
				this._financialDetailsGrid._saveFinancials();
			}
		},
		
		_saveProject: function(params){
			
			//lock the page
			Util._lockPage(Messages.SAVING_CODES);			
			var self = this;
			var persistRowFunctions = {
				_persistRowSuccess: function(savedElement){
					self._getResource(self.identifier, Constants.HANDLE_REFRESH_CODES);
				},
				
				_persistRowFailure: function(errorMsg){
					EPMErrorHandler.setError(errorMsg.errorMessage, errorMsg.errorCode);
					// Set an interval and clear the error message
		            var intervalID = setInterval(function(){
		                EPMErrorHandler.clearError();
		                clearInterval(intervalID);
		            }, 10000);
					self._getResource(self.identifier, Constants.HANDLE_REFRESH_CODES);
					self._releaseAndSaveBudgets();					
				}
			};
			
			var srh = new ServiceResponseHandler(persistRowFunctions, "_persistRowSuccess", "_persistRowFailure");
			if (this.identifier != null) {
				gridModelClient.postURI(srh, this.identifier, params);
			}
			
		},
		
		_removeFinancials: function() {
			this._financialDetailsGrid._removeFinancials();
		},
		
		_handleProject: function(loadedElement){
			if(loadedElement != null){
				this.project = loadedElement;
				//Allow adding category only for main project
				if(this.project.epmProjectType  == "SUB_PROJECT"){
					this.addButtonBudget.disabled = true;
				}
				else{
					this.addButtonBudget.disabled = false;
				}
				var budgetlink = getLink(loadedElement, "EPMBudget");
				if(budgetlink != null){
					this.budgetUri = budgetlink.relativeHref;
					this._getResource(budgetlink.relativeHref, Constants.HANDLE_BUDGET);
				}
			}					
		},
		
		_handleBudget: function(loadedElement){
			if (loadedElement != null) {
				this.budget = loadedElement;
				this.intialize = true;
                this.refresh();
				Util.hideMessageArea();
				this._toggleRefreshButton(false);
			}					
		},
		
		_handleCodes: function(loadedElement){
			if(loadedElement != null){
				this.project = loadedElement;
				//Update with the latest values
				this._releaseAndSaveBudgets();
				this._destroyCodesGrid();
				this._createCodesWidget();
			}					
		},
		
		_releaseAndSaveBudgets: function(){
			//release the page lock
			Util._releasePage();
			//save the financial only after saving the project assocation codes
			this._financialDetailsGrid._saveFinancials();
		},
		
		_getResource: function(uri, type){
            var self = this;
            var generalFunctions = {
                success: function(loadedElement){
                    if (loadedElement) {
						if (type == Constants.HANDLE_PROJECT) {
							self._handleProject(loadedElement);
						}
						else if(type == Constants.HANDLE_BUDGET){
							self._handleBudget(loadedElement);
						}
						else if(type == Constants.HANDLE_REFRESH_CODES){
							self._handleCodes(loadedElement);
						}
                    }
                },
                failure: function(errorMsg){
                	Util.hideMessageArea();
                	self.intialize = true;
					EPMErrorHandler.setError(errorMsg.errorMessage, errorMsg.errorCode);
					// Set an interval and clear the error message
		            var intervalID = setInterval(function(){
		                EPMErrorHandler.clearError();
		                clearInterval(intervalID);
		            }, 10000);
					if(type == Constants.HANDLE_BUDGET){
						self.refresh();
						self._toggleRefreshButton(false);
					}
					else if(type == Constants.HANDLE_REFRESH_CODES){
						self._releaseAndSaveBudgets();
					}
					else if (self._financialDetailsGrid) {
		                self.refresh();
		            }
                }
            };
            
            var srh = new ServiceResponseHandler(generalFunctions, "success", "failure");
            gridModelClient.getURI(srh, uri);            
        },
		
		_getAccountHeads: function(){
            var self = this;
			var jsonExpressionVal = '{"attributeExpressions": [{"attributeId": "accountHead", "operator": "is", "values": [true], "variables": []}], "termExpressions": [], "similarityExpressions": [], "operator": "AND"}';
			var URI = "epm/query/";
            URI += "?" + "type" + "=" + "chargecodes&execute=true&jsonExpression=";
            URI += jsonExpressionVal;            
            URI += '&';
            
            var generalFunctions = {
                success: function(loadedElement){
                    if (loadedElement) {
						var chargecodes = loadedElement.QueryResult;
						var accountchargecodes = new Array();
						var accountchargecodeUris = new Array();
						for(var j in chargecodes){
							var tempChargecode = chargecodes[j];
							accountchargecodes.push(tempChargecode.name);
							accountchargecodeUris.push(tempChargecode.self.relativeHref);
						}
						self.accountHeads = accountchargecodes;
						self.accountHeadUris = accountchargecodeUris;
                    }
                },
                failure: function(errorMsg){
                    console.log(errorMsg);
					alert(errorMsg);
                }
            };
            
            var srh = new ServiceResponseHandler(generalFunctions, "success", "failure");
            gridModelClient.getQueryURI(srh, URI);            
        }
    });
})();
