/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.work.web.ui.internal.GanttDependency");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");
dojo.require("dijit.Tooltip");
dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");


/*
 * This is a widget for showing dependency
 */
(function(){

    var hLineWidth = 10;
    var border = 2;
    var rowHeight = 22;
    var OFFSET_TARGETNODE = 2;
    
    var Constants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    
    /* defineWidget */
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttDependency", [dijit._Widget, dijit._Templated], {
        templateString: '<div  tabindex="0" class="com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv" dojoAttachPoint="_outerDiv">' +
        '</div>',
        
        visible: false, //To store the visibility state of the gantt dependency
        type: null, //type of dependency
        source: null, //Source of dependency
        target: null, //Target of dependency			
        cachedSourceTop : -1,
        cachedSourceLeft : -1,
        cachedSourceWidth : -1,
        cachedSourceHeight : -1,
       	cachedTargetTop : -1,
        cachedTargetLeft : -1,
        cachedTargetWidth : -1,
        cachedTargetHeight : -1,
        		
        /**
         *  postCreate : Handle initialisation
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
            //Draw line for showing dependency			
            this._drawDependencyLine();
        },
        
		/**
		 * method to remove the dependency between two given nodes. <br>
		 */
		removeDependency: function(){
            while (this._outerDiv.firstChild) {
				this._outerDiv.removeChild(this._outerDiv.lastChild);
			}
		},
        /**
         *  redraw : destroy the child divs and redraw the dependency
         *  Incoming : none
         *  Outgoing : none
         *
         */
        redraw: function(){   
                  
            if (this.source && this.target) {
            	var source_visible = true;
                if (dojo.style(this.source.domNode, "left") == 0 && dojo.style(this.source.domNode, "top") == 0) {
                    source_visible = false;
                }
                
                var target_visible = true;
                if (dojo.style(this.target.domNode, "left") == 0 && dojo.style(this.target.domNode, "top") == 0) {
                    target_visible = false;
                }  
                  
	            if (source_visible && target_visible) {
	            	if(!(this.cachedSourceTop == dojo.style(this.source.domNode,"top") &&
						this.cachedSourceLeft == dojo.style(this.source.domNode,"left") &&            	 
						this.cachedSourceWidth == dojo.style(this.source.domNode,"width") &&            	 
						this.cachedSourceHeight == dojo.style(this.source.domNode,"height") &&            	 
						this.cachedTargetTop == dojo.style(this.target.domNode,"top") &&
						this.cachedTargetLeft == dojo.style(this.target.domNode,"left") &&            	 
						this.cachedTargetWidth == dojo.style(this.target.domNode,"width") &&            	 
						this.cachedTargetHeight == dojo.style(this.target.domNode,"height"))) {
						    //Delete previously drawn lines and arrow 
				            while (this._outerDiv.firstChild) {
				                this._outerDiv.removeChild(this._outerDiv.lastChild);
				            }   					
							this._drawDependencyLine();
					}
	            }
	            else {
					//Delete previously drawn lines and arrow 
		            while (this._outerDiv.firstChild) {
		                this._outerDiv.removeChild(this._outerDiv.lastChild);
		            }  	            
	            }
	            
            }
        },
        
        /**
         *  _drawDependencyLine : create dependency line according to the dependency type
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _drawDependencyLine: function(){
            if (!this.source || !this.target) {
                return;
            }
            if (this.source == this.target) {
                return;
            }
            
            //console.debug("_drawDependencyLine:" + this.source.index + ":" + this.target.index);
			this.cachedSourceTop = dojo.style(this.source.domNode,"top");
			this.cachedSourceLeft = dojo.style(this.source.domNode,"left");            	 
			this.cachedSourceWidth = dojo.style(this.source.domNode,"width");            	 
			this.cachedSourceHeight = dojo.style(this.source.domNode,"height");            	 
			this.cachedTargetTop = dojo.style(this.target.domNode,"top");
			this.cachedTargetLeft = dojo.style(this.target.domNode,"left");            	 
			this.cachedTargetWidth = dojo.style(this.target.domNode,"width");            	 
			this.cachedTargetHeight = dojo.style(this.target.domNode,"height");    
				            
            //Create outer div
            var oTop = -1;
            var oLeft = -1;
            var oWidth = -1;
            var oHeight = -1;
            
            if (this.type == Constants.FINISH_TO_START) {
            
               /* console.log("Source:: Left:"  +  dojo.style(this.source.domNode,"left") +
                 "Top:" + dojo.style(this.source.domNode,"top") +
                 "Width:" + dojo.style(this.source.domNode,"width") +
                 "Height:" + dojo.style(this.source.domNode,"height"));
                 console.log("Target:: Left:"  +  dojo.style(this.target.domNode,"left") +
                 "Top:" + dojo.style(this.target.domNode,"top") +
                 "Width:" + dojo.style(this.target.domNode,"width") +
                 "Height:" + dojo.style(this.target.domNode,"height"));*/
                if (dojo.style(this.source.domNode, "top") < dojo.style(this.target.domNode, "top")) {
                    oTop = dojo.style(this.source.domNode, "top") + (dojo.style(this.source.domNode, "height") / 2) + border / 2 + border;
                }
                else {
                    oTop = dojo.style(this.target.domNode, "top") + (dojo.style(this.target.domNode, "height") / 2) + border / 2 + border;
                }
                
                oLeft = dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left") ? dojo.style(this.target.domNode, "left") - hLineWidth + border / 2 : dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + border / 2;
                
                oWidth = dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left") ? ((dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + hLineWidth) -
                (dojo.style(this.target.domNode, "left") - hLineWidth)) : (dojo.style(this.target.domNode, "left") - ((dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width"))));
                
                oHeight = Math.abs(dojo.style(this.source.domNode, "top") - dojo.style(this.target.domNode, "top"));
                
                /*console.log("Dependency:: Left:"  +  oLeft  +
                
                
                 "Top:" + oTop +
                
                
                 "Width:" + oWidth+
                
                
                 "Height:" + oHeight);*/
                
                
            }
            
            
            //Set the position and size of the box			
            dojo.style(this._outerDiv, "top", oTop + 'px');
            dojo.style(this._outerDiv, "left", oLeft + 'px');
            dojo.style(this._outerDiv, "width", oWidth + 'px');
            dojo.style(this._outerDiv, "height", oHeight + 'px');
            
            
            //Add div for displaying lines
            if (this.type == Constants.FINISH_TO_START) {
                //LINE 1
                var line1 = document.createElement('DIV');
                dojo.addClass(line1, "gantt-Horizontal-Line-Div");
                
                if (dojo.style(this.source.domNode, "top") < dojo.style(this.target.domNode, "top")) {
                    dojo.style(line1, "top", 0 + 'px');
                }
                else {
                    dojo.style(line1, "top", dojo.style(this._outerDiv, "height") + 'px');
                }
                
                if (dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left")) {
                    dojo.style(line1, "left", (dojo.style(this._outerDiv, "width") - hLineWidth) + 'px');
                }
                else {
                    dojo.style(line1, "left", 0 + 'px');
                }
                
                dojo.style(line1, "width", hLineWidth + 'px');
                
                
                this._outerDiv.appendChild(line1);
                
                //LINE 2
                var line2 = document.createElement('DIV');
                dojo.addClass(line2, "gantt-Vertical-Line-Div");
                
                if (dojo.style(this.source.domNode, "top") < dojo.style(this.target.domNode, "top")) {
                    dojo.style(line2, "top", 0 + 'px');
                }
                else {
                    dojo.style(line2, "top", (dojo.style(this._outerDiv, "height") - rowHeight / 2) + 'px');
                }
                
                
                if (dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left")) {
                    dojo.style(line2, "left", dojo.style(this._outerDiv, "width") + 'px');
                }
                else {
                    dojo.style(line2, "left", hLineWidth + 'px');
                }
                
                dojo.style(line2, "height", (rowHeight / 2 + 1) + 'px');
                
                this._outerDiv.appendChild(line2);
                
                
                //LINE 3
                if (dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left")) {
                    var line31 = document.createElement('DIV');
                    
                    dojo.addClass(line31, "gantt-Horizontal-Line-Div");
                    dojo.style(line31, "width", dojo.style(this._outerDiv, "width") + 'px');
                    
                    if (dojo.style(this.source.domNode, "top") < dojo.style(this.target.domNode, "top")) {
                        dojo.style(line31, "top", (rowHeight / 2) + 'px');
                    }
                    else {
                    //tmp* , checking for negative values [ results pop ups in IE]
                        var tmpHeight1 = dojo.style(this._outerDiv, "height") - rowHeight / 2;
                        dojo.style(line31, "top", ((tmpHeight1 >= 0)? tmpHeight1 : 0 )+ 'px');
                    }
                    
                    dojo.style(line31, "left", 0 + 'px');
                    this._outerDiv.appendChild(line31);
                    
                    var line32 = document.createElement('DIV');
                    dojo.addClass(line32, "gantt-Vertical-Line-Div");
                    var tmpHeight2 = dojo.style(this._outerDiv, "height") - rowHeight;
                    dojo.style(line32, "height", ((tmpHeight2 >= 0)? tmpHeight2 : 0 )+ 'px');
                    
                    dojo.style(line32, "top", (rowHeight / 2) + 'px');
                    
                    dojo.style(line32, "left", 0 + 'px');
                    this._outerDiv.appendChild(line32);
                    
                }
                else {
                
                    var line3 = document.createElement('DIV');
                    dojo.addClass(line3, "gantt-Vertical-Line-Div");
                    var tmpHeight3 = dojo.style(this._outerDiv, "height") - rowHeight;
                    
                    dojo.style(line3, "height", ((tmpHeight3 >= 0)? tmpHeight3 : 0 )+ 'px');
                    dojo.style(line3, "top", (rowHeight / 2) + 'px');
                    
                    dojo.style(line3, "left", hLineWidth + 'px');
                    this._outerDiv.appendChild(line3);
                }
                
                
                //LINE 4
                var line4 = document.createElement('DIV');
                dojo.addClass(line4, "gantt-Vertical-Line-Div");
                
                if (dojo.style(this.source.domNode, "top") < dojo.style(this.target.domNode, "top")) {
                    dojo.style(line4, "top", (dojo.style(this._outerDiv, "height") - rowHeight / 2) + 'px');
                }
                else {
                    dojo.style(line4, "top", 0 + 'px');
                }
                
                
                if (dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left")) {
                    dojo.style(line4, "left", 0 + 'px');
                }
                else {
                    dojo.style(line4, "left", hLineWidth + 'px');
                }
                
                
                dojo.style(line4, "height", (rowHeight / 2) + 'px');
                
                this._outerDiv.appendChild(line4);
                
                //LINE 5
                var line5 = document.createElement('DIV');
                dojo.addClass(line5, "gantt-Horizontal-Line-Div");
                
                if (dojo.style(this.source.domNode, "top") < dojo.style(this.target.domNode, "top")) {
                    dojo.style(line5, "top", dojo.style(this._outerDiv, "height") + 'px');
                }
                else {
                    dojo.style(line5, "top", 0 + 'px');
                }
                
                if (dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left")) {
                    dojo.style(line5, "left", 0 + 'px');
                }
                else {
                    dojo.style(line5, "left", hLineWidth + 'px');
                }
                
                if (dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left")) {
                    dojo.style(line5, "width", hLineWidth - OFFSET_TARGETNODE + 'px');
                }
                else {
                    dojo.style(line5, "width", (dojo.style(this._outerDiv, "width") - hLineWidth - OFFSET_TARGETNODE) + 'px');
                }
                
                this._outerDiv.appendChild(line5);
                
                //ARROW
                var arrow = document.createElement('DIV');
                dojo.addClass(arrow, "gantt-Right-Arrow");
                
                if (dojo.style(this.source.domNode, "top") < dojo.style(this.target.domNode, "top")) {
                    dojo.style(arrow, "top", (dojo.style(this._outerDiv, "height") - 3) + 'px');
                } //Image height is 7
                else {
                    dojo.style(arrow, "top", (-3) + 'px');
                }
                
                if (dojo.style(this.source.domNode, "left") + dojo.style(this.source.domNode, "width") + 2 * hLineWidth > dojo.style(this.target.domNode, "left")) {
                    dojo.style(arrow, "left", (hLineWidth - 4 - OFFSET_TARGETNODE) + 'px');
                } //Image width is 4
                else {
                    dojo.style(arrow, "left", (dojo.style(this._outerDiv, "width") - 4 - OFFSET_TARGETNODE) + 'px');
                }
                
                this._outerDiv.appendChild(arrow);
                
            }
            
            
        }
    });
    
})();
