/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(
function() {
    dojo.provide("com.ibm.epm.work.web.ui.internal.GanttDependencyHandle");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttDependency");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttBar");
    dojo.require("com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub");
    
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var WBSGridGanttPubSub = com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub;
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttDependencyHandle", [dijit._Widget, dijit._Templated], {
        // summary: A dragable handle with rubber band effect of resize.
        // description:
        //	
        //
        
        listener: null,
        container: null,
        // sourceId: String
        //	
        sourceNode: null,
        
        // targetId: String
        //
        targetNode: null,
        
        templateString: '<div tabindex="0" dojoAttachPoint="resizeHandle" class="GanttDependencyHandle"><div></div></div>',
        
        /**
         *  postCreate : handle all the initialisation
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
            if (!this.listener) {
                alert("document is needed to create the gantt dependency widget");
            }
            if (!this.container) {
                alert("document is needed to create the gantt dependency widget");
            }
            this._pconnects = [];
            // summary: setup our one major listener upon creation
            this.mouseOverEvent = this.connect(this.listener, "onmouseover", "_onHover"); 
			this.mouseOutEvent = this.connect(this.listener, "onmouseout", "_onMouseOut");
            this.mouseDownEvent = this.connect(this.listener, "onmousedown", "_selectNodes");
            this.keyDownEvent = this.connect(this.listener, "onkeypress", "_drawdependency");
            this.keyBoardEvent = dojo.connect(document, "onkeypress", this, "_abortSizing");
        },
        
		/**
		 * onhover over the gantt main content
		 * @param {Object} e
		 */
		_onHover: function(e){
			this._semiHighlightBar(e, true);
		},

		/**
		 * * onmouse out from the gantt main content.
		 * @param {Object} e
		 */
		_onMouseOut: function(e){
			this._semiHighlightBar(e, false);
		},
		
		/**
		 * Highlight/de-highlight the gantt bar on mouseover/mousout when the dependency icon is checked.<br>
		 * @param {Object} event: Mousehover/mouseout event <br>
		 * @param {Object} bHighlight: true to highlight the gantt bar.
		 */
		_semiHighlightBar: function(event, bHighlight){
            var focussedNode = event.target;
			//Fetch the outer div if the mouse is hovered over the inner div and highlight it.
            if (dojo.hasClass(focussedNode, "com-ibm-epm-work-web-ui-internal-GanttBar-innerBarDiv")) {
				var semiHighlightNode = dijit.byId(focussedNode.parentNode.id);
				this._hoverNode = semiHighlightNode;
				if (!this.sourceNode) {
					this._semiHighLightPredecessorNode(semiHighlightNode, bHighlight);	
				}
                else {
                    this._semiHighLightSuccessorNode(semiHighlightNode, bHighlight);
				}
			}
			//If the mouse is hovered over the outer div, fetch the node and highlight.
			else if (dojo.hasClass(focussedNode, "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv")) {
				var semiHighlightNode = dijit.byId(focussedNode.id);
				this._hoverNode = semiHighlightNode;
				if (!this.sourceNode) {
					this._semiHighLightPredecessorNode(semiHighlightNode, bHighlight);
				}
            	else{
					this._semiHighLightSuccessorNode(semiHighlightNode, bHighlight);
            	}
			}
		},

		/**
		 * Change the border and the cursor if the mouse is hovered over the predecessor node.
		 * @param {Object} node: node on which mouse is hovered on. 
		 * @param {Object} bHighlight: True for hover, false for mouseout. <br>
		 */
		_semiHighLightPredecessorNode: function(node, bHighlight){
			if (node) {
				if (this._allowDependency(node)) {
					if (bHighlight) {
						dojo.addClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedPredecessor");
					}
					else 
						if (dojo.hasClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedPredecessor")) {
							dojo.removeClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedPredecessor");
						}
				}
			}
			
		},
		
		/**
		 * Change the border and the cursor if the mouse is hovered over the successor node.
		 * @param {Object} node: node on which mouse is hovered on. 
		 * @param {Object} bHighlight: True for hover, false for mouseout. <br>
		 */
		_semiHighLightSuccessorNode: function(node, bHighlight){
			if (this._allowDependency(node)) {
				if (bHighlight) {
					dojo.addClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedSuccessor");
				}
				else 
					if (dojo.hasClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedSuccessor")) {
						dojo.removeClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedSuccessor");
					}
			}
		},
		

			
		/**
		 * We will highlight the predecessor once it is selected. <br>
		 * @param {Object} node : node to be highlighted. <br>
		 * @param {Object} bHighlight: true to highlight the gantt bar. <br>
		 */
		_highLightSelectedNode: function(node, bHighlight){
			if (node) {
				if (bHighlight) {
					dojo.addClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-selectedPredecessor");
					var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/selectSuccessor.cur") + "'),pointer";
					dojo.style(this.dragPane.domNode,"cursor",cursorUrl);
				}
				else {
					if (dojo.hasClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-selectedPredecessor")) {
						dojo.removeClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-selectedPredecessor");
					}
					if (dojo.hasClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedPredecessor")) {
						dojo.removeClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedPredecessor")
					}
					if (dojo.hasClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedSuccessor")) {
						dojo.removeClass(node.domNode, "com-ibm-epm-work-web-ui-internal-GanttDependency-semiSelectedSuccessor")
					}
					//Change the dragPane as well as the node cursor to predecessor cursor.
					var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/selectPredeccessor.cur") + "'),pointer";
					dojo.style(this.dragPane.domNode,"cursor",cursorUrl);
					dojo.style(node.domNode,"cursor",cursorUrl);
				}
			}
		},

        /**
         *  _selectNodes : called on mouse down and selects the source and the target node
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _selectNodes: function(/*Event*/e){
            // summary: setup movement listeners and calculate initial size
            if ((this.sourceNode) && (this.targetNode)) {
                this.sourceNode = null;
                this.targetNode = null;
                dojo.forEach(this._pconnects, dojo.disconnect);
                this._pconnects = [];
                if (this.dependencyArrow) {
                    this.container.removeChild(this.dependencyArrow.domNode);
                }
            }
            var focussedNode = e.target;
            if (dojo.hasClass(focussedNode, "com-ibm-epm-work-web-ui-internal-GanttBar-innerBarDiv")) {
                if (!this.sourceNode) {
                    this.sourceNode = dijit.byId(focussedNode.parentNode.id);
                    if (this._allowDependency(this.sourceNode)) {
						
						this._highLightSelectedNode(this.sourceNode, true);
						
                        this.dependencyArrow = new com.ibm.epm.work.web.ui.internal.GanttDependency({
                            source: this.sourceNode,
                            target: this.targetNode,
                            type: WBSConstants.FINISH_TO_START
                        });
                        this.container.appendChild(this.dependencyArrow.domNode);
                    }
                    else {
                        this.sourceNode = null;
                    }
                }
                else {
                    this.targetNode = dijit.byId(focussedNode.parentNode.id);
                    if (!this._allowDependency(this.targetNode)) {
                        this.targetNode = null;
                    }
					//We have selected a target as well now, hence deselect both the bars.
					this._highLightSelectedNode(this.sourceNode, false);
					this._highLightSelectedNode(this.targetNode, false);
                }
            }
            if (dojo.hasClass(focussedNode, "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv")) {
				if (!this.sourceNode) {
					this.sourceNode = dijit.byId(focussedNode.id);
					if (this._allowDependency(this.sourceNode)) {
						
						this._highLightSelectedNode(this.sourceNode, true);
						
						this.dependencyArrow = new com.ibm.epm.work.web.ui.internal.GanttDependency({
							source: this.sourceNode,
							target: this.targetNode,
							type: WBSConstants.FINISH_TO_START
						});
						this.container.appendChild(this.dependencyArrow.domNode);
					}
					else {
						this.sourceNode = null;
					}
				}
				else {
					this.targetNode = dijit.byId(focussedNode.id);
					if (!this._allowDependency(this.targetNode)) {
						this.targetNode = null;
					}
					//We have selected a target as well now, hence deselect the dependency.
					this._highLightSelectedNode(this.sourceNode, false);
					this._highLightSelectedNode(this.targetNode, false);
				}
			}
            
            if ((this.sourceNode) && (this._pconnects.length === 0)) {
                this._pconnects.push(dojo.connect(this.listener, "onmousemove", this, "_updateSizing"));
				this._pconnects.push(dojo.connect(this.listener, "onkeypress", this, "_updateSizingOnkeypress"));
            }
            
            if (this.targetNode) {
                this._pconnects.push(dojo.connect(this.listener, "onmouseup", this, "_endSizing"));
            }else if(this.sourceNode){
				//listen to second mousedown.
				this._pconnects.push(dojo.connect(document, "onmousedown", this, "_removePreview"));
			}
            
            e.stopPropagation();
            e.preventDefault();
            dojo.stopEvent(e);
        },

		/**
		 * Method to remove the preview if the second mousedown is not on the ganttBar.
		 * @param {Object} e: mousedown event
		 */
		_removePreview: function(e){
            var focussedNode = e.target;
			//If the second mousedown is on the ganttBar, create the dependency.
            if (dojo.hasClass(focussedNode, "com-ibm-epm-work-web-ui-internal-GanttBar-innerBarDiv") ||
						dojo.hasClass(focussedNode, "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv"))
			{
				this._selectNodes();
				return;	
			}
			//Else remove the preview.
			if(this.dependencyArrow){
				this.container.removeChild(this.dependencyArrow.domNode);
				this.dependencyArrow = null;
			}
			//Remove the selection classes from source and target
			this.deselectSelectedNodes(this.sourceNode, this.targetNode);			
            this.sourceNode = null;
            this.targetNode = null;
			
            dojo.forEach(this._pconnects, dojo.disconnect);
            this._pconnects = [];
		},
        
		/**
		 * deselectSelectedNodes: deselect the selected nodes, if user cancels the preview. <br>
		 * @param {Object} source: source of the dependency previewed <br>
		 * @param {Object} target: destination of the dependency previewed <br>
		 */
		deselectSelectedNodes: function(source, target){
			this._highLightSelectedNode(source, false);
			this._highLightSelectedNode(target, false);
		},
		
        /**
         *  _updateSizing : after source node is selected draw the dependency line if the target node
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _updateSizing: function(/*Event*/e){
            if (e.target) {
                var hoverNode = null;
                if (dojo.hasClass(e.target, "com-ibm-epm-work-web-ui-internal-GanttBar-innerBarDiv")) {
                    hoverNode = e.target.parentNode;
                }
                else {
                    if (dojo.hasClass(e.target, "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv")) {
                        hoverNode = e.target;
                    }
                }
                if (hoverNode) {
                    hoverNode = dijit.byId(hoverNode.id);
                }
                
                if (this.dependencyArrow && hoverNode && (hoverNode != this.sourceNode)) {
                    if (this._allowDependency(hoverNode)) {
                        this.dependencyArrow.target = hoverNode;
                        this.dependencyArrow.redraw();
                    }
                }
            }
            e.stopPropagation();
            e.preventDefault();
            dojo.stopEvent(e);
        },
        
        /**
         *  _endSizing : after the sizing disconnect all mouse listeners
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _endSizing: function(/*Event*/e){
            // summary: disconnect listenrs and cleanup sizing
            dojo.forEach(this._pconnects, dojo.disconnect);
            this._pconnects = [];
            this.onResize(e);
            
            e.stopPropagation();
            e.preventDefault();
            dojo.stopEvent(e);
        },
        
        /**
         *  onResize : after the resize of the dependency is completed, delete the preview and publish the event for the dependency creation
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        onResize: function(e){
            if (this.dependencyArrow) {
                this.container.removeChild(this.dependencyArrow.domNode);
            }
            this.dependencyArrow = null;
            var data = {
                sourceId: this.sourceNode.ganttBarInfo.id,
                targetId: this.targetNode.ganttBarInfo.id,
				controllerID: com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants.ScheduleGridController
            };
 
            // summary: Stub fired when sizing is done, for things like Grid
            if (data.sourceId != data.targetId) {
                data.uniqueIdentifier = {
                    treeID: WBSConstants.WBS_TREE_ID
                };
                dojo.publish(WBSGridGanttPubSub.ON_CREATE_DEPENDS, [data]);
            }
            
            this.sourceNode = null;
            this.targetNode = null;
        },
        
        /**
         *  destroyWidget : destructor
         *  Incoming : none
         *  Outgoing : none
         *
         */
        destroyWidget: function(){
            if (this.dependencyArrow) {
                this.container.removeChild(this.dependencyArrow.domNode);
            }
			if(this.mouseOverEvent){
				this.disconnect(this.mouseOverEvent);
			}
			if(this.mouseOutEvent){
				this.disconnect(this.mouseOutEvent);
			}
            if (this.mouseDownEvent) {
                this.disconnect(this.mouseDownEvent);
            }
            if (this.keyDownEvent) {
                this.disconnect(this.keyDownEvent);
            }
            if (this.keyBoardEvent) {
                dojo.disconnect(this.keyBoardEvent);
            }
            dojo.forEach(this._pconnects, dojo.disconnect);
            this._pconnects = [];
        },
        
        /**
         *  _allowDependency : All logic of allowing dependency creation between nodes
         *  Incoming : gantt Widget
         *  Outgoing : Boolean ( Valid = True, Invalid = False )
         *
         */
        _allowDependency: function(ganttBar){
            if (ganttBar) {
                if (ganttBar.ganttBarInfo.workItemType) {
                    if (ganttBar.ganttBarInfo.workItemType == WBSConstants.EPM_TASK_TYPE ||
                    ganttBar.ganttBarInfo.workItemType == "task") {
                        return true;
                    }
                }
            }
            return false;
        },
        
        /**
         *  _abortSizing : abort the sizing if user presses Escape
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _abortSizing: function(/*Event*/e){
            var key = e.keyCode || e.charCode;
            var k = dojo.keys;
            
            if (key == k.ESCAPE) {
                if (this.dependencyArrow) {
                    this.container.removeChild(this.dependencyArrow.domNode);
                    this.dependencyArrow = null;
                }
				//Deselect the source and target nodes.
				if (!this.targetNode && this._hoverNode) {
					this.deselectSelectedNodes(this.sourceNode, this._hoverNode);
				}
				else {
					this.deselectSelectedNodes(this.sourceNode, this.targetNode);
				}	
                this.sourceNode = null;
                this.targetNode = null;
            }			
        },
		/* Selecting source Gantt bar on keypress */
		_drawdependency : function(/*Event*/e){           
			if(e.keyCode === 32 || e.keyCode === 13){					
				this._selectNodes(e);							
			}
		},
		/* Creating dependency on keypress on target gantt bar*/
		_updateSizingOnkeypress : function(/*Event*/e){           
			if(e.keyCode === 32 || e.keyCode === 13){					
				this._updateSizing(e);
				this._endSizing(e);							
			}
		}
		
        
    });
})();
