/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.GanttRowBackground");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    
    /* defineWidget */
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttRowBackground", [dijit._Widget], {
        rowFlag: true,
        container: null,
        
        /**
         *  postCreate : initialisation
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
            if (!this.container) {
                alert("Row background widget needs a container");
            }
            this.rowWidgetArray = new Array();
        },
        
        /**
         *  addRow : Add a row to the document with either colored or plain style
         *  Incoming : none
         *  Outgoing : none
         *
         */
        addRow: function(){
            var rowDiv = document.createElement('DIV');
            if (this.rowFlag) {
                dojo.addClass(rowDiv, "com-ibm-epm-work-web-ui-internal-GanttRowBar-plainDiv");
            }
            else {
                dojo.addClass(rowDiv, "com-ibm-epm-work-web-ui-internal-GanttRowBar-colorDiv");
            }
            this.container.appendChild(rowDiv);
            
            this.rowWidgetArray.push(rowDiv);
            this.rowFlag = !this.rowFlag;
        },
        
        /**
         *  removeRows : remove a bunch of rows from the document
         *  Incoming : none
         *  Outgoing : none
         *
         */
        removeRows: function(numberOfRowsToRemove){
            for (var i = 0; i < numberOfRowsToRemove; i++) {
                var rowDiv = this.rowWidgetArray.pop();
                this.container.removeChild(rowDiv);
                this.rowFlag = !this.rowFlag;
            }
        },
        
        /**
         *  highlight : highlight/unhighlight the row by adding/removing a css class
         *  Incoming : rowIndex, highlight or unhighlight flag
         *  Outgoing : none
         *
         */
        highlight: function(index, /*hightlight boolean*/ flag){
            if ((index >= 0) && (index < this.rowWidgetArray.length)) {
                var row = this.rowWidgetArray[index];
                if (flag) {
                    dojo.addClass(row, "com-ibm-epm-work-web-ui-internal-GanttRowBar-highlightDiv");
                }
                else {
                    dojo.removeClass(row, "com-ibm-epm-work-web-ui-internal-GanttRowBar-highlightDiv");
                }
            }
        },
        
        /**
         *  totalRows : returns the total number of background rows
         *  Incoming : none
         *  Outgoing : total number of background rows
         *
         */
        totalRows: function(){
            return this.rowWidgetArray.length;
        }
    });
    
})();
