/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.GanttTimeBlock");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojo.date");
    // Localization
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    var WMMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    //CONSTANTS CAN BE DEFINED HERE
    var month_array = [WMMessages.MONTH_SHORTNAME_JANUARY, WMMessages.MONTH_SHORTNAME_FEBRUARY, WMMessages.MONTH_SHORTNAME_MARCH, WMMessages.MONTH_SHORTNAME_APRIL, WMMessages.MONTH_SHORTNAME_MAY, WMMessages.MONTH_SHORTNAME_JUNE, WMMessages.MONTH_SHORTNAME_JULY, WMMessages.MONTH_SHORTNAME_AUGUST, WMMessages.MONTH_SHORTNAME_SEPTEMBER, WMMessages.MONTH_SHORTNAME_OCTOBER, WMMessages.MONTH_SHORTNAME_NOVEMBER, WMMessages.MONTH_SHORTNAME_DECEMBER];
    
    var quarter_array = [WMMessages.QUARTER_ONE, WMMessages.QUARTER_TWO, WMMessages.QUARTER_THREE, WMMessages.QUARTER_FOUR];
    
    var week_array = [WMMessages.WEEKDAY_SHORTNAME_MONDAY, WMMessages.WEEKDAY_SHORTNAME_TUESDAY, WMMessages.WEEKDAY_SHORTNAME_WEDNESDAY, WMMessages.WEEKDAY_SHORTNAME_THURSDAY, WMMessages.WEEKDAY_SHORTNAME_FRIDAY, WMMessages.WEEKDAY_SHORTNAME_SATURDAY, WMMessages.WEEKDAY_SHORTNAME_SUNDAY];
    
    /*
     * This is a widget to create a timeblock
     */
    /* defineWidget */
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttTimeBlock", [dijit._Widget, dijit._Templated], {
        templateString: '<div class="com-ibm-epm-work-web-ui-internal-GanttTimeBlock">' +
        '<div class="com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_topRow" dojoAttachPoint="_timeline_topRow">' +
        '</div>' +
        '<div class="com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_bottomRow" dojoAttachPoint="_timeline_bottomRow">' +
        '</div>' +
        '</div>',
        
        //
        // summary: 
        //
        // description:
        // properties: comma separated
        blockStartDate: null, /*Date*/
        dayWidth: 0, //This is passed during the construction
        blockWidth: 0, //This is computed and cached ( should not be passed as argument )
        /**
         *  postCreate : checks if the constructor was passed with correct initial values
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
            if ((this.dayWidth <= 0) || (!this.blockStartDate)) {
                alert("Creation problem in GanttTimeBlock, cannot have zero duration");
            }
        },
        
        /**
         *  getBlockWidth : return the current timeblock width
         *  Incoming : none
         *  Outgoing : blockwidth ( Integer )
         *
         */
        getBlockWidth: function(){
            return this.blockWidth;
        },
        
        /**
         *  createYearBlock : one year block is created in the top row and 12 months are created in the bottom row
         *  Incoming : none
         *  Outgoing : none
         *
         */
        createYearBlock: function(){
            var num_Months = 12;
            var yearWidth = 0;
            
            //Create the months for the year
            for (var i = 0; i < num_Months; i++) {
                //construct a date to denote the first day of the month of that year
                var daysInMonth = dojo.date.getDaysInMonth(new Date(this.blockStartDate.getFullYear() + Math.floor(i / 12), (i % 12), 1));
                yearWidth += daysInMonth * this.dayWidth;
                var monthWidth = daysInMonth * this.dayWidth - 1;
                
                var mon_cell = document.createElement('DIV');
                dojo.addClass(mon_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_bottomRow_cell');
                dojo.style(mon_cell, "width", monthWidth + 'px');
                //Display the name of the month e.g., Jan..Dec
                var text = document.createTextNode(month_array[(i % 12)]);
                mon_cell.appendChild(text);
                this._timeline_bottomRow.appendChild(mon_cell);
            }
            
            //Create the year
            var year_cell = document.createElement('DIV');
            dojo.addClass(year_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_topRow_cell');
            dojo.style(year_cell, "width", (yearWidth - 1) + 'px');
            //Display the year e.g., 2008
            var text1 = document.createTextNode(this.blockStartDate.getFullYear());
            year_cell.appendChild(text1);
            this._timeline_topRow.appendChild(year_cell);
            
            //Apply the width to the containers
            dojo.style(this._timeline_topRow, 'width', yearWidth + 'px');
            dojo.style(this._timeline_bottomRow, 'width', yearWidth + 'px');
            
            this.blockWidth = yearWidth + 1;
        },
        
        /**
         *  createQuarterBlock : one quarter block is created in the top row and 3 months are created in the bottom row
         *  Incoming : none
         *  Outgoing : none
         *
         */
        createQuarterBlock: function(quarterIndex){ //quarter index will be 0,1,2,3 for Q1 Q2 Q3 Q4 respectively
            var num_Months = 3;
            var quarterWidth = 0;
            //Create the months for the quarter
            for (var i = 0; i < num_Months; i++) {
                //construct a date to denote the first day of the month of that year
                var monthIndex = quarterIndex * 3 + i;
                var daysInMonth = dojo.date.getDaysInMonth(new Date(this.blockStartDate.getFullYear(), monthIndex, 1));
                quarterWidth += daysInMonth * this.dayWidth;
                var monthWidth = daysInMonth * this.dayWidth - 1;
                
                var mon_cell = document.createElement('DIV');
                dojo.addClass(mon_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_bottomRow_cell');
                dojo.style(mon_cell, "width", monthWidth + 'px');
                //Display the month name e.g., Jan..Dec
                var text = document.createTextNode(month_array[monthIndex]);
                mon_cell.appendChild(text);
                this._timeline_bottomRow.appendChild(mon_cell);
                
            }
            
            //Create the quarter
            var quarter_cell = document.createElement('DIV');
            dojo.addClass(quarter_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_topRow_cell');
            dojo.style(quarter_cell, "width", (quarterWidth - 1) + 'px');
            //Display the name of the quarter with the year e.g., Q1 2008
            var text1 = document.createTextNode(quarter_array[quarterIndex] + ' ' + this.blockStartDate.getFullYear());
            quarter_cell.appendChild(text1);
            this._timeline_topRow.appendChild(quarter_cell);
            
            //Apply the width to the containers		
            dojo.style(this._timeline_topRow, 'width', quarterWidth + 'px');
            dojo.style(this._timeline_bottomRow, 'width', quarterWidth + 'px');
            
            this.blockWidth = quarterWidth + 1;
        },
        
        /**
         *  createMonthBlock : one month block is created in the top row and as many days in the month are created in the bottom row
         *  Incoming : none
         *  Outgoing : none
         *
         */
        createMonthBlock: function(monthIndex){ //month index will be 0..11
            var daysInMonth = dojo.date.getDaysInMonth(this.blockStartDate);
            
            //Create the days of the month
            for (var i = 0; i < daysInMonth; i++) {
                var day_Width = this.dayWidth - 1;
                
                var day_cell = document.createElement('DIV');
                dojo.addClass(day_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_bottomRow_cell');
                dojo.style(day_cell, "width", day_Width + 'px');
                //Display the date e.g., 1..31
                var text = document.createTextNode(i + 1);
                day_cell.appendChild(text);
                this._timeline_bottomRow.appendChild(day_cell);
            }
            
            //Create the month
            var monthWidth = daysInMonth * this.dayWidth;
            var month_cell = document.createElement('DIV');
            dojo.addClass(month_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_topRow_cell');
            dojo.style(month_cell, "width", (monthWidth - 1) + 'px');
            //Display the name of the month with the year e.g., Sep 2008
            var text1 = document.createTextNode(month_array[monthIndex] + ' ' + this.blockStartDate.getFullYear());
            month_cell.appendChild(text1);
            this._timeline_topRow.appendChild(month_cell);
            
            //Apply the width to the containers
            dojo.style(this._timeline_topRow, 'width', monthWidth + 'px');
            dojo.style(this._timeline_bottomRow, 'width', monthWidth + 'px');
            
            this.blockWidth = monthWidth + 1;
        },
        
        /**
         *  createWeekBlock : one week block is created in the top row and 7 days in the week are created in the bottom row
         *  Incoming : none
         *  Outgoing : none
         *
         */
        createWeekBlock: function(){
            var num_DaysInWeek = 7;
            var week_Width = num_DaysInWeek * this.dayWidth;
            var dayWidth = this.dayWidth - 1;
            
            //Create the days for this week
            for (var i = 0; i < num_DaysInWeek; i++) { //create days for this week
                var day_cell = document.createElement('DIV');
                dojo.addClass(day_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_bottomRow_cell');
                dojo.style(day_cell, "width", dayWidth + 'px');
                //Display the week day e.g., S ( Sunday ) ..
                var text = document.createTextNode(week_array[i]);
                day_cell.appendChild(text);
                this._timeline_bottomRow.appendChild(day_cell);
            }
            
            //Create the week	
            var week_cell = document.createElement('DIV');
            dojo.addClass(week_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_topRow_cell');
            dojo.addClass(week_cell, 'com-ibm-epm-work-web-ui-internal-GanttTimeBlock-timeline_topRow_cell_left_aligned');
            dojo.style(week_cell, "width", (week_Width - 1) + 'px');
            //Display the start date of the week e.g., Sep 21, 2008
            var text1 = document.createTextNode(month_array[this.blockStartDate.getMonth()] + ' ' + this.blockStartDate.getDate() + ',' + this.blockStartDate.getFullYear());
            week_cell.appendChild(text1);
            this._timeline_topRow.appendChild(week_cell);
            
            //Apply the width to the containers
            dojo.style(this._timeline_topRow, 'width', week_Width + 'px');
            dojo.style(this._timeline_bottomRow, 'width', week_Width + 'px');
            
            this.blockWidth = week_Width + 1;
        },
        
        /**
         *  destroy : Destructor
         *  		This function removes the timeline rows. This is called before switching to any other timeline
         *  Incoming : none
         *  Outgoing : none
         *
         */
        destroy: function(){
            while (this._timeline_topRow.firstChild) {
                this._timeline_topRow.removeChild(this._timeline_topRow.lastChild);
            }
            while (this._timeline_bottomRow.firstChild) {
                this._timeline_bottomRow.removeChild(this._timeline_bottomRow.lastChild);
            }
            this.inherited(arguments);
        }
        
    });
    
})();
