/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.risk.web.ui.internal.RiskStrategies");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGrid");
    dojo.require("com.ibm.epm.web.core.ui.internal.DojoData");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskUtil");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
	dojo.require("com.ibm.epm.risk.web.ui.internal.RiskStrategiesGridView");
	dojo.require("com.ibm.epm.risk.web.ui.internal.RiskActionsGridView");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskGridView");
	dojo.require("dijit.form.Textarea");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.risk.web.ui", "Messages");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskConstants");
    
    var RiskConstants = com.ibm.epm.risk.web.ui.internal.RiskConstants;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var dateUtil = com.ibm.epm.risk.web.ui.internal.RiskUtil;
	var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
	var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants; 
    var RiskMessages = dojo.i18n.getLocalization("com.ibm.epm.risk.web.ui", "Messages");
    var rownode = null;
	var strevent = null;
	var actevent = null;
	
    dojo.declare("com.ibm.epm.risk.web.ui.internal.RiskStrategies", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.risk.web", "ui/internal/templates/RiskStrategies.html"),
        wbsNodeType: "",
        wbsNodeName: "",
		selectedStrategy: null,
        
        
        
        /**
         *  postCreate : postCreate for the viewlet.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        postCreate: function(){
			this.rownode = null;
            this._createRiskPlansToolbar();
            this._createRiskStrategiesGrid();
			this._createRiskActionsToolbar();
			this._createRiskActionsGrid();
			this._createPlanDescription();
            this.riskMatrixCellClicked = false;
			this.strevent = dojo.subscribe(RiskMessages.RISKUPDATE, this, "UpdateRiskStrategies");
			this.actevent = dojo.subscribe(RiskConstants.RISK_STRATEGIES_GRID_ROW_CLICK, this, "UpdateRiskActions");
        },
        
        
        /**
         *  UpdateRiskStrategies : This function is to populate various attributes
         *  Incoming Arguments : Data from server to populate the fields.
         *  Outgoing Argument : none
         *
         */
        
		UpdateRiskStrategies: function(data){
			if (this.RiskPlanGrid && data && this.typeSelect) {
				/* if(this.rownode && this.rownode.id == data.id)
				{
					return;
				} */
				this.rownode = data;
				this.RiskPlanGrid.UpdateStrategies(data, this.typeSelect.selectedIndex);
			}
        },
		
		 /**
         *  UpdateRiskActions : This function is to populate various attributes
         *  Incoming Arguments : Data from server to populate the fields.
         *  Outgoing Argument : none
         *
         */
		UpdateRiskActions: function(data){
			if (this.RiskActionsGrid) {
               this.RiskActionsGrid.UpdateActions(data, this.rownode);
            }
			this.selectedStrategy = data;
			//Update the description box
			var description = "";
			if(data && data.description)
			{
				description = data.description;
			}
			this.StrategyDescriptionEditor.attr('value',description);
        },
		
		/**
         *  _createRiskStrategiesGrid : This function is for creating Risk Strategies Grid.
         *  Incoming Arguments : Uri.
         *  Outgoing Argument : none
         */
        _createRiskStrategiesGrid: function(link){
			this.RiskPlanGrid = new com.ibm.epm.risk.web.ui.internal.RiskStrategiesGridView({
                containerViewlet: this.containerViewlet
            });
            this.RiskPlanGrid.buildRiskStrategiesViewGrid();
            this.RiskPlanGrid._rowIndexSelected = 0;
			this._riskPlansGrid.appendChild(this.RiskPlanGrid.domNode);
        },
		
		/**
         *  _createRiskActionsGrid : This function is for creating Risk actions Grid.
         *  Incoming Arguments : Uri.
         *  Outgoing Argument : none
         */
        _createRiskActionsGrid: function(link){
			this.RiskActionsGrid = new com.ibm.epm.risk.web.ui.internal.RiskActionsGridView({
                containerViewlet: this.containerViewlet
            });
            this.RiskActionsGrid.buildRiskActionsGridView();
            this.RiskActionsGrid._rowIndexSelected = 0;
			this._riskActionsGrid.appendChild(this.RiskActionsGrid.domNode);
        },
              
        /**
         *  _createDescription : This function is to create the Description Text, Edit button  and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        _createPlanDescription: function(){
			this.descriptionLabel = document.createElement("label");
            this.descriptionLabel.innerHTML = RiskMessages.DESCRIPTION ;
			this._strategyDescriptionToolBar = new EPMToolbar({
                leftContentList: [this.descriptionLabel]
            });            
            
            dojo.addClass(this._strategyDescriptionToolBar.domNode, RiskConstants.TRANSPARENT_TOOLBAR_CLASS);
			this._strategyDescriptionToolBar.domNode.title = RiskMessages.STRATEGY_DESCRIPTION_OF + RiskMessages.DESCRIPTION_OF_MITIGATION_TITLE;
            this._strategyDescLabel.appendChild(this._strategyDescriptionToolBar.domNode);			

			var self = this;
            self.StrategyDescriptionEditor = new dijit.form.Textarea({
                onBlur: function(value){
                    if (self.rownode !== null) {
                        self.updateStartegyDescriptionInModel(this.value);
                    }
                },
                disabled: false,
				intermediateChanges: true
            });
			this._strategyDescEditor.appendChild(this.StrategyDescriptionEditor.domNode);
        },
		
		updateStartegyDescriptionInModel:function(value)
		{
			if (this.selectedStrategy) {
				this.RiskPlanGrid.updateStartegyDescriptionInModel(this.selectedStrategy.id, value);
			}
		},
		
        /**
         *  destroy : Destroy function for viewlet.
         *  Incoming Arguments : none.
         *  Outgoing Argument : none
         */
        destroy: function(){
			if(this.actevent)
			{
				dojo.unsubscribe(this.actevent);
			}
			if(this.strevent)
			{
				dojo.unsubscribe(this.strevent);
			}
			if (this.RiskPlanGrid) {
                this.RiskPlanGrid.destroy();
                delete this.RiskPlanGrid;
            } 
			if (this.RiskActionsGrid) {
                this.RiskActionsGrid.destroy();
                delete this.RiskActionsGrid;
            }          
        },
		
		/**
         *  _createBasicToolbar : This function is create toolbar.
         *  Incoming Arguments : none.
         *  Outgoing Argument : none
         *
         */
		_createRiskPlansToolbar: function(){ 
		
            this.typeSelect = document.createElement("select");
            this.typeSelect.options[0] = new Option(RiskMessages.RISK_MITIGATION_STRATEGIES);
			this.typeSelect.options[1] = new Option(RiskMessages.RISK_CONTINGENCY_PLANS);
			
			this.typeSelect.title = RiskMessages.SELECT_STRATEGY_TYPE_TITLE;
			this.typeSelect.options[0].title = RiskMessages.SELECT_MITIGATION_STRATEGIES_TITLE;
			this.typeSelect.options[1].title = RiskMessages.SELECT_CONTINGENCY_PLANS_TITLE;
			
			
            dojo.style(this.typeSelect, "width", "15em");
			dojo.style(this.typeSelect, "horizontal-align", "left");		
			dojo.connect(this.typeSelect, "onchange", dojo.hitch(this, this._onTypeChange)); 
			
			this.addButton = new dijit.form.Button({
                title: RiskMessages.ADD_STRATEGY_TITLE + RiskMessages.DESCRIPTION_OF_MITIGATION_TITLE,
				label : RiskMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS,
				disabled: true,
                onClick: dojo.hitch(this, this._addRiskStrategy)
            }); 
			
			this.refreshButton = new dijit.form.Button({
                title: RiskMessages.REFRESH_STRATEGY_TITLE,
				label : RiskMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS,
				disabled: true,
                onClick: dojo.hitch(this, this._refreshRiskStrategy)
            });
            
			var rightContentList = [this.addButton,this.refreshButton];
            var leftContentList = [this.typeSelect];
            this._riskStrategiesToolBar = new EPMToolbar({
                rightContentList: rightContentList,
				leftContentList: leftContentList
            });            
            
            dojo.addClass(this._riskStrategiesToolBar.domNode, RiskConstants.TRANSPARENT_TOOLBAR_CLASS);
            this._riskPlansBar.appendChild(this._riskStrategiesToolBar.domNode);			
        },
		
		toggleAddButton: function(state){
			var oldState = this.addButton.attr('disabled');
            if (oldState != state) {
                this.addButton.attr('disabled',state);
				this.refreshButton.attr('disabled',state);
				this.StrategyDescriptionEditor.attr('readOnly',state);
				if (state) {
					this.RiskPlanGrid.removeAllRows();
					this.RiskActionsGrid.removeAllRows();
					this.StrategyDescriptionEditor.attr('value',"");
					
					this.addActionButton.attr('disabled', state);
					this.refreshActionButton.attr('disabled', state);
					this.addButton.attr('iconClass', IconClassConstants.EPM_ADD_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
					this.refreshButton.attr('iconClass', IconClassConstants.EPM_REFRESH_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
					this.addActionButton.attr('iconClass', IconClassConstants.EPM_ADD_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
					this.refreshActionButton.attr('iconClass', IconClassConstants.EPM_REFRESH_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
				}
				else {
					this.addButton.attr('iconClass', IconClassConstants.EPM_ADD_ICON_CLASS);
					this.refreshButton.attr('iconClass', IconClassConstants.EPM_REFRESH_ICON_CLASS);
				}
            }
        },
		toggleButtons: function(state){
				this.RiskPlanGrid.rowSelectionChanged(state);
				
				this.addActionButton.attr('disabled',state);
				this.refreshActionButton.attr('disabled',state);
				this.StrategyDescriptionEditor.attr('readOnly',state);
                if (state) {
					this.RiskActionsGrid.removeAllRows();
					this.StrategyDescriptionEditor.attr('value',"");
					this.addActionButton.attr('iconClass', IconClassConstants.EPM_ADD_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
					this.refreshActionButton.attr('iconClass', IconClassConstants.EPM_REFRESH_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
				}
				else {
					this.addActionButton.attr('iconClass', IconClassConstants.EPM_ADD_ICON_CLASS);
					this.refreshActionButton.attr('iconClass', IconClassConstants.EPM_REFRESH_ICON_CLASS);
				}
        },
		
		/**
         * Adds a risk strategy in the grid
         */
		_addRiskStrategy:function(){
			if( this.RiskPlanGrid){
				this.RiskPlanGrid.addRiskStrategy();
				this.containerViewlet.toggleSaveButton(false);								
			}
		},
		
		/**
         * Deletes the selected risk strategy
         */
        _deleteRiskStrategy: function(){
			if( this.RiskPlanGrid){
				 this.RiskPlanGrid.deleteRiskStrategy();				
			}
        },
		
		/**
         *  _refreshRiskStrategy : Refreshes the startegy grid
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
		_refreshRiskStrategy:function(){
			if( this.RiskPlanGrid){
				 this.RiskPlanGrid.refreshRiskStrategy();				
			}
		},
		
		/**
         *  _onTypeChange : This function is to create the Precision Combobox and label.
         *  Incoming Arguments : type value
         *  Outgoing Argument : none
         *
         */
        _onTypeChange: function(value){
			if (this.RiskPlanGrid) {
				this.RiskPlanGrid.UpdateStrategies(this.rownode, value.target.selectedIndex);
				if (value.target.selectedIndex === RiskConstants.TYPE_MITIGATION) {
					this._strategyDescriptionToolBar.domNode.title = RiskMessages.STRATEGY_DESCRIPTION_OF + RiskMessages.DESCRIPTION_OF_MITIGATION_TITLE;
					this.addButton.domNode.title = RiskMessages.ADD_STRATEGY_TITLE + RiskMessages.DESCRIPTION_OF_MITIGATION_TITLE;
				}
				else {
					this._strategyDescriptionToolBar.domNode.title = RiskMessages.STRATEGY_DESCRIPTION_OF + RiskMessages.DESCRIPTION_OF_CONTINGENCY_TITLE;
					this.addButton.domNode.title = RiskMessages.ADD_STRATEGY_TITLE + RiskMessages.DESCRIPTION_OF_CONTINGENCY_TITLE;
				}
			}
        },
        
        /**
         *  _createRiskActionsToolbar : Creates the tool bar from the actions grid.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
		_createRiskActionsToolbar: function(){ 
		
			this.actionLabel = document.createElement("label");
            this.actionLabel.innerHTML = RiskMessages.ACTIONS ;
            var leftContentList = [this.actionLabel];
            
            
			
			this.addActionButton = new dijit.form.Button({
                title: RiskMessages.ADD_ACTIONS_TITLE,
				label : RiskMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS,
				disabled: true,
                onClick: dojo.hitch(this, this._addRiskAction)
            }); 
			
            this.refreshActionButton = new dijit.form.Button({
                title: RiskMessages.REFRESH_ACTIONS_TITLE,
				label : RiskMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS,
				disabled: true,
                onClick: dojo.hitch(this, this._refreshRiskAction)
            });
            
			var rightContentList = [this.addActionButton,this.refreshActionButton];
            
            this._riskActionsToolBar = new EPMToolbar({
				leftContentList: leftContentList,
                rightContentList: rightContentList
            });            
            this._riskActionsToolBar.domNode.title = RiskMessages.RISK_ACTIONS_GRID_TITLE;
            dojo.addClass(this._riskActionsToolBar.domNode, RiskConstants.TRANSPARENT_TOOLBAR_CLASS);
            this._riskActionsBar.appendChild(this._riskActionsToolBar.domNode);			
        },
		
		/**
         * Adds a risk action in the grid
         */
		_addRiskAction:function(){
			if( this.RiskActionsGrid){
				this.RiskActionsGrid.addRiskAction();
				this.containerViewlet.toggleSaveButton(false);				
			}
		},
		
		/**
         * Deletes the selected risk strategy
         */
        _deleteRiskAction: function(){
			if( this.RiskActionsGrid){
				 this.RiskActionsGrid.deleteRiskAction();				
			}
        },
		
		/**
         *  _refreshRiskAction : Refreshes the startegy grid
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
		_refreshRiskAction:function(){
			if( this.RiskActionsGrid){
				 this.RiskActionsGrid.refreshRiskAction();				
			}
		}
    });
})();
