/**
 * @author paul
 */
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.risk.web.ui.internal.RiskStrategiesGridView");
	dojo.require("com.ibm.epm.web.core.grid.controller.EPMRiskStrategiesGridController");
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskStrategiesGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskConstants");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	// Localization
    dojo.requireLocalization("com.ibm.epm.risk.web.ui", "Messages");
    
    var ControllersFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var RiskMessages = dojo.i18n.getLocalization("com.ibm.epm.risk.web.ui", "Messages");
    var RiskConstants = com.ibm.epm.risk.web.ui.internal.RiskConstants;
	var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
	var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
	var strategyType = RiskConstants.TYPE_MITIGATION;
	 
    dojo.declare("com.ibm.epm.risk.web.ui.internal.RiskStrategiesGridView", abstractView, {
        templateString: '<div class="com-ibm-epm-risk-web-ui-internal-RiskStrategiesGridView" dojoAttachPoint="_RiskStrategiesViewAttachPoint"></div>',
        
		/* 
         * buildRatesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with RiskGrid.
         */
        buildRiskStrategiesViewGrid: function(){
			this._rowHoverArray = [];
        	this._gridID=RiskConstants.RISK_STRATEGIES;
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.risk.web.ui.internal.RiskStrategiesGridModel({_containingGrid: this});
            }
			if (!this._controller) {
				this._controller = ControllersFactory.getEPMRiskStrategiesGridController(RiskConstants.RISK_STRATEGIES_GRID_CONTROLLER);
			}
			this.buildGrid(this, this._gridModel, this._controller);
        },

        /*
         * This is name the Grid.
         */
        getGridTitle: function(){
            return RiskMessages.RISK_STRATEGIES_GRID_TITLE;
        },
        
         /*
         * This is unique ID of the Grid.
         */
        getGridInternalID: function(){
            return RiskConstants.RISK_STRATEGIES;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this._RiskStrategiesViewAttachPoint;
        },
        
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var RiskStrategiesViewGridLayout = [{
                noscroll: false,
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [[{
                    name: RiskMessages.STRATEGIES,
                    width: 'auto',
                    field: RiskConstants.RISK_STRATEGY_FIELD_SUMMARY,
                    editor: dojox.grid.editors.Input,
					title: RiskMessages.RISK_STRATEGIES_GRID_TITLE
                },
				{
	                name: " ",
	                width: '1em',
	                field: RiskConstants.RISK_STRATEGY_FIELD_ACTION,
					title: RiskMessages.REMOVE_STRATEGY_TITLE,
					value: ""
                }]]
            }];
            
            return RiskStrategiesViewGridLayout;
        },
		onCellClick: function(event){
			this._onClick(event);
		},
		 _onClick: function(event){
            if (event && event.cell.field == "action"  && event.target.name == "deleteImage") {
				this.deleteRiskStrategy(event);
			}
            event.stopPropagation();
            event.preventDefault();
            dojo.stopEvent(event);
        },
		
		        /*
         * Private Method
         * _MouseOut: Handle the on mouse over event on the grid row and add the action images to action column
         * Input: event e
         * outPut: None
         */
        _onMouseOver: function(e){
            if (!this._generalGrid || (e.rowIndex < 0) || (this._generalGrid.edit && this._generalGrid.edit.isEditing())
			|| (e.gridID != RiskConstants.RISK_STRATEGIES) ) {
				return;		//Donot process the row if an editor is open
			}
			this._clearAllActionRows();
			var row = this._generalGrid.model.getRow(e.rowIndex);
			if (row && (this._generalGrid.hasLayout() > 0)) {
				var cellobj = this._generalGrid.getCell(this._getColumnIndex("action"));
				var cellDomNode = cellobj.getNode(e.rowIndex);
				cellDomNode.innerHTML = '<img name="deleteImage" style="margin-right:7px;cursor: pointer;" title="' 
					+ RiskMessages.REMOVE_STRATEGY_TITLE + '" alt="' + RiskMessages.REMOVE_STRATEGY_TITLE + '" src =' +
					dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/actions/action_delete.gif") + " />";
				this._rowHoverArray.push(row.id);
			}
        },
        
        /*
         * Private Method
         * _MouseOut: Handle the on mouse out event on the grid row and remove the action images from action column
         * Input: event e
         * outPut: None
         */
        _onMouseOut: function(e){
            if (!this._generalGrid || (e.rowIndex < 0) || (this._generalGrid.edit && this._generalGrid.edit.isEditing())
			|| (e.gridID != RiskConstants.RISK_STRATEGIES)) {
                return;	//Do not process the row if an editor is open
            }
            this._clearAllActionRows();
        },
        
        /*
         * Private Method
         * _clearOneActionRow: Clear action from this row
         * Input: event e
         * outPut: None
         */
        _clearOneActionRow: function(rowId){
			if (this._generalGrid) {
				var row = this._generalGrid.model.store._getItemByIdentity(rowId);
				if (row && (this._generalGrid.hasLayout() > 0)) {
					var cellobj = this._generalGrid.getCell(this._getColumnIndex("action"));
					var rowIndex = this._controller.getRowIndex(rowId);
					var cellDomNode = cellobj.getNode(rowIndex);
					cellDomNode.innerHTML = "";
				}
			}
        },
        
        /*
         * Private Method
         * _getColumnIndex: Returns the  column index of Action Column
         * Input: Column name
         * outPut: Integer index
         */
		_getColumnIndex: function(columnName) {
			for (var i = 0; i < this._generalGrid.layout.cellCount; i++) {
				if (this._generalGrid.layout.cells[i].field == columnName) {
					return i;
				}
			}
			return this._generalGrid.layout.cellCount - 1; 
		},
        
        /*
         * Private Method
         * _clearAllActionRow: Clear all actions from this row
         * Input: event e
         * outPut: None
         */
        _clearAllActionRows: function(){
            for (var i=0 ; i < this._rowHoverArray.length; i++) {
                this._clearOneActionRow(this._rowHoverArray[i]);
            }
            this._rowHoverArray = [];
        },
        
		/**
         *  postCreate : postCreate for the viewlet.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        postCreate: function(){
			this.rownode = null;
			this.strategyType = RiskConstants.TYPE_MITIGATION;
        },
		/**
		 * This method is called by the framework, after the grid is created.
		 * Subscribes to the required events 
		 */
		createView: function(){
            this.inherited('createView', arguments);
			dojo.subscribe(GridEvents.FIELD_CHANGED, this, this.updateStrategiesInModel);
			this._pconnects.push(dojo.connect(this._generalGrid, "onRowMouseOver", this, this._onMouseOver));
            this._pconnects.push(dojo.connect(this._generalGrid, "onRowMouseOut", this, this._onMouseOut));
			this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_CLICK, this, this.onCellClick));
			this._pconnects.push(dojo.connect(this._generalGrid, "onStartEdit", this, this.onStartEdit));
			this._pconnects.push(dojo.connect(this._generalGrid, "onApplyCellEdit", this, this.onApplyCellEdit));
		},

		/**
		 * Save the current values for the edited cell, locally, to be able to revert to this 
		 * value if the validation fails in 'onApplyCellEdit'
		 */
		onStartEdit: function(inCell, inRowIndex){
			var row = this._generalGrid.model.getRow(inRowIndex);
			if (row) {
				this.curCellField = inCell.field;
			}
		},
		
		/**
		 * Validate the fields for valid values, if the validation fails revert to the previous value,
		 * which was saved in 'onStartEdit'
		 */
		onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
			//IE Issue : For the first edit the 0th index is undefined
			if (this._generalGrid && (this._generalGrid._click[0] || this._generalGrid._click[1])) {
				var row = this._generalGrid.model.getRow(inRowIndex);
				if (row) {
					var cellFieldName = this.curCellField;
					var item = this._generalGrid.model.store._getItemByIdentity(row.id);
					if (item && cellFieldName === RiskConstants.RISK_STRATEGY_FIELD_SUMMARY) {
						inValue = inValue ? inValue.replace(/^\s*/, "").replace(/\s*$/, "") : inValue;
						if (!inValue || inValue.length === 0) {
							this._generalGrid.edit.cancel();
						}
					}
				}
			}
		}, 
		 /**
         * getRowIndex: This is returns the index of the strategy in the array by id
         */
		getRowIndex: function(id){
			var strategies = null;
			if (this.rownode) {
				if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
					strategies = this.rownode.mitigationstrategy;
				}
				else {
					strategies = this.rownode.contingencyplan;
				}
			}
			if (strategies && strategies.length >= 0 && id) {
				for (var i = 0; i < strategies.length; i++) {
					if (strategies[i].id == id) {
						return i;
					}
				}
			}
			return -1;
        },
		
		/**
         *  updateStartegyDescriptionInModel : updates the changes to the strategy description
         *  in the model
         *  
         *
         */
		updateStartegyDescriptionInModel:function(id, value)
		{
			var strategyIndex = this.getRowIndex(id);
			if (strategyIndex >= 0) {
				if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
					if (this.rownode.mitigationstrategy[strategyIndex].description !== value){
						this.rownode.mitigationstrategy[strategyIndex].modifieddescription = value;
						this.rownode.mitigationstrategy[strategyIndex].description = value;
						this.containerViewlet.toggleSaveButton(false);
					}
				}
				else
				{
					if (this.rownode.contingencyplan[strategyIndex].description !== value){
						this.rownode.contingencyplan[strategyIndex].modifieddescription = value;
						this.rownode.contingencyplan[strategyIndex].description = value;
						this.containerViewlet.toggleSaveButton(false);
					}
				}
			}
		},
			
		/**
         *  updateStrategiesInModel : Updates the changes to strategy in the grid
         *  to the model
         *  
         *
         */	
		updateStrategiesInModel: function(data){
			if (data.controllerID == RiskConstants.RISK_STRATEGIES_GRID_CONTROLLER) {
				var strategyIndex = this.getRowIndex(data.row.id);
				if (strategyIndex >= 0 && data.newValue) {
					var inValue = data.newValue;
					inValue = inValue ? inValue.replace(/^\s*/, "").replace(/\s*$/, "") : inValue;
					if (inValue.length > 0) {
						if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
							if ((data.fieldName === "summary") && (this.rownode.mitigationstrategy[strategyIndex].summary !== inValue)) {
								this.rownode.mitigationstrategy[strategyIndex].modifiedsummary = inValue;
								this.rownode.mitigationstrategy[strategyIndex].summary = inValue;
							}
						}
						else {
							if ((data.fieldName === "summary") && (this.rownode.contingencyplan[strategyIndex].summary !== inValue)) {
								this.rownode.contingencyplan[strategyIndex].modifiedsummary = inValue;
								this.rownode.contingencyplan[strategyIndex].summary = inValue;
							}
						}
					}
				}
			}
		},
        
        /**
         *  insertStrategiesInModel : insert a new strategy in the model
         *  if it is inserted in the grid
         *  
         *
         */
		insertStrategiesInModel: function(row){
			if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
						if (!this.rownode.mitigationstrategy) {
							this.rownode.mitigationstrategy = [];
							}
								var newrow = {
								"active": false,
								"description": row.description,
								"id": row.relativeHref,
								"summary": row.summary,
								"modifiedsummary":row.summary
								};
							this.rownode.mitigationstrategy.push(newrow);
					}
					else
					{
						if (!this.rownode.contingencyplan) {
							this.rownode.contingencyplan = [];
							}
							var newrow1 = {
								"active": false,
								"description": row.description,
								"id": row.relativeHref,
								"summary": row.summary,
								"modifiedsummary":row.summary
								};
							this.rownode.contingencyplan.push(newrow1);
					}
		},
        /**
         *  rowSelectionChanged : This method is called form the risk strategies when the row selection chnages in the 
         *  strategies grid.
         */
		rowSelectionChanged: function(state){
            if (!state && this._controller && this._generalGrid) {
				var rowIndex = this._generalGrid.selection.selectedIndex;
				var row = this._controller.getRowByIndex(rowIndex);
				
				if(!row){return;}
				
				var strategyIndex = this.getRowIndex(row.id);
				if (strategyIndex >= 0) {
					var data = null;
					if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
						data = this.rownode.mitigationstrategy[strategyIndex];
					}
					else {
						data = this.rownode.contingencyplan[strategyIndex];
					}
					if (data) {
						data.rowIndex = data.rowIndex;
						data.strategytype = this.strategyType;
						dojo.publish(RiskConstants.RISK_STRATEGIES_GRID_ROW_CLICK, [data]);
					}
				}
            }
        },

        /**
         *  UpdateStrategies : This function is to update the grid row.
         *  Incoming Arguments : The modofied row values is paased.
         *  Outgoing Argument : none
         *
         */
        UpdateStrategies: function(data, type){
			
			if (this._controller && data) {
				this.strategyType = type;
				this.rownode = data;
				this.refresh();
			}
			
        },
		
		refresh: function(){
			if (this.rownode) {
				this.removeAllRows();
				if (this.strategyType === RiskConstants.TYPE_MITIGATION) {
					this._controller.strategies = this.rownode.mitigationstrategy;
				}
				else {
					this._controller.strategies = this.rownode.contingencyplan;
				}
				this.buildGrid(this, this._gridModel, this._controller);
				this.defaultSelect();
			}
		},
        
        /*
         * This sets the first row in the grid as selected when the grid is loaded.
         *
         */
		defaultSelect: function() {
			if (this._controller) {
				this._generalGrid.selection.unselectAll();
				this._generalGrid.selection.select(0);
			}
		},
		
        /*
         * This is get the first row of the grid.
         *
         */
        getfirstRow: function(){
            var data1;
            if (!this._controller) {
                this._controller = ControllersFactory.getEPMRiskStrategiesGridController(RiskConstants.RISK_STRATEGIES_GRID_CONTROLLER);
            	data1 = this._controller.getRowByIndex(0);
            }
            else {
                data1 = this._controller.getRowByIndex(0);
            }
            return data1;
        },
		
		/*
         * This is to add a new row in the grid.
         *
         */
		addRiskStrategy: function(){
            var rowID = this._controller.getUniqueRowID("_unsaved");
            
            var rows = {
                "relativeHref": rowID,
                "summary": RiskMessages.NEW_RISK_STRATEGY,
                "description": ""
            };
			this.insertStrategiesInModel(rows);
            this._controller._insertElement(rows, null, false);
			this.scrollToSelectedRow();
			this.startFirstCellEdit();
        },
		
		/**
		 * When new risk strategy is added. Select 1st cell text in editable mode.
		 */
		startFirstCellEdit: function() {
		    if (this._generalGrid) {
				var rowIndex = this._generalGrid.selection.selectedIndex;
				var cell = this._generalGrid.getCell(0);
				
				if (rowIndex >= 0 && cell) {
					this._generalGrid.focus.setFocusCell(cell, rowIndex);
					this._generalGrid.edit.setEditCell(cell, rowIndex);
				}else {
					console.log("startFirstCellEdit: no selected row to edit");
				}
			}	
		},
		
		/**
         *  deleteRiskStrategy : Throws a Delete Confirmation and then deletes
         *
         */
		deleteRiskStrategy: function(){
			var selectedRows = this._generalGrid.selection.getSelected();
			if (selectedRows && selectedRows.length > 0) {
				var dialogContent;
				var buttonlist = [{
					label: RiskMessages.YES,
					action: dojo.hitch(this, this.deleteStrategies)
				}, {
					label: RiskMessages.NO
				}];
				dialogContent = RiskMessages.CONFIRM_STRATEGY_DELETE_MESSAGE;
				
				var labelList = [{
					label: dialogContent
				}];
				
				this.confirmDialog = new com.ibm.epm.web.core.ui.internal.EPMConfirmDialog({
					labelList: labelList,
					title: RiskMessages.DIALOG_TITLE,
					buttonlist: buttonlist
				});
				this.confirmDialog.show();
			}
		},
		
		/**
         *  deleteStrategies : Delete selected strategies from the grid
         *
         */
		deleteStrategies: function(){
			var workitemRiskStrategy = [];  
			var error = null;
			var errorMesg = null;
			var _gridData = null;
			var rowToBeDeleted = [];
			var indx = null;
			var selectedRows = this._generalGrid.selection.getSelected();
			if (this._controller && selectedRows && selectedRows.length > 0) {
				for (var i = 0; i < selectedRows.length; i++) {
					var row = this._controller.getRowByIndex(selectedRows[i]);
					errorMesg = this.hasLinkedWIs(row);
					if(errorMesg){
						workitemRiskStrategy.push(errorMesg);
						continue;
					}
					if (row.id) {
						indx = this.getRowIndex(row.id);
					}
					if (indx>=0) {
						if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
							if(row.id.indexOf(RiskConstants.NEW_ELEMENT_PREFIX) >= 0 ){
								this.rownode.mitigationstrategy.splice(indx,1);
							}else{
								this.rownode.mitigationstrategy[indx].deleted = true;
								this.containerViewlet.toggleSaveButton(false);
							}
						}
						else {
							if(row.id.indexOf(RiskConstants.NEW_ELEMENT_PREFIX) >= 0 ){
								this.rownode.contingencyplan.splice(indx,1);
							}else{
								this.rownode.contingencyplan[indx].deleted = true;
								this.containerViewlet.toggleSaveButton(false);
							}
						}
					}
					rowToBeDeleted.push(selectedRows[i]);
				}
				if(this._controller){
					_gridData = this._controller.getDataModel(); 
				}
				if(this._generalGrid){
					this._generalGrid.selection.unselectAll();
					this._generalGrid.model.remove(rowToBeDeleted);
				}
				if (_gridData) {
					_gridData.store.save();
				}
				if( workitemRiskStrategy && workitemRiskStrategy.length > 0){
					error = {errorMessage: workitemRiskStrategy[0]};
					this._displayError(error);
				}
				this.defaultSelect();
				this.closeDialog();
			}
		},
		
		/**
		 * destroy risk confirm dialog.
		 */
		closeDialog: function(){
			this.confirmDialog.hide();
			this.confirmDialog.destroy();
			
        },
		/**
         *  removeAllRows : Remove all rows from the grid, called during refresh
         */
		removeAllRows: function(){
			try {
				if (this._controller) {
					var _gridData = this._controller.getDataModel();
					if (_gridData && this._generalGrid) {
						var count = _gridData.getRowCount();
						this._generalGrid.beginUpdate();
						for (var i = 0; i < count; i++) {
							var row = _gridData.data[0];
							if (row) {
								_gridData.remove([0]);
							}
						}
						//save the model
						_gridData.store.save();
					}
				}
			} 
			catch (Error) {
				console.log("Error during removing all rows :" + Error.message);
			}
			finally {
				if (this._generalGrid) {
					this._generalGrid.endUpdate();
				}
			}
		},
		/**
		 * getAction :  get all the actions for strategy.
		 */
		hasLinkedWIs: function(strategy){
			if(strategy && strategy.actions && strategy.actions.length > 0)
			{
				for(var x = 0 ;x < strategy.actions.length ;x++)
				{
					if(strategy.actions[x].workitem)
					{
						var errorMessage = dojo.string.substitute(RiskMessages.DELETE_STRATEGY_WITH_ACTION_WORKITEM);
						return errorMessage;
					}
				}
			}
			return null;
		},
		
		getSortableColums: function(){
			return {	summary:"summary"	};
		},
		
		refreshRiskStrategy: function(){
			if (this.rownode && this.rownode.loadedelement && this.rownode.loadedelement[0] && this.rownode.loadedelement[0].riskdetails) {
				if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
					this.rownode.mitigationstrategy = dojo.clone(this.rownode.loadedelement[0].riskdetails.mitigationStrategies);
				}
				else {
					this.rownode.contingencyplan = dojo.clone(this.rownode.loadedelement[0].riskdetails.contingencyPlans);
				}
			}
			this.refresh();
		}
    });
})();
