/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.resource.web.ui.internal.StaticPoolsGridModel");
    
    dojo.require("com.ibm.epm.web.core.grid.model.AbstractGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    // Constants
    dojo.require("com.ibm.epm.resource.web.ui.internal.ResourceConstants");
    
    //client
    dojo.require("com.ibm.epm.resource.web.client.internal.StaticPoolClient");
    
    //service
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    
    // Error Dialogs
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.resource.web.ui", "Messages");
	
    var StaticPoolMessages = dojo.i18n.getLocalization("com.ibm.epm.resource.web.ui", "Messages");
    
    var ErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    var abstractGridModel = com.ibm.epm.web.core.grid.model.AbstractGridModel;
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    var Constants = com.ibm.epm.resource.web.ui.internal.ResourceConstants;
    var StaticPoolClient = com.ibm.epm.resource.web.client.internal.StaticPoolClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    
    
    
    dojo.declare("com.ibm.epm.resource.web.ui.internal.StaticPoolsGridModel", abstractGridModel, {
        newElementPrefix: Constants.NEW_ELEMENT_PREFIX,
        titleString: "",
        poolTypeToHrefMap: [],
        poolsController:null,
		poolsTypeSaveParamsMap :null,
		oldPoolTypeHref:"",
        constructor: function(){
        
            this.selectionHandler=dojo.subscribe(Constants.ON_STATICPOOLTYPE_SELECTION, this, "_setPoolTypeToHrefMap");
           
        },
        
				
        /*
         * set the pooltypeToHrefMap required for posting the type href
         * Incoming : publishedItem
         * Outgoing : None
         */
        _setPoolTypeToHrefMap: function(publishedItem){
            this.poolTypeToHrefMap = publishedItem.map;
            
        },
        /*
         * setting the pools controller handle which is required for setting the child pool type
         * Incoming: controller
         * Outgoing: None
         */
		setPoolsController:function(controller){
			if(controller){
			 this.poolsController = controller;
			}
		},
		
        /*
         * This is called by controller. Returns the empty row structure to the controller.
         */
        getEmptyRow: function(rowID, treeElementHandle){
            var row = {
                "id": rowID,
                "href": rowID,
                "parentId": null,
                "name": treeElementHandle.title,
                "resourcetype": treeElementHandle.resourcetype,
                "description": "",
                "type": "",
                "loadedelement": null
            };
			
			   if (treeElementHandle.parentIdentifier) {
                if (dojo.isArray(treeElementHandle.parentIdentifier)) {
                    row.parentId = treeElementHandle.parentIdentifier[0];
                }
                else {
                    row.parentId = treeElementHandle.parentIdentifier;
                }
				//setting the child pool type to that of the parent
				if(this.poolsController){
					var rowIndex=this.poolsController. getRowIndex(row.parentId);
					var parentRow =this.poolsController.getRowByIndex(rowIndex);
					// Get the Pool Type information from the parent
					if(parentRow){
						row.type=parentRow.type;
					}
				}

            }
            
            return row;
        },
        
        
        
        /*
         * This is called by controller.
         * After inserting a row in Grid, the getAndUpdateRow() calls updateRowData with RowURI, RowID params.
         */
        updateRowData: function(row, loadedElement, linkName){
			if (linkName === null) {
                row.name = loadedElement.name;
                row.description = loadedElement.description;
                row.type = loadedElement.type;
                row.loadedelement = [loadedElement];
                
                //Load the title of static pool type link as Pool Type info
                var linkType = getLink(loadedElement, "StaticPoolType");
                if (linkType) {
					row.type = linkType.title;
				}
				this._updateParent(row, loadedElement);
            }
        },
        
        _updateParent: function(row, loadedElement){
            //5780 fix
            var parentLink = getLink(loadedElement, Constants.PARENT);
			if (parentLink && parentLink.relativeHref) {
                row.parentId = parentLink.relativeHref;
		    }
        },
        
        /*
         * sets the arguments which have to be posted to the pool service
         */
        prepareSaveParams: function(row){
        
            var poolTyperelativeHref;
			this.poolsTypeSaveParamsMap=new Array();
            
			if (row.type) {
                poolTyperelativeHref = this.poolTypeToHrefMap[row.type];
            }
			//error to be thrown only for the parent pool
			/*if ((row.parentId === null || row.parentId === "") && (
		  	    row.type===null || row.type==="") && (row.href.indexOf("_unsaved")==-1)) {
				console.log("row.parentId==>"+row.parentId);
				console.log("row.type==>"+row.type);	
                this._displayErrorMessage(StaticPoolMessages.CANNOT_SAVE_STATICPOOL_WITHOUT_POOL_TYPE_VALUE);
                return null;
            }*/
            var saveParams = {
                name: row.name,
                description: row.description
        		
            };
			if(poolTyperelativeHref)
			{
				var loadedElement = row.loadedelement;
				if (loadedElement != "") {
					//get the previous pooltypehref
					var previousPoolsTypeHref = loadedElement[0].StaticPoolType.relativeHref;
	                if (previousPoolsTypeHref) {
						if(previousPoolsTypeHref != poolTyperelativeHref){
							this.poolsTypeSaveParamsMap.push(previousPoolsTypeHref + ":del");
							this.poolsTypeSaveParamsMap.push(poolTyperelativeHref + ":add");
							
						}
					}
					
				}
				else {
					this.poolsTypeSaveParamsMap.push(poolTyperelativeHref + ":add");
				}
				saveParams.StaticPoolType = this.poolsTypeSaveParamsMap;
			}
			
            if (row.parentId !== null && row.parentId !== "") {
			
				saveParams.ParentStaticPool = row.parentId + ":add";
				saveParams.parent=row.parentId;
			}
			else
			{
				saveParams.parent="";
			}
			
            return saveParams;
        },
        
        _getNewElementURI: function(row){
            return Constants.STATICPOOL_URI;
        },
        
        getTitleString: function(){
            return this.titleString;
        },
        
        /*
         * Displays the Error Message
         */
        _displayErrorMessage: function(errorMsg){
        
            ErrorHandler.setError(errorMsg);
			// Set an interval and clear the error message
            var intervalID = setInterval(function(){
                ErrorHandler.clearError();
                clearInterval(intervalID);
            }, 10000);
            
        },
		
		destroy:function(){
			if(this.selectionHandler){
				dojo.unsubscribe(this.selectionHandler);
			}
		}
        
    });
})();

