/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.resource.web.ui.internal.StaticPoolsGridView");
    
    
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    
    dojo.require("com.ibm.epm.resource.web.ui.internal.StaticPoolsGridModel");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	
    // Constants
    dojo.require("com.ibm.epm.resource.web.ui.internal.ResourceConstants");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.resource.web.ui", "Messages");
    
	//dialog
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
	
    var Constants = com.ibm.epm.resource.web.ui.internal.ResourceConstants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var StaticPoolMessages = dojo.i18n.getLocalization("com.ibm.epm.resource.web.ui", "Messages");
	
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    
    //
    var selectOptions = [];
    
    /*
     * Define Custom editor to populate dynamic select options
     */
    dojo.declare("com.ibm.epm.resource.assignment.web.ui.internal.DynamicSelect", dojox.grid.editors.Select, {
    
        format: function(inDatum, inRowIndex){
        
            var newOptions = [];
		
            //Copy the options
            for (var i = 0; i < selectOptions.length; i++) {
				newOptions[i] = selectOptions[i];
            }
            
            this.options = this.values = newOptions;
            
            return this.inherited("format", arguments);
            
        }
    });
    
    var dynamicSelectEditor = com.ibm.epm.resource.assignment.web.ui.internal.DynamicSelect;
    //
    
    dojo.declare("com.ibm.epm.resource.web.ui.internal.StaticPoolsGridView", abstractView, {
    
        templateString: '<div class="com-ibm-epm-resource-web-ui-internal-StaticPoolsGrid"><div class="com-ibm-epm-web-core-ui-internal-templates-EPMGrid" dojoAttachPoint="staticpoolsGridAttachPoint"/></div>',
        
        /*
         * Passing in a constructor arg of modifyAllowed = false will prevent the editing of
         * this view. This is needed for when a user is not allowed to make changes due to
         * having lack of permissions.
         */
        _modifyAllowed: true,
        
        constructor: function(args) {
            /*
             * If modifyAllowed is passed in and set to false, it'll disable
             * the editing of this grid.
             */
            if (args && (args.modifyAllowed !== undefined))
                this._modifyAllowed = args.modifyAllowed;
        },
       
        postCreate: function(){
        	this._eventsubscriptions = [];
			this.eventSubscribe();
        },
		
		/**
         * eventSubscribeToAllEvents: Method to subscribe to all events.
         */
        eventSubscribe: function(){
            if (this._modifyAllowed) {
                this._eventsubscriptions = [
                    dojo.subscribe(EPMTreePubSub.ON_RENAME_NODE, this, "_renameNode"),
                    dojo.subscribe(EPMTreePubSub.ON_CREATE_NODE, this, "_createNode"),
                    dojo.subscribe(EPMGridPubSub.ON_APPLY_CELL_EDIT, this, "_onApplyCellEdit")];
            }
        },
		
		/**
		 * _renameNode: Event handler when the tree publishes the EPMTreePubSub.ON_RENAME_NODE event.<br>
		 * 
		 * @param {Object} publishObj - The published object contains the tree id, the node.item, old and new values of the name field.
		 */
		_renameNode: function(publishObj) {
			 if ((publishObj) && (publishObj.treeID == Constants.STATICPOOL_TREE_TITLE)) {
			 	var id = publishObj.item.identifier[0];
			 	if(this.StaticPoolsController){
                  if (this.containerViewlet)
                      this.containerViewlet.enableSaveButton(true);
				  var dataModel=this.StaticPoolsController.getDataModel();
				  var item;
				  if(dataModel && id)
				  {
				  	 item=dataModel.store._getItemByIdentity(id);
				  }
                  if (item) {
					dataModel.store._setValueOrValues(item, Constants.FIELD_NAME, publishObj.newValue, true);
				  }
				}
			 }
		},
		
		/*
		 * Called whenever a tree node is created (whether via Add button or via
		 * context menu). Will enable save button.
		 */
        _createNode: function(publishObj) {
			if (publishObj && publishObj.treeID == Constants.STATICPOOL_TREE_TITLE) {
				if (this.containerViewlet) 
					this.containerViewlet.enableSaveButton(true);
			}
        },
        		
        /* 
         * buildStaticPoolsGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with StaticPoolsGrid.
         */
        buildStaticPoolsGrid: function(){
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.resource.web.ui.internal.StaticPoolsGridModel();
            }
            this.StaticPoolsController = controllerUtil.getTreeBasedGridController(Constants.STATICPOOL_CONTROLLER);
			this.StaticPoolsController._buildModel = false;
			//set the controller handle to model
			this._gridModel.setPoolsController(this.StaticPoolsController);
            this.buildGrid(this, this._gridModel, this.StaticPoolsController);
        },
		       
        /* Gets the StaticPoolsModel 
         * Incoming : None
         * Outgoing : this._gridModel-model of static pools grid
         */
        getStaticPoolsModel: function(){
            if (this._gridModel) {
                return this._gridModel;
            }
        },
        
        /*
         * getGridTitle funtion - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return StaticPoolMessages.STATICPOOL_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.staticpoolsGridAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var gridLayout = [{
                noscroll: false,
                rows: [[{
                    name: StaticPoolMessages.POOL_TYPE,
                    width: "20%",
                    field: "type",
                    editor: dynamicSelectEditor
                
                
                }, {
                    name: StaticPoolMessages.POOL_DESC,
                    width: "30%",
                    field: "description",
                    editor: dojox.grid.editors.Input
                }]]
            }];
            
            return gridLayout;
        },
        
        handleError: function(errorObject){
            this.inherited("handleError", arguments);
            this._displayError(errorObject);
        },
        
        /*
         * Sets the select option
         */
        setSelectOptions: function(options){
        
            selectOptions = options;
            
        },
		
        destroy: function(){
            this._unsubscribeToAllEvents();
			this.inherited(arguments);
        },
		
		/**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEvents: function(){
            if (this._eventsubscriptions != null) {
                for (var topic in this._eventsubscriptions) {
                    var handler = this._eventsubscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._eventsubscriptions = [];
        },
        
		 /*
         * Override the function defined in EPMGrid.
         * Make poolType as uneditable for child static pool
         */
        isCellEditable: function(event){
            // Never allow cells to be edited if the grid is marked readonly
            if (!this._modifyAllowed)
                return false;
                        
			if (event.cell.field == "type") {
				var rowIndex = parseInt(event.rowIndex, 10);
				if (isNaN(rowIndex)) {
					return true;
				}
				var row = this.StaticPoolsController.getRowByIndex(rowIndex);
				//parent Id would be there if its a child pool type
				if ((row.parentId !== null && row.parentId !== "") )
				{
					return false;
				}
				else {
					return true;
				}
			}
			else
			{
				return true;
			}
        },
        
        _validateController: function(data){
        
            if (!this._controller) {
                return false;
            }
            
            if (data) {
                var resourcetype = null;
                if (dojo.isArray(data.resourcetype)) {
                    resourcetype = data.resourcetype[0];
                }
                else {
                    resourcetype = data.resourcetype;
                }
                if (resourcetype == Constants.SP_RESOURCE_TYPE) {
                    return true;
                }
            }
            return false;
        },
        
        _onMouseOver: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onMouseOut: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onSelect: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onBlur: function(){
        
            return;
            
        },
        
        /*
         * On notification of a cell's content changing, notify containing page that
         * the page is now dirty. (This will enable the save button.)
         */
        _onApplyCellEdit: function(e){
            if (this.containerViewlet)
                this.containerViewlet.enableSaveButton(true);
        },
        
        /*
         * User pressed Save button; save just the static pools and disable the
         * Save button.
         */
        _saveStaticPools: function(){
            if (this.containerViewlet)
                this.containerViewlet.enableSaveButton(false);
            this.persistGrid();
        },
		/*
         * User pressed Delete button; Delete pool which is does not have user or child pool
         * Incoming :None
         * Outgoing :None
         */
		_deleteStaticPools:function(){
			// Get selected Rows and then call delete on them in a loop
			if((!this._generalGrid) || (!this.StaticPoolsController)){
				return;
			}
			var selectedRows = this._generalGrid.selection.getSelected();
			if(!selectedRows)
				return;
			if (selectedRows.length !== 0) {
                var buttonlist;
                var dialogContent;
                if (selectedRows.length > 1) {
					//Multiple selection row delete is not supported.
					//clearing the selection and selecting only one node
					this._generalGrid.selection.unselectAll();
					this._generalGrid.selection.addToSelection(selectedRows[0]);
					
                    buttonlist = [{
                        label: StaticPoolMessages.YES,
                        action: dojo.hitch(this, this.deletePool)
                    }, {
                        label: StaticPoolMessages.NO,
                        action: dojo.hitch(this, this.noDelete)
                    }];
                    dialogContent = StaticPoolMessages.DIALOG_CONTENT_MULTIPLE_SELECTION_NOT_SUPPORTED + StaticPoolMessages.DIALOG_CONTENT_SINGLE_POOL_SELECTED;
                }
			else {
                    buttonlist = [{
                        label: StaticPoolMessages.YES,
                        action: dojo.hitch(this, this.deletePool)
                    }, {
                        label: StaticPoolMessages.NO
                    }];
                    dialogContent = StaticPoolMessages.DIALOG_CONTENT_SINGLE_POOL_SELECTED;
                }
                
                var labelList = [{
                    label: dialogContent
                }];
                
                this.confirmDialog = new com.ibm.epm.web.core.ui.internal.EPMConfirmDialog({
                    labelList: labelList,
                    title: StaticPoolMessages.DIALOG_TITLE,
                    buttonlist: buttonlist
                });
                this.confirmDialog.show();
            }
			
		},
			
		/**
		 * The method deleteAll will delete all the selected rows in the grid
		 */
		deletePool: function(){
        	
			var selectedRows = this._generalGrid.selection.getSelected();
			if (selectedRows.length !== 0) {
				this._generalGrid.selection.unselectAll();
	        	//this._containingPage.disableDeleteButton(true);	        	
				this._controller._deleteElement(selectedRows[0]);
			}
	        if (this.confirmDialog) {
			 	this.confirmDialog.hide();
			 	this.confirmDialog.destroy();
			}
        },
        
		/**
		 * The method noDelete will close the confirmation dialog and won't do anything
		 */
        noDelete: function(){
           if (this.confirmDialog) {
		   	this.confirmDialog.hide();
		   	this.confirmDialog.destroy();
		   }            
        },
		
		 /**
         * getGridInternalID: Used to set the unique ID of the grid, needs to be overridden by the inheriting view
         * Incoming:
         *	gridModel
         * Outgoing:
         *  none
         */
        getGridInternalID: function(){            
            return Constants.STATICPOOL_GRID_TITLE;
        },
		
		/**
		 * createView - method will call the createView of the abstractView
		 * And also set the grid to grow auto.
		 */
		createView: function(){
			this.inherited('createView', arguments);
			
			this._generalGrid.autoHeight = true;
			this._generalGrid.update();
		}
		
    });
})();

