/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.resource.web.ui.internal.StaticPoolsViewlet");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
    
    // Viewlet
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    
    // Widgets
    dojo.require("com.ibm.epm.resource.web.ui.internal.StaticPoolsGridView");
    dojo.require("com.ibm.epm.resource.web.ui.internal.StaticPoolUsersGridView");
    dojo.require("com.ibm.epm.resource.web.ui.internal.StaticPoolsTreeWidget");
    dojo.require("com.ibm.epm.resource.web.ui.internal.StaticPoolAssignedUsersGridView");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    
    // Constants
    dojo.require("com.ibm.epm.resource.web.ui.internal.ResourceConstants");
    
    //client
    dojo.require("com.ibm.epm.resource.web.client.internal.StaticPoolClient");
    
    //service
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.resource.web.ui", "Messages");
    
	 // Error handler
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
   
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
	
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var StaticPoolViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.resource.web.ui", "Messages");
	var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    
    var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var Constants = com.ibm.epm.resource.web.ui.internal.ResourceConstants;
    var StaticPoolClient = com.ibm.epm.resource.web.client.internal.StaticPoolClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
    
    dojo.declare("com.ibm.epm.resource.web.ui.internal.StaticPoolsViewlet", [Viewlet], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.resource.web", "ui/internal/templates/StaticPoolsViewlet.html"),
		contextHelpId:Constants.STATIC_POOL_VIEWLET_CONTEXT_HELP_ID,
		
        _staticpoolsTree: null,
        _staticpoolsGrid: null,
        _staticpoolUsersGrid: null,
        _staticpoolAssignedUsersGrid: null,
		 gridRowClickHandler:null,
        
        // Set to true if user has JazzAdmins rights, allowing user to edit the resources
        _userIsAdmin: false,
        
         selectedStaticPoolHref:null,
		 
		 
        init: function(){
			this.inherited(arguments);
			this._staticpoolsTree = null;
            this._staticpoolsGrid = null;
            this._staticpoolUsersGrid = null;
            this._staticpoolAssignedUsersGrid = null;
			this.selectedStaticPoolHref=null;
        },
        
        postCreate: function(){
            /*
             * Determine if user is an admin and thus has rights to modify pool info. If not,
             * the viewlet will be rendered readonly.
             */
            this._userIsAdmin = Util.userIsAdmin();

        	this._createToolbar();
            dojo.subscribe(EPMGridPubSub.ON_CELL_CLICK, this, this.onCellClick);
        },
        
        //TODO: High Priority - topNodeItem.href is undefined
        //[Break on this error] relativeHref: topNodeItem.href[0],
        /*
         * Constructor
         */
        constructor: function(){
            this.Assign = StaticPoolViewletMessages.ASSIGN_BUTTON_LABEL;
            this.Unassign = StaticPoolViewletMessages.UNASSIGN_BUTTON_LABEL;
			this.saveBtn=StaticPoolViewletMessages.SAVE;
			this.saveTitle=StaticPoolViewletMessages.SAVE_TITLE;
			this.assignedResources=StaticPoolViewletMessages.ASSIGNED_RESOURCES;
			this.add=StaticPoolViewletMessages.ADD_ANCHOR
        },
        
        /*
         * Viewlet refresh , destroy if present and create widgets
         * Incoming : None
         * Outgoing : None
         */
        refresh: function(){
            this.enableSaveButton(false);
            this.destroyWidgets();
            this._createUserAssignToolbar();
            this._createWidgets();
            this._addLabel();
        },
        
        destroy: function(){
        
            this.destroyWidgets();
            
            //unsubs - todo
            
            this.inherited("destroy", arguments);
        },
        
        /* Creates static pools tree, grid and also users grid.
         * Incoming: None
         * Outgoing : None
         */
        _createWidgets: function(){
            if (this._staticpoolsTree == null) {
                this._staticpoolsTree = new com.ibm.epm.resource.web.ui.internal.StaticPoolsTreeWidget({
                    containerViewlet: this,
                    modifyAllowed: this._userIsAdmin
                });
                this._treeContent.appendChild(this._staticpoolsTree.domNode);
            }
            this._staticpoolsTree._buildStaticPoolsTree();
            
            if (this._staticpoolsGrid == null) {
                this._staticpoolsGrid = new com.ibm.epm.resource.web.ui.internal.StaticPoolsGridView({
                    _treeID: "StaticPoolsTree",
                    containerViewlet: this,
                    modifyAllowed: this._userIsAdmin
                });
                this._gridContent.appendChild(this._staticpoolsGrid.domNode);
                
            }
            this._staticpoolsGrid.buildStaticPoolsGrid();
            
            //loading the pool type options data
            this._loadPoolTypeOptionsData();
            
            //subscribe treenode select click event
            if (!this.treeSelectHandler) {
				this.treeSelectHandler = dojo.subscribe(EPMTreePubSub.ON_SELECT, this, "_createStaticPoolAssignedUsersGrid");
			//				this.treeSelectHandler = dojo.subscribe(Constants.ON_STATICPOOL_TREECLICK, this, "_createStaticPoolAssignedUsersGrid");
			}
			//subscribe grid row click event
            if (!this.gridRowClickHandler) {
				this.gridRowClickHandler=dojo.subscribe(EPMGridPubSub.ON_ROW_CLICK, this, "_selectTreeNodeOnRowClick");
			}
			
			
        },
        
		/*
         *  _selectTreeNodeOnRowClick : Event handler when there is a row click event on the Grid
         *  Incoming : publishedObject
         *  Outgoing : none
        */
        _selectTreeNodeOnRowClick: function(publishedItem){
            var row = this._staticpoolsTree._validateAndGetRow(publishedItem);
            if (row && row.id) {
                this._staticpoolsTree._selectNodeOnRowClick(row);
                var relatedHref = row.href;
                if (relatedHref.toString().indexOf(Constants.NEW_ELEMENT_PREFIX) == -1) {
                    if (this._staticpoolAssignedUsersGrid) {

                        this._staticpoolAssignedUsersGrid.removeAllRows();
                    }
                    this.selectedStaticPoolHref = relatedHref;
                    this._buildStaticPoolAssignedUsersGrid();
                }
				if (this._staticpoolAssignedUsersGrid) {

                        this._staticpoolAssignedUsersGrid.removeAllRows();
                }
            }
        },
		
        /*
         * Creates a pooltypeToHrefMap and publishes so that model makes use of it
         * Incoming : titleArray holds pool type titles
         * 			  relativeHrefArray pool type href array
         */
        _createTypeMapAndPublish: function(titleArray, relativeHrefArray){
        
            var map = [];
            if (titleArray && relativeHrefArray) {
                for (var i = 0; i < titleArray.length; i++) {
                
                    map[titleArray[i]] = relativeHrefArray[i];
                }
            }
            //set it to the static pools grid view
            var publishobj = [];
            publishobj.map = map;
            publishobj.treeID = Constants.STATICPOOL_TREE_TITLE;
            publishobj.gridID = Constants.STATICPOOL_GRID_TITLE;
            
            dojo.publish(Constants.ON_STATICPOOLTYPE_SELECTION, [publishobj]);
            
        },
        
        
        /*
         * Gets and loads all the pool types from the server and sets it the type combo
         * Incoming : None
         * Outgoing : None
         */
        _loadPoolTypeOptionsData: function(){
        
            var self = this;
            
            var serviceObject = {
            
                success: function(data){
                    var map = [];
                    if (!data) {
                    
                        return null;
                    }
                    
                    var title = "", id = "";
                    var relativeHref = "";
                    
                    for (var i = 0; i < data.length; i++) {
                        title = title + "," + data[i].title;
                        relativeHref = relativeHref + "," + data[i].relativeHref;
                        id = id + "," + relativeHref.substring(relativeHref.lastIndexOf("/") + 1);
                    }
                    if (title != "") {
                        title = title.substring(1);
                    }
                    
                    if (id != "") {
                    
                        id = id.substring(1);
                    }
                    if (relativeHref != "") {
                        relativeHref = relativeHref.substring(1);
                    }
                    var titleArray = title.split(",");
                    var relativeHrefArray = relativeHref.split(",");
                    self._createTypeMapAndPublish(titleArray, relativeHrefArray);
                    self._staticpoolsGrid.setSelectOptions(titleArray);
                    
                },
                
                failure: function(errObj){
                    return null;
                }
                
            };
            var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
            var relativeURI = Constants.STATICPOOLTYPE_URI;
            StaticPoolClient.getURI(srh, relativeURI);
            
        },
        
        /*
         * Set whether save button is enabled or disabled.
         * Pass true to enable, false to disable.
         */
        enableSaveButton: function(state) {
            this.saveButton.disabled = !state;
        },
        
        /*
         * Creating the toolbar. Pools tree-grid views, tree & grid will not have separate toolbar.
         * It's a common toolbar will have Add, Save, Refresh icons.
         *  Incoming : None
         *  Outgoing : None
         */
        _createToolbar: function(){
            
			 this._staticpoolsGridToolBar = new dijit.Toolbar({
                layoutAlign: "top",
                style: "background : transparent; border: none;float:right"
            });
			
			  this.addButton = new dijit.form.Button({
                title: StaticPoolViewletMessages.ADD,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addStaticPools)
            });
			this._staticpoolsGridToolBar.addChild(this.addButton);
			
			this.deleteButton = new dijit.form.Button({
                title: StaticPoolViewletMessages.DELETE_POOL,
				label : StaticPoolViewletMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deletePools)
            });
			
			
		
			this._staticpoolsGridToolBar.addChild(this.deleteButton);
			
			this.poolSeparator = new dijit.ToolbarSeparator();
            
			this._staticpoolsGridToolBar.addChild(this.poolSeparator);
			
			this.setCSHelpIdInToolbar(this.contextHelpId, this._staticpoolsGridToolBar);
			
            this.refreshButton = new dijit.form.Button({
                title: StaticPoolViewletMessages.REFRESH,
				label : StaticPoolViewletMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshStaticPools)
            });
			
			this._staticpoolsGridToolBar.addChild(this.refreshButton);
			
		    this._staticpoolsGridImageButtons.appendChild(this._staticpoolsGridToolBar.domNode);
        },
        /**
		 * The method _deletePools will be called whenever the delete button is clicked
		 */
		_deletePools: function(){
			// Only admin can delete user
            if (!this._userIsAdmin) {
				this._displayErrorMessage(StaticPoolViewletMessages.CANNOT_ADD_OR_DELETE_POOL_NO_ADMIN_ACCESS);
				return;
			}
			 if (this._staticpoolsGrid) {
			 	this._staticpoolsGrid._deleteStaticPools();
            }
			
		},
		
        /*
         * Creating the toolbar. Pools tree-grid views, tree & grid will not have separate toolbar.
         * It's a common toolbar will have Add, Delete, Save, Refresh icons.
         *  Incoming : None
         *  Outgoing : None
         */
        _createUserAssignToolbar: function(){
            //			this._staticpoolUsersGridToolBar1 = new dijit.Toolbar({layoutAlign: "top", style: "align:center"});
            //			this.addButton = new dijit.form.Button({title: "Add User", iconClass: "AddButtonIcon",onClick: dojo.hitch(this, this._addUserToStaticPool)});
            //			this._staticpoolUsersGridToolBar1.addChild(this.addButton);				
            //			this._userAssignmentButton1.appendChild(this._staticpoolUsersGridToolBar1.domNode);
            //			
            //			this._staticpoolUsersGridToolBar2 = new dijit.Toolbar({layoutAlign: "top", style: "align:center"});
            //			this.removeButton = new dijit.form.Button({title:"Remove User",iconClass: "DeleteButtonIcon", onClick: dojo.hitch(this, this._removeUserFromStaticPool)});
            //			this._staticpoolUsersGridToolBar2.addChild(this.removeButton);
            //			this._userAssignmentButton2.appendChild(this._staticpoolUsersGridToolBar2.domNode);
        
        },
        
        /*
         * creates static pool assigned users grid
         * Incoming : publishedItem static pool selected in the tree
         * Outgoing : None
         */
        _createStaticPoolAssignedUsersGrid: function(publishedItem){
        
        	
            if (publishedItem.treeID == Constants.STATICPOOL_TREE_TITLE && publishedItem.identifier !== null) {
                
				//checking if the static pool is unsaved
                var relatedHref = publishedItem.href;
                if (relatedHref.toString().indexOf(Constants.NEW_ELEMENT_PREFIX) == -1) {
                    if (this._staticpoolAssignedUsersGrid) {
                    
                        this._staticpoolAssignedUsersGrid.removeAllRows();
                    }
                    this.selectedStaticPoolHref = publishedItem.identifier;
                    this._buildStaticPoolAssignedUsersGrid();
                }
				else {
					this.selectedStaticPoolHref=relatedHref;
				}
            }
            
        },
        
        /*
         * If user clicked on delete button in the grid, unassign the user from the pool.
         * Do not allow if user is not an admin.
         */
        onCellClick: function(event){
            if (event.gridID == "StaticPoolAssignedUsersGrid") {
                if (this._userIsAdmin && (event.cell.name == StaticPoolViewletMessages.DELETE)) {
                    var rowIndex = event.rowIndex;
                    this._removeUserFromStaticPool(rowIndex);
                }
            }
        },
        
        
        /*
         * Method which builds the static pool assigned users grid if not present
         * Incoming : None
         * Outgoing : None
         */
        _buildStaticPoolAssignedUsersGrid: function(){
            if (this._staticpoolAssignedUsersGrid == null) {
                this._staticpoolAssignedUsersGrid = new com.ibm.epm.resource.web.ui.internal.StaticPoolAssignedUsersGridView({
                    _treeID: "user_unsaved",
                    containerViewlet: this,
                    modifyAllowed: this._userIsAdmin
                });
                this._userAssocgridContent.removeChild(this.noGridLabelDiv);
                this.noGridLabelDiv = null;
                this._userAssocgridContent.appendChild(this._staticpoolAssignedUsersGrid.domNode);
                
                this._staticpoolAssignedUsersGrid.buildStaticPoolAssignedUsersGrid("xx");
            }
            
            this._staticpoolAssignedUsersGrid.addUsers(this.selectedStaticPoolHref);
            
        },
		
         /*
         * Displays the Error Message
         * Incoming : Error message
         * Outgoing : None
         */
        _displayErrorMessage: function(errorMessage){
            this._displayAndClearErrorMessage(errorMessage);
        },
		/*
		 * Clears the error Message
		 * Incoming : None
		 * Outgoing : None
		 */
		 _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
        /*
         * Display and clear the error message
         * incoming : Error Message Object
         * OutGoing : None
         */
        _displayAndClearErrorMessage: function(errorMessage){
            var self = this;
            EPMErrorHandler.setError(errorMessage);
            
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);
        },
        /*
         * Adding an user to the selected static pool, ie tree node selected
         * Incoming : None
         * Outgoing : None
         */
        _addUserToStaticPool: function(){
            // Only admin can add user
            if (!this._userIsAdmin) {
				this._displayErrorMessage(StaticPoolViewletMessages.CANNOT_ADD_OR_DELETE_POOL_NO_ADMIN_ACCESS);
				return;
			}

			if(!this.selectedStaticPoolHref && (this.saveButton.disabled=true)){
				this._displayErrorMessage(StaticPoolViewletMessages.NO_POOL_SELECTED_WHILE_ASSIGNING_USER);
				return;
			}
			
			if ( (this.selectedStaticPoolHref && 
			   (this.selectedStaticPoolHref.toString().indexOf(Constants.NEW_ELEMENT_PREFIX) != -1 ))){
				 this._displayErrorMessage(StaticPoolViewletMessages.SAVE_POOL_BEFORE_ASSIGNING);
				   return;
			}
                
            if (this.selectedStaticPoolHref) {
                var disableAlreadyAssignedUsers = this._staticpoolAssignedUsersGrid.getAllAssignedUsers();
                this.ownerListWidget = new SelectContributorDialog({
                    hideAdminGuest: true,
                    allowMultiple: false,
                    disableUsers: disableAlreadyAssignedUsers
                });
                this.connect(this.ownerListWidget, "onOK", dojo.hitch(this, function(selectedUserList){
                
                    this._staticpoolAssignedUsersGrid._addUserToStaticPool(selectedUserList, this.selectedStaticPoolHref);
                }));
                this.ownerListWidget.searchText.focus();
            }
            //			if(this._staticpoolUsersGrid)
            //			{
            //				var selectedUsers=this._staticpoolUsersGrid.getSelectedUsers();
            //				
            //				if(this._staticpoolAssignedUsersGrid && this.selectedStaticPoolHref){
            //					this._staticpoolAssignedUsersGrid._addUserToStaticPool(selectedUsers,this.selectedStaticPoolHref);
            //				}
            //				
            //			}
            //reload the grid  - TODO : reload not required.. reload the grid only when tree selection changes
            //this._removeStaticPoolAssignedUsersGrid();
            //this._buildStaticPoolAssignedUsersGrid();
        
        },
        /*
         * Removing an user from the selected static pool
         * Incoming : rowIndex
         * Outgoing : None
         */
        _removeUserFromStaticPool: function(rowIndex){
            if (this._staticpoolAssignedUsersGrid && this.selectedStaticPoolHref) {
                this._staticpoolAssignedUsersGrid._removeUsersFromStaticPool(this.selectedStaticPoolHref, rowIndex);
            }
            //reload the grid   
            //this._removeStaticPoolAssignedUsersGrid();
            //this._buildStaticPoolAssignedUsersGrid();
        },
        
        
        /*
         * calls destroy for removing widgets and rebuild them afresh
         * Incoming:None
         * Outgoing :None
         */
        _refreshStaticPools: function(){
            this.enableSaveButton(false);
            this.destroyWidgets();
            this._createWidgets();
            this._addLabel();
        },
        
        /*
         * Adds a Label stating that no pool is selected
         * Incoming: None
         * Outgoing : None
         */
        _addLabel: function(){
            if (this._staticpoolAssignedUsersGrid == null) {
                this.noGridLabelDiv = document.createElement("div");
                dojo.addClass(this.noGridLabelDiv, "com-ibm-epm-resource-web-ui-internal-staticPoolsViewlet-labelDiv");
                var noGridLabel = document.createElement("label");
                noGridLabel.innerHTML = StaticPoolViewletMessages.NO_POOL_SELECTED;
                this.noGridLabelDiv.appendChild(noGridLabel);
                this._userAssocgridContent.appendChild(this.noGridLabelDiv);
            }
        },
        
        /*
         * Adds a static pool as a root or a child depending on user request
         * Incoming: None
         * Outgoing : None
         */
        _addStaticPools: function(){
            if (this._staticpoolsTree) {
				if (!this._userIsAdmin) {
					this._displayErrorMessage(StaticPoolViewletMessages.CANNOT_ADD_OR_DELETE_POOL_NO_ADMIN_ACCESS);
					return;
				}

                this._staticpoolsTree._createRootStaticPool();
            }
        },
        
        /* To persist the static pool data in the db.Calls grid save.
         * Incoming : None
         * Outgoing : None
         */
        _saveStaticPools: function(){
            if (this._staticpoolsGrid) {
                this._staticpoolsGrid._saveStaticPools();
            }
        },
        /*
         * removing the selected users from the selected static pool
         * Incoming : None
         * Outgoing : None
         */
        _removeStaticPoolAssignedUsersGrid: function(){
        
            if (this._staticpoolAssignedUsersGrid) {
                this._userAssocgridContent.removeChild(this._staticpoolAssignedUsersGrid.domNode);
                
                this._staticpoolAssignedUsersGrid.destroy();
                
                delete this._staticpoolAssignedUsersGrid;
            }
        },
        /*
         * Destroying the widgets and dettaching all the grids, tree's domnode ,tool bar
         * Incoming : None
         * Outgoing : None
         */
        destroyWidgets: function(){
        
            if (this.treeSelectHandler) {
            
                dojo.unsubscribe(this.treeSelectHandler);
                this.treeSelectHandler = null;
            }
			if (this.gridRowClickHandler){
				 dojo.unsubscribe(this.gridRowClickHandler);
                this.gridRowClickHandler = null;
			}
            
            if (this._staticpoolsTree) {
                this._staticpoolsTree.destroy();
				if(this._treeContent && this._treeContent.children && this._treeContent.children.length > 0){
                	this._treeContent.removeChild(this._staticpoolsTree.domNode);
				}
                delete this._staticpoolsTree;
                
            }
		    if (this._staticpoolsGrid) {
				this._staticpoolsGrid.destroy();
				if(this._gridContent && this._gridContent.children && this._gridContent.children.length > 0){
                	this._gridContent.removeChild(this._staticpoolsGrid.domNode);
				}
                delete this._staticpoolsGrid;
            }
            
            if (this._staticpoolUsersGrid) {
                this._staticpoolUsersGrid.destroy();
				if (this._usergridContent && this._usergridContent.children && this._usergridContent.children.length > 0) {
					this._usergridContent.removeChild(this._staticpoolUsersGrid.domNode);
				}
                delete this._staticpoolUsersGrid;
            }
            
            if (this._staticpoolAssignedUsersGrid) {
                this._staticpoolAssignedUsersGrid.destroy();
                if (this._userAssocgridContent && this._userAssocgridContent.children && this._userAssocgridContent.children.length > 0) {
					this._userAssocgridContent.removeChild(this._staticpoolAssignedUsersGrid.domNode);
				}
                delete this._staticpoolAssignedUsersGrid;
            }
            if (this.noGridLabelDiv) {
                this._userAssocgridContent.removeChild(this.noGridLabelDiv);
            }
            
            
        }//destroy
    });
})();
