/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){

    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimecodeEditor");
    
    dojo.require("dojox.grid.compat._data.dijitEditors");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreeDialog");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
    dojo.require("com.ibm.epm.web.core.ui.internal.MouseTooltip");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes");
    
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    
    var TimecodesTreeWidget = com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget;
    var TimecodesGridView = com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView;
    var EPMTreeDialog = com.ibm.epm.web.core.ui.internal.EPMTreeDialog;
    var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var MouseTooltip = com.ibm.epm.web.core.ui.internal.MouseTooltip;
    var TimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    var TimesheetAttributes = com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes;
    var HIDE_DELAY = 5000;
    var SHOW_DELAY = 500;
    
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimecodeEditor", [dijit._Widget], {
    
        _containerViewlet: null,
        _selectedRow: null,
        _grid: null,
		_editable: true,
        
        postCreate: function(){
        
            this.inherited(arguments);
            dojo.subscribe(EPMTreePubSub.ON_MOUSE_OVER, this, "_onTreeMouseOver");
            dojo.subscribe(EPMTreePubSub.ON_MOUSE_OUT, this, "_onTreeMouseOut");
            this.mouseTooltip = new MouseTooltip();
            this._loadProjectTimecodeDialog();
        },
        
        /**
         * _onGridCellMouseOver: To display the timecode attributes as tooltip
         * @param {Object} e
         */
        _onTreeMouseOver: function(e){
            if (this._TimecodesGrid) {
                if (dojo.isArray(e.href)) {
                    this._selectedrowID = e.href[0];
                }
                else {
                    this._selectedrowID = e.href;
                }
				var domTreeNode = this._getTreeDomNode();
                if ((this._TimecodesTree) && (this._TimecodesTree.menuImage)){					
					dojo.style(this._TimecodesTree.menuImage,"visibility","hidden");
				}
            
                this._computeHint(domTreeNode);
            }
        },

		/**
         * _getTreeDomNode: Method to get the tree node's domNode
         */
        _getTreeDomNode: function(){
            var domTreeNode = null;            
            if ((this._TimecodesTree) && (this._TimecodesTree._timecodesTree)) {
                domTreeNode = this._TimecodesTree._timecodesTree.getNodeHover();
            }                        
            return domTreeNode ? domTreeNode.contentNode : null;
        },
		
        /**
         * 	_computeHint: 	Retrieve the attributes of the selected timecode in the grid,
         * 					copies to tooltip
         * 	@param {Object} e
         * 					event e is used to get the selected row of timecode grid,
         * 					copies the required attributes to toolTipInfo
         */
        _computeHint: function(e){
        
            if (this._TimecodesGrid._controller && this._selectedrowID) {
            
                var rowIndex = this._TimecodesGrid._controller.getRowIndex(this._selectedrowID);
                var selectedRow = rowIndex!=null ? this._TimecodesGrid._controller.getRowByIndex(rowIndex) :null;
                
                if (selectedRow) {
                    if (!this._showTimer) {
                        this._showTimer = setTimeout(dojo.hitch(this, function(){
                            var toolTipInfo = {
                                "Name": selectedRow.name,
                                "Description": selectedRow.description,
                                "Code": selectedRow.code
                            };
                            this.showTooltip(toolTipInfo, e);
                        }), SHOW_DELAY);
                    }
                    
                    if (!this._hideTimer) {
                        this._hideTimer = setTimeout(dojo.hitch(this, function(){
                            this.hideTooltip(e);
                        }), HIDE_DELAY);
                    }
                }
            }
        },
        
        /**
         * showTooltip: Creates and displays the tooltip
         * 				Uses mouseTooltip to display tooltip
         * @param {Object} toolTipInfo
         * @param {Object} e
         */
        showTooltip: function(toolTipInfo, e){
            //ToolTipInfo structure is :
            //"Name"
            //"Description"
            var msg = "<b>";
            if (toolTipInfo.Name) {
                msg += TimesheetViewletMessages.TIMECODE_NAME + ": " + toolTipInfo.Name;
                msg += "</b><br>";
            }
            if (toolTipInfo.Description) {
                msg += TimesheetViewletMessages.TIMECODE_DESCRIPTION + ": " + toolTipInfo.Description;
                msg += "<br>";
            }
            if (toolTipInfo.Code) {
                msg += TimesheetViewletMessages.TIMECODE_CODE + ": " + toolTipInfo.Code;
            }
            
            this.mouseTooltip._open({
                innerHTML: msg,
                target: e,
                zIndex: 10001
            });
        },
        
        /**
         * _onGridCellMouseOut: To remove the tooltip on mouse over
         * @param {Object} e
         */
        _onTreeMouseOut: function(e){
            if (this._showTimer) {
                clearTimeout(this._showTimer);
                delete this._showTimer;
            }
            if (this._hideTimer) {
                clearTimeout(this._hideTimer);
                delete this._hideTimer;
            }
			var domTreeNode = this._getTreeDomNode();
            if (this.mouseTooltip) {
                this.hideTooltip(domTreeNode);
            }
        },
        
        /**
         * hideTooltip: To close the created mouseTooltip
         * 				Uses mouseTooltip close() method
         * @param {Object} e
         */
        hideTooltip: function(e){
            this.mouseTooltip._close(e);
            dijit._masterTT._onDeck = null;
        },
        
        /**
         * Sets the project href of the task assignment.
         * Created the timecode widget using the project href as one of the
         * parameter. This ensures that only the timecode assigned to that
         * particular project are displayed.
         */
        _loadProjectTimecodeDialog: function(){
        
            this._projectURI = this._selectedRow[TimesheetAttributes.containingProject];
            this._createTimecodeTreeWidget();
        },
        
        /**
         *  Creates the timecode tree widget and the timecode grid widget using the project href.
         *  Appends the tree widget to the dialog box.
         *  2 buttons are also appended to assign and remove the timecode to/from the task assignment.
         */
        _createTimecodeTreeWidget: function(){
        
			this._destroyTimecodeWidget();
            this._TimecodesTree = new TimecodesTreeWidget({
                _projectID: this._projectURI
            });
            
            this._TimecodesTree._buildTimecodesTree(/*renameAllowed*/false);
            this._TimecodesGrid = new TimecodesGridView({
                _treeID: "TimecodesTree"
            });
            this._TimecodesGrid.buildTimecodesGrid();
            
            var buttonlist = [{
                label: TimesheetViewletMessages.TIMECODE_ASSIGN,
				disabled: this._editable,
                action: dojo.hitch(this, this._assignTimecode)
            }, {
                label: TimesheetViewletMessages.TIMECODE_REMOVE,
				disabled: this._editable,
                action: dojo.hitch(this, this._removeTimecode)
            }];
            
            this.timecodeTreeDialog = new EPMTreeDialog({
                tree: this._TimecodesTree,
                title: TimesheetViewletMessages.TIMECODE_TREE_TITLE,
                buttonlist: buttonlist
            });

			/*
			 * Earlier, onCancel was not destroying the treeWidget. Hence the cached widget was getting called
			 * when user tries to populate timecode/chargecode tree from the financial tab.
			 */ 
            dojo.connect(this.timecodeTreeDialog.dialog, "onCancel", this, 
                this._closedialog);
            
            this.timecodeTreeDialog.dialog.closeButtonNode.title = TimesheetViewletMessages.CLOSE;
            this.timecodeTreeDialog.show();
            
        },
        
        /**
         * It checks if the cell clicked is editable or not.
         * If the timesheet is submitted/approved or if the logged in user is the approver,
         * then user should not be allowed to assign/reject timecode
         */
        _checkIfCellEditable: function(){
        
            var timesheetApprovalState = "";
            timesheetApprovalState = this._selectedRow.timesheetApprovalState;
            if (this._selectedRow[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED ||
            this._selectedRow[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_APPROVED) {
                return false;
            }
            return true;
        },
		
		_destroyTimecodeWidget: function(){
			
			if(this._TimecodesTree){
				this._TimecodesTree.destroy();
				delete this._TimecodesTree;
				
			}
			if(this._TimecodesGrid){
				
				this._TimecodesGrid.destroy();
				delete this._TimecodesGrid;
			}
			if(this._TimecodesGrid){
				this._TimecodesGrid = null;
			}
		},
		
		_destroyTimecodeTreeDialog: function(){
			
			this._destroyTimecodeWidget();
			
			this.timecodeTreeDialog.destroy();
		},
		
		_closedialog: function(){
            this._destroyTimecodeTreeDialog();
        },
        
        /**
         *  This function is called when the assign button is clicked.
         *  The user selects the timecode from the tree displayed and clicks the assign button.
         *  Note: Presently only one timecode can be selected. Multi select cannot be done. It is
         *  a restriction in tree.
         */
        _assignTimecode: function(){
        
            if (!this._checkIfCellEditable()) {
                this._destroyTimecodeTreeDialog();
                return; //TODO: Display the appropriate message.
            }
            if ((this._TimecodesGrid) && (this._TimecodesGrid._generalGrid)) {
            
                var selectedRowIndexes = this._TimecodesGrid._generalGrid.selection.getSelected();
                if (!selectedRowIndexes || selectedRowIndexes === null) {
                    this._destroyTimecodeTreeDialog();
                    return;
                }
                var numberOfSelectedTimecodes = selectedRowIndexes.length;
                for (var i = 0; i < numberOfSelectedTimecodes; i++) {
                    var selectedRow = this._TimecodesGrid._generalGrid.model.getRow(selectedRowIndexes[i]);
                    this._addToGrid(selectedRow);
                }
            }
            this._destroyTimecodeTreeDialog();
        },
		
        /**
         *
         * @param {Object} selectedRow : The corresponding row of the grid when the tree node is selected.
         *
         * Because of the support of multiple timecode assignment, the list of timecodes are displayed as
         * comma separated. The values are set in the timesheet grid.
         */
        _addToGrid: function(selectedRow){
        
            var existingTimecode = this._selectedRow[TimesheetAttributes.timecode];
            var existingTimecodeURI = this._selectedRow[TimesheetAttributes.timecodeURI];
            var item = null;
            
            if (existingTimecodeURI === "" || existingTimecodeURI === null) {
                this._grid.edit.apply();
                if (this._containerViewlet.containerViewlet.toggleSaveButton) {
                    this._containerViewlet.containerViewlet.toggleSaveButton(false);
                }
                
                // This is required in order to update the timesheet store and push the modified data 
                // to the pending items of the store.
                item = this._grid.model.store._getItemByIdentity(this._selectedRow[TimesheetAttributes.assignmentURI]);
				
                this._grid.model.store._setValueOrValues(item, TimesheetAttributes.timecodeURI, selectedRow.href, true);
                this._grid.model.store._setValueOrValues(item, TimesheetAttributes.timecode, selectedRow.name, true);
            }
            else {
            
                if (!this._checkIfTimecodePresent(existingTimecodeURI, selectedRow.href)) {
                
                    var tempTimecode = this._selectedRow[TimesheetAttributes.timecode];
                    tempTimecode = tempTimecode + "," + selectedRow.name;
                    var tempTimecodeURI = this._selectedRow[TimesheetAttributes.timecodeURI];
                    tempTimecodeURI = tempTimecodeURI + "," + selectedRow.href;
                    this._grid.edit.apply();
                    if (this._containerViewlet.containerViewlet.toggleSaveButton) {
                        this._containerViewlet.containerViewlet.toggleSaveButton(false);
                    }
                    
                    // This is required in order to update the timesheet store and push the modified data 
                    // to the pending items of the store.
                    item = this._grid.model.store._getItemByIdentity(this._selectedRow[TimesheetAttributes.assignmentURI]);
					
					var timecodeURI = selectedRow.href;
					
                    this._grid.model.store._setValueOrValues(item, TimesheetAttributes.timecodeURI, tempTimecodeURI, true);
                    this._grid.model.store._setValueOrValues(item, TimesheetAttributes.timecode, tempTimecode, true);
                }
            }
            this._grid.render();
        },
        
        /**
         *
         * @param {Object} listOfTimecodeURI
         * @param {Object} selectedTimecodeURI
         *
         * It checks if the timecode which needs to be assigned/removed from the task assignment exists or not.
         */
        _checkIfTimecodePresent: function(listOfTimecodeURI, selectedTimecodeURI){
        
            var tempTimecodeURIArray = new Array(1);
            var isTimecodePresent = false;
            tempTimecodeURIArray = listOfTimecodeURI.split(",");
            var numberOfTimecodes = tempTimecodeURIArray.length;
            for (var i = 0; i < numberOfTimecodes; i++) {
            
                if (tempTimecodeURIArray[i] === selectedTimecodeURI) {
                    isTimecodePresent = true;
                    break;
                }
            }
            return isTimecodePresent;
        },
        
        /**
         * This function is called when the remove button is clicked.
         * The selected timecode is removed from the timesheet grid.
         */
        _removeTimecode: function(){
        
            if (!this._checkIfCellEditable()) {
                this._destroyTimecodeTreeDialog();
                return; //TODO: Display the appropriate message.
            }
            if ((this._TimecodesGrid) && (this._TimecodesGrid._generalGrid)) {
                var selectedRowIndexes = this._TimecodesGrid._generalGrid.selection.getSelected();
                if (!selectedRowIndexes || selectedRowIndexes === null) {
                    this._destroyTimecodeTreeDialog();
                    return;
                }
                var numberOfSelectedTimecodes = selectedRowIndexes.length;
                for (var i = 0; i < numberOfSelectedTimecodes; i++) {
                    var selectedRow = this._TimecodesGrid._generalGrid.model.getRow(selectedRowIndexes[i]);
                    this._removeFromGrid(selectedRow);
                }
            }
            this._destroyTimecodeTreeDialog();
        },
        
        /**
         *
         * @param {Object} selectedRow : The corresponding row of the grid when the tree node is selected.
         */
        _removeFromGrid: function(selectedRow){
        
            var existingTimecode = this._selectedRow[TimesheetAttributes.timecode];
            var existingTimecodeURI = this._selectedRow[TimesheetAttributes.timecodeURI];
            
            if (existingTimecodeURI !== "" || existingTimecodeURI !== null) {
            
                this._checkAndRemoveIfTimecodePresent(existingTimecodeURI, existingTimecode, selectedRow);
                this._grid.render();
            }
            
        },
        
        /**
         *
         * @param {Object} listOfTimecodeURI
         * @param {Object} existingTimecode
         * @param {Object} selectedRow
         *
         * It removes the timecode selected from the task assignment. First it checks if the timecode exists or not.
         */
        _checkAndRemoveIfTimecodePresent: function(listOfTimecodeURI, existingTimecode, selectedRow){
        
            var tempTimecodeURIArray = new Array(1);
            var tempTimecodeNameArray = new Array(1);
            var newListOfTimecodeURI = "";
            var newListOfTimecodeName = "";
            var isTimecodePresent = false;
            tempTimecodeURIArray = listOfTimecodeURI.split(",");
            tempTimecodeNameArray = existingTimecode.split(",");
            var numberOfTimecodes = tempTimecodeURIArray.length;
            for (var i = 0; i < numberOfTimecodes; i++) {
            
                if (tempTimecodeURIArray[i] !== selectedRow.href) {
                
                    if (newListOfTimecodeURI !== "") {
                        newListOfTimecodeURI = newListOfTimecodeURI + "," + tempTimecodeURIArray[i];
                        newListOfTimecodeName = newListOfTimecodeName + "," + tempTimecodeNameArray[i];
                    }
                    else {
                        newListOfTimecodeURI = tempTimecodeURIArray[i];
                        newListOfTimecodeName = tempTimecodeNameArray[i];
                    }
                }
            }
            this._grid.edit.apply();
            if (this._containerViewlet.containerViewlet.toggleSaveButton) {
                this._containerViewlet.containerViewlet.toggleSaveButton(false);
            }
            
            // This is required in order to update the timesheet store and push the modified data 
            // to the pending items of the store.
            var item = this._grid.model.store._getItemByIdentity(this._selectedRow[TimesheetAttributes.assignmentURI]);
            this._grid.model.store._setValueOrValues(item, TimesheetAttributes.timecodeURI, newListOfTimecodeURI, true);
            this._grid.model.store._setValueOrValues(item, TimesheetAttributes.timecode, newListOfTimecodeName, true);
        }
        
    });
})();
