/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridController");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.EPMGridController");
    dojo.require("com.ibm.epm.web.core.client.QueryUtil");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
	
    var EPMGridController = com.ibm.epm.web.core.grid.controller.EPMGridController;
    var QueryUtil = com.ibm.epm.web.core.client.QueryUtil;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var GridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
	
	
	
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridController", EPMGridController, {
    
		containerViewlet : "",
		numberOfTaskAssignment: 0,
		
		_postParams:"",
        /* 
         * postCreate calling the super function
         */
        postCreate: function(){
            this.inherited(arguments);
        },
		
		getNumberOfTaskAssignment: function(){
			
			return this.numberOfTaskAssignment;
		},
        
        _get_n_updateModel: function(){
           
		    var self = this;
            
            var getAllElementsHandlers = {
                
				_updateAllElements: function(epmQueryResult){
                
                    var queryResult = null;
					
					if (epmQueryResult) {
                        queryResult = QueryUtil.getQueryResultSet(epmQueryResult);
                    }
					
					if (queryResult !== null) {
						
						self.numberOfTaskAssignment = queryResult.length;
						var collectionElements = [];
                        for (var i = 0; i < queryResult.length; i++) {
                    
					        collectionElements.push(queryResult[i].self);
                        }
					}	
					
                    if (collectionElements) {
                    
                        if (!dojo.isArray(collectionElements)) { // for single elements it will not be a array.
                            collectionElements = [collectionElements];
                        }
                        // If the Reuslt doesnt contain any information then create blank grid
                        if (collectionElements.length === 0) {
                            self._createBlankRow();
                        }
                        else {
                            var massagedLinks = this.massageLinks(collectionElements);
                            self.updateModel(false, massagedLinks);
                        }
                    }
                    else {
                        self._createBlankRow();
                    }
                },
				
                _getAllElementsFailure: function(errorMsg){
					self._propogateErrorMessage(errorMsg);
                    self._createBlankRow();
                    
                },
                
				massageLinks: function(collectionElementsLinks){
                    if (!collectionElementsLinks || collectionElementsLinks.length === 0) {
                        return null;
                    }
                    var massagedLinks = new Array(1);
                    for (var i in collectionElementsLinks) {
                        massagedLinks.push(collectionElementsLinks[i]);
                    }
                    return massagedLinks;
                }
                
            };
            var srh = new ServiceResponseHandler(getAllElementsHandlers, "_updateAllElements", "_getAllElementsFailure");
            
            //Fetch the data from the server
            // Display blank grid if the data is not available
            if (this._allElementsURI !== null && this._allElementsURI !== "" && this._allElementsURI.indexOf(this.newElementPrefix) === -1) {            
               
			   var tempPostParam = {};
			   tempPostParam["jsonExpression"] = this._postParams;
			   GridModelClient.postQueryURI(srh, this._allElementsURI, tempPostParam);
            }
            else {
            
                self._createBlankRow();
                
            }
        },
		
		_persistRow: function(saveObject, row, rowIndex, linkName){
            var rowID = row.id;
            var self = this;
            var reloadURI = saveObject.assignmentURI;
            
            var persistRowFunctions = {
                
				_persistRowSuccess: function(savedElement){
                    
					if (savedElement) {
                        if (saveObject.isNew === true) {
                            var publishObj = self.getReplaceNodePublishObject(row, rowID, rowIndex, savedElement);
                            dojo.publish(GridEvents.REPLACE_NODE, [publishObj]);
                        }
                        else {
                            self._saveLinks(null, row);
                        }
                        if (savedElement.relativeHref) {
                            reloadURI = savedElement.relativeHref;
                        }
                    }
                    self._getAndUpdateRow(saveObject.assignmentURI, rowID, linkName);
                },
                
                _persistRowFailure: function(errorMsg){
                    
                    //Update the row with the correct data.
            		//Defect 13226 : Disable the get call for all failures
//                    if (saveObject.isNew === false) {
//                        self._getAndUpdateRow(reloadURI, rowID, linkName);
//                    }
                    self._propogateErrorMessage(errorMsg, row, rowID);
                }
            };
            
            var srh = new ServiceResponseHandler(persistRowFunctions, "_persistRowSuccess", "_persistRowFailure");
            if (saveObject && saveObject.rowURI && saveObject.params) {
                GridModelClient.postURI(srh, saveObject.rowURI, saveObject.params);
            }
        },
		
		getPostParams: function()
		{
			return this._postParams;
		},
		
		setPostParams: function(postParams)
		{
			this._postParams = postParams;
		}
        
    });
})();
