/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    
	dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridView");
    
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridModel");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridController");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetTree");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimecodeEditor");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetNotesView");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreeDialog");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
	dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	
	dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	
	dojo.require("jazz.ui.Dialog");
	
	dojo.require("dijit.form.Textarea");
	dojo.require("dijit.Tooltip");
	
    // Localization
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    
    var AbstractGridView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var TimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
	var TimesheetAttributes = com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes;
	var TimesheetGridModel = com.ibm.epm.timesheet.web.ui.internal.TimesheetGridModel;
	var TimesheetGridController = com.ibm.epm.timesheet.web.ui.internal.TimesheetGridController;
	var TimesheetUtil = com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil;
	var TimesheetTree = com.ibm.epm.timesheet.web.ui.internal.TimesheetTree;
	var TimecodeEditor = com.ibm.epm.timesheet.web.ui.internal.TimecodeEditor;
	var EPMTreeDialog = com.ibm.epm.web.core.ui.internal.EPMTreeDialog;
	var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
	var EPMDomNodeConstants = com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants;
	var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
	var DateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
	var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
	var TimesheetNotesView = com.ibm.epm.timesheet.web.ui.internal.TimesheetNotesView;
	
	var SelfSizingTextAreaWidget = com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget;
	
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridView", AbstractGridView, {
		
        templatePath: dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/templates/TimesheetGridView.html"),
		
		containerViewlet: "",
		_rowIndexSelected: 0,
		_postParams:"",
		
		_rowHoverArray : [],
		_hoverRowIndex:-1,
		/**
         * buildTimesheetGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with TimesheetGridView.
         */
        buildTimesheetGrid: function(){			
			
            if (!this._gridModel) {
                this._gridModel = new TimesheetGridModel({
					containerViewlet: this.containerViewlet
				});
            }
            
            this.timesheetGridController = new TimesheetGridController({
				containerViewlet: this.containerViewlet
				});
			
			this.timesheetGridController.setPostParams(this._postParams);
			
			if (this.timesheetGridController.isInitialized() === false) {
                this.timesheetGridController.initializeController(this._treeID, this, this._gridModel, [TimesheetAttributes.Timesheets, TimesheetAttributes.TaskAssigned]);
            }
            this.buildGrid(this, this._gridModel, this.timesheetGridController);
			
        },
		
		
		/**
		 * This function is called in AbstractGridView only when the generic grid is ready.
		 * So, in this function, we can connect to any events related to grid object.
		 * @param {Object} gridModel
		 */
		createView: function(gridModel){
		
			this.inherited(arguments);
			
			
			
			/*
             * There are many listeners also associated with the grid.
             * onStyleRow listener is called whenever the grid cell is
             * updated. Depending upon some condition, appropriate css
             * can be applied to the row or the cell.
             */
            dojo.connect(this._generalGrid, 'onStyleRow', this, this._styleRow);
            dojo.connect(this._generalGrid, 'onHeaderClick', this, this._setGridSort);
            dojo.connect(this._generalGrid, 'onCellClick', this, this._handleCellClick);
			dojo.connect(this._generalGrid, 'onCellDblClick', this, this._handleCellDblClick);
			dojo.connect(this._generalGrid, 'onRowClick', this, this._handleRowClickEvent);
			dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_MOUSE_OVER, this, this._onMouseOver);
			dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_MOUSE_OUT, this, this._onMouseOut);
			dojo.connect(this._generalGrid, "onRowMouseOver", this, this._onRowMouseOver);
            dojo.connect(this._generalGrid, "onRowMouseOut", this, this._onRowMouseOut);
			dojo.connect(this._generalGrid, EPMGridPubSub.ON_KEY_DOWN, this, this._onKeyDown);			
			
			this.taskTooltip = new dijit.Tooltip();
			
            this._generalGrid.isCellEditable = dojo.hitch(this, this.isCellEditable);			
						
			/*
             * There are various observers associated with the dojox grid.
             * Depending upon the requirement, we can associate any observer with the
             * grid and override the function.
             *
             * modelDatumChange observer is called whenever there is a change in any
             * of the grid cells.
             *
             * Here, we are using this observer to enable and disable the save button.
             */
			var self = this;
            var modelObservers = {
                modelDatumChange: function(value, row, column){
					
					var totalWork = self.getTotalWork(row);
					
					var tempRow = self.timesheetGridController.getRowByIndex(row);
					tempRow.totalWorkPerWeek = totalWork;
                    
					if (value === null) {
                        return;
                    }
                    var fieldName = self._generalGrid.model.fields.values[column].name;
                    if (!self._generalGrid.model.cache[row]) {
                        return;
                    }
                    var fieldValue = self._generalGrid.model.cache[row][fieldName];
                    if (!dojo.isString(value)) {
                        value = value.toString();
                    }
					
                    if (fieldValue.toString() !== value) {
                        if (self.containerViewlet.toggleSaveButton) {
                            self.containerViewlet.toggleSaveButton(false);
                        }
                    }
                }
    		};
    
    		this._generalGrid.model.observer(modelObservers);
			
			
			var intervalID = setInterval(function(){
            	
				var numberOfTaskAssignment = self.timesheetGridController.getNumberOfTaskAssignment();
				var rowCount = self._generalGrid.model.getRowCount();
                if (rowCount === numberOfTaskAssignment) {
                    
					clearInterval(intervalID);
					self.containerViewlet.toggleBusyStatusOfViewlet(false);
					self._selectFirstRowByDefault();
                }
            }, 50);
			
		},
		
		_onRowMouseOut: function(e)
		{
			if (e.rowIndex != -1) {
                
                //this._clearAllActionRows();
                this._clearOneActionRow(e.rowIndex);
            }
		},
		
		_onRowMouseOver: function(e)
		{
			if (e.rowIndex == -1){
                    return;
                }
			var row = this._generalGrid.model.getRow(e.rowIndex);
			if(!row){
				return;
			}
			
			this._clearAllActionRows();
			var disableApproveButton = false;
			var disableRejectButton = false;
			
			var timesheetState = row[TimesheetAttributes.timesheetApprovalState];
			
			if (timesheetState !== TimesheetConstants.TIMESHEET_STATUS_APPROVED) {
				disableApproveButton= false;
				
				if (timesheetState === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED) {
					disableRejectButton = false;
				} else {
					disableRejectButton = true;
				}
			} else {
				disableApproveButton = true;
				disableRejectButton = true;
			}								
			
			//TODO change from hardcoded row to actual one
			var cellobj = this._generalGrid.getCell(3);
            var cellDomNode = cellobj.getNode(e.rowIndex);
			
			cellDomNode.innerHTML = '<img name="contextTree" style="margin-right: 2px; margin-left: 1px; vertical-align:middle; cursor: pointer;" title="' + TimesheetViewletMessages.CONTEXT_TREE_LABEL +
	            '" alt="' + TimesheetViewletMessages.CONTEXT_TREE_LABEL + '" src =' +
	            dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/images/actions/contexttree_co.gif") +' />'+
				'&nbsp <img name="notesEditor" style="margin-right: 2px; margin-left: 2px; vertical-align:middle; cursor: pointer;" title="' + TimesheetViewletMessages.TIMESHEET_NOTES +
	            '" alt="' + TimesheetViewletMessages.TIMESHEET_NOTES + '" src =' +
	            dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/images/actions/notes_co.gif") +' />';
				
				if (this.containerViewlet.isProjectManager && !disableApproveButton) {
					cellDomNode.innerHTML +=
					'&nbsp <img name="approveTask" style="margin-right: 2px; margin-left: 2px; vertical-align:middle; cursor: pointer;" title="' + TimesheetViewletMessages.APPROVE_TASK_ACTION +
					'" alt="' + TimesheetViewletMessages.APPROVE_TASK_ACTION + '" src =' +
					dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/images/actions/approvetask_co.gif") +
					' />';
				}
				
				if (this.containerViewlet.isProjectManager && !disableRejectButton) {
					cellDomNode.innerHTML +=
					'&nbsp <img name="rejectTask" style="margin-right: 1px; margin-left: 2px; vertical-align:middle; cursor: pointer;" title="' + TimesheetViewletMessages.REJECT_TASK_ACTION +
					'" alt="' + TimesheetViewletMessages.REJECT_TASK_ACTION + '" src =' +
					dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/images/actions/rejecttask_co.gif") +
					' />';
				}
								
			this._rowHoverArray.push(e.rowIndex);	
			this._hoverRowIndex=e.rowIndex;
			
			e.stopPropagation();
            e.preventDefault();
            dojo.stopEvent(e);					
		},
		
		 _clearAllActionRows: function(){
            for (var i = 0; i < this._rowHoverArray.length; i++) {
                var rowIndex = this._rowHoverArray[i];
                this._clearOneActionRow(rowIndex);
            }
            this._rowHoverArray = [];
        },
		
		 _clearOneActionRow: function(rowIndex){
            var row = this._generalGrid.model.getRow(rowIndex);
            if (row) {
                var actionValue = "";
               
                    var cellobj = this._generalGrid.getCell(3);
                    var cellDomNode = cellobj.getNode(rowIndex);
                    cellDomNode.innerHTML = actionValue;
                
            }
        },
		
		_onMouseOver: function(e)
		{
			if (e.rowIndex == -1) {
				return;
			}
			
			if (e.cell.field == "title") {
                    var row = this._generalGrid.model.getRow(e.rowIndex);
                    
                    if (row) {
                        if (row.name != "") {
                            this.taskTooltip.label = this._createTooltipHTML(row).toString();//row.tooltip.toString();
                            this.taskTooltip.open(e.cellNode);
                        }
                    }
                }
		},
		
		_onKeyDown: function(e){
			//Donot process the row if an editor is open
            if (this._generalGrid.edit && this._generalGrid.edit.isEditing()) {
                return;
            }
			this._hoverRowIndex = e.rowIndex;
			var keys = dojo.keys;
			switch (e.keyCode){
				case 65 /*a - approve*/:
					if(this._canDoAction(e, TimesheetViewletMessages.APPROVE_TASK_ACTION)){
						this.approveTimesheet(true);
					}
					break;
				case 84 /*t - context tree*/:
					this._showContextTree(); 
					break;
				case 78 /*n - notes*/:
					this._showNotesDialog();
					break;
				case 82 /*r - reject time sheets*/:
					if(this._canDoAction(e, TimesheetViewletMessages.REJECT_TASK_ACTION)){
						this.rejectTimesheet(true);
					}
					break;
				case keys.ENTER:
					this._handleCellDblClick(e);
					break;
			}			
		},
		
		_canDoAction: function(e, action){
			var row = this._generalGrid.model.getRow(e.rowIndex);
            if (row) {
				var disableApproveButton = false;
				var disableRejectButton = false;
			
				var timesheetState = row[TimesheetAttributes.timesheetApprovalState];			
				if (timesheetState !== TimesheetConstants.TIMESHEET_STATUS_APPROVED) {
					disableApproveButton= false;
					
					if (timesheetState === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED) {
						disableRejectButton = false;
					} else {
						disableRejectButton = true;
					}
				} else {
					disableApproveButton = true;
					disableRejectButton = true;
				}								
				if (this.containerViewlet.isProjectManager && !disableApproveButton) {
					if(action == TimesheetViewletMessages.APPROVE_TASK_ACTION){
						return true;
					}
				}
				
				if (this.containerViewlet.isProjectManager && !disableRejectButton) {
					if(action == TimesheetViewletMessages.REJECT_TASK_ACTION){
						return true;
					}
				}
			}
			return false;
		},
		
		_createTooltipHTML:function(taskItem){
			var resultString = "";
			var div = '<div class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipMessage">'+TimesheetViewletMessages.TASK_SCHEDULE_DETAILS+ '</div>';
			resultString += div;
			var table = '<table class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTable">';
			table += '<tr>';			
			table += '<TD colspan="2" class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipHeader">'+TimesheetViewletMessages.PLANNED+'</TD>';
			
			table += '<TD colspan="2" class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipHeader">'+TimesheetViewletMessages.ACTUAL+'</TD>';
			
			table += '<TD colspan="2" class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipHeader">'+TimesheetViewletMessages.WORK+'</TD>';
			
			table += '</TR>';
			table += '<TR>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+TimesheetViewletMessages.STARTDATE+":"+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">'+DateUtil.formatDate(taskItem[TimesheetAttributes.plannedScheduleStartDate])+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+TimesheetViewletMessages.STARTDATE+":"+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">'+DateUtil.formatDate(taskItem[TimesheetAttributes.actualScheduleStartDate])+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+TimesheetViewletMessages.REMAINING+":"+'</TD>';
			
			if (taskItem[TimesheetAttributes.taskAssignmentType] == TimesheetAttributes.taskAssignmentTypeEffort) {
				table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipEffortCell">' + TimesheetUtil.convertMinutesToHours(taskItem[TimesheetAttributes.remainingWork]) + " " + TimesheetViewletMessages.HOURS + '</TD>';
			}
			else{
				table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipEffortCell">' + taskItem[TimesheetAttributes.remainingWork]+" " ;				
				table += taskItem[TimesheetAttributes.remainingWork]!=1? TimesheetViewletMessages.DAYS: TimesheetViewletMessages.DAY ;
				table += '</TD>';				
			}
			
			table += '</TR>';
			table += '<TR>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+TimesheetViewletMessages.FINISHDATE+":"+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">'+DateUtil.formatDate(taskItem[TimesheetAttributes.plannedScheduleFinishDate])+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+TimesheetViewletMessages.FINISHDATE+":"+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">'+DateUtil.formatDate(taskItem[TimesheetAttributes.actualScheduleFinishDate])+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+ TimesheetViewletMessages.COMPLETION+":"+'</TD>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipEffortCell">'+taskItem[TimesheetAttributes.percentageProgress]+ TimesheetViewletMessages.PERCENT + '</TD>';
			table += '</TR>';
			
			table += '<TR>';
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+TimesheetViewletMessages.WORK+":"+'</TD>';
			if (taskItem[TimesheetAttributes.taskAssignmentType] == TimesheetAttributes.taskAssignmentTypeEffort) {
				table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">' + taskItem[TimesheetAttributes.plannedScheduleEffort] + " " + TimesheetViewletMessages.HOURS + '</TD>';
			}
			else{
				table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">' + taskItem[TimesheetAttributes.plannedScheduleDuration] +" " ;
				table += taskItem[TimesheetAttributes.plannedScheduleDuration]!=1? TimesheetViewletMessages.DAYS: TimesheetViewletMessages.DAY ;
				table += '</TD>';
			}
			table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipTaskNameCell">'+"Work:"+'</TD>';
			if(taskItem[TimesheetAttributes.taskAssignmentType] == TimesheetAttributes.taskAssignmentTypeEffort){
				table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">'+taskItem[TimesheetAttributes.totalActualHours]+ " " + TimesheetViewletMessages.HOURS + '</TD>';	
			}
			else{
				table += '<TD class="com-ibm-epm-timesheet-web-ui-internal-TimesheetGridView-tooltipDatesCell">'+taskItem[TimesheetAttributes.totalActualDuration]+" ";
				table += taskItem[TimesheetAttributes.totalActualDuration]!=1? TimesheetViewletMessages.DAYS: TimesheetViewletMessages.DAY ;
				table += '</TD>';
			}
			
			table += '</TR>';
			table += '</TABLE>';
			resultString +=table;			
			return resultString;
		},
		
		_onMouseOut: function(e)
		{
			this.taskTooltip.close();
		},
		
		/**
		 * Reset the row selected index to 0. 
		 *  
		 */
		resetRowSelected: function(){
			
			this._rowIndexSelected = 0;
		},
		
		/**
		 * Select the first row of the timesheet by default
		 */
		_selectFirstRowByDefault: function(){
			
			if(this._generalGrid){
				
				if (this._generalGrid.selection.getSelectedCount() > 0) {
					
					var rowIndexSelected = 0;
					if(this._generalGrid.selection.selectedIndex <= 0){
						this._generalGrid.selection.select(0);						
					} else {
						var lastRowSelected = this._generalGrid.selection.selectedIndex;
						this._generalGrid.selection.clear();
						this._generalGrid.selection.select(lastRowSelected);
						
						rowIndexSelected = lastRowSelected;
					}
	            } else {
					this._generalGrid.selection.select(0);					
				}				
			}
		},
		
		_showContextTree: function(){
			
			if (this._hoverRowIndex == -1) {
				return;
			}
			
            var row = this.timesheetGridController.getRowByIndex(this._hoverRowIndex);
            if (row) {
            
                /*
                 * Since there can be more than one timesheet grid(approver and submitter timesheet),
                 * first check if the event is generated from the same grid. If yes, then proceed
                 * else return.
                 */
                var taskURI = row[TimesheetAttributes.taskURI];
                this._timesheetTree = new TimesheetTree({
                    taskURI: taskURI
                });
                
                this.timesheetTreeDialog = new EPMTreeDialog({
                    tree: this._timesheetTree,
                    title: TimesheetViewletMessages.CONTEXT_TREE_LABEL
                });
                
				this.timesheetTreeDialog.dialog.closeButtonNode.title = TimesheetViewletMessages.CLOSE;
                this.timesheetTreeDialog.show();
                
            }
		},       
		
		
		/**
         *
         * @param {Object} row
         * This function is called from the onStyleRow listener event.
         * Depending upon the state of the task or if the timesheet is
         * for the project manager view, approriate class is attached to
         * the row node or the cell node.
         */
        _styleRow: function(row){
        	
			if(this._generalGrid){
				
				var item = this.timesheetGridController.getRowByIndex(row.index);
	            
	            if (item) {
	                var timesheetApprovalState = item[TimesheetAttributes.timesheetApprovalState];
	                if (!timesheetApprovalState) {
	                    return;
	                }
	                if (timesheetApprovalState === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED || timesheetApprovalState === TimesheetConstants.TIMESHEET_STATUS_APPROVED) {
	                    row.customClasses = row.customClasses + " " + "rowNonEditable";
	                }
	            }
			}
        },
		
		/**
         *
         * @param {Object} event
         * When the mouse is on any cell of the grid, the title of the cell node is set with the
         * content of the cell. On doing this, on mouse over, the content of the cell is shown
         * as tooltip (similar to tooltip).
         * Also, check if the cell is taskName cell. If it is, then add a new css class to the
         * cell to change its behavior. On clicking the taskName cell, the context tree is shown.
         */
        _showCellTitle: function(event){
        
            if (event && event.cellNode) {
                event.cellNode.title = event.cellNode.textContent;
            }

            event.stopPropagation();
            event.preventDefault();
            dojo.stopEvent(event);
        },
		
		/**
         *
         * @param {Object} event
         * On clicking the header, the columns in the grid should be sorted.
         * Presently after catching this event, it is not passed to the
         * parent grid class, as no sorting is implemented.
         */
        _setGridSort: function(event){
            //Grid sorting happens at abstract level, the timesheet details doesnt refresh accordingly
			//doing them here.
			this._selectFirstRowByDefault();
			
            event.stopPropagation();
            event.preventDefault();
            dojo.stopEvent(event);
        },
		
		/**
         *
         * @param {Object} event
         * On clicking the task name cell in the grid, the context tree is shown.
         */
        _handleCellClick: function(event){
		  	
            if (!event.cell.field) {
                return;
            }
            /*
             * Check if the cell clicked is the task name cell.
             * If yes, then proceed and show the tree, else return.
             */
			if(event.cell.field === "actions"){
				
				if (event.target.name === "contextTree") {
						this._showContextTree();                        
                    }
				else if (event.target.name === "notesEditor"){
					this._showNotesDialog();
				}
				else if (event.target.name === "approveTask"){				
					this.approveTimesheet(true);				
				}			
				else if (event.target.name === "rejectTask") {				
					this.rejectTimesheet(true);				
				}				
			}
			
            event.stopPropagation();
            event.preventDefault();
            dojo.stopEvent(event);
        },
		
		/**
         *
         * @param {Object} event
         * On clicking the task name cell in the grid, the context tree is shown.
         */
		_handleCellDblClick: function(event){
            if (!event.cell.field) {
                return;
            }
            /*
             * Check if the cell clicked is the task name cell.
             * If yes, then proceed and show the tree, else return.
             */
            var rowIndex = event.rowIndex;
            var row = this.timesheetGridController.getRowByIndex(rowIndex);
			
			var timecodeEditable = true;
			
			if(this.isCellEditable(event)){
				timecodeEditable = false;
			}
            
            if (event.cell.field === TimesheetAttributes.timecode) {
                this.timecodeEditor = new TimecodeEditor({
                    _containerViewlet: this,
                    _grid: this._generalGrid,
					_editable: timecodeEditable,
                    _selectedRow: row
                });
                
            }
            event.stopPropagation();
            event.preventDefault();
            dojo.stopEvent(event);
		},
		
		_showNotesDialog:function()		
		{
			
			if (this._hoverRowIndex == -1) {
				return;
			}
			
			if(this.timesheetNotesViewDlg)
			{
				this.timesheetNotesViewDlg.destroy();
                this.timesheetNotesViewDlg = null;
			}
			
			var row = this.timesheetGridController.getRowByIndex(this._hoverRowIndex);
            if (row) {
				
				this.timesheetNotesView = new TimesheetNotesView({				
				rowIndexSelected : this._hoverRowIndex,
				timesheetGridController : this.timesheetGridController,
				timesheetGrid: this._generalGrid,
				containerViewlet: this.containerViewlet
				});		

            this.timesheetNotesViewDlg = new jazz.ui.Dialog({
                contentNode : this.timesheetNotesView.domNode,
				primaryTitle: TimesheetViewletMessages.NOTES_FOR+" '"+row[TimesheetAttributes.title]+"'",
				width: "340px"
            });

			this.timesheetNotesView.load(this.timesheetNotesViewDlg);
				}
		},
		
		/**
         *
         * @param {Object} event
         * On row click, show the submitter and approval notes of the task.
         */
        _handleRowClickEvent: function(event){
        
            event.stopPropagation();
            event.preventDefault();
            dojo.stopEvent(event);
        },
		
		/**
         *
         * @param {Object} event
         * Overwrite the function defined in EPMGrid.
         * Checks if the given cell in the grid should be editable or not.
         * Return false if non-editable else true.
         */
        isCellEditable: function(event){
        
            var rowIndex = parseInt(event.rowIndex, 10);
            if (isNaN(rowIndex)) {
                return true;
            }
            var row = this.timesheetGridController.getRowByIndex(rowIndex);
            var timesheetApprovalState = "";
            timesheetApprovalState = row[TimesheetAttributes.timesheetApprovalState];
            if (row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED || row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_APPROVED) {
                return false;
            }
          return true;
        },
        
		
		/**
         * Returns the title of the grid
         */
        getGridTitle: function(){
            return TimesheetViewletMessages.TIMESHEET;
        },
		
		/**
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this._timesheetGridDIV;  
        },
		
		_effortOrDuration:function(/*Integer*/rowIndex)
		{
			var returnString = "";
            if (this.grid) {
				var row = this.grid.model.getRow(rowIndex);
				if (row) {
					
					if(TimesheetAttributes.taskAssignmentTypeEffort==row[TimesheetAttributes.taskAssignmentType])
					{
						returnString = '<img name="'+TimesheetViewletMessages.EFFORT+'" style="margin-right: 5px; margin-left: 5px; cursor: pointer;" title="' + TimesheetAttributes.taskAssignmentTypeEffort +
	                                '" alt="' + TimesheetAttributes.taskAssignmentTypeEffort + '" src =' +
	                                dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/images/efforttask_obj.gif") +
	                                ' />';
					}
					else{
						returnString = '<img name="'+TimesheetViewletMessages.DURATION+'" style="margin-right: 5px; margin-left: 5px; cursor: pointer;" title="' + TimesheetAttributes.taskAssignmentTypeDuration +
	                                '" alt="' + TimesheetAttributes.taskAssignmentTypeDuration + '" src =' +
	                                dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/images/durationtask_obj.gif") +
	                                ' />';
					}

				}
			}
			return returnString;
		},
		
		/**
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
 
             /*
             * This view contains all columns of the tasks. 
             */
            var gridColumns = {
                rows: [[				
				{					
					name: " ",
                    width: 1.5,
					get: this._effortOrDuration
				},				
				{
                    name: TimesheetViewletMessages.TASKHEADER,
                    width: 14,
                    cellClasses: 'cellNonEditable',                    
                    field: TimesheetAttributes.title
                },				
				{
                    name: TimesheetViewletMessages.STATE,
                    width: 7,
                    cellClasses: 'cellNonEditable',
                    field: TimesheetAttributes.timesheetApprovalState
                    
                },
				{
                    name: TimesheetViewletMessages.ACTIONS,
                    width: 9,
                    cellClasses: 'cellNonEditable',                    
                    field: "actions"
                },				
				{
                    name: TimesheetViewletMessages.SUNDAYREGULAR,
                    width: 3,
                    formatter: this.validateHours,
					editor: dojox.grid.editors.Input,
                    cellClasses: 'regularHours', // For regular hours of the timesheet, a box is shown around the cell for better UI.
                    field: TimesheetAttributes.sundayRegular
                }, 
				{
                    name: TimesheetViewletMessages.MONDAYREGULAR,
                    width: 3,
                    formatter: this.validateHours,
					editor: dojox.grid.editors.Input,
                    cellClasses: 'regularHours',
                    field: TimesheetAttributes.mondayRegular					
                }, 
				{
                    name: TimesheetViewletMessages.TUESDAYREGULAR,
                    width: 3,
                    formatter: this.validateHours,
					editor: dojox.grid.editors.Input,
                    cellClasses: 'regularHours',
                    field: TimesheetAttributes.tuesdayRegular
                }, 
				{
                    name: TimesheetViewletMessages.WEDNESDAYREGULAR,
                    width: 3,
                    formatter: this.validateHours,
					editor: dojox.grid.editors.Input,
                    cellClasses: 'regularHours',
                    field: TimesheetAttributes.wednesdayRegular
                }, 
				{
                    name: TimesheetViewletMessages.THURSDAYREGULAR,
                    width: 3,
                    formatter: this.validateHours,
					editor: dojox.grid.editors.Input,
                    cellClasses: 'regularHours',
                    field: TimesheetAttributes.thursdayRegular
                }, 
				{
                    name: TimesheetViewletMessages.FRIDAYREGULAR,
                    width: 3,
                    formatter: this.validateHours,
					editor: dojox.grid.editors.Input,
                    cellClasses: 'regularHours',
                    field: TimesheetAttributes.fridayRegular
                }, 
				{
                    name: TimesheetViewletMessages.SATURDAYREGULAR,
                    width: 3,
                    formatter: this.validateHours,
					editor: dojox.grid.editors.Input,
                    cellClasses: 'regularHoursLast',
                    field: TimesheetAttributes.saturdayRegular
                }, 
				{
                    name: TimesheetViewletMessages.TOTAL,
                    width: 4,
					formatter: this.validateHours,
                    cellClasses: 'numericField cellNonEditable',
					field: "totalWorkPerWeek"
                }, 
				{
                    name: TimesheetViewletMessages.PERCENTAGECOMPLETE,
                    width: 7,
                    cellClasses: 'numericField',
                    formatter: this.checkHours,
					editor: dojox.grid.editors.Input,
                    field: TimesheetAttributes.percentageComplete
                }, 
				{
                    name: TimesheetViewletMessages.ADDITIONAL_EFFORT,
                    width: 9,
                    field: TimesheetAttributes.additionalEffort,
					editor: dojox.grid.editors.Input,
                    formatter: this.validateHours,
                    cellClasses: 'numericField'
                }, 
				{
                    name: TimesheetViewletMessages.TIMECODE,
                    width: 6,
                    field: TimesheetAttributes.timecode,
					cellClasses: 'cellEditable'
                }
				]]
            };
            
            var structure = [gridColumns];
            
            return structure;
        },
		
		/**
		 * @return Retrieves the sortable columns for this grid view
		 */
		
		getSortableColums:function()
		{
			var sortableFields ={};
			sortableFields[TimesheetAttributes.title] ="name";
			sortableFields["totalWorkPerWeek"] = "totalWorkPerWeek";
			sortableFields[TimesheetAttributes.percentageComplete] = TimesheetAttributes.percentageComplete;
			sortableFields[TimesheetAttributes.additionalEffort] = TimesheetAttributes.additionalEffort;
			sortableFields[TimesheetAttributes.timesheetApprovalState] = TimesheetAttributes.timesheetApprovalState;
			
			return sortableFields;
		},
		
		/**
		 * @param {Object} currentItemValue
		 * @param {Object} rowIndex
		 * 
		 * Validate the hours entered by the user and displays the appropriate tooltip accordingly.
		 */
		validateHours: function(currentItemValue, rowIndex){
			
			/*
			 *Allows only hours:minutes format. For example: 1:1, 1:01, 122:23, 122:1222 
			 */
			var regExp = new RegExp("(^([0-9]*):([0-9]*)$)|(^([0-9]+)$)");
			var flag = regExp.test(currentItemValue);
			
			if(flag){
				if(this.lastHoursNodeFocussed){
					
					dijit.hideTooltip(this.lastHoursNodeFocussed);
					dijit._masterTT._onDeck=null;
					this.lastHoursNodeFocussed = null;	
				}
				return currentItemValue;
			} else {
				
				var node = this.getNode(rowIndex);
				if(node){
					var self = this;
					
					if(this.lastHoursNodeFocussed){
					
						dijit.hideTooltip(this.lastHoursNodeFocussed);
						dijit._masterTT._onDeck=null;
						this.lastHoursNodeFocussed = null;	
					}
                    this._showTimer = setTimeout(dojo.hitch(this, function(){
						
						var node = self.getNode(rowIndex);
						self.lastHoursNodeFocussed = node;
						var tooltipInfo = TimesheetViewletMessages.HOURS_FORMAT_INVALID + "<br>" + TimesheetViewletMessages.FORMAT;
						tooltipInfo += ": " + TimesheetViewletMessages.HOURS_FORMAT + "<br>";
						tooltipInfo +=  TimesheetViewletMessages.EXAMPLE + ": " + TimesheetViewletMessages.HOURS_FORMAT_EXAMPLE;
                        dijit.showTooltip(tooltipInfo, node);
                    }), 5);
                    this._hideTimer = setTimeout(dojo.hitch(this, function(){
						var node = self.getNode(rowIndex);
                        dijit.hideTooltip(this.lastHoursNodeFocussed);
						dijit._masterTT._onDeck=null;
                    }), 5000);
				
				}
				return currentItemValue;
			}
			
		},
		
		/*
             * The total work corresponds to the total work done for the corresponding task
             * in that week. It is a calculated field and is the summation of all the regular
             * hours of that row.
             */
		getTotalWork: function(inRowIndex){
			
			if(this._generalGrid){
				var row = this.timesheetGridController.getRowByIndex(inRowIndex);
	            if (row) {
	                
					var sundayRegular = TimesheetUtil.convertHoursToMinutes(row[TimesheetAttributes.sundayRegular]);
					var mondayRegular = TimesheetUtil.convertHoursToMinutes(row[TimesheetAttributes.mondayRegular]);
					var tuesdayRegular = TimesheetUtil.convertHoursToMinutes(row[TimesheetAttributes.tuesdayRegular]);
					var wednesdayRegular = TimesheetUtil.convertHoursToMinutes(row[TimesheetAttributes.wednesdayRegular]);
					var thursdayRegular = TimesheetUtil.convertHoursToMinutes(row[TimesheetAttributes.thursdayRegular]);
					var fridayRegular = TimesheetUtil.convertHoursToMinutes(row[TimesheetAttributes.fridayRegular]);
					var saturdayRegular = TimesheetUtil.convertHoursToMinutes(row[TimesheetAttributes.saturdayRegular]);
					
					totalWork = sundayRegular 
								+ mondayRegular 
								+ tuesdayRegular 
								+ wednesdayRegular 
								+ thursdayRegular 
								+ fridayRegular 
								+ saturdayRegular;
					
		           
		            return TimesheetUtil.convertMinutesToHours(totalWork);
		            
	            }
			}
        },
		
		/*
         * Due to the defect in the NumberedTextBox editor, checkHours is required.
         * NumberTextBoxEditor, on deleting the value, returns NaN.
         * But, an empty field is a valid entry and user should not
         * be shown NaN for that.
         * So, this function will make sure that if any fields has NaN value, the user
         * will be shown empty field.
         */
        checkHours: function(currentItemValue, rowIndex){
            if (isNaN(currentItemValue)) {
                return "";
            }
            else {
                return currentItemValue;
            }
            
        },
		
		/**
         *
         * @param {Object} saveAndSubmit
         * Save the timesheet details entered by the user.
         */
        saveTimesheet: function(saveAndSubmit){
        	
            /*
             * If the user is submitting the timesheet, and if the timesheet is changed and not saved for the current weekk,
             * save all the rows first and send "Submit" as parameter.
             */
            if (saveAndSubmit) {
            	
				this._gridModel.submit = true;
				this.persistGrid(false,true);
                    
            }
            else {
                this._gridModel.submit = false;
				this.persistGrid(false,true);
            }
        },
		
		/**
         *
         * @param {Object} saveAndSubmit
         * Save the timesheet details entered by the user.
         */
        approveTimesheet: function(ifSelectedRows){
			
			if (!this.containerViewlet.getSaveButtonStatus()) {
				
				this.containerViewlet._showTimesheetDirtyDialog();
				return;
			}
        
			this._gridModel.approve = true;
			this._gridModel.reject = false;
            if (ifSelectedRows) {
            	
				var selectedRows = this._generalGrid.selection.getSelected();
				
				if(selectedRows.length){
					for(var rowIndex = 0; rowIndex < selectedRows.length; rowIndex++){
					
						var row = this.timesheetGridController.getRowByIndex(selectedRows[rowIndex]);
						if (row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED ||
						row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_UNSUBMITTED ||
						row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_REJECTED) {
							
							row[TimesheetAttributes.timesheetApprovalState] = TimesheetConstants.TIMESHEET_STATUS_APPROVED;
						}	
					}
					
					
					this._gridModel.actionOnSelected = true;
					this.persistGrid(false,true);	
				}
				
            }
            else {
				this._gridModel.actionOnSelected = false;
				this.persistGrid(false,true);
            }
        },
		
		/**
         *
         * @param {Object} saveAndSubmit
         * Save the timesheet details entered by the user.
         */
        rejectTimesheet: function(ifSelectedRows){
			
			if (!this.containerViewlet.getSaveButtonStatus()) {
				
				this.containerViewlet._showTimesheetDirtyDialog();
				return;
			}
        
			this._gridModel.approve = false;
			this._gridModel.reject = true;
            if (ifSelectedRows) {
            	
				var selectedRows = this._generalGrid.selection.getSelected();
				
				if(selectedRows.length){
					for(var rowIndex = 0; rowIndex < selectedRows.length; rowIndex++){
					
						var row = this.timesheetGridController.getRowByIndex(selectedRows[rowIndex]);
						if (row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED) {
							
							row[TimesheetAttributes.timesheetApprovalState] = TimesheetConstants.TIMESHEET_STATUS_REJECTED;
						}	
					}
					
					this._gridModel.actionOnSelected = true;
					this.persistGrid(false,true);	
				}
                    
            }
            else {
				this._gridModel.actionOnSelected = false;
				this.persistGrid(false,true);
            }
        },
		
		/**
		 * Any error generated while building the grid
		 * @param {Object} errorMessage
		 */
		handleError: function(errorMessage){
            this.inherited("handleError", arguments);
            this._displayError(errorMessage);
        },
		
		getPostParams: function()
		{
			return this._postParams;
		},
		
		setPostParams: function(postParams)
		{
			this._postParams = postParams;
		}
	
	});
})();