/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/*******************************************************************************
 
 Team Member Timesheet Grid Widget
 
 *******************************************************************************/
(function(){

    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetService");
    
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    
    dojo.require("com.ibm.epm.timesheet.web.client.internal.TimesheetClient");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes");
    
    dojo.require("dojo.parser");
    dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
    
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var TimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    var TimesheetClient = com.ibm.epm.timesheet.web.client.internal.TimesheetClient;
    var TimesheetUtil = com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil;
    var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
    var TimesheetAttributes = com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes;
    
    com.ibm.epm.timesheet.web.ui.internal.TimesheetService = {
    
        /**
         *
         * @param {Object} assignmentURI
         * @param {Object} viewlet
         * @param {Object} uri
         */
        TimesheetServiceObj: function(assignmentURI, viewlet, uri){
        
            var self = this;
            var serviceObj = {
                success: function(timesheet){
                
                    if (!timesheet) {
                        self.viewlet.containerViewlet._displayErrorMessage(TimesheetViewletMessages.TIMESHEET_OBJECT_FAILED);
                        return;
                    }
                    var intervalID = setInterval(function(){
                    
                        var row = null;
                        if (!viewlet._timesheetGrid) {
                            return;
                        }
                        if (viewlet._timesheetGrid.model === null) {
                            return;
                        }
                        var item = null;
                        if (viewlet._timesheetGrid.model._rowIdentities[assignmentURI]) {
                        
                            var rowIndex = viewlet._timesheetGrid.model.getRowByIdentity(assignmentURI);
                            item = viewlet._timesheetGrid.model.getRow(rowIndex);
                            
                        }
                        
                        if (item !== null) {
                        
                            clearInterval(intervalID);
                            
                            item[TimesheetAttributes.remainingWork] = timesheet[TimesheetAttributes.remainingWork];
                            item[TimesheetAttributes.percentageComplete] = timesheet[TimesheetAttributes.percentageComplete];
                            item[TimesheetAttributes.percentageProgress] = timesheet[TimesheetAttributes.percentageProgress];
                            item[TimesheetAttributes.percentageDC] = timesheet[TimesheetAttributes.percentageDC];
                            item[TimesheetAttributes.billable] = timesheet[TimesheetAttributes.billable];
                            item[TimesheetAttributes.summaryNotes] = timesheet[TimesheetAttributes.summaryNotes];
                            item[TimesheetAttributes.approvalNotes] = timesheet[TimesheetAttributes.approvalNotes];
                            item[TimesheetAttributes.timesheetApprovalState] = timesheet[TimesheetAttributes.timesheetApprovalState];
                            item[TimesheetAttributes.totalActualHours] = timesheet[TimesheetAttributes.totalActualHours];
                            item[TimesheetAttributes.eetc] = timesheet[TimesheetAttributes.eetc];
                            item[TimesheetAttributes.timesheeturi] = uri;
                            
                            if (item[TimesheetAttributes.timesheetApprovalState] === "" || item[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_UNSUBMITTED || item[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_REJECTED) {
                                if (viewlet.containerViewlet.toggleSubmitButton) {
								//	alert("hello");
                                    viewlet.containerViewlet.toggleSubmitButton(false);
                                }
                            }
                            
                            if (timesheet[TimesheetAttributes.forecastFinish]) {
                                item[TimesheetAttributes.forecastFinish] = TimesheetUtil.utilGetDateObjFromServerDate(timesheet[TimesheetAttributes.forecastFinish]);
                            }
                            
                            else {
                                item[TimesheetAttributes.forecastFinish] = "";
                            }
                            if (timesheet[TimesheetAttributes.forecastStart]) {
                                item[TimesheetAttributes.forecastStart] = TimesheetUtil.utilGetDateObjFromServerDate(timesheet[TimesheetAttributes.forecastStart]);
                            }
                            
                            else {
                                item[TimesheetAttributes.forecastStart] = "";
                            }
                            
                            item[TimesheetAttributes.sundayRegular] = timesheet[TimesheetAttributes.hours][TimesheetAttributes.sundayRegular];
                            item[TimesheetAttributes.mondayRegular] = timesheet[TimesheetAttributes.hours][TimesheetAttributes.mondayRegular];
                            item[TimesheetAttributes.tuesdayRegular] = timesheet[TimesheetAttributes.hours][TimesheetAttributes.tuesdayRegular];
                            item[TimesheetAttributes.wednesdayRegular] = timesheet[TimesheetAttributes.hours][TimesheetAttributes.wednesdayRegular];
                            item[TimesheetAttributes.thursdayRegular] = timesheet[TimesheetAttributes.hours][TimesheetAttributes.thursdayRegular];
                            item[TimesheetAttributes.fridayRegular] = timesheet[TimesheetAttributes.hours][TimesheetAttributes.fridayRegular];
                            item[TimesheetAttributes.saturdayRegular] = timesheet[TimesheetAttributes.hours][TimesheetAttributes.saturdayRegular];
                            
                            if (timesheet[TimesheetAttributes.TimeCode]) {
                            
                                var numberOfTimecodes = timesheet[TimesheetAttributes.TimeCode].length;
                                for (var i = 0; i < numberOfTimecodes; i++) {
                                
                                    if (item[TimesheetAttributes.timecodeURI] !== "") {
                                        item[TimesheetAttributes.timecodeURI] = item[TimesheetAttributes.timecodeURI] + "," + timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.relativeHref];
                                        item[TimesheetAttributes.timecode] = item[TimesheetAttributes.timecode] + "," + timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.title];
                                    }
                                    else {
                                        item[TimesheetAttributes.timecodeURI] = timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.relativeHref];
                                        item[TimesheetAttributes.timecode] = timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.title];
                                    }
                                }
                                
                            }
                            if (viewlet._timesheetGrid.selection.selected[0] === true) {
                            
                                viewlet._showSummary(0);
                            }
                            
                            viewlet._timesheetStore.save();
                            viewlet._timesheetGrid.model.rowChange(item, rowIndex);
                            
                        }
                    }, 10);
                },
                failure: function(errorObj){
                    self.viewlet.containerViewlet._displayErrorMessage(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            TimesheetClient.getTimesheetDetail(srh, uri, "get");
        },
        
        
        /**
         *
         * @param {Object} viewlet
         * @param {Object} item
         * @param {Object} modifiedItem
         * @param {Object} uri
         */
        TimesheetServiceSaveObj: function(viewlet, item, modifiedItem, uri){
        
            var self = this;
            var serviceObj = {
                success: function(responseVal){
                
                    if (viewlet.containerViewlet.toggleSaveButton) {
                        viewlet.containerViewlet.toggleSaveButton(true);
                    }
                    TimesheetUtil.clearTimesheetRowData(item);
                    viewlet._updateTaskAssignmentDetails(item[TimesheetAttributes.assignmentURI]);
                },
                
                failure: function(errorObj){
                    if (viewlet.containerViewlet.toggleSaveButton) {
                        viewlet.containerViewlet.toggleSaveButton(false);
                    }
                    viewlet.clearRow(item);
                    viewlet._updateTaskAssignmentDetails(item.taskassignmenturi);
                    self.viewlet.containerViewlet._displayErrorMessage(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            TimesheetClient.saveTimesheet(srh, uri, modifiedItem, "post");
        }
    };
    
})();
