/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/*******************************************************************************
 
 Timesheet Tree
 
 *******************************************************************************/
(function(){

    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetTree");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTimesheetTree_temp");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
    
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    
    dojo.require("dojo.data.ItemFileWriteStore");
    
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    
    dojo.require("com.ibm.epm.timesheet.web.client.internal.TimesheetClient");
    
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var BasicTimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    var TimesheetClient = com.ibm.epm.timesheet.web.client.internal.TimesheetClient;
    var EPMTree = com.ibm.epm.web.core.ui.internal.EPMTree;//com.ibm.epm.web.core.ui.internal.EPMTimesheetTree_temp;
    var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimesheetTree", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/templates/TimesheetTree.html"),
        
        taskURI: "",
        
        postCreate: function(){
        
            this._cleanUpTree();
            this.parentLoaded = false;
            this.parentMap = new Array();
            this.parentMapCounter = 0;
            this._buildContextTree(this.taskURI);
        },
        
        /**
         * Clean the tree related objects
         */
        _cleanUpTree: function(){
        
            if (this._taskTree) {
                this._taskTree.destroyRecursive();
            }
            if (this._taskTreeStore) {
                delete this._taskTreeStore;
            }
            
            if (this._temporaryStore) {
                delete this._temporaryStore;
            }
            
            if (this._taskTreeData) {
                delete this._taskTreeData;
            }
            
            if (this._temporaryData) {
                delete this._temporaryData;
            }
            
            if (this._taskTree) {
            
                this._timesheetTreeContent.removeChild(this._taskTree.domNode);
            }
            
        },
        
        /**
         *
         * @param {Object} uri
         */
        _getResourceType: function(uri){
            var tempIndex = uri.lastIndexOf("/");
            var tempString = uri.substring(0, tempIndex);
            tempIndex = tempString.lastIndexOf("/");
            var tempStringLength = tempString.length;
            var resourceType = tempString.substring(tempIndex + 1, tempStringLength);
            return TimesheetConstants.EPM + resourceType;
            
        },
        
        /**
         *
         * @param {Object} taskURI
         */
        _buildContextTree: function(taskURI){
        
            if (!taskURI || taskURI === null) {
                return;
            }
            
            var self = this;
            
            var serviceObj = {
            
                success: function(taskDetail){
                
                    if (!taskDetail) {
                        console.log("TaskDetail is empty!!!");
                        return;
                    }
                    
                    var epmTask = taskDetail;
                    var taskName = epmTask.summary;
                    
                    var resourceType = "";
                    resourceType = epmTask.workItemType;
                    var item = new Array(1);
                    item[0] = {
                    
                        "id": taskURI,
                        "name": taskName,
                        "resourcetype": resourceType,
                        "iconClass": resourceType
                    };
                    
                    self._temporaryData = {
                        identifier: 'id',
                        label: 'name',
                        items: item
                    };
                    
                    self._temporaryStore = new dojo.data.ItemFileWriteStore({
                        data: self._temporaryData
                    });
					//If iteration based work item obtain parent information from PlannedFor field
					if(epmTask.PlannedFor){
                        var parentHref = epmTask.PlannedFor[0].relativeHref;
                        self.parentMap[self.parentMapCounter] = taskURI;
                        
                        self._getParentDetail(parentHref);
                        
                        self._buildTree();
                    }
					//For non-iteration based WIs, obtain parent info from Parent field
                    else if (epmTask.Parent) {
                        
						var parentHref = epmTask.Parent[0].relativeHref;
                        self.parentMap[self.parentMapCounter] = taskURI;
                        
                        self._getParentDetail(parentHref);
                        
                        self._buildTree();
                    }                    
                },
                failure: function(errorObj){
                    console.log("_getTaskDetail failed!!!");
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            TimesheetClient.getTaskDetail(srh, taskURI, "get");
            
        },
        
        /**
         *
         */
        _buildTree: function(){
        
            var self = this;
            
            
            var intervalID = setInterval(function(){
            
                if (self.parentLoaded) {
                
                    clearInterval(intervalID);
                    
                    self.parentLoaded = false;
                    
                    for (var i = self.parentMapCounter; i > -1; i--) {
                    
                        var identifier = self.parentMap[i];
                        
                        var item = self._temporaryStore._getItemByIdentity(identifier);
                        
                        var treeItem = null;
                        treeItem = {
                            "id": item.id,
                            "name": item.name,
                            "resourcetype": item.resourcetype[0],
                            "iconClass": item.resourcetype[0] + "Icon"
                        };
                        
                        if (!self._taskTreeData) {
                            var parentTemp = new Array(1);
                            
                            parentTemp[0] = treeItem;
                            
                            self._taskTreeData = {
                                identifier: 'id',
                                label: 'name',
                                items: parentTemp
                            };
                            
                            var itemFileWriteStore = new dojo.data.ItemFileWriteStore({
                                data: self._taskTreeData
                            });
                            
                            self._taskTreeStoreModel = new dijit.tree.ForestStoreModel({
                                store: itemFileWriteStore,
                                rootLabel: "Root",
                                rootId: "Root"
                            });
                            
                            if (self._taskTreeStoreModel) {
                                self._taskTreeStore = self._taskTreeStoreModel.store;
                                self._taskTreeStore._forceLoad();
                            }
                            
                            if (self._taskTreeStore) {
                                var props = {
                                    model: self._taskTreeStoreModel,
                                    labelAttr: "name",
									showRoot: false,
                                    persist: false
                                };
                                self._taskTree = new EPMTree(props);
                                if (self._taskTree) {
                                    self._timesheetTreeContent.appendChild(self._taskTree.domNode);
                                }
                            }
                        }
                        else {
							
                            treeItem = {
                                "id": item.id[0],
                                "name": item.name[0],
                                "resourcetype": item.resourcetype[0],
								"iconClass": item.resourcetype[0] + "Icon"
                            };
                            
                            var parentNode = self._taskTreeStore._getItemByIdentity(self.parentMap[i + 1]);
                            
                            if (!self._taskTreeStore._getItemByIdentity(self.parentMap[i])) {
                            
                                self._taskTreeStore.newItem(treeItem, {
                                    parent: parentNode,
                                    attribute: "children"
                                });
                               
                            }							
                        }
                    }
					
					/*
					 * Since, showRoot is false, the root node wan't be shown.
					 * But, the display property of the child nodes of the root
					 * also inherits the property of the root node.
					 * 
					 * So, explicitly making the display as block
					 */
					if (self._taskTree){
						if (!dojo.isIE){						
							var treeContainer = self._taskTree.domNode.getElementsByClassName("dijitTreeContainer");
							//console.log(treeContainer);
							dojo.style(self._taskTree.domNode, "display", "block");
							for (var j = 0; j < treeContainer.length; j++) {
								dojo.style(treeContainer[j], "display", "block");
							}							
						}
						else {
							var treeContainer = self.getElementsByClass("dijitTreeContainer", self._taskTree.domNode, "div");
							for (var j = 0; j < treeContainer.length; j++) {
								dojo.style(treeContainer[j], "display", "block");
							}							
						}
						self._taskTree._expandAllTopLevelNodes();
					}                    
                }
            }, 1);
        },
		
		/**
		 * 
		 * @param {Object} searchClass: The css class to search for element
		 * @param {Object} node: The node under which the element to be found
		 * @param {Object} tag : The type of element to be searched for
		 * 
		 * Since getElementsByClassName is not supported by IE, writing a function which mimic this 
		 * functionality in IE.
		 */
		getElementsByClass: function(searchClass,node,tag) {
			
			var classElements = new Array();
			if ( node == null ) {
				
				node = document;
			}
				
			if ( tag == null ) {
				
				tag = '*';
			}
				
			var els = node.getElementsByTagName(tag);
			var elsLen = els.length;
			var pattern = new RegExp('(^|\\\\s)'+searchClass+'(\\\\s|$)');
			
			for (i = 0, j = 0; i < elsLen; i++) {
				
				if ( pattern.test(els[i].className) ) {
					
					classElements[j] = els[i];
					j++;
				}
			}
			return classElements;
		},
        
        /**
         *
         * @param {Object} parentHref
         */
        _getParentDetail: function(parentHref){
        
            var self = this;
            
            var serviceObj = {
            
                success: function(parentDetail){
                
                    if (!parentDetail) {
                        console.log("Parent detail is empty!!!");
                        return;
                    }
                    
                    var parentObject = parentDetail;
                    var parentName = parentObject.summary ? parentObject.summary : parentObject.name;
                    var resourceType = "";
					
                    if (parentObject.workItemType) {
                        resourceType = parentObject.workItemType;
                    }
                    else if (parentObject.relativeHref.substr(0, 7) == 'program') {
                    	resourceType = 'program';
                    }
					else if (parentObject.relativeHref.substr(0, 9) == 'iteration') {
                    	resourceType = 'iteration';
                    }
                    self.parentMapCounter++;
                    self.parentMap[self.parentMapCounter] = parentHref;
                    
                    var item = {
                        "id": parentHref,
                        "name": parentName,
                        "resourcetype": resourceType,
                        "iconClass": resourceType
                    };
                    
                    if (self._temporaryData) {
                        if (!self._temporaryStore._getItemByIdentity(parentHref)) {
                            self._temporaryStore.newItem(item);
                        }
                    }
                    var tempHref;
					
					//Ensure the following order of IF's is preserved
					//PlannedFor should always come first as for iteration based tasks,
					//Iteration data is obtained from PlannedFor field.
					
					if (parentObject.PlannedFor) {
                            tempHref = parentObject.PlannedFor[0].relativeHref;
                            self._getParentDetail(tempHref);
                            self.parentLoaded = false;
                        }
                    else if (parentObject.Parent) {
                        tempHref = parentObject.Parent[0].relativeHref;
                        self._getParentDetail(tempHref);
                        self.parentLoaded = false;
                    }
                    else if (parentObject.Program) {
                            tempHref = parentObject.Program[0].relativeHref;
                            self._getParentDetail(tempHref);
                            self.parentLoaded = false;
                    }					
                    else {
                            self.parentLoaded = true;
                   	}
                },
                
                failure: function(errorObj){
                    console.log("No parent Href");
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            TimesheetClient.getParentDetails(srh, parentHref, "get");
            
        },
        
        /**
         *
         * @param {Object} errorMsg
         */
        _displayErrorMessage: function(errorMsg){
            EPMErrorHandler.setError(errorMsg);
        },
        
        /**
         *
         */
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
        
        /**
         *
         */
        update: function(){
        
        }
    });
})();

