/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil");
    
    dojo.require("dojo.parser");
    dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes");
    
    var _TIMESHEET_URI = "timesheet"; // TODO: to be pushed to timesheet constants. Need some investigation whether timesheet or timesheets
    var TimesheetAttributes = com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes;
    
    com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil = {
    
        getTimesheetEmptyRow: function(assignmentURI){
        
            var item = {};
            item[TimesheetAttributes.assignmentURI] = assignmentURI;
            
            item[TimesheetAttributes.hoursID] = "";
            
            item[TimesheetAttributes.sundayRegular] = "";
            item[TimesheetAttributes.mondayRegular] = "";
            item[TimesheetAttributes.tuesdayRegular] = "";
            item[TimesheetAttributes.wednesdayRegular] = "";
            item[TimesheetAttributes.thursdayRegular] = "";
            item[TimesheetAttributes.fridayRegular] = "";
            item[TimesheetAttributes.saturdayRegular] = "";
            item[TimesheetAttributes.eetc] = "";
            item[TimesheetAttributes.remainingWork] = "";
            item[TimesheetAttributes.percentageComplete] = "";
            item[TimesheetAttributes.percentageProgress] = "";
            item[TimesheetAttributes.percentageDC] = "";
            item[TimesheetAttributes.billable] = "";
            item[TimesheetAttributes.totalActualHours] = "";
			item[TimesheetAttributes.totalActualDuration] = "";
            item[TimesheetAttributes.containingProject] = "";
            item[TimesheetAttributes.summaryNotes] = "";
            item[TimesheetAttributes.approvalNotes] = "";
            item[TimesheetAttributes.timesheetApprovalState] = "";
            item[TimesheetAttributes.timesheeturi] = "";
            item[TimesheetAttributes.timesheeturilist] = "";
            item[TimesheetAttributes.weekOf] = "";
            item[TimesheetAttributes.action] = "";
            item[TimesheetAttributes.title] = "";
            item[TimesheetAttributes.timecode] = "";
            item[TimesheetAttributes.timecodeURI] = "";
            item[TimesheetAttributes.taskURI] = "";
            item[TimesheetAttributes.actualScheduleStartDate] = "";
            item[TimesheetAttributes.actualScheduleFinishDate] = "";
            item[TimesheetAttributes.plannedScheduleStartDate] = "";
            item[TimesheetAttributes.plannedScheduleFinishDate] = "";
            item[TimesheetAttributes.plannedScheduleEffort] = "";
			item[TimesheetAttributes.plannedScheduleDuration] = "";
            item[TimesheetAttributes.forecastFinish] = "";
            item[TimesheetAttributes.forecastStart] = "";
            item[TimesheetAttributes.ExpectedWorkWeek] = "";
			item[TimesheetAttributes.taskAssignmentType] = TimesheetAttributes.taskAssignmentTypeEffort;
            
            return item;
        },
        
        clearTimesheetRowData: function(item){
        
            item[TimesheetAttributes.sundayRegular] = "";
            item[TimesheetAttributes.mondayRegular] = "";
            item[TimesheetAttributes.tuesdayRegular] = "";
            item[TimesheetAttributes.wednesdayRegular] = "";
            item[TimesheetAttributes.thursdayRegular] = "";
            item[TimesheetAttributes.fridayRegular] = "";
            item[TimesheetAttributes.saturdayRegular] = "";
            item[TimesheetAttributes.eetc] = "";
            item[TimesheetAttributes.remainingWork] = "";
            item[TimesheetAttributes.percentageComplete] = "";
            item[TimesheetAttributes.percentageProgress] = "";
            item[TimesheetAttributes.percentageDC] = "";
            item[TimesheetAttributes.billable] = "";
            item[TimesheetAttributes.totalActualHours] = "";
            item[TimesheetAttributes.containingProject] = "";
            item[TimesheetAttributes.summaryNotes] = "";
            item[TimesheetAttributes.approvalNotes] = "";
            item[TimesheetAttributes.timesheetApprovalState] = "";
            item[TimesheetAttributes.timesheeturi] = "";
            item[TimesheetAttributes.timesheeturilist] = "";
            item[TimesheetAttributes.weekOf] = "";
            item[TimesheetAttributes.action] = "";
            item[TimesheetAttributes.timecode] = "";
            item[TimesheetAttributes.timecodeURI] = "";
            item[TimesheetAttributes.forecastFinish] = "";
            item[TimesheetAttributes.forecastStart] = "";
			item[TimesheetAttributes.totalActualDuration] = "";
			item[TimesheetAttributes.plannedScheduleDuration] = "";
            item[TimesheetAttributes.ExpectedWorkWeek] = "";
            item[TimesheetAttributes.hoursID] = "";
			item[TimesheetAttributes.taskAssignmentType] = TimesheetAttributes.taskAssignmentTypeEffort;
            
            return item;
        },
		
		/**
		 * 
		 * @param {Object} currentItemValue
		 * 
		 * Data entered in the format hours:minutes will be 
		 * converted into minutes.
		 */
		convertHoursToMinutes: function(currentItemValue){
			
			if(currentItemValue){
				
				var minutes = 0;
				var hoursArray = new Array();
				hoursArray = currentItemValue.split(":");
				
				if(hoursArray[0]){
					
					if (!isNaN(hoursArray[0])) {
						
						minutes = parseInt(hoursArray[0], 10) * 60;
					} else {
						return minutes;
					}
				}
				if(hoursArray[1]){
					
					if (!isNaN(hoursArray[1])) {
						
						minutes = parseInt(hoursArray[1], 10) + minutes;
					}
					else {
						return minutes;
					}
				}
				return minutes;
			}
		},
		
		/**
		 * 
		 * @param {Object} currentItemValue
		 * 
		 * Data fetched from the server are in minutes.
		 * For display purpose it has to be converted into
		 * hours:minutes format. 
		 */
		convertMinutesToHours: function(currentItemValue){
			
			if(currentItemValue){
				
				currentItemValue = parseInt(currentItemValue, 10); 
				var hours = parseInt(currentItemValue/60, 10);
				var minutes = currentItemValue%60;
				
				if(minutes === 0){
					minutes = "00"
				} else {
					if(minutes < 10){
						minutes = "0" + minutes;
					}	
				}
				
				return hours + ":" + minutes;
			} else {
				return "00:00";
			}
		},
		
		
        
        /**
         *
         * @param {Object} inDateString
         */
        utilGetDateObjFromServerDate: function(inDateString){
            if (!inDateString || inDateString === "") {
                return "";
            }
            inDateString = inDateString.toString();
            
            return dojo.date.locale.parse(inDateString.substring(0, 10), {
                selector: 'date',
                datePattern: "yyyy-MM-dd"
            });
            
        },
        
        /**
         *
         * @param {Object} inDatum
         */
        formatDate: function(inDatum){
        
            if (inDatum === "?") {
                return;
            }
            if (inDatum !== "") {
            
                var date = dojo.date.locale.format(new Date(inDatum), this.constraint);
                var leastDate = com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil.utilGetDateObjFromServerDate("1980-12-31");
                if (dojo.date.compare(leastDate, inDatum, 'date') === 1) {
                    return "";
                }
                return date;
            }
            else {
                return ("");
            }
        },
        
        /**
         *
         * @param {Object} taskassignmenturi
         */
        getTimesheetUri: function(taskassignmenturi){
        
            var temp = taskassignmenturi.split('taskAssignment');
            return temp[0] + _TIMESHEET_URI;
        }
    };
})();
