/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/**
 *
 * Timesheet Viewlet
 *
 */
(function(){

    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetViewlet");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    dojo.require("com.ibm.epm.web.core.app.internal.Util");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridView");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.WeekTextBox");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
	dojo.require("com.ibm.epm.timesheet.web.client.internal.TimesheetClient");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
	dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants");
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
	
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    
    dojo.require("dojo.parser");
    dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
    
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
	
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var TimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    var WeekTextBox = com.ibm.epm.timesheet.web.ui.internal.WeekTextBox;
    var epmUtil = com.ibm.epm.web.core.app.internal.Util;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
	var TimesheetClient = com.ibm.epm.timesheet.web.client.internal.TimesheetClient;
	var TimesheetGridView = com.ibm.epm.timesheet.web.ui.internal.TimesheetGridView;
	var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
	var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	var EPMDomNodeConstants = com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants;
	var DateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
	var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
	var DashboardUtil = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
	
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimesheetViewlet", Viewlet, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/templates/TimesheetViewlet.html"),
        contextHelpId:TimesheetConstants.SUBMITTER_TIMESHEET_VIEWLET_CONTEXT_HELP_ID,
		
        init: function(){
            this.inherited(arguments);

			this.weekSelected = null;
			this._newWeekSelected = null;
            this.isProjectManager = false;
            this.resourceID = null;
            this.setResourceID();
            this._createBasicToolbar();
            this._createWeekOfTextBox();
            this._createTimesheetGrid();
        },
        
        /**
         * Week text box will enable the user to select the week of the timesheet.
         * This will trigger an action that will help to reload the data in the grid.
         */
        _createWeekOfTextBox: function(){
        
            var self = this;
            
            var weekLabel = document.createElement(EPMDomNodeConstants.LABEL);
            weekLabel.innerHTML = TimesheetViewletMessages.WEEK;
            this._weekLabelDIV.appendChild(weekLabel);
            
            this.weekTextBox = new WeekTextBox({
            
                onChange: function(value){
					
					if(value){
						self._reloadTimesheet(value);
					}
					this.weekPicker.focusNode.blur();
					//Explicitly setting the focus else where as the above call is not working
					self.refreshButton.focus();
					
                }
            });
            
            this.weekTextBox.buildView();
            this._weekPickerDIV.appendChild(this.weekTextBox.domNode);
			this.weekSelected = this.weekTextBox.getValue();
        },
        
        /**
         *
         * @param {Object} date
         * If the week is changed, fetch the new timesheet of the resource for the new week selected.
         */
        _reloadTimesheet: function(date){
			if (this.weekSelected !== null) {
            
                var date1 = DateUtil.formatDate_Short(new Date(date));
                var date2 = DateUtil.formatDate_Short(new Date(this.weekSelected));
				
                if (date1 === date2) {
                    return;
                }
            }
            
			if (this.timesheetGrid) {
                this._weekChange(date);
            }
            
        },
		
		/**
         *
         * @param {Object} value
         * This event is triggered when the week is changed.
         * The new set of values are rendered in the grid taking one row at a time.
         */
        _weekChange: function(date){
        
            if (!this.getSaveButtonStatus()) {
				this._newWeekSelected = date;
                this._createTimesheetSaveDialog();
            } else {
				
				if (this.toggleBusyStatusOfViewlet) {
                    this.toggleBusyStatusOfViewlet(true);
                }
				this.weekSelected = date;
                this._refresh();
				
				if (this.toggleSaveButton) {
	                this.toggleSaveButton(true);
	            }
	            if (this.toggleSubmitButton) {
	                this.toggleSubmitButton(true);
	            }
			}
        },
		
		/**
         * On week change, if the user has updated any values in the grid for
         * the current week, then the user will prompted with the dialog;
         * The dialog contains a message asking the user whether to continue
         * or not.
         * The user can save the timesheet before continuing or
         * can discard the changes made in the grid.
         */
        _createTimesheetSaveDialog: function(){
        
            var buttonlist = [{
                label: TimesheetViewletMessages.YES,
                action: dojo.hitch(this, this._yesButtonPressed)
            }, {
                label: TimesheetViewletMessages.NO,
                action: dojo.hitch(this, this._noButtonPressed)
            }, {
                label: TimesheetViewletMessages.CANCEL,
                action: dojo.hitch(this, this._resetPreviousSelection)
            }];
            
            var labelList = [{
                label: TimesheetViewletMessages.TIMESHEET_NOT_SAVED_DIALOG
            }, {
                label: TimesheetViewletMessages.TIMESHEET_SAVE_CONFIRMATION
            }];
            
            this.timesheetSaveDialog = new EPMConfirmDialog({
                labelList: labelList,
                title: TimesheetViewletMessages.TIMESHEET_DIALOG_NOT_SAVED,
                buttonlist: buttonlist
            });
			
			dojo.connect(this.timesheetSaveDialog.dialog, "onCancel", this, this._resetPreviousSelection);
            
            this.timesheetSaveDialog.show();
        },
		
		/**
		 * 
		 * @param {Object} event
		 */
		_resetPreviousSelection: function(event){
			 
		 	this.timesheetSaveDialog.destroy();
			this._resetWeek();
			
		},
		
		/**
		 * Reset the value of the week to previous week selected.
		 * If the user don't want to perform any operation, then 
		 * it is required the set the week value to previously selected week.
		 */
		_resetWeek: function(){
			
			this.weekTextBox.weekPicker._setValueAttr(this.weekSelected, true);
			if(this.weekTextBox.weekPicker._opened){
				this.weekTextBox.weekPicker.toggleOpenClose();	
				this.weekTextBox.weekPicker.focusNode.blur();
			}

		},
        
        /**
         * When the "Yes" button is pressed in the Timesheet save dialog,
         * the timesheet for the current week is saved first and then
         * fresh timesheet is loaded for the week selected.
         */
        _yesButtonPressed: function(){
        
            this.timesheetSaveDialog.destroy();
            this._saveTimesheet(false);
            this._refresh();
			if(this._newWeekSelected !==null){
				this.weekSelected = this._newWeekSelected;	
			}
        },
        
        /**
         * When the "No" button is pressed in the Timesheet save dialog,
         * the changes in the Timesheet grid for the current week are discarded
         * and the fresh timesheet is loaded for the week selected.
         */
        _noButtonPressed: function(){
        	
            this.timesheetSaveDialog.destroy();
            this._refresh();
			if(this._newWeekSelected !==null){
				this.weekSelected = this._newWeekSelected;	
			}
        },
        
        /**
         * Returns the value of the week selected
         */
        getWeekOfValue: function(){
          
		    if (this.weekTextBox.getValue()) {
			  return this.weekSelected;
            }
        },
        
        /**
         * Returns the list of project to which the user is assigned under the specified project manager.
         * Not required for team member timesheet, so returns empty string.
         */
        getProjectList: function(){
            return "";
        },
        
        /**
         * Fetch the currently logged in user
         */
        getResourceID: function(){
        
            return this.resourceID;
        },
        
        /**
         * Set the currently logged in user to resourceID
         */
        setResourceID: function(){
        
            this.resourceID = epmUtil.getLoggedInUser();
        },
        
        /**
         *
         * @param {Object} status
         * It shows the message while the viewlet is loading
         */
        toggleBusyStatusOfViewlet: function(status){
           // this.getSite().setLoading(status);
			if(status){
				DashboardUtil.showMessage(TimesheetViewletMessages.LOADING);
			}
			else{
				DashboardUtil.hideMessageArea();
			}
        },
        
        /**
         * For the currently logged in user, get the timesheet grid and display in the viewlet.
         */
        _createTimesheetGrid: function(){
        
            this.toggleBusyStatusOfViewlet(true);
			
			var queryURI = TimesheetClient.getUserQueryURIForPost(this.getResourceID(), this.getWeekOfValue());
			
			this.timesheetGrid = new TimesheetGridView({
                _treeID: queryURI, containerViewlet: this
            });
			
			this.timesheetGrid.setPostParams(TimesheetClient.getJSONExpression(this.getResourceID(), this.getWeekOfValue()));
			
            this.timesheetGrid.buildTimesheetGrid();
			
            this._timesheetContentDIV.appendChild(this.timesheetGrid.domNode);
        },
		
        /**
         * The basic toolbar will contain all the image buttons which are specific to any viewlet
         */
        _createBasicToolbar: function(){
        
			this.saveButton = document.createElement(EPMDomNodeConstants.BUTTON);
            this.saveButton.innerHTML = TimesheetViewletMessages.SAVE;
            this.saveButton.title = TimesheetViewletMessages.SAVE;
			this.toggleSaveButton(true);
            dojo.connect(this.saveButton, EPMDomNodeConstants.ON_CLICK, this, this._saveTimesheet);
            
			this.submitButton = document.createElement(EPMDomNodeConstants.BUTTON);
            this.submitButton.innerHTML = TimesheetViewletMessages.SUBMIT;
            this.submitButton.title = TimesheetViewletMessages.SUBMIT;
			this.toggleSubmitButton(true);
            dojo.connect(this.submitButton, EPMDomNodeConstants.ON_CLICK, this, this._submitTimesheet);
			
            this.refreshButton = new dijit.form.Button({
                title: TimesheetViewletMessages.REFRESH,
				label : TimesheetViewletMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._confirmAndRefresh)
            });
            
			var rightContentList = [this.refreshButton, this.submitButton, this.saveButton];
            
            this._timesheetToolBar = new EPMToolbar({
                rightContentList: rightContentList
            });

			this.setCSHelpIdInToolbar(this.contextHelpId, this._timesheetToolBar);            
            
            this.toolbarDIV.appendChild(this._timesheetToolBar.domNode);
        },
		
		_confirmAndRefresh: function()
		{
			if(!this.getSaveButtonStatus()){
				this._createTimesheetSaveDialog();
				return;
			}else{
				this._refresh();
			}
		},		
		        
        /**
         * Reload the timesheet grid of the logged in user.
         */
        _refresh: function(){
			
			this.weekSelected = this.weekTextBox.getValue();
			this._newWeekSelected = null;
			
            this.toggleSaveButton(true);
			
			var numberOfChildNodes = this._timesheetContentDIV.childNodes.length;
                        
            for (var i = 0; i < numberOfChildNodes; i++) {
                this._timesheetContentDIV.removeChild(this._timesheetContentDIV.childNodes[i]);
            }
            if (this.timesheetGrid) {
				
				delete this.timesheetGrid;
            }
            this._createTimesheetGrid();
        },
		
        /**
         *
         * @param {Object} state
         * Save button is enabled once there is any updation in the grid,
         * else it is set to disabled.
         */
        toggleSaveButton: function(state){
        	
			this.saveButton.disabled = state;
			if(state==false)
			{
				this.setDirtyFlag();
			}
			else
			{
				this.resetDirtyFlag();
			}
        },
        
        /**
         * Returns the status of the save button
         */
        getSaveButtonStatus: function(){
            return this.saveButton.disabled;
        },
        
        /**
         *
         * @param {Object} state
         * Change the state of the submit button depending upon the value.
         */
        toggleSubmitButton: function(state){
        
			this.submitButton.disabled = state;
        },
		
		/**
		 * refresh 
		 * Called from framework, useful while switching views/tabs
		 * to clear unsaved or unwanted grid data
		 */
        refresh: function(){
			
			//Just call internal implementation of timesheet viewlet refresh 
			//as all the states of widgets are restored here. 
			this._refresh();
			
		},
        /**
         * Save the timesheet
         */
        _saveTimesheet: function(){
        	this.toggleSaveButton(true);
            if (this.timesheetGrid) {
                this.timesheetGrid.saveTimesheet(false);
            }
        },
        
        /**
         * Submit the timesheet
         */
        _submitTimesheet: function(){
        	this.toggleSaveButton(true);
            if (this.timesheetGrid) {
                this.timesheetGrid.saveTimesheet(true);
            }
        },
        
        /**
         *
         * @param {Object} errorMsg
         */
        _displayErrorMessage: function(errorMsg){
            EPMErrorHandler.setError(errorMsg);
			var self = this;
			
			var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);
        },
        
        /**
         *
         */
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
		
		/**
		 * 
		 * @param {Object} active
		 * 
		 * The week text box if clicked and no date is changed and at the same time if the user clicks on some other tab, 
		 * the weekPicker dialog is still open.
		 * So, check if the viewlet is active or not. If not active, check if the week picker is open.
		 * If open close the week picker dialog and remove the focus.
		 */
		setActive: function(active){
			this.inherited(arguments);
			if(!active){
				if(this.weekTextBox.weekPicker._opened){
					this.weekTextBox.weekPicker.toggleOpenClose();	
					this.weekTextBox.weekPicker.focusNode.blur();
				}
			}
		},
		
        /**
         * Destroy all the objects created.
         */
        destroy: function(){
        
            if (this.weekTextBox) {
                this.weekTextBox.destroy();
            }
            if (this.timesheetGrid) {
            
                this.timesheetGrid.destroy();
            }
        }
    });
    
})();
