/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function() {
	
	dojo.provide("com.ibm.epm.timesheet.web.ui.internal.WeekTextBox");
	
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
	dojo.require("dojo.parser");
	dojo.require("dojo.date");
	dojo.require("dojo.date.locale");
	dojo.require("dijit.form.Button");
	dojo.require("dijit.form.DateTextBox");
	
	var DateUtil = com.ibm.epm.web.core.util.internal.DateUtil;

 	dojo.declare("com.ibm.epm.timesheet.web.ui.internal.WeekTextBox", [dijit._Widget, dijit._Templated], {

    	templatePath: dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/templates/WeekTextBox.html"),

    	constructor: function() {

        	this.weekPicker = new WeekDateTextBox({
        		isDate: true,
        		containerToggle: "fade",
				onKeyPress: dojo.hitch(this, this._keyPressed),
        		onChange: dojo.hitch(this, this.onChange1)
      		});
    	},
		
		/**
		 * 
		 * @param {Object} event
		 * If enter key is pressed, call the onChange event explicitly
		 * 
		 */
		_keyPressed: function(event){
			
			var keys = dojo.keys;
			
			if(event.keyCode === keys.ENTER){
				
				var value = this.getValue();
				this.onChange1(value);
			}
			
		},
    
    	onChange1: function(value) {
    	
    		// directly calling onChange is not working when the event has to be caught from BasicTimesheetViewlet, so indirection
    		this.onChange(value);
    	},

    	onChange: function(value) {
    	    	
    	},

    	buildView: function() {

      		this.weekTextBoxAttPt.appendChild(this.weekPicker.domNode);
    	},

    	getValue: function() {
    	
			return this.weekPicker.value;
    	}
    
	});

  	dojo.declare("WeekDateTextBox", [dijit.form.DateTextBox], {
  	
    	validator: function(/*anything*/value, /*dijit.form.ValidationTextBox.__Constraints*/ constraints) {
		
      		var regExp = new RegExp(dojo.date.locale.regexp( {formatLength:'medium', selector:'date', locale:''}));

     		var dates = value.split(' - '); // 8/6/2008 - 8/12/2008
      
	  		return (!this._isEmpty(value) && // cannot be empty
      				(this._isEmpty(dates[0]) || regExp.test(dates[0])) &&
					//TODO need to understand and fix the below line
      				//((new Date(dates[0])).getDay() == 1) &&//doesnt work when different locale is chosen
      				((dates[1] == undefined) || this._isEmpty(dates[1]) || regExp.test(dates[1])) && 
      				(!this.required || !this._isEmpty(value)) && 
      				(this._isEmpty(value) || (this.parse(value, constraints) !== undefined))); // Boolean

    	},
	
    	getWeekOf: function(d, startDayOfWeek) {

      		var beginOffset = 0;
	        var endOffset = 0;
	
	      	var t = startDayOfWeek - d.getDay();
	
	      	if (d.getDay() < startDayOfWeek) {
		  	
		  		beginOffset = t - 7;
	        	endOffset = t - 1;
	      	} 
		  	else {
	        
				beginOffset = t;
	        	endOffset = 6 + t;
	      	}
      		var beginOfWeek = dojo.date.add(d, "day", beginOffset);
      		var endOfWeek = dojo.date.add(d, "day", endOffset);

      		var returnVal = {
        		"weekBegin": beginOfWeek,
        		"weekEnd": endOfWeek
      		};
      		return returnVal;
    	},
    
    	startDayOfWeek : 0, // Sunday is start day of week for now, get from constraints later
    
    
    	format: function(/*Date*/ value,/*dojo.date.locale.__FormatOptions*/ constraints) {
			
			var formattedVal = "";
    		//summary: formats the value as a Date, according to constraints
      		if (value) {
				var weekDetails = this.getWeekOf(value, this.startDayOfWeek);

				formattedVal = DateUtil.formatDate(weekDetails.weekBegin);
				formattedVal += ' - ' + DateUtil.formatDate(weekDetails.weekEnd);
      		}
			return formattedVal;
    	},
    
		filter: function(val){
			
			if(val === null || val === undefined){ 
				return ""; 
			}
            var weekDetails = this.getWeekOf(val, this.startDayOfWeek);
			return weekDetails.weekBegin;
		},
				
    	parse: function(/*String*/ value,/*dojo.date.locale.__FormatOptions*/ constraints) {
      		
			//summary: parses the value as a Date, according to constraints

      		var dates = value.split(' - '); // 8/6/2008 - 8/12/2008
      		var parse1 = false;
      		var parse2 = false;
      		if (dates[0]){
      	
				parse1 = dojo.date.locale.parse(dates[0], {formatLength:'medium', selector:'date', locale:''});
      		}	 
      		if (dates[1]){
				
      			parse2 = dojo.date.locale.parse(dates[1], {formatLength:'medium', selector:'date', locale:''});
      		}
      	 

      		// make sure the difference between the dates is 6
      		if ((parse1 && parse2) && (dojo.date.difference(parse1, parse2) == 6)) {
        		
				return parse1; // return first day of the week
      		} else{
      			
				return undefined;	
      		}
      	/* can't return null to getValue since that's special */
    	},

    	serialize: dojo.date.stamp.toISOString,

    	toggleOpenClose: function() {
      
	  		if (!this._opened) {
        		this._open();
        		this._opened = true;
      		} else {
        		this._close();
        		this._opened = false;
      		}
    	},
		
		_handleOnChange: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
			
			this.inherited(arguments);
			this.focusNode.blur();
		},
		
    	postCreate: function() {
      		
			this.inherited(arguments);

      		dojo.subscribe("onCalImageClick", this, this.toggleOpenClose);

      		// default is the week today belongs to
	  		this.attr("value", new Date());
    	}
  	});
})();


