/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.admin.EPMSettings");
    
    dojo.require("dijit._Widget");
    dojo.require("dojox.data.dom");
    
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("jazz.ui.SideNavigationLayout");

	dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView"); //$NON-NLS-1$
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGrid");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUser");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.calendar.Calendar");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarWithGrid");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.Timecodes");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.chargecodes.Chargecodes");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.rates.Rates");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.skills.SkillsGrid");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.proficiency.ProficiencyGrid");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGrid");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGrid");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.staticpooltype.StaticPoolTypeGrid");

    dojo.require("jazz.app.ua.SectionHoverHelp");

    // Constants
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
	// Localization
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
	var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
	var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
	
    dojo.declare("com.ibm.epm.admin.web.ui.internal.admin.EPMSettings", dijit._Widget, {
    
        //
        // Methods intended to be called by child pages
        //
        
        reloadCurrent: function(){
            this._previousActionEntry = this._currentActionEntry || null;
            this._showCurrentWidget(/*isReload*/true);
        },
        
        //
        // Dojo lifecycle and internal methods
        //
        
        postCreate: function(){
            // call superclass's postCreate method
            this.inherited(arguments);
            
            // initialize widget cache
            this._widgetCache = {};
            this.userEditorCache = {};
            
            // set the root style for the Server page and all of its sub-pages
            this.domNode.className = "com-ibm-epm-admin";
            this._appView = new jazz.ui.SideNavigationLayout();
            this.domNode.appendChild(this._appView.domNode);
            
            // register actions and create hyperlinks
            ActionRegistry = getWorkbench().getActionRegistry();
            ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction"); //$NON-NLS-1$ //$NON-NLS-2$
            
            var self = this;
            this.allLinks = [];
            dojo.forEach(ACTION_SETS, function(actionSet){
                self._createNavSection(actionSet);
            });
        },

		_createNavSection: function(actionSet) {
			var self = this;
			var navSection = this._appView.addNavigationSection(document.createElement("div"), actionSet.label, true); //$NON-NLS-1$
			dojo.forEach(actionSet.actions, function(action){
                var id = PREFIX + action.shortId;
				ACTION_MAP[action.shortId] = action;
				if (action.showInNav) {
					this.csHelp = new jazz.app.ua.SectionHoverHelp({
    	                uaKey: "com.ibm.rational.epm.cshelp." + action.csHelpId
        	        });
            	    this.csHelp.domNode.className = "com-ibm-epm-web-core-ui-contextHelp";
					var link = navSection.addSectionRow(action.label, "#action=" + id); //$NON-NLS-1$
					link.__jazzActionId = action.shortId;
					
					// TODO: Need to add the help icon properly using addArtifactCommand
/*					var span = document.createElement("span");
                    span.__jazzReload = true;
                    link.appendChild(this.csHelp.domNode);
                    link.appendChild(span);					*/
					
					self.allLinks.push(link);
					navSection.addArtifactCommand(link, AdminMessages.NAV_REFRESH, dojo.hitch(self, self._onNavSectionClick), "admin-refresh-command"); //$NON-NLS-1$
					link.appendChild(this.csHelp.domNode);
				}
				ActionRegistry.registerAction(id, self, "_handleAction");  //$NON-NLS-1$
			});
		},
        
         _defaultAction: function(){
            // by default, show the first section from the left-nav
            this._handleAction({
                action: PREFIX + ACTION_SETS[0].actions[0].shortId
            });
        },
        
        _initAdminUI: function(){
            // by default, show the first section from the left-nav
            this._handleAction({
                action: PREFIX + ACTIONS[0].shortId
            });
        },
        
        _handleAction: function(args){
            if (!isAdmin()) {
                this._showUnauthorized();
                return;
            }
            
            if (!args.action) {
                console.debug("EPMSettings.js#_handleAction: missing required param 'args.action'");
                return;
            }
            
            // cache the previous entry to see if the widget should be destroyed or
            // not
            this._previousActionEntry = this._currentActionEntry || null;
            
            // parse out the short id from the fully-qualified id
            var shortId = args.action.substring(args.action.lastIndexOf(".") + 1); //$NON-NLS-1$
            this._currentActionEntry = ACTION_MAP[shortId];
            this._currentArgs = args;
            this._currentArgs.parent = this;
            
            // set the title for the action
            getWorkbench().setTitle(this._currentActionEntry.label);
            
            // highlight the selected link and unhighlight the others
            dojo.forEach(this.allLinks, function(link){
                if (link.__jazzActionId === shortId) {
                    dojo.toggleClass(link, "selected", true); //$NON-NLS-1$
					dojo.style(link.lastChild,"visibility", "inherit");
                }
                else {
                    if (link.aliases != null) {
                        var found = false;
                        for (var i = 0; i < link.aliases.length; i++) {
                            if (link.aliases[i] === shortId) {
                                dojo.toggleClass(link, "selected", true); //$NON-NLS-1$
                                found = true;
                                break;
                            }
                        }
                        if (!found) {
                            dojo.toggleClass(link, "selected", false); //$NON-NLS-1$
                        }
                    }
                    else {
                        dojo.toggleClass(link, "selected", false); //$NON-NLS-1$
                        dojo.style(link.lastChild,"visibility", "hidden");
                    }
                }
            });
            
            // call the actual method implementation
            if (!this._currentActionEntry.simpleAction || this._currentContentWidget == null ||
            this._currentContentWidget.id.replace(/_\d+$/, "").replace(/_/g, ".") !== SERVER_PACKAGE + this._currentActionEntry.widget) { //$NON-NLS-1$ //$NON-NLS-2$
                this._showCurrentWidget();
            }
            if (this._currentActionEntry.simpleAction) {
                this._currentActionEntry.simpleAction(args, this._currentContentWidget);
            }
        },
        
        _onNavSectionClick: function(e){
            dojo.stopEvent(e);
            this.reloadCurrent();
        },
        
        _showCurrentWidget: function(isReload){
            // save a reference to the prior content widget
            this._previousContentWidget = this._currentContentWidget || null;
             // help the garbage collector do its job
            if (this._previousContentWidget && !this._previousActionEntry.cached) {
                this._previousContentWidget.destroyRecursive();
            }
            if (this._currentActionEntry.cached) {
                this._showCachedWidget(isReload);
            }
            else {
                this._showUncachedWidget();
            }
            
           
        },
        
        _showCachedWidget: function(isReload){
            var className = this._currentActionEntry.widget;
            var shortId = this._currentActionEntry.shortId;
            if (this._widgetCache[shortId + "." + className]) {
                // get widget from the cache if it exists
                this._currentContentWidget = this._widgetCache[shortId + "." + className];
            }
            else {
                // otherwise instantiate it and add it to the cache
                var constructor = dojo.getObject(SERVER_PACKAGE + shortId + "." + className);
                alert(constructor);
                this._currentContentWidget = new constructor(this._currentArgs);
                dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
                this._widgetCache[shortId + "." + className] = this._currentContentWidget;
            }
            if (isReload) {
                this._currentContentWidget.reload(this._currentArgs);
            }
            else {
                this._currentContentWidget.show(this._currentArgs);
                this._appView.setMainContent(this._currentContentWidget.domNode);
            }
        },
        
        _showUncachedWidget: function(){
            var shortId = this._currentActionEntry.shortId;
            var constructor = dojo.getObject(SERVER_PACKAGE + shortId + "." + this._currentActionEntry.widget);
            this._currentContentWidget = new constructor(this._currentArgs);
            
            dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
            this._appView.setMainContent(this._currentContentWidget.domNode);
        },
        
        _showUnauthorized: function(){
            if (!this._unauth) {
                this._unauth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
            }
            dojox.data.dom.removeChildren(this.domNode);
            this.domNode.appendChild(this._unauth.domNode);
        }
	});
    
    //
    // aliases, constants, and module-private variables
    //
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    
    var ActionRegistry = null;
    var PREFIX = "com.ibm.epm.admin.";
    var SERVER_PACKAGE = "com.ibm.epm.admin.web.ui.internal.view.";
    var ACTION_MAP = {};
    
    //
    // The following defines the left-nav links and their bindings to actions and
    // widgets.
    //
    var ACTION_SETS = [{
        label: "Resources",
        actions: [{
            label: AdminMessages.ACTION_SETS_USERS_LABEL,
            shortId: "user",
            widget: "EPMUser",
			showInNav: true,
			csHelpId:Constants.USERS_CONTEXT_HELP_ID
	    }, {
            label: AdminMessages.ACTION_SETS_RESOURCES_SKILLS_LABEL,
            shortId: "skills",
            widget: "SkillsGrid",
			showInNav: true,
			csHelpId:Constants.SKILLS_CONTEXT_HELP_ID
        }, {
            label: AdminMessages.ACTION_SETS_RESOURCES_COMPETENCIES_LABEL,
            shortId: "competencies",
            widget: "CompetenciesGrid",
			showInNav: true,
			csHelpId:Constants.COMPETENCIES_CONTEXT_HELP_ID
        },{
            label: AdminMessages.ACTION_SETS_RESOURCES_PROFICIENCIES_LABEL,
            shortId: "proficiency",
            widget: "ProficiencyGrid",
			showInNav: true,
			csHelpId:Constants.PROFICIENCIES_CONTEXT_HELP_ID
        }, {
            label: AdminMessages.ACTION_SETS_RESOURCES_POOLTYPE_LABEL,
            shortId: "staticpooltype",
            widget: "StaticPoolTypeGrid",
			showInNav: true,
			csHelpId:Constants.STATIC_POOL_TYPE_CONTEXT_HELP_ID
        }, {
            label: AdminMessages.ACTION_SETS_RESOURCES_PROFILES_LABEL,
            shortId: "systemprofile",
            widget: "SystemProfileGrid",
			showInNav: true,
			csHelpId:Constants.SYSTEM_PROFILE_CONTEXT_HELP_ID
		}]
        },{
        label: "Projects",
        actions: [{
            label: AdminMessages.ACTION_SETS_CALENDAR_LABEL,
            shortId: "calendar",
			aliases: ["calendar"], //$NON-NLS-1$
            widget: "CalendarWithGrid",
            showInNav: true,
			csHelpId:Constants.CALENDAR_CONTEXT_HELP_ID
        },{
            label: AdminMessages.ACTION_SETS_RISK_CATEGORY_LABEL,
            shortId: "risk",
            widget: "EPMRiskCategoryGrid",
			showInNav: true,
			csHelpId:Constants.RISK_CATEGORY_CONTEXT_HELP_ID
        }]
    }, {
        label: AdminMessages.ACTION_SETS_FINANCIAL_LABEL,
        actions: [{
            label: AdminMessages.ACTION_SETS_FINANCIAL_TIMECODES_LABEL,
            shortId: "timecodes",
            widget: "Timecodes",
			showInNav: true,
			csHelpId:Constants.TIMECODE_CONTEXT_HELP_ID
        }, {
            label: AdminMessages.ACTION_SETS_FINANCIAL_CHARGECODES_LABEL,
            shortId: "chargecodes",
            widget: "Chargecodes",
			showInNav: true,
			csHelpId:Constants.CHARGECODE_CONTEXT_HELP_ID
        }, {
            label: AdminMessages.ACTION_SETS_FINANCIAL_RATES_LABEL,
            shortId: "rates",
            widget: "Rates",
			showInNav: true,
			csHelpId:Constants.RATES_CONTEXT_HELP_ID
        }]
    }];
    	
	function getWorkbench(){
        return PlatformUI.getWorkbench();
    } 
        
    function isAdmin(){
        return com.ibm.team.repository.web.ui.internal.Util.userIsAdmin();
    }
    
})();
