/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.work.web.ui.internal.details.WBSItemPageController");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController");
dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
dojo.require("com.ibm.epm.work.web.ui.internal.details.EPMWorkItemUtils");
dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");

(function(){

    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var WorkItemController = com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var EPMWorkItemUtils = com.ibm.epm.work.web.ui.internal.details.EPMWorkItemUtils;
	var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
	var WORK_ITEM_EDITOR_ID = "__jazzWorkItemEditor";

    dojo.declare("com.ibm.epm.work.web.ui.internal.details.WBSItemPageController", WorkItemController, {
        selectedNode: null,
        selectedNodeID: null,
        
		/**
		 * viewWorkItem: This function is the first entry point while displaying a work item.
		 * 				 Overwritting this function from parent WorkItemController.
		 * @param {Object} kwArgs
		 * @param {Object} isRefresh
		 */
        viewWorkItem: function(kwArgs, isRefresh){
			var workItemEditorWidget = null;
			if (kwArgs.nodeSelected){
				this.selectedNode = kwArgs.nodeSelected;
			}
			else {
				this.selectedNode = EPMWorkItemUtils.selectedWBSTreeNode;
			}
			
			if (!this.selectedNode) {
				this.inherited('viewWorkItem', arguments);
				return;
			}
			var wiHref = dojo.isArray(this.selectedNode.href) ? this.selectedNode.href[0] : this.selectedNode.href;
            var wiId = this._getResourceIDFromURI(wiHref);
            this.selectedNodeID = wiId;
			kwArgs.id = wiId;
            this._isSubsequentView = true;
            checkNotNull(kwArgs, "kwArgs");
            if (kwArgs.nodeSelected == null) {
                kwArgs.nodeSelected = EPMWorkItemUtils.selectedWBSTreeNode;
            }
            checkNotNull(kwArgs.nodeSelected, "kwArgs.nodeSelected");
            
            var id = parseInt(wiId, 10);
            if (!id || id <= 0) {
                this.handleError({
                    message: this.messages.errorOpeningWorkItem
                }, dojo.string.substitute(this.messages.errorOpeningWorkItemMessage, [("" + wiId)]));
            }
            
			if (this._workItemPageView.isWorkItemEditorAvailable(id)) {
				workItemEditorWidget = this._workItemPageView._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, id);
				if (workItemEditorWidget && workItemEditorWidget.domNode == null) {
					this._workItemPageView._multipaneContentWidget.controller.removeCachedWidget(WORK_ITEM_EDITOR_ID, id);
					isRefresh = true;
				}
			}

            // if the work item editor is already open for the specified id, and
            // we're not doing an explicit refresh, then simply change the tab
            if (this._workItemPageView.isWorkItemEditorAvailable(id) && !isRefresh) {
                this._workItemPageView.showEditorTab(id, kwArgs.tab, kwArgs.doSave);
                return;
            }
			
			if (this._workItemPageView.isWorkItemEditorAvailable(id)) {
				workItemEditorWidget = this._workItemPageView._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, id);
				if (workItemEditorWidget) {
					this._workItemPageView._multipaneContentWidget.controller.removeCachedWidget(WORK_ITEM_EDITOR_ID, id);
				}
			}
			this._workItemPageView.showWorkItemEditor(wiId, "", isRefresh);
			this._getWorkItemData();
        },
       
        _getWorkItemData: function(){
			EPMWorkItemUtils.wiLoading = true;
            var self = this;
            var rowURI = dojo.isArray(self.selectedNode.href) ? self.selectedNode.href[0] : self.selectedNode.href;
            var getWBSItemDataFunctions = {
                success: function(loadedElement){
                    if (loadedElement) {
                        EPMWorkItemUtils.controllerNode = loadedElement;
						EPMWorkItemUtils.wiLoading = false;
                    }
                },
                failure: function(errorObj){
                    self.handleError(errorObj.errorMessage, self.errorOpeningWorkItem);
					EPMWorkItemUtils.wiLoading = false;
                }
            };
            
            var srh = new ServiceResponseHandler(getWBSItemDataFunctions, "success", "failure");
			gridModelClient.getURI(srh, rowURI);
        },
        
        _getResourceIDFromURI: function(workitemHref){
            var index = workitemHref.lastIndexOf("/");
            if (index > 0) {
                return workitemHref.substring(index + 1, workitemHref.length);
            }
        },
        
		/**
		 * _internalViewWorkItem: This method registers the action & page id by using Util methods
		 * 							Called from details viewlet on Tree node click.
		 * @param {Object} kwArgs
		 */
		_internalViewWorkItem: function(kwArgs) {
			if (kwArgs.nodeSelected){
				this.selectedNode = kwArgs.nodeSelected;
			}
			else {
				this.selectedNode = EPMWorkItemUtils.selectedWBSTreeNode;
			}
			var wiHref = dojo.isArray(this.selectedNode.href) ? this.selectedNode.href[0] : this.selectedNode.href;
	        var wiId = this._getResourceIDFromURI(wiHref);
			checkNotNull(wiId, "id");
			var numId= parseInt(wiId, 10);
	
			if (!numId || numId <= 0) {
				this.handleError({message: this.messages.errorOpeningWorkItem}, dojo.string.substitute(this.messages.errorOpeningWorkItemMessage, [ ("" + wiId)]));
			}
			window.location.href = EPMWorkItemUtils.getViewWorkItemURI(wiId, null, kwArgs.tab);
		},
		
		/**
		 * preventWIAction: Registered method for all the details view actions for which propagation needs to be stopped.  
		 * @param {Object} e
		 */
		preventWIAction: function(e) {
			alert(WorkMessages.ACTION_NOT_SUPPORTED_MSG);
			if ( (e)&& (e.stopPropagation) ) {
				e.stopPropagation();
				e.preventDefault();
			}
		}

    });
})();
