/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.grid.BaselineViewlet");
    
    // Viewlet
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    
    // Grids
    dojo.require("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSBaselineGridView");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
	
	
	//ServiceResponseHandler
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    
    // Events published by the Grid View
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.TreeEventListener");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub"); 
    // Events published by the WBS Tree Widget
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSTreePubSub");
    
    // Constants
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    dojo.require("com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants");
	
	// Web core constant for the tree id 
	dojo.require("com.ibm.epm.web.core.util.internal.Constants");
		
	 // Localization
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var GridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var TreeEventListener = com.ibm.epm.web.core.ui.internal.TreeEventListener;
    var _wbsGeneralGrid = com.ibm.epm.work.web.ui.internal.wbsGrids.WBSBaselineGridView;
	var WBSTreePubSub = com.ibm.epm.work.web.ui.internal.WBSTreePubSub;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var WBSGridGanttPubSub = com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub;
	var Constants = com.ibm.epm.web.core.util.internal.Constants; // For Tree id 
    var WorkViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
   
    
	
    dojo.declare("com.ibm.epm.work.web.ui.internal.grid.BaselineViewlet", [Viewlet, TreeEventListener], {
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/templates/BaselineViewlet.html"),
        
        contextHelpId: WBSConstants.WBS_BASELINE_VIEWLET_CONTEXT_HELP_ID, //	The Context Sensitive Help ID for the WBS grid viewlet. 
        noWBSTree: false,
        _wbsGeneralGrid: null,
        
		/**
		 * 
		 * Subscribe to the on select event of the EPMTree
		 */
        init: function() {
			this.inherited(arguments);
			
            this._createInformationLabel();
            if (this.subscribeToOnselectOftree == null) {
                this.subscribeToOnselectOftree = dojo.subscribe(EPMTreePubSub.ON_SELECT, this, "onSelectionOfRow");
            }
            // Disable viewlet if no WBS tree
            if (this._disableViewletIfNoWBSTree())
                return;        
        },

        /*
         * Create information label text. Do this just one time for the life of the
         * viewlet.
         */
        _createInformationLabel: function() {
            this.informationLabel = document.createElement("label");
            this.informationLabel.innerHTML = "";
            this.setCSHelpIdInToolbar(this.contextHelpId, this._informationLabel);
            this._informationLabel.appendChild(this.informationLabel);
        },
         
		 /**
		  *
		  */  
		  onSelectionOfRow: function() {
		  	if(this._wbsGeneralGrid)
			{
				this._wbsGeneralGrid.onRowSelect();
			}
		  },
        /*
         * Disable viewlet and display an appropriate message if there is no WBS Tree
         * for the viewlet to make use of. Returns true if viewlet was disabled, signifying
         * to caller that further setup is not necessary.
         */
        _disableViewletIfNoWBSTree: function() {
            var tree = dijit.byId(Constants.WBS_TREE_ID);
            if (tree)
                return false;

            this.noWBSTree = true;
            this._displayInformationLabel(WorkViewletMessages.WBS_TREE_MISSING);
            return true;
        },
        
        /*
         * Displays information label and hides the rest of the viewlet.
         */
        _displayInformationLabel: function(text) {
            dojo.addClass(this._content, "HiddenClass");
            this.informationLabel.innerHTML = text;
            dojo.removeClass(this._informationLabel, "HiddenClass");
        },
        
        /*
         * Hides information label and shows the rest of the viewlet.
         */
        _hideInformationLabel: function() {
            dojo.addClass(this._informationLabel, "HiddenClass");
            this.informationLabel.innerHTML = "";
            dojo.removeClass(this._content, "HiddenClass");
        },
        
        /**
         *  persistViewlet : Saves the modified data in the grid/gantt
         *  Incoming : state
         *  Outgoing : none
         *
         */
        persistViewlet: function(){
            if (this._wbsGeneralGrid != null) {
                this._wbsGeneralGrid.persistGrid();
            }
        },
        
		/*
		 * Destroy the created grid and error label.
		 */
        destroy: function(){
            this._destroyGrids();
            
            for (var i=0; i < this._informationLabel.childNodes.length; i++) {
                this._informationLabel.removeChild(this._informationLabel.childNodes[i]);
            }
        },
        
		/**
		 * Check and destroy and unsubscribe
		 */
        _destroyGrids: function(){
            if (this._wbsGeneralGrid != null) {
                this._wbsGeneralGrid.destroy();
                delete this._wbsGeneralGrid;
            }
        },
        
		/*
		 * Add the baseline view.
		 */
        refresh: function() {
            // Only refresh if the viewlet is not disabled due to having no WBS tree
            if (!this.noWBSTree) {
                if (this._wbsGeneralGrid == null) {
                    var args = {
                        topToolbar: this.topToolbar,
                        _treeID: "WBSTree"
				    };
						
			        this._wbsGeneralGrid = new com.ibm.epm.work.web.ui.internal.wbsGrids.WBSBaselineGridView(args);
					
					this.helpButtonToolbar = new dijit.Toolbar({
		                layoutAlign: "top",
		                style: "background : transparent; border: none"
		            });
            		if (this.helpButtonToolbar) {
						this.setCSHelpIdInToolbar(this.contextHelpId, this.helpButtonToolbar);
						this._wbsGeneralGrid.helpButton.appendChild(this.helpButtonToolbar.domNode);
					}

			        this._content.appendChild(this._wbsGeneralGrid.domNode);
				
				    this._redrawGrids();
			    }
                else {
                    this._redrawGrids();
                }
            }
        },
        
		/**
		 * Validate the project
		 * This checks if there is a node selected in the wbs tree
		 * If its a program, show the tip of selecting the tree
		 * else hide it and show the grid 
		 */
		 _validateForProject: function(){
            var tree = dijit.byId(Constants.WBS_TREE_ID);

            if (tree && tree._selectedNode)
			{				
                var item = tree._selectedNode.item;
                if (item && item.resourcetype == WBSConstants.EPM_PROGRAM_TYPE) 
                    this._displayInformationLabel(WorkViewletMessages.SELECT_PROJECT_TO_POPULATE_BASELINE);
				else
                    this._hideInformationLabel();
            }
            else
			{
				this._displayInformationLabel(WorkViewletMessages.SELECT_PROJECT_TO_POPULATE_BASELINE);
			}
		},
		
        /**
         * refreshViewlet: refresh the wbs tree, general grid as well as the schedule grid. <br>
         * Incoming: none.
         */
        refreshViewlet: function(){
            if (this._wbsGeneralGrid != null) {
                this._wbsGeneralGrid.refreshGrid();
			}
        },
        
		/**
		 * Redraw the grid
		 * Call the onRowSelect fun of BaselineGridView 
		 * to Populate the select tags if a project is already selected
		 */
        _redrawGrids: function(){
            if (this._wbsGeneralGrid != null) {
				this._wbsGeneralGrid.onRowSelect();
				this._validateForProject();
			}
        },
        
        /**
         *  afterTreeModified : Overridden function from TreeEventListener
         *  Incoming : none
         *  Outgoing : none
         *
         */
        afterTreeModified: function(){
		   this.update();
        },

		subscribeToTreeEvents: function(){
            this._eventSubscriptionHandler = [
				dojo.subscribe(EPMTreePubSub.ON_SELECT, this, "_validateForProject")
			];
			 if (this.subscribeToOnselectOftree == null) {
                this.subscribeToOnselectOftree = dojo.subscribe(EPMTreePubSub.ON_SELECT, this, "onSelectionOfRow");
            }
        },
        
        unSubscribeFromTreeEvents: function(){
            if (this._eventSubscriptionHandler != null) {
                for (var i in this._eventSubscriptionHandler) {
                    var handler = this._eventSubscriptionHandler[i];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._eventSubscriptionHandler = [];
			dojo.unsubscribe(this.subscribeToOnselectOftree);
            this.subscribeToOnselectOftree = null;
        },
				
        /**
         * setActive: Overridden function from Viewlet.
         * active: Boolean value whether tab is active or not.
         */
        setActive: function(active){
			console.log("Baseline VIEWLET IS NOW ACTIVE = " + active );

            // Only necessary if the viewlet is not disabled due to having no WBS tree
			if (!this.noWBSTree) {   
           		if (active) {
 					if (this._wbsGeneralGrid != null) {
	                    this._wbsGeneralGrid.updateWithAutoHeight();
    	            	this._wbsGeneralGrid.onRowSelect();
					}
                    this.subscribeToTreeEvents();
				} else {
                    this.unSubscribeFromTreeEvents();
				}
            }
            this.inherited(arguments);
        }
    });
})();