/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.grid.OverViewViewlet");
    
    // Viewlet
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    
    // Grids
    dojo.require("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridView");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
	
    // Events published by the Grid View
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.TreeEventListener");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
	
    // Events published by the WBS Tree Widget
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSTreePubSub");
    
    // Constants
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
	// Web core constant for the tree id 
	dojo.require("com.ibm.epm.web.core.util.internal.Constants");
		
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	
	//localization
	dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    
	var WorkViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var TreeEventListener = com.ibm.epm.web.core.ui.internal.TreeEventListener;
    var _wbsGeneralGrid = com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridView;
    var WBSTreePubSub = com.ibm.epm.work.web.ui.internal.WBSTreePubSub;
	var Constants = com.ibm.epm.web.core.util.internal.Constants; // For Tree id
	var controllerID = com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants.WBSGeneralGridController;
	
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var WBSGridGanttPubSub = com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub;

    var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
	
    dojo.declare("com.ibm.epm.work.web.ui.internal.grid.OverViewViewlet", [Viewlet, TreeEventListener], {
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/templates/OverViewViewlet.html"),
        
        SHOW_HIDE_CTRL: 9, //	Width of the Splitter which need to be subtracted from the max limit of left Nav
        contextHelpId: WBSConstants.WBS_OVERVIEW_VIEWLET_CONTEXT_HELP_ID, //	The Context Sensitive Help ID for the WBS grid viewlet. 
        noWBSTree: false,       
        
       
        constructor: function() {
			
			this.saveButton=WorkViewletMessages.SAVE;
        },
		
		
        init: function(){
			this.inherited(arguments);
			
            this._wbsGeneralGrid = null;

            this._createInformationLabel();
            
			this.tree = dijit.byId(Constants.WBS_TREE_ID);
						
            // Disable viewlet if no WBS tree
            if (this._disableViewletIfNoWBSTree())
                return;            
          
            this.subscribeToEventsForSaveButton();            
            this._createToolbar();
			this._addRemoveButton();
			this._addRefreshButton();
			         
            // Variable to indicate that a save is in-progress
            this.saveInProgressFlag = false;
            
            // Variable to indicate that there is a pending save
            this.pendingSaveFlag = false;
			
            // Variable to indicate if we need to delete all grid data and fetch
            // or just discard unsaved data and fetch the rows that were changed
            this.deleteAndFetch = false;
        },
		
        /*
         * Create information label text. Do this just one time for the life of the
         * viewlet.
         */
        _createInformationLabel: function() {
            this.informationLabel = document.createElement("label");
            this.informationLabel.innerHTML = "";
            this._informationLabel.appendChild(this.informationLabel);
        },
               
        /*
         * Disable viewlet and display an appropriate message if there is no WBS Tree
         * for the viewlet to make use of. Returns true if viewlet was disabled, signifying
         * to caller that further setup is not necessary.
         */
        _disableViewletIfNoWBSTree: function() {
            if (this.tree) {
                return false;
            }

            this.noWBSTree = true;
            this._displayInformationLabel(WorkViewletMessages.WBS_TREE_MISSING);
            return true;
        },
        
        /*
         * Displays information label and hides the rest of the viewlet.
         */
        _displayInformationLabel: function(text) {
            dojo.addClass(this._content, "HiddenClass");
            this.informationLabel.innerHTML = text;
            dojo.removeClass(this._informationLabel, "HiddenClass");
        },
        
        /*
         * Hides information label and shows the rest of the viewlet.
         */
        _hideInformationLabel: function() {
            dojo.addClass(this._informationLabel, "HiddenClass");
            this.informationLabel.innerHTML = "";
            dojo.removeClass(this._content, "HiddenClass");
        },
        
        /**
         *  enableSaveButton : Event handler when there is any updation in Grid
         *  Incoming : publishedObj
         *  Outgoing : none
         *
         */
        enableSaveButton: function(publishedObj){
			if (!publishedObj) {
				return;
			}
			
			// Check if the event was due to changes in the overview grid
			if (publishedObj.controllerID != controllerID) {
				return;
			}
			
            if (publishedObj.uniqueIdentifier) {
                if (publishedObj.uniqueIdentifier.treeID == WBSConstants.WBS_TREE_ID) {
                    this._checkToEnableSaveButton();    
                }
            }else if (publishedObj.declaredClass == WBSConstants.DND_MOVER_CLASS) {            	
            	// event from Gantt for drag and drop
            	this._checkToEnableSaveButton();
            }
        },

        /**
         *  _checkToEnableSaveButton : Checks if there is a save in-progress. If
         * there is one, it sets a flag that defers enabling the save button.
         * If no save is in-progress, it enables the save button.
         *  Incoming : none
         *  Outgoing : none
         *
         */        
        _checkToEnableSaveButton: function() {
            // Check if there is a save in progress
            if (!this.saveInProgressFlag) {
                // No saves in progress; enable the save button
                this.toggleSaveButton(false);
            } else {
                 // Indicate that the save button must be enabled after the
                 // in-progress save is complete
                this.pendingSaveFlag = true;
            }	
        },
        
        /*
         * isSaveInProgress: Flag that indicates that a save is in progress
         * Incoming: none
         * Outgoing: boolean (saveInProgressFlag)
         */
        isSaveInProgress: function() {
            return this.saveInProgressFlag;    
        },
        
        /**
         *  saveInProgress : Indicates that a save is in progress
         *  Incoming : none
         *  Outgoing : none
         *
         */
        saveInProgress: function() {
            
            // Check if the button was left enabled by mistake.
            // If yes, then disable it.
            if (this.isButtonEnabled()) {
                this.toggleSaveButton(true);
            }
            
            // Indicate that there is a save in-progress
            this.saveInProgressFlag = true;
        },
        
        /**
         *  saveComplete : Handles the pending request to enable the save button
         *  Incoming : none
         *  Outgoing : none
         *
         */
        saveComplete: function() {
        	
        	// Indicate that the in-progress save has been handled
            this.saveInProgressFlag = false;
            
            // Check if there is a pending save 
            if (this.pendingSaveFlag) {
               // Enable the save button
               this.toggleSaveButton(false);
               
               // Indicate that the pending option was handled
               this.pendingSaveFlag = false;
            }   
        },
        
        
        /**
         *  _createToolbar : Creates the toolbar.
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createToolbar: function(){
            this.topToolbar = new dijit.Toolbar({
                layoutAlign: "top",
                style: "background : transparent; border: none"
            });
			this.imageButtons.appendChild(this.topToolbar.domNode);			
        },
                
        /**
         *  _addRemoveButton : Appends the remove button to the toolbar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _addRemoveButton: function(){
            if (this.topToolbar) {
				this.removeButton = new dijit.form.Button({
                    title: WorkViewletMessages.REMOVE_WBS,
                    label : WorkViewletMessages.REMOVE_BUTTON,
					showLabel : false,
                    iconClass: WBSConstants.REMOVE_WBS_ICON_CLASS,
                    onClick: dojo.hitch(this, this._removeWBSCommand)
                });
				this.topToolbar.addChild(this.removeButton);
            }
        },
        
        /**
         *  _removeRemoveButton : Removes the remove button from the toolbar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _removeRemoveButton: function(){
            if (this.topToolbar) {
                if (this.removeButton) {
                    this.topToolbar.removeChild(this.removeButton);
                    this.removeButton.destroy();
					this.removeButton = null;
                }
            }
        },

		/**
		 * Method called when the user wants to remove WBS command is fired.
		 */
		_removeWBSCommand: function(){
			if ((this._wbsGeneralGrid) && (this._wbsGeneralGrid._controller)) {
            	if (this.tree) {
					this._wbsGeneralGrid._controller._removeWBSElement(this.tree._selectedNode, false);
				}
			}
		},
		        
        /**
         *  _addRefreshButton : Appends the refresh button to the toolbar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _addRefreshButton: function(){
            if (this.topToolbar) {
				// Add the CSH button
				this.setCSHelpIdInToolbar(this.contextHelpId, this.topToolbar);

                this.refreshButton = new dijit.form.Button({
                    title: WorkViewletMessages.REFRESH,
                    label : WorkViewletMessages.REFRESH,
					showLabel : false,
                    iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                    onClick: dojo.hitch(this, this.confirmAndRefreshViewlet)
                });
                this.topToolbar.addChild(this.refreshButton);
            }
        },
        
        /**
         *  _removeRefreshButton : Removes the refresh button from the toolbar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _removeRefreshButton: function(){
            if (this.topToolbar) {
                if (this.refreshButton) {
                    this.topToolbar.removeChild(this.refreshButton);
                    this.refreshButton.destroy();
					this.refreshButton = null;
                }
            }
        },
        
        /**
         *  persistViewlet : Saves the modified data in the grid/gantt
         *  Incoming : state
         *  Outgoing : none
         *
         */
        persistViewlet: function(){
            
            // Call the view to save the grid changes to the model first
            if (this._wbsGeneralGrid) {
                this._wbsGeneralGrid.saveChangesToModel();
            }
            
            // Disable save button
            this.toggleSaveButton(true);
            
            if (this._wbsGeneralGrid) {
                this._wbsGeneralGrid.persistGrid();
            }
        },
        
        // Destroy needs to be implemented by the viewlet as its invoked by the framework on cleanup
        destroy: function(){
            this._destroyGrids();
            if (this.informationLabel != null) {
				this._informationLabel.removeChild(this.informationLabel);
				this.informationLabel = null;
			}
        },
        
        _destroyGrids: function(){
            if (this.noWBSTree)
                return;
                
            if (this._wbsGeneralGrid) {
                // uncommenting the below line will cause problems is switching the views. 
                //        		this._layout._leftNav.removeChild(this._wbsGeneralGrid.domNode);
                
                this._wbsGeneralGrid.destroy();
                delete this._wbsGeneralGrid;
            }
			
			this._removeRemoveButton();
            this._removeRefreshButton();

            if (this.topToolbar) {
                this.topToolbar.destroy();
				this.topToolbar = null;
            }
       },
        
        refresh: function(){
			this.toggleSaveButton(true);
            if (this.noWBSTree)
                return;

			if (!this.topToolbar) {
				this._createToolbar();				
			}
			if(!this.removeButton){
				this._addRemoveButton();
			}
			if(!this.refreshButton){
				this._addRefreshButton();
			}
			if(!this.informationLabel){
				this._createInformationLabel();
			}
            if (this._wbsGeneralGrid == null) {
                var args = {
                    topToolbar: this.topToolbar,
					_viewlet: this,
                    _treeID: "WBSTree"
                };
				this._wbsGeneralGrid = new com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridView(args);
				this._gridContent.appendChild(this._wbsGeneralGrid.domNode);   
			} else {
                if (this.deleteAndFetch) {
                    //Destroy the grid, toolbar and related buttons and then recreated them.
                    this.deleteAndFetch = false;
                    this.destroy();
                    this.refresh();
                }
                else {
                    // Refetch just the rows that have changed
                    this.discardUnsavedGridChanges();
                }
            }
			
			if (this._wbsGeneralGrid) {
				this.rowUpdatedEvent = dojo.connect(this._wbsGeneralGrid, "rowUpdated", dojo.hitch(this, this.initViewBasedonTree));
			}
			this.enableSaveForNewElement();
        },
        
		/**
		 * Discard unsaved grid changes.
		 */
		discardUnsavedGridChanges: function(){
            if (this._wbsGeneralGrid) {
	        	this._wbsGeneralGrid.discardUnsavedGridChanges();
    		}
		},

		
        /**
         * confirmAndRefreshViewlet: Confirm and refresh the grid. <br>
         * Incoming: none.
         */
        confirmAndRefreshViewlet: function(){
            if (this.noWBSTree)
                return;
            if(!this.saveBtn.disabled){
				this.discardUnsavedChanges();
				return;
			}else{
				this.refreshViewlet();
			}
			    
        },
        
		/**
		 * refreshViewlet: Publish an event to which tree has subscribed to refresh the tree and grid subsequently. <br>
		 * Incoming: none
		 */
		refreshViewlet: function(){
            this.toggleSaveButton(true);
            dojo.publish(WBSGridGanttPubSub.ON_REFRESH);
		},
		
		discardUnsavedChanges: function(){
            var buttonlist = [{
                label: WorkViewletMessages.YES,
                action: dojo.hitch(this, this._discardChanges)
            }, {
                label: WorkViewletMessages.NO,
                action: dojo.hitch(this, this._cancelOperation)
            }];
            
            var labelList = [{
                label: WorkViewletMessages.CONFIRM_UNSAVED_CHANGES
            }];
            
			//TODO: Check if this confirm dialog can be moved to a common place for overview and scnedule viewlet.
	        this.discardUnsavedDialog = new EPMConfirmDialog({
	            labelList: labelList,
	            title: WorkViewletMessages.DISCARD_CHANGES_TITLE,
	            buttonlist: buttonlist
	        });
	        
	        this.discardUnsavedDialog.show();
		},
		
		_discardChanges: function(){
			this.discardUnsavedDialog.destroy();
			this.refreshViewlet();
		},
		
		_cancelOperation: function(){
			this.discardUnsavedDialog.destroy();
		},
		
        _redrawGrids: function(){
            if (this._wbsGeneralGrid) {
                this._wbsGeneralGrid.updateWithAutoHeight();
            }
        },
        
        /**
         *  toggleSaveButton : Save button is enabled once there is any updation in the grid
         *  Incoming : state
         *  Outgoing : none
         *
         */
        toggleSaveButton: function(state){
            this.saveBtn.disabled = state;
            
            if (state == true) {
                this.subscribeToEventsForSaveButton();
				this.resetDirtyFlag();
            }
            else {
                this.unsubscribeToEventsForSaveButton();
				this.setDirtyFlag();
            }
            
        },
        
        /**
         *  subscribeToEventsForSaveButton : Function to subscribe to all events needed for enabling Save button
         *  Incoming : none
         *  Outgoing : none
         *
         */
        subscribeToEventsForSaveButton: function(){
            // connect to the events as the button is disabled
            if (this._fieldChangeEvent == null) {
                this._fieldChangeEvent = dojo.subscribe(GridEvents.FIELD_CHANGED, this, this.enableSaveButton);
            }
            
            if (this._onCreateNewElement == null) {
                this._onCreateNewElement = dojo.subscribe(WBSTreePubSub.ON_CREATE_NODE, this, this.onCreateNode);
            }
        },
        
        /**
         *  unsubscribeToEventsForSaveButton : Function to unsubscribe to all events needed for enabling Save button, once the button is enabled
         *  Incoming : none
         *  Outgoing : none
         *
         */
        unsubscribeToEventsForSaveButton: function(){
            // disconnect from the events when the button is enabled
            dojo.unsubscribe(this._fieldChangeEvent);
            this._fieldChangeEvent = null;
            
            dojo.unsubscribe(this._onCreateNewElement);
            this._onCreateNewElement = null;
			
        },

        /*
         * When the viewlet gets loaded completely, this method is called to check if the Save button needs to be enabled
         */
        enableSaveForNewElement: function() {
			// Check if the WBS Tree has any unsaved element added
			var tree = this.tree;
            if (tree && tree.hasNewNode) {
				this._checkToEnableSaveButton();
            }
        },
        
		/**
		 * Enable the save button for all the work items except the project.
		 * @param {Object} publishedObject
		 */
		onCreateNode: function(publishedObject){
			var resourceType = publishedObject.resourcetype;
			if(resourceType && resourceType !== WBSConstants.EPM_PROJECT_TYPE){
				this._checkToEnableSaveButton();
			}
		},
		
        /**
         * setActive: Overridden function from Viewlet.
         * active: Boolean value whether tab is active or not.
         */
        setActive: function(active){
            this.inherited(arguments);
            if ( (active == true) && this._wbsGeneralGrid) {
				dojo.subscribe(GridEvents.REFRESH_ROW_DATA, this, this.updateRowData);
                this._wbsGeneralGrid.updateWithAutoHeight();
                this._wbsGeneralGrid.subscribeToKeyboardEvents();
				if(this._wbsGeneralGrid._controller._isHidden_n_Dirty){
					console.debug("viewlet was dirty when hidden hance redrawing");
					this._wbsGeneralGrid._controller._wait_n_updateModel();
					this._wbsGeneralGrid._controller._isHidden_n_Dirty = false;
				}
			} else if(active === false){
				dojo.unsubscribe(GridEvents.REFRESH_ROW_DATA, this, this.updateRowData);
	            if (this._wbsGeneralGrid) {
					this._wbsGeneralGrid.unsubscribeToKeyboardEvents();
				}
				var tree = this.tree;
				if(this.isDirty() == true && tree && tree.hasNewNode){
					this.toggleSaveButton(true);
					tree.hasNewNode = false;
					dojo.publish(WBSGridGanttPubSub.DISCARD_UNSAVED_NODES);
				}
			}
        },
		
		/**
		 * updateRowData: 
		 * 		Calls controller's _getAndUpdateRow() for specific grid row. 
		 * @param {Object} data
		 */
		updateRowData: function(data) {
			if (!data) {
				return;
			}
			var itemIdentifier = null;
			if (data.identifier) {
				itemIdentifier = dojo.isArray(data.identifier) ? data.identifier[0] : data.identifier;
			} 
			else if (data.relativeHref) {
				itemIdentifier = dojo.isArray(data.relativeHref) ? data.relativeHref[0] : data.relativeHref;
			}
			this._wbsGeneralGrid._controller._getAndUpdateRow(itemIdentifier, itemIdentifier, null);
			this._wbsGeneralGrid.updateWithAutoHeight();
		},

		/*
         * Get the current selected tree node
         * Incoming : None
         * Outgoing : None
         */
        initViewBasedonTree: function(){
			if (this.rowUpdatedEvent) {
				dojo.disconnect(this.rowUpdatedEvent);
				this.rowUpdatedEvent = null;
			} else {
				return;
			}
            var tree = this.tree;
            if (tree && tree._selectedNode) {
                var item = tree._selectedNode.item;
                if (item && item.identifier) {
                    var obj = {
                        treeID: item.treeID,
                        identifier: item.identifier,
                        name: item.name,
                        resourcetype: item.resourcetype
                    };
					
					if (this._wbsGeneralGrid) {
						this._wbsGeneralGrid._onSelect(obj);
					}
                }
            }
        },
		      
        /*
         * Returns the state of the grid contained in the viewlet
         * true: Button is enabled (there are unsaved changes)
         * false: Button is disabled (unsaved changes exist) 
         */
        isButtonEnabled: function() {
            return (!this.saveBtn.disabled);
        },
        
        /*
         * Set the deleteAndFetch flag that is used to indicate to the refresh() method
         * that the whole grid must be destroyed and refetched from the server.
         */
        setDeleteAndFetch: function() {
            this.deleteAndFetch = true;
        }
    });
})();