/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.grid.ScheduleGanttViewlet");
    
    // Viewlet
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    dojo.require("com.ibm.epm.web.core.ui.internal.ViewletSplitter");
    
    // Widgets
    //Gantt
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttView");
    // Grids
    
    dojo.require("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGanttGridView");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    
    // Events published by the Grid/Gantt View
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.TreeEventListener");
    // Events published by the WBS Tree Widget
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSTreePubSub");
    
    // Constants
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    dojo.require("com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants");
	// Web core constant for the tree id 
	dojo.require("com.ibm.epm.web.core.util.internal.Constants");
		
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	
	//Confirmation Dialog
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");	
	 // Localization
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    
	var WorkViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var TreeEventListener = com.ibm.epm.web.core.ui.internal.TreeEventListener;
    var WBSGanttGrid = com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGanttGridView;
    var WBSTreePubSub = com.ibm.epm.work.web.ui.internal.WBSTreePubSub;
	
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    
    var ViewletSplitter = com.ibm.epm.web.core.ui.internal.ViewletSplitter;
    var GanttView = com.ibm.epm.work.web.ui.internal.GanttView;
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var WBSGridGanttPubSub = com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub;
    var Constants = com.ibm.epm.web.core.util.internal.Constants; // For Tree id 
    
    var controllerID = com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants.ScheduleGridController;
    var controllerFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
	
	var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.grid.ScheduleGanttViewlet", [Viewlet, TreeEventListener], {
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/templates/ScheduleGanttViewlet.html"),
        
        SHOW_HIDE_CTRL: 9, //	Width of the Splitter which need to be subtracted from the max limit of left Nav
        INITIAL_LHS_VISIBILITY : 1.5, // Defines amount of LHS of the splitter to be visible, units in parts of the entire pane for the LHS
        contextHelpId:WBSConstants.WBS_SCHEDULE_VIEWLET_CONTEXT_HELP_ID, //	The Context Sensitive Help ID for the WBS grid viewlet. 
        _GanttController: null,
        noWBSTree: false,
		
		constructor: function() {			
			this.saveBtn=WorkViewletMessages.SAVE;
        },
        
        init: function(){
			this.inherited(arguments);
			
            this.ganttView = null;
			this._wbsGanttGrid = null;
            
            this._createInformationLabel();
            
            // Disable viewlet if no WBS tree
            if (this._disableViewletIfNoWBSTree())
                return;            
            
			//Schedule viewlet is subscribed to events related to save button, splitter dragging and key events.
			this.subscribeToEventsForSplitter();
            this.subscribeToEventsForSaveButton();
            this._subscribeToEventsFromDateEditor();
            
            this._createToolbar();
            this._layout = new ViewletSplitter({});
            this._gridContent.appendChild(this._layout.domNode);
                                 
            // Variable to indicate that a save is in-progress
            this.saveInProgressFlag = false;
            
            // Variable to indicate that there is a pending save
            this.pendingSaveFlag = false;
            
            // Variable to indicate if we need to delete all grid data and fetch
            // or just discard unsaved data and fetch the rows that were changed
            this.deleteAndFetch = false;
            
            // Variable to indicate if we need to resize the gantt view when
            // the viewlet becomes active again
            this.pendingResize = false;
        },
		
        /*
         * Create information label text. Do this just one time for the life of the
         * viewlet.
         */
        _createInformationLabel: function() {
            this.informationLabel = document.createElement("label");
            this.informationLabel.innerHTML = "";
            this._informationLabel.appendChild(this.informationLabel);
        },
               
        /*
         * Disable viewlet and display an appropriate message if there is no WBS Tree
         * for the viewlet to make use of. Returns true if viewlet was disabled, signifying
         * to caller that further setup is not necessary.
         */
        _disableViewletIfNoWBSTree: function() {
            var tree = dijit.byId(Constants.WBS_TREE_ID);
            if (tree) {
                return false;
            }

            this.noWBSTree = true;
            this._displayInformationLabel(WorkViewletMessages.WBS_TREE_MISSING);
            return true;
        },
        
        /*
         * Displays information label and hides the rest of the viewlet.
         */
        _displayInformationLabel: function(text) {
            dojo.addClass(this._content, "HiddenClass");
            this.informationLabel.innerHTML = text;
            dojo.removeClass(this._informationLabel, "HiddenClass");
        },
        
        /*
         * Hides information label and shows the rest of the viewlet.
         */
        _hideInformationLabel: function() {
            dojo.addClass(this._informationLabel, "HiddenClass");
            this.informationLabel.innerHTML = "";
            dojo.removeClass(this._content, "HiddenClass");
        },
        
        //Reset the Left Nav size when we drag the Splitter of then Main Navigation area or Viewlet.. 
        _onLeftNavResize: function(data){
			if (this.domNode.clientHeight == 0 && this.domNode.clientWidth == 0) {
				//domNode creation is not complete -- exiting to resolve IE7 issues
				return;
			}
            var viewletDom = dojo.contentBox(this.domNode);
            var lefNavDom = dojo.contentBox(this._layout._leftNav);
            if ((data) && (data.type == "ViewletSplitter")) {
                if (viewletDom.w > lefNavDom.w) {
                    this._layout.LEFT_NAV_MAX_WIDTH = viewletDom.w - this.SHOW_HIDE_CTRL;
					this._layout._contentWrapper.style.width = viewletDom.w - lefNavDom.w + "px";
                }
            }
            else {
                if (viewletDom.w < lefNavDom.w) {
                    this._layout.LEFT_NAV_MAX_WIDTH = viewletDom.w - this.SHOW_HIDE_CTRL;
                    this._layout._leftNav.style.width = viewletDom.w - this.SHOW_HIDE_CTRL + "px";
                }
            }
            this._redrawGrids();
        },
        
        /**
         * Reset the Gantt size when we drag or open/collapse the tree/grid splitter
         * 
         * @param {Object} data = {
         *      type Type of splitter
         *      navState State of the splitter
         *          open splitter is open
         *          close splitter is collapsed
         *      splitterDom Dom of the splitter
         * }
         */ 
        _onTreeGridLeftNavResize: function(data){
            var active = this.getSite().isActive();
            if (active) {
                var viewletDom = dojo.contentBox(this.domNode);
                var leftNavDom = dojo.contentBox(this._layout._leftNav);
                if (data && data.type == "TreeGridSplitter") {
                    if (data.splitterDom && viewletDom.w && leftNavDom.w) {
                    
                        // Set the maximum width of the Gantt view
                        this._layout.LEFT_NAV_MAX_WIDTH = viewletDom.w - this.SHOW_HIDE_CTRL;
                        
                        var width = viewletDom.w;
                        
                        // If the grid/gantt splitter is open, then deduct the width of the leftNav;
                        // else occupy the full length of the viewlet
                        if (this._layout._currentLeftNavState === "open") {
                            // If the grid/gantt splitter is open, deduct the width of the leftNav from total
                            width = width - leftNavDom.w;
                        }
                        
                        // If the tree/grid splitter is closed, then add the width of the
                        // tree navigation pane to the total width of the gantt
                        if (data.navState === "close") {
                            // We need to add the width of the tree navigation view to the total
                            width = width + data.splitterDom.w;
                        }
                        
                        // Set the width of the gantt pane
                        this._layout._contentWrapper.style.width = width + "px";
                        this._redrawGrids();
                    }
                }
            } else {
                
                // Since the viewlet is not active, save the publish data
                // This data will be used when the viewlet becomes active again
                this.pendingResize = true;
                this.pendingResizeData = data;
            }
        },
        
        _onSplitterMouseDown: function(data){
			if (this.domNode.clientHeight == 0 && this.domNode.clientWidth == 0) {
				//domNode creation is not complete -- exiting to resolve IE7 issues
				return;
			}
            var viewletDom = dojo.contentBox(this.domNode);
            var lefNavDom = dojo.contentBox(this._layout._leftNav);
            if ((data) && (data.type == "ViewletSplitter")) {
                if (viewletDom.w > lefNavDom.w) {
                    this._layout.LEFT_NAV_MAX_WIDTH = viewletDom.w - this.SHOW_HIDE_CTRL;
                }
            }
        },
        
        //Reset the Left Nav Max width when we double click on the Viewlet Splitter.. 
        _onLeftNavDblClick: function(data){
			if (this.domNode.clientHeight == 0 && this.domNode.clientWidth == 0) {
				//domNode creation is not complete -- exiting to resolve IE7 issues
				return;
			}
            if ((data) && (data.type == "ViewletSplitter")) {
                var viewletDom = dojo.contentBox(this.domNode);
                var gridDom = dojo.contentBox(this._wbsGanttGrid.domNode);
                if (viewletDom.w > gridDom.w) {
                    this._layout.LEFT_NAV_MAX_WIDTH = gridDom.w - this.SHOW_HIDE_CTRL;
                }
				if (this._layout._showHideButton.className.indexOf("showHideButtonExpanded") != -1) {	//Collapsing
					this._layout._contentWrapper.style.width = viewletDom.w + "px";
				} 
				else if (this._layout._showHideButton.className.indexOf("showHideButtonCollapsed") != -1) {	//Expanding
					this._layout._contentWrapper.style.width = (viewletDom.w - gridDom.w)+ "px";
				}
            }
        },
	        
        // overriden function of base viewlet class to handle resize of panes on both side of the splitter
        postZoom: function(){
            if (this.noWBSTree)
                return;
                
            var viewletDom = dojo.contentBox(this.domNode);
            var lefNavDom = dojo.contentBox(this._layout._leftNav);
            if (viewletDom.w < lefNavDom.w) {
                this._layout.setSplitterAttributes(this.domNode, this.INITIAL_LHS_VISIBILITY);
            }
            else {
                this._layout.LEFT_NAV_MAX_WIDTH = viewletDom.w - this.SHOW_HIDE_CTRL;
            }
        },
        
        /**
         *  enableSaveButton : Event handler when there is any updation in Grid/Gantt
         *  Incoming : publishedObj
         *  Outgoing : none
         *
         */
        enableSaveButton: function(publishedObj){
			if (!publishedObj) {
				return;
			}
            if (publishedObj.uniqueIdentifier) {           			
				// Check if the event was due to changes in the overview grid
				if (publishedObj.controllerID != controllerID) {
					return;
				}
            
                if (publishedObj.uniqueIdentifier.treeID == WBSConstants.WBS_TREE_ID) {
                	this._checkToEnableSaveButton();
                }
            }
            else if (publishedObj.declaredClass == WBSConstants.DND_MOVER_CLASS) {
            	// event from Gantt for drag and drop
            	this._checkToEnableSaveButton();
            }    
        },
        
        /**
         *  _checkToEnableSaveButton : Checks if there is a save in-progress. If
         * there is one, it sets a flag that defers enabling the save button.
         * If no save is in-progress, it enables the save button.
         *  Incoming : none
         *  Outgoing : none
         *
         */        
        _checkToEnableSaveButton: function() {
            // Check if there is a save in progress
            if (!this.saveInProgressFlag) {
                // No saves in progress; enable the save button
                this.toggleSaveButton(false);
            } else {
                 // Indicate that the save button must be enabled after the
                 // in-progress save is complete
                this.pendingSaveFlag = true;
            }   
        },
        
        /*
         * isSaveInProgress: Flag that indicates that a save is in progress
         * Incoming: none
         * Outgoing: boolean (saveInProgressFlag)
         */
        isSaveInProgress: function() {
            return this.saveInProgressFlag;    
        },
        
        /**
         *  saveInProgress : Indicates that a save is in progress
         *  Incoming : none
         *  Outgoing : none
         *
         */
        saveInProgress: function() {
            
            // Check if the button was left enabled by mistake.
            // If yes, then disable it.
            if (this.isButtonEnabled()) {
                this.toggleSaveButton(true);
            }
            
            // Indicate that there is a save in-progress
            this.saveInProgressFlag = true;
        },
        
        /**
         *  saveComplete : Handles the pending request to enable the save button
         *  Incoming : none
         *  Outgoing : none
         *
         */
        saveComplete: function() {
            
            // Indicate that the in-progress save has been handled
            this.saveInProgressFlag = false;
            
            // Check if there is a pending save 
            if (this.pendingSaveFlag) {
               // Enable the save button
               this.toggleSaveButton(false);
               
               // Indicate that the pending option was handled
               this.pendingSaveFlag = false;
            }   
        },
        
        /**
         *  _createToolbar : Creates the common toolbar which is populated by the Grid and the Gantt view
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createToolbar: function(){
            this.topToolbar = new dijit.Toolbar({
                layoutAlign: "top",
                style: "background : transparent; border: none"
            });
            if (this.topToolbar) {
                this.timelineSeparator = new dijit.ToolbarSeparator();
                this.topToolbar.addChild(this.timelineSeparator);
            }
        },

        /**
         *  _addRefreshCSHButton : Appends the refresh and csh button to the toolbar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _addRefreshCSHButton: function(){
            if (this.topToolbar) {
				// Add the CSH button
				this.setCSHelpIdInToolbar(this.contextHelpId, this.topToolbar);
				
                this.refreshButton = new dijit.form.Button({
                    title: WorkViewletMessages.REFRESH,
					label : WorkViewletMessages.REFRESH,
					showLabel : false,
                    iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                    onClick: dojo.hitch(this, this.confirmAndRefreshViewlet)
                });
                this.topToolbar.addChild(this.refreshButton);
				
                this.imageButtons.appendChild(this.topToolbar.domNode);
            }
        },

        /**
         * confirmAndRefreshViewlet: Confirm and refresh the grid. <br>
         * Incoming: none.
         */
        confirmAndRefreshViewlet: function(){
            if (this.noWBSTree)
                return;
            if(!this.saveButton.disabled){
				this.discardUnsavedChanges();
				return;
			}else{
				this.refreshGanttViewlet();
			}
			    
        },
        
		/**
		 * refreshViewlet: Publish an event to which tree has subscribed to refresh the tree and grid subsequently. <br>
		 * Incoming: none
		 */
        refreshGanttViewlet: function(){
            this.toggleSaveButton(true);
			dojo.publish(WBSGridGanttPubSub.ON_REFRESH);        
		},

		discardUnsavedChanges: function(){
            var buttonlist = [{
                label: WorkViewletMessages.YES,
                action: dojo.hitch(this, this._discardChanges)
            }, {
                label: WorkViewletMessages.NO,
                action: dojo.hitch(this, this._cancelOperation)
            }];
            
            var labelList = [{
                label: WorkViewletMessages.CONFIRM_UNSAVED_CHANGES
            }];
            
			
	        this.discardUnsavedDialog = new EPMConfirmDialog({
	            labelList: labelList,
	            title: WorkViewletMessages.DISCARD_CHANGES_TITLE,
	            buttonlist: buttonlist
	        });
	        
	        this.discardUnsavedDialog.show();
		},
		
		_discardChanges: function(){
			this.discardUnsavedDialog.destroy();
			this.refreshGanttViewlet();
		},
		
		_cancelOperation: function(){
			this.discardUnsavedDialog.destroy();
		},

        /**
         *  persistViewlet : Saves the modified data in the grid/gantt
         *  Incoming : state
         *  Outgoing : none
         *
         */
        persistViewlet: function(){
            
            // Call the view to save the grid changes to the model first
            if (this._wbsGanttGrid) {
                this._wbsGanttGrid.saveChangesToModel();
            }
            
            // Disable save button
            this.toggleSaveButton(true);
            
            if (this._wbsGanttGrid) {
                this._wbsGanttGrid.persistGrid();
            }
        },
        
        // Destroy needs to be implemented by the viewlet as its invoked by the framework on cleanup
        destroy: function(){
            this._destroyGrids();
            
            if (this.informationLabel != null)
                this._informationLabel.removeChild(this.informationLabel);
        },
        
        _destroyGrids: function(){
            if (this.noWBSTree)
                return;
                
            if (this._wbsGanttGrid) {
                this._wbsGanttGrid.destroy();
                delete this._wbsGanttGrid;
            }
            
            if (this.ganttView) {
                this.ganttView.destroy();
                delete this.ganttView;
            }
        },
        
		/**
         * refresh: Extends the viewlet.refresh function, reloads the data for the schedule viewlet
         * 			which include the schedule grid, gantt and viewlet splitter attributes.
         */
        refresh: function(){
			this.toggleSaveButton(true);
            if (this.noWBSTree)
                return;
                
            if (this._wbsGanttGrid == null) {
                var args = {
                    topToolbar: this.topToolbar,
                    _treeID: "WBSTree"
                };
                this._addWBSGrids(args);
                
                if (this.ganttView == null) {
                    args.viewButtons = this.viewButtons;
                    args.imageButtons = this.imageButtons;
					args.saveButton = this.saveButton;

                    if (this._GanttController == null) {
                        this._GanttController = controllerFactory.getGanttController(controllerID);
                    }
                    args._controller = this._GanttController;
                    
                    // Add the Gantt view	
                    this.ganttView = new GanttView(args);
                    
                    //Gantt view added here
                    this._layout.setMainContent(this.ganttView.domNode);
                    
                }
								
                // add the refresh and CSH button to the end of the toolbar
                this._addRefreshCSHButton();
                this._layout.setSplitterAttributes(this.domNode, this.INITIAL_LHS_VISIBILITY);
            } else {
                if (this.deleteAndFetch) {
                    this.deleteAndFetch = false;
                    this.destroy_n_refresh();
                }
                else {
                    this.discardUnsavedGridChanges();
                }
            }
			
			if (this._wbsGanttGrid) {
				this.rowUpdatedEvent = dojo.connect(this._wbsGanttGrid, "rowUpdated", dojo.hitch(this, this.initViewBasedonTree));
			}
			this.enableSaveForNewElement();
        },
        
        /**
         *  destroy_n_refresh : Refreshes the contents of the grid and gantt
         *  Incoming : state
         *  Outgoing : none
         *
         */
        destroy_n_refresh: function(){
            // Disable save button
            this.toggleSaveButton(true);
            
            this._GanttController = null;
            this._destroyGrids();
            if (this.gridScrollDiv) {
                this._layout._leftNav.removeChild(this.gridScrollDiv);
                delete this.gridScrollDiv;
            }
			if(this.refreshButton){
				this.refreshButton.destroy();
			}
            if (this.topToolbar) {
                this.topToolbar.destroy();
            }
            this._createToolbar();
            this.refresh();
        },
        
		/**
		 * Discard the unsaved changes.
		 */
		discardUnsavedGridChanges: function(){
            if (this._wbsGanttGrid) {
	        	this._wbsGanttGrid.discardUnsavedGridChanges();
    		}
		},

        _addWBSGrids: function(args){
            // Create the scrollable div into which the grids will be placed
            var gridDiv = this.createScrollableDiv();
            
            if (this._GanttController == null) {
                this._GanttController = controllerFactory.getGanttController(controllerID);
            }
            args._controller = this._GanttController;
			args._viewlet = this;
            
            // To get the gantt grid below the general grid uncomment below lines
            this._wbsGanttGrid = new WBSGanttGrid(args);
            dojo.style(this._wbsGanttGrid.domNode, "width", "450px");
            gridDiv.appendChild(this._wbsGanttGrid.domNode);
            
            this._redrawGrids();
        },
        
        _redrawGrids: function(){
        
            if (this._wbsGanttGrid) {
                this._wbsGanttGrid.updateWithAutoHeight();
            }
            
            if (this.ganttView) {
                this.ganttView.update();
            }        
        },
        
        /**
         *  toggleSaveButton : Save button is enabled once there is any updation in the grid/gantt
         *  Incoming : state
         *  Outgoing : none
         *
         */
        toggleSaveButton: function(state){
            this.saveButton.disabled = state;
            
            if (state == true) {
				this.resetDirtyFlag();
                this.subscribeToEventsForSaveButton();
            }
            else {
				this.setDirtyFlag();
                this.unsubscribeToEventsForSaveButton();
            }
            
        },
        
		/**
		 * subscribeToEventsForSplitter: subscribe to splitter events.
		 */
		subscribeToEventsForSplitter: function(){
			if(this.navResizeHandler == null){
				this.navResizeHandler = dojo.subscribe("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onEnd", this, "_onLeftNavResize"); //subscribe to the drag event of splitter	
			}
            if(this.mouseDownHandler == null){
				this.mouseDownHandler = dojo.subscribe("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onStart", this, "_onSplitterMouseDown"); //subscribe to the drag event of splitter	
			}
            if (this.navDblClickHandler == null) {
				dojo.subscribe("com.ibm.epm.web.core.ui.internal.ViewletSplitter.onEnd", this, "_onLeftNavDblClick"); //subscribe to the double click event of splitter.
			}
            if(this.leftNavResizeHandler == null){
                //subscribe to the drag (or open/close) event of the tree/grid splitter
                this.leftNavResizeHandler = dojo.subscribe("com.ibm.epm.web.core.ui.internal.EPMView.LeftNavigationLayout.onEnd", 
                    this, "_onTreeGridLeftNavResize");  
            }
		},
		
        /**
         *  subscribeToEventsForSaveButton : Function to subscribe to all events needed for enabling Save button
         *  Incoming : none
         *  Outgoing : none
         *
         */
        subscribeToEventsForSaveButton: function(){
            // connect to the events as the button is disabled
            if (this._fieldChangeEvent == null) {
                this._fieldChangeEvent = dojo.subscribe(GridEvents.FIELD_CHANGED, this, this.enableSaveButton);
            }
            
            if (this._ganttDependencyCreation == null) {
                this._ganttDependencyCreation = dojo.subscribe(WBSGridGanttPubSub.ON_CREATE_DEPENDS, this, this.enableSaveButton);
            }
            
            if (this._ganttDependencyDeletion == null) {
                this._ganttDependencyDeletion = dojo.subscribe(WBSGridGanttPubSub.ON_DELETE_DEPENDENCY, this, this.enableSaveButton);
            }

            if (this._ganttBarUpdate == null) {
                this._ganttBarUpdate = dojo.subscribe(WBSConstants.ON_AFTER_UPDATE_GANTTBAR, this, this.enableSaveButton);
            }
            
            if (this._onCreateNewElement == null) {
                this._onCreateNewElement = dojo.subscribe(WBSTreePubSub.ON_CREATE_NODE, this, this.onCreateNode);
            }
        },
        
        /**
         *  unsubscribeToEventsForSaveButton : Function to unsubscribe to all events needed for enabling Save button, once the button is enabled
         *  Incoming : none
         *  Outgoing : none
         *
         */
        unsubscribeToEventsForSaveButton: function(){
            // disconnect from the events when the button is enabled
            dojo.unsubscribe(this._fieldChangeEvent);
            this._fieldChangeEvent = null;
            
            dojo.unsubscribe(this._ganttDependencyCreation);
            this._ganttDependencyCreation = null;
            
			dojo.unsubscribe(this._ganttDependencyDeletion);
			this._ganttDependencyDeletion = null;
			
            dojo.unsubscribe(this._ganttBarUpdate);
            this._ganttBarUpdate = null;
            
            dojo.unsubscribe(this._onCreateNewElement);
            this._onCreateNewElement = null;
			
        },
        
        createScrollableDiv: function(){
            this.gridScrollDiv = document.createElement("div");
            dojo.addClass(this.gridScrollDiv, "com-ibm-epm-work-web-ui-internal-ScheduleGanttViewlet-gridsScheduleScrollPane");
            
            var gridContentDiv = document.createElement("div");
            dojo.addClass(gridContentDiv, "com-ibm-epm-work-web-ui-internal-ScheduleGanttViewlet-gridsScheduleContentPane");
            
            this.gridScrollDiv.appendChild(gridContentDiv);
            
            // Attach the Grid domNode to the left of the splitter
            this._layout._leftNav.appendChild(this.gridScrollDiv);
            return gridContentDiv;
        },
        
        /**
         * setActive: Overridden function from Viewlet.
         * active: Boolean value whether tab is active or not.
         */
        setActive: function(active){
            this.inherited(arguments);

			if (active === true) {
				if (this._GanttController) {
					dojo.subscribe(GridEvents.REFRESH_ROW_DATA, this, this.updateRowData);
					if(this._GanttController._isHidden_n_Dirty){
						console.debug("viewlet was dirty when hidden hance redrawing");
						this._GanttController._wait_n_updateModel();
						this._GanttController._isHidden_n_Dirty = false;
					}
				}
				
				if (this._wbsGanttGrid) {
					this._wbsGanttGrid.updateWithAutoHeight();
					this._wbsGanttGrid.subscribeToKeyboardEvents();
					this._subscribeToEventsFromDateEditor();
				}
				
				if (this.ganttView) {
					this.ganttView.update();
                    
                    // Check if we need to resize the gantt view
                    if (this.pendingResize) {
                        
                        // Resize the gantt view with pending resize data
                        this._onTreeGridLeftNavResize(this.pendingResizeData);
                        
                        // Reset the pending resize variables
                        this.pendingResize = false;
                        this.pendingResizeData = null;
                    }
					
					this.ganttView._subscribeToEvents();
				}
			} else if (active == false) {
				dojo.unsubscribe(GridEvents.REFRESH_ROW_DATA, this, this.updateRowData);
                if (this._wbsGanttGrid) {
					this._wbsGanttGrid.unsubscribeToKeyboardEvents();
					this._unsubscribeToEventsFromDateEditor();
				}
				var tree = dijit.byId(WBSConstants.WBS_TREE_ID);
				if(this.isDirty() == true && tree && tree.hasNewNode){
					tree.hasNewNode = false;
					dojo.publish(WBSGridGanttPubSub.DISCARD_UNSAVED_NODES);
				}
				
				if (this.ganttView) {
					this.ganttView._unsubscribeToAllEvents();
				}
			} 
        },

        /*
         * When the viewlet gets loaded completely, this method is called to check if the Save button needs to be enabled
         */
        enableSaveForNewElement: function() {
			// Check if the WBS Tree has any unsaved element added
            var tree = dijit.byId(WBSConstants.WBS_TREE_ID);
            if (tree && tree.hasNewNode) {
				this.toggleSaveButton(true);
				this._checkToEnableSaveButton();
            }
        },

		/**
		 * Enable the save button for all the work items except the project.
		 * @param {Object} publishedObject
		 */
		onCreateNode: function(publishedObject){
			var resourceType = publishedObject.resourcetype;
			if(resourceType && resourceType !== WBSConstants.EPM_PROJECT_TYPE){
				this._checkToEnableSaveButton();
			}
		},
		        
		/**
		 * updateRowData: 
		 * 		Calls controller's _getAndUpdateRow() for specific grid row. 
		 * @param {Object} data
		 */
		updateRowData: function(data) {
			if (!data) {
				return;
			}
			var itemIdentifier = null;
			if (data.identifier) {
				itemIdentifier = dojo.isArray(data.identifier) ? data.identifier[0] : data.identifier;
			} 
			else if (data.relativeHref) {
				itemIdentifier = dojo.isArray(data.relativeHref) ? data.relativeHref[0] : data.relativeHref;
			}
			
			if (this._wbsGanttGrid) {
				this._wbsGanttGrid._controller._getAndUpdateRow(itemIdentifier, itemIdentifier, null);
				this._wbsGanttGrid.updateWithAutoHeight();
			}
		},
		
		/*
         * Get the current selected tree node
         * Incoming : None
         * Outgoing : None
         */
        initViewBasedonTree: function(){
			if (this.rowUpdatedEvent) {
				dojo.disconnect(this.rowUpdatedEvent);
				this.rowUpdatedEvent = null;
			} else {
				return;
			}
			
            var tree = dijit.byId(Constants.WBS_TREE_ID);
            if (tree && tree._selectedNode) {
                var item = tree._selectedNode.item;
                if (item && item.identifier) {
                    var obj = {
                        treeID: item.treeID,
                        identifier: item.identifier,
                        name: item.name,
                        resourcetype: item.resourcetype
                    };
					
					if (this._wbsGanttGrid) {
						this._wbsGanttGrid._onSelect(obj);
					}
					if (this.ganttView) {
						this.ganttView.onRowSelect(obj);
					}
                }
            }
        },
        
        /*
         * Returns the state of the grid contained in the viewlet
         * true: Button is enabled (there are unsaved changes)
         * false: Button is disabled (unsaved changes exist) 
         */
        isButtonEnabled: function() {
            return (!this.saveButton.disabled);
        },
        
        /*
         * Set the deleteAndFetch flag that is used to indicate to the refresh() method
         * that the whole grid must be destroyed and refetched from the server.
         */
        setDeleteAndFetch: function() {
            this.deleteAndFetch = true;
        },
        
        /*
         * Handles if the save button should be disabled when the date picker is shown
           in the grid. If the save button was already enabled, this method disables it
           but adds a listener for the apply/cancel methods of the date editor. When either
           of these methods fire, the save button is enabled again.
         */
        _handleSaveButtonOnDateEdit: function(inCellEditor) {
            
            // Check if the save button is enabled or not        
            if (this.isButtonEnabled()) {
            
                // Disable the save button
                this.toggleSaveButton(true);
                
                // Mark the viewlet as dirty even though we have disabled the save button
                // This is to trigger the confirmation dialog if the user attempts to change
                // the viewlet tab when the date picker is still active.
                // The save button will be enabled as soon as the date picker is done.
                this.setDirtyFlag();
                
                // Add a listener for the apply method on the date editor
                if (this._dateEditorApplyHandle == null) {
                    this._dateEditorApplyHandle = dojo.connect(inCellEditor, "apply", this, function(event) {
                        
                        console.log("ScheduleGanttViewlet._handleSaveButtonOnDateEdit: In Apply");
                        
                        // Enable the save button
                        this.toggleSaveButton(false);
                        
                        // Remove this listener
                        dojo.disconnect(this._dateEditorApplyHandle);
                        
                        // Initialize the event handlers to null
                        this._dateEditorApplyHandle = null;
                        
                        if (this._dateEditorCancelHandle != null) {
                            dojo.disconnect(this._dateEditorCancelHandle);
                        }
                        this._dateEditorCancelHandle = null;
                    });
                }
                
                // Add a listener for the cancel method on the date editor
                if (this._dateEditorCancelHandle == null) {    
                    this._dateEditorCancelHandle = dojo.connect(inCellEditor, "cancel", this, function(event) {
                        
                        console.log("ScheduleGanttViewlet._handleSaveButtonOnDateEdit: In Cancel");
                        
                        // Enable the save button
                        this.toggleSaveButton(false);
                        
                        // Remove this listener
                        dojo.disconnect(this._dateEditorCancelHandle);
                        
                        this._dateEditorCancelHandle = null;
                        
                        // Initialize the event handlers to null
                        if (this._dateEditorApplyHandle != null) {
                            dojo.disconnect(this._dateEditorApplyHandle);
                        }
                        this._dateEditorApplyHandle = null;
                        
                    });
                }    
            }
        },
        
        /*
         * Method for subscribing to an event for when the date picker is displayed in the grid.
         */
        _subscribeToEventsFromDateEditor: function() {
        
            // Subscribe to the event
            if (this._onDateEditor == null) {
                this._onDateEditor = dojo.subscribe(WBSGridGanttPubSub.ON_DATE_EDITOR, this, this._handleSaveButtonOnDateEdit);
                
                // Initialize the event handlers to null
                if (this._dateEditorApplyHandle != null) {
                    dojo.disconnect(this._dateEditorApplyHandle);
                }
            
                if (this._dateEditorCancelHandle != null) {
                    dojo.disconnect(this._dateEditorCancelHandle);
                }
            
                this._dateEditorApplyHandle = null;
                this._dateEditorCancelHandle = null;
            }
        },
        
        /*
         * Method for unsubscribing to an event for when the date picker is displayed in the grid.
         */
        _unsubscribeToEventsFromDateEditor: function() {
            
            // Unsubscribe to the event
            dojo.unsubscribe(this._onDateEditor);
            this._onDateEditor = null;
            
            // Initialize the event handlers to null
            if (this._dateEditorApplyHandle != null) {
                dojo.disconnect(this._dateEditorApplyHandle);
            }
            
            if (this._dateEditorCancelHandle != null) {
                dojo.disconnect(this._dateEditorCancelHandle);
            }
            
            this._dateEditorApplyHandle = null;
            this._dateEditorCancelHandle = null;
        }
    });
})();