/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

 (function() {
dojo.provide("com.ibm.epm.web.core.ui.internal.page.QueryEditorPageController");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.i18n");
dojo.require("com.ibm.epm.web.core.client.QueryUtil");
dojo.require("com.ibm.epm.web.core.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil"); // TODO: promote mergeObjects
dojo.require("com.ibm.epm.web.core.client.internal.QueryClient");
//dojo.require("com.ibm.team.workitem.web.ui.internal.view.query.QueryUtility");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.requireLocalization("com.ibm.epm.web.core.ui.internal.page", "QueryEditorMessages");



var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WorkItemClient			= com.ibm.epm.web.core.client.internal.WorkItemClient;
var QueryClient				= com.ibm.epm.web.core.client.internal.QueryClient;
var QueryUtil			= com.ibm.epm.web.core.client.QueryUtil;
var EditorUtil            = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var LayoutUtil            = com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;
// matches any text other than a digit
var NON_NUMERIC_REGEX		= /[^0-9]/;

var DEFAULT_ORDER_ATTRIBUTE	= "id";
var DEFAULT_DIRECTION		= "descending";
var DEFERRED_MOUSEOVER_DELAY = 600; // milliseconds to delay the deferred mouseOver
var SERVER_MISSING_MSG = "The server is not responding or a page navigation occurred before an operation completed.\n"
							+ "Check that your server connection is working and that the server is operational,\n" 
							+ "then try the action again or refresh the page.";
var QueryEditorMessages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui.internal.page", "QueryEditorMessages");



dojo.declare("com.ibm.epm.web.core.ui.internal.page.QueryEditorPageController", null, {


	/*
	 * Internal constructor; should not be called by *any* non-Dojo code.
	 */
	constructor: function(queryEditorPageView) {
		this._currentOperation= null;
		this._defaultQuery= null;
		this.suspendDelaysUntil = 0;
		this._queryEditorPageView = queryEditorPageView;
		//this.queryUtil= new com.ibm.team.workitem.web.ui.internal.view.query.QueryUtility(this);
		this.pageController = this;
		// Handle mouse motion events
		this.mouseListeners = [];
		var self = this;
		this._mousemove_handle = dojo.connect(document.body, "onmousemove",  // must be dojo.connect
				function(e) {
				try{ 
					self._handleMouseMove(e);
					} catch(e) { }
				});	
	},
	destroyController: function() {
		dojo.disconnect(this._mousemove_handle);
	},
	/*
	 * Once mouseovers are shown, yhou want them to show immediatleyon subspequent items.,
	 * so a client can suspend delays for a while
	 */
	suspendMouseOverDelays: function(milliseconds) {
		milliseconds = milliseconds ? milliseconds : 500;
		this.suspendDelaysUntil = (new Date()).getTime() + milliseconds;
	},
	isUsingDelay: true, // Unique id for each listener, used to remove
	mouseListenerHandle: 0, // Unique id for each listener, used to remove
	addMouseListener: function(receiver, domNode,  modifiers ) {
		/*
		 * Set the default values
		 */
		listenerObject = {
			parent: this,
			handle: this.mouseListenerId,
			receiver: receiver,
			domNode: domNode,
			isMouseOver:false,
			isDeferredMouseOverHandled:false,
			overDelay: DEFERRED_MOUSEOVER_DELAY, // milliseconds to delay the deferred mouseOver
			mouseMoveHandler: "handlePageMouseMove",
			mouseOverHandler: "handlePageMouseOver",
			mouseOutHandler: "handlePageMouseOut",
			deferredMouseOverHandler: "handleDeferredPageMouseOver",
			lastListenerCheckScheduled: 0
// Future function...
//			outDelay: 300, // milliseconds to delay the deferred mouseOut			
//			deferredMouseOutHandler: "handlePageDeferredMouseOut"
		}
		LayoutUtil.mergeObjects(listenerObject, modifiers);
		this.mouseListeners.push(listenerObject);
		return this.mouseListenerHandle++;
	},
	/* 
	 * Remove all the mouse listeners associated with a parent widget
	 */
	removeAllMouseListeners: function(receiver) {
		var newMouseListeners = [];
		for (var i = 0; i < this.mouseListeners.length; i++) {
			if(this.mouseListeners[i].receiver != receiver) {
				newMouseListeners.push(this.mouseListeners[i]);
			}
		}
		this.mouseListeners = newMouseListeners;
	},
	removeMouseListener: function(handle) {
		for (var i = 0; i < this.mouseListeners.length; i++) {
			if(this.mouseListeners[i].handle == handle) {
				this.mouseListeners.splice(i,1);
				return;
			}
		}
	},
	/*
	 * On a mouse move, propogate the event to registered listeners
	 * A listener gets called if the mouse is over that listener or
	 * over any of its children. 
	 */
	_handleMouseMove: function(e) {
		// Ignore if we are over a popup widget (e.g., date widget, progressive disclosure)
			if(QueryUtil.isChildOfPopup(e.target)) return;
	
			var timeNow = (new Date()).getTime();
			for(var i=0;i<this.mouseListeners.length;i++) {
				var listener = this.mouseListeners[i];
				// Optimiation: If the element is not visible, then its width is zero so skip it.
				if(!listener || !listener.domNode || !listener.domNode.parentNode) continue;
				if(listener.domNode.tagName.toLowerCase() == "a") {
					if(listener.domNode.parentNode.clientWidth ==0) continue;
					} else {
						if(listener.domNode.clientWidth ==0) continue;
					}
				
				var isTargetChildOfDomNode = QueryUtil.isChildOf(e.target, listener.domNode);

				// Send Mouse Move
				if (listener.receiver[listener.mouseMoveHandler]) {
					listener.receiver[listener.mouseMoveHandler](e, listener, isTargetChildOfDomNode);
				}
				/*
				* Notify the listener on mouse over and on deferred mouse over
				* Deferred mouse over delays a few hundres ms
				*/
				if (isTargetChildOfDomNode) {
					// Handle Mouse Over
					
					// If the first time it is over
					if (!listener.isMouseOver) {
						listener.mouseOverTime = timeNow;
						listener.isMouseOver = true;
 						if(listener.receiver[listener.mouseOverHandler]) {
							listener.receiver[listener.mouseOverHandler](e, listener);							
						}
						
						/* 
						 * If delays are suspended, fire deferred mouse overs immediately
						 */
						if((timeNow < this.suspendDelaysUntil) && !listener.isDeferredMouseOverHandled && listener.receiver[listener.deferredMouseOverHandler] ) {
//							console.debug("mouse over  listener : "  + listener.label + " " + listener.isDeferredMouseOverHandled);
							listener.isDeferredMouseOverHandled = true;
							listener.receiver[listener.deferredMouseOverHandler](e, listener);
						}
						
						
						// Handle the case where the mouse stops over the listener
						// Only schedule one check per 100 ms
 						if(!listener.isDeferredMouseOverHandled && listener.receiver[listener.deferredMouseOverHandler] && (timeNow - listener.lastListenerCheckScheduled > 100) ) {
							listener.lastListenerCheckScheduled = timeNow;
							setTimeout(dojo.hitch(this, function(e1, listener1){
								this._checkIfMouseIsStillOverListener(e1, listener1);
							}, e, listener), listener.overDelay);
						} 

					} else {
						/*
						 *  If it has been over before, but the defer has not fired.... 
						 * If delays are suspended, fire deferred mouse overs immediately
						 */
						if (listener.receiver[listener.deferredMouseOverHandler]) {
 							if((timeNow < this.suspendDelaysUntil) || (timeNow - listener.mouseOverTime  >= listener.overDelay)) {
								listener.isDeferredMouseOverHandled = true;
								listener.isMouseOver = true;
								listener.receiver[listener.deferredMouseOverHandler](e, listener);
							}
						} 				
					}
				} else {
					// Handle Mouse Out
					if (listener.isMouseOver) {
						if (listener.receiver[listener.mouseOutHandler]) {
							listener.receiver[listener.mouseOutHandler](e, listener);
						}
					} else {
						// Handle deferred mouse out
						// Not supported yet
					}
//					console.debug("mouse out listener : "  + listener.label + " " + listener.isDeferredMouseOverHandled);
					listener.isDeferredMouseOverHandled = false;
					listener.isMouseOver = false;
					listener.mouseOverTime = 0;
				}

			}
	},
	/*
	 * Fun with setTimeout
	 */
	_checkIfMouseIsStillOverListener: function(e, listener) {
			listener.lastListenerCheckScheduled = 0;
			var timeNow; // save a few cycles by deferring unless needed
//			console.debug("check listener : "  + listener.label + " " + listener.isDeferredMouseOverHandled);
			if (listener.receiver[listener.deferredMouseOverHandler] && !listener.isDeferredMouseOverHandled && listener.isMouseOver) {
				timeNow = timeNow ? timeNow : (new Date()).getTime();
				if ((timeNow < this.suspendDelaysUntil) || (timeNow - listener.mouseOverTime >= listener.overDelay)) {
					listener.isDeferredMouseOverHandled = true;
					listener.isMouseOver = true;
					listener.receiver[listener.deferredMouseOverHandler](listener.lastEvent, listener);
				}
			}
	},
	_setPageTitle: function(title) {
	    PlatformUI.getWorkbench().setTitle(title);
	}, 

	_setWorkItemPageTitle: function(workItem) {
		if(workItem && workItem.attributes.workItemType) {
				if (workItem.id < 0) {
					this._setPageTitle("New Work Item");
				}
				else {
					var summary = ": ";
					if (workItem.attributes.summary.content) {
						var summaryContent = EditorUtil.restoreTags(workItem.attributes.summary.content);
						summary = summary + summaryContent.substring(0, Math.min(workItem.attributes.summary.content.length, 200));
						if (Math.min(summaryContent.length, 200) == 200) {
							summary = summary + "...";
						}
					}
					this._setPageTitle(workItem.attributes.workItemType.label + " " + workItem.id + summary);
				}
		} else {
			this._setPageTitle("Work Item");
		} 
	},  
	 _updateURL: function(url) {
 		 // IE uses an iframe to track history
 		 if (dojo.isIE) {
 		 		 // see net.jazz.ajax.core.internal.IEUriMonitor
 		 		 var iframe = window["net.jazz.ajax.iframe"];
 		 		 if (iframe) {
 		 		 		 var href = iframe.location.href;
 		 		 		 var index = href.indexOf('?');
 		 		 		 if (index != -1) {
 		 		 		 		 // main frame will detect and update itself
 		 		 		 		 iframe.location.replace(href.substring(0, index) + "?" + url.substring(1));
 		 		 		 		 return;
 		 		 		 }
 		 		 }
 		 }
 		 location.replace(url);
 	},
	
	/*
	 * Special method called by the Jazz web ui framework when the URL contains
	 * no 'action' request parameter.  In this scenario, we run the default
	 * pre-defined query.
	 * 
	 * This function name can be changed if we change 
	 * the page's defaultAction attribue in workitem.web/plugin.xml.
	 */
	_showDefaultView: function(args) {
		
		this._queryEditorPageView._registerNavigation();
		
		// If we are returning here from another page (e.g., planning), keep context
 		if(!this._isSubsequentView || this._forceShowingDefaultView){ // first time or forced
        	this._isSubsequentView = true;
//        	this._updateURL(QueryUtil.getRunDefaultQueryURI (/* include timestamp */ false));
        	this._updateURL(QueryUtil.getWelcomePageURI (this._forceShowingDefaultViewMsg));
			this._forceShowingDefaultViewMsg = null;
    	} else { // next time
			var info = this.getLastQueryInfo();
			if(info && info.url) {
				this._updateURL(info.url);
			}
		}
		// show whatever was the current view
	},

	
	_fetchAndRunDefaultQuery: function() {
		var self= this;
		var handlerObj= {
			success: function(query) {
				if (!query) {
					self.handleError(
						{message: "No default query found"},
						"Error running default query");
					dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
					return;
				}
				self._defaultQuery= query;
				self._internalRunQuery(self._defaultQuery);
				// initialize queries done in _internalRunQUery
			},
			failure: function(errorObj) {
				self.handleError(
					errorObj,
					"Error running default query");
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				self._isReloading = false;
			}
		};
		var handler= new ServiceResponseHandler(handlerObj, "success", "failure");
		QueryClient.getDefaultQuery(this._queryEditorPageView.queryType, handler);
	},

	/*
	 * Creates an operation specified by the given name and id.
	 */
	createOperation: function(/* string */ name, /* string */ id) {
		checkNotNull(name, "name");
		checkNotNull(id, "id");
		return { name: name, id: id };
	},
	
	/*
	 * Marks the start of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.epm.web.core bundle.
	 */
	startOperation: function(operation) {
		PlatformUI.getWorkbench().showMessage(operation.name);
	},

	/*
	 * Marks the end of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.epm.web.core bundle.
	 */
	finishOperation: function(operation) {
		PlatformUI.getWorkbench().hideMessageArea();
	},

	/*
	 * Handles an error in a standard way.
	 */
	handleError: function(obj, msg) {

		if(this._isReloading) return; // avoid repeated errors
		var humanFriendlyMsg= null;
		if (!obj) { obj= {}; }
//		if (obj.status === 0) {
//			humanFriendlyMsg= SERVER_MISSING_MSG;
//		}

//		var summaryMsg= msg && dojo.isString(msg) ? msg : null;

//		var detailedMsg= humanFriendlyMsg ? humanFriendlyMsg : obj.message;
//		if(!detailedMsg || detailedMsg.toLowerCase() == "not found") {
//			detailedMsg =SERVER_MISSING_MSG;
//		}
//		var alertMsg= (summaryMsg ? summaryMsg + "\n(" : "") +	detailedMsg +
//				(summaryMsg ? ")" : "");
		
//		console.debug("*** ERROR: " +  alertMsg);
		this._queryEditorPageView._displayError(msg);
//		this._isReloading = true;
//		this._openDefaultView(detailedMsg);
	},
	_openDefaultView: function(msg) {
		this._forceShowingDefaultView = true;
		this._forceShowingDefaultViewMsg = msg;
		var split = window.location.href.split("#"); 
		window.location.href = split[0] + "#action=jazz.viewPage&id=com.ibm.team.workitem"; // show the default view
		// probably does not return
	},
	/*
	 * QUICK SEARCH-related methods.
	 */

	/*
	 * Responds to the submission of the "quick search" form.  Any non-null,
	 * non-empty is valid.  If input is a positive int, the app will attempt to
	 * open a work item with that id; if input is anything else, the app will
	 * run a textual search against the input.
	 *
	 * This method should not be called directly.
	 */
	_handleQuickSearch: function(searchText, openOnly) {
		checkNotNull(searchText, "searchText");
		var searchText= dojo.trim(searchText);
		var quoteLess = dojo.trim(searchText.replace(/\"/g, "").replace(/\'/g, ""));
		if(quoteLess.length == 0) {
			this._openDefaultView();
			return;
			}
		try{
			var value = parseInt(searchText);
			if(value > 2147483647 || value < 1) {
				searchText = '"' + value + '"';
			}
		} catch(e) {} // ignore parseInt failure
		this.resetLastQueryString();
		if (NON_NUMERIC_REGEX.test(searchText)) {
			this._internalSearch(searchText, openOnly);
		} else {
			this._internalViewWorkItem(searchText);
		}
	},
	
	/*
	 * Sets the browser's URI so that the Jazz framework calls the 'search'
	 * method.  May be called by other code within the com.ibm.epm.web.core
	 * bundle.
	 */
	openSimilarityQuery: function(/* itemId: , summary: , content: */ parms) {
		window.location.href= QueryUtil.getSimilarityQueryURI(parms);
	},
	
	/*
	 * Sets the browser's URI so that the Jazz framework calls the 'search'
	 * method.  May be called by other code within the com.ibm.epm.web.core
	 * bundle.
	 */
	_internalSearch: function(input, openOnly) {
		checkNotNull(input, "input");
		window.location.href= QueryUtil.getSearchURI(input, openOnly, true);
	},
	resetLastQueryString: function() {
		this._lastQueryKey = "";
	},
	
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Show the welcome page
	 */
	viewWelcome: function( kwArgs) {
		dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
		this._queryEditorPageView.showWelcomePage(kwArgs.error);
	},
	updateWelcomePageQueries: function(queryList) {
		this._queryEditorPageView.welcomePageWidget.initializeQueryLinks(queryList);
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Searches for work items matching the string specified by 'kwArgs.q'.
	 */
	runDefaultQuery: function( kwArgs) {
		this._isSubsequentView = true;
		this._lastColumnArgs=null;
  		kwArgs.tq = kwArgs.tq == null ? "" : kwArgs.tq; 
		var queryKey = "runDefaultQuery:"+ kwArgs.tq;
		//cache If the search text is unchanged, show the same page again
		
		if (!kwArgs.index) {kwArgs.index= null;}
		if (!kwArgs.orderBy) {kwArgs.orderBy= null;}
		if (!kwArgs.direction) {kwArgs.direction= null;}
		if (!kwArgs.filterAttribute) {kwArgs.filterAttribute= null;}
		if (!kwArgs.filterValue) {kwArgs.filterValue= null;}
		if (kwArgs.preserveFilter == "true") {kwArgs.preserveFilter = true}

		if (queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastFilterAttribute == kwArgs.filterAttribute
				&& this._lastFilterValue == kwArgs.filterValue) {

			this._queryEditorPageView.showLastQueryResults();
			return;
		}
		
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryKey = queryKey;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastFilterAttribute = kwArgs.filterAttribute;
		this._lastFilterValue = kwArgs.filterValue;
		
		
		var op = this.createOperation("Running Default Query", "showDefaultViewOperation");
		this.startOperation(op);
		this._queryEditorPageView.showLastQueryResults();
		// run default query
		this._setPageTitle(""); // Blank for default view
		if (this._defaultQuery) {
			this._internalRunQuery(this._defaultQuery);
		} else {
			this._fetchAndRunDefaultQuery();
		}
		this.finishOperation(op);
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Searches for work items matching the string specified by 'kwArgs.q'.
	 */
	search: function(kwArgs) {
		checkNotNull(kwArgs.q, "kwArgs.q");
		this._isSubsequentView = true;
		var searchText= dojo.trim(kwArgs.q);
		var quoteLess = dojo.trim(searchText.replace(/\"/g, "").replace(/\'/g, ""));
		if(quoteLess.length == 0) {
			this._openDefaultView();
			return;
			}
		
		var queryKey = "Search:"+searchText;
		//cache If the search text is unchanged, show the same page again
		var openOnly = kwArgs.o ? true : false;
		
		// if we're already caching the query editor editor, and the timestamp is the same, simply show the cached page
		if (queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastQueryItemsPerPage == kwArgs.itemsPerPage) {
			this._queryEditorPageView.showLastQueryResults();
			return;
		}
		
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastQueryItemsPerPage = kwArgs.itemsPerPage;
		
		this._lastQueryURL = window.location.href;

	// If its the same query key, but different other factors, rerun the query
		if(queryKey == this._lastQueryKey) {
			this._internalRunQuery(this._lastQueryRun, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, true);
			return;
		}

		this._lastQueryKey = queryKey;
		
		
		if (NON_NUMERIC_REGEX.test(searchText)) {
			
			var self= this;
			 
			if (openOnly) {
				var queryName= "Search Results for '" + kwArgs.q + "' in Open Work Items";	
			} else {
				var queryName= "Search Results for '" + kwArgs.q + "'";	
			}
			var query= QueryClient.createFulltextQuery(this._queryEditorPageView.queryType, queryName, searchText, openOnly);
			
			self._lastQueryRun = query;
			self._internalRunQuery(query, 0);
			
		} else {
			this._internalViewWorkItem(searchText);
			// initialize queries is done in viewWorkItem... which gets called eventually from internalViewWorkItem
		}
	},
	
	/*
	 * QUERY-related methods.
	 */

	/*
	 * Sets the browser's URI so that the Jazz framework calls the 'simpleQuery'
	 * method.  May be called by other code within the com.ibm.epm.web.core
	 * bundle.
	 */
	_internalSimpleQuery: function(query) {
		checkNotNull(query, "query");
		window.location.href= QueryUtil.getQueryURI(query);
	},

	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Runs an old style query using the specified query object and displays
	 * query results in a table.
	 * <p>
	 * Note that this 'query' object very different from the 'query' objects
	 * used in other parts of the work item web app.
	 * </p>
	 */
	simpleQuery: function(query) {
		this._isSubsequentView = true;
		this._lastColumnArgs=null;
		var args = { 
			query: query,
			table: this._queryResultsTable
		};
		if(!query.name) {
			query.name = "Simple Query";
		}
		if(!this._currentOperation) {
			this._currentOperation= this.createOperation("Running Query '" + query.name + "'", "simpleQuery");
 			this.startOperation(this._currentOperation);
		}

		var self= this;
		var successCallback=
			function(queryResultsSummaryPage) {
				self._querySuccessCallback(queryResultsSummaryPage, query);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			};
		var failureCallback=
			function(error) {
				self._queryFailureCallback(error);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			};

		//this.queryUtil.runQuery(query, (query.index ? query.index - 1 : 0) , -1 , false, successCallback, failureCallback, args);
	},

	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Runs the saved query with itemId specified by the paramter 'kwArgs.id'.
	 * Displays query results in a table.
	 */
  	runSavedQuery: function(kwArgs) {
		this._isSubsequentView = true;
		this._lastColumnArgs=null;
  		// kwArgs.action=runSavedQuery kwArgs.id=_uuid kwArgs.tq=[timestamp]
  		checkNotNull(kwArgs, "kwArgs");
  		checkNotNull(kwArgs.id, "kwArgs.id");
		
  		kwArgs.tq = kwArgs.tq == null ? "" : kwArgs.tq; 
		var queryKey = "runSaved:"+kwArgs.id + ":" + kwArgs.tq;
		//cache If the search text is unchanged, show the same page again
		
		if (!kwArgs.index) {kwArgs.index= null;}
		if (!kwArgs.orderBy) {kwArgs.orderBy= null;}
		if (!kwArgs.direction) {kwArgs.direction= null;}
		if (!kwArgs.itemsPerPage) {kwArgs.itemsPerPage= null;}
		if (!kwArgs.filterAttribute) {kwArgs.filterAttribute= null;}
		if (!kwArgs.filterValue) {kwArgs.filterValue= null;}
		if (kwArgs.preserveFilter == "true") {
			if(!kwArgs.filterAttribute && this._lastFilterAttribute) {
				kwArgs.filterAttribute = this._lastFilterAttribute;
			}
			if(!kwArgs.filterValue && this._lastFilterValue) {
				kwArgs.filterValue = this._lastFilterValue;
			}
			kwArgs.preserveFilter = true
			}

		if (queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastQueryItemsPerPage == kwArgs.itemsPerPage
				&& this._lastFilterAttribute == kwArgs.filterAttribute
				&& this._lastFilterValue == kwArgs.filterValue) {

			this._queryEditorPageView.showLastQueryResults();
			return;
		}
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryKey = queryKey;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastQueryItemsPerPage = kwArgs.itemsPerPage;
		this._lastFilterAttribute = kwArgs.filterAttribute;
		this._lastFilterValue = kwArgs.filterValue;
		
  		var self= this;
		
		var serviceObject= {
			success: function(query) {
				if (!query) {
					self._handleQueryNotFound();
					return;
				}
				query.itemId = query.id;
				self._internalRunSavedQuery(query, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, false, kwArgs.preserveFilter, kwArgs.filterAttribute, kwArgs.filterValue);
				// Initialize queries is done in internalRunQuery
			},
			error: function(error) { 
				self.handleError(error, "Error running saved query");
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			}
		};
			
		//WI : 4423 - The Query execution URI has changed after query server revamp.Making the required changes
		//QueryClient.getQuery(this._queryEditorPageView.queryType, new ServiceResponseHandler(serviceObject, "success", "error"), kwArgs.id);
		
		QueryClient._getQueryDetails(new ServiceResponseHandler(serviceObject, "success", "error"),kwArgs.id);
  	},
	
	/*
	 * Internal Run saved query. The query would be executed by passing the id to the server
	 * TODO : investigate why earlier the execute query always passes json expression and not the id
	 */
	_internalRunSavedQuery:function(query, index, orderByAttrib, orderDirection, itemsPerPage, keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs){
		
		if (!this._currentOperation) {
				var message= query.name ? "Running Query '" + query.name + "'" : "Running Query";
				this._currentOperation= this.createOperation(message, query.itemId);
				this.startOperation(this._currentOperation);
			}
			this._lastQueryName = query.name ? query.name : "New Query";
			this._lastQueryURL = window.location.href;
			// If the last query url doesn't have an "action" in it, then it was the default query and we need to pick it up differently
			if(this._lastQueryURL.indexOf("action=") == -1) {
				this._lastQueryURL = QueryUtil.getQueryURI(query)
			}
			
			// Run query
			this._internalGetResultsSetForSavedQuery(query, index, orderByAttrib, orderDirection, itemsPerPage,
				keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs);
				
	},
	
	/*
	 * Internal get result for Saved query
	 */
	_internalGetResultsSetForSavedQuery: function(query, index, orderByAttrib, orderDirection, itemsPerPage,
			keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs) {

   		var self= this;
		var handlerObj= {
			success: function(queryResultsSummaryPage) {
				
				// TODO : Remove this code once the server support returning headers for query with no result
				if(!queryResultsSummaryPage){
					
					queryResultsSummaryPage = {   
					"EPMHeaders": { "FieldDefinitions": [ {"attributeType": "smallString", "label": "Query" } ]},
					"QueryResult": [  { "Query": "No Resource matches the current filter criteria."  } ]
					};
					
				};
				self._querySuccessCallback(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor);
				if (!preserveFilter || !self._lastFilter || self._lastFilter == "") {
					self._queryEditorPageView.resetFilter();
				} else {
					if(self._lastFilter) {
						self._queryEditorPageView.setFilter(self._lastFilter)				
					}
				}
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				self._isReloading = false;
			},
			error: function(error) { 
				self._queryFailureCallback(error);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				self._isReloading = false;
			}
		};

		var handler= new ServiceResponseHandler(handlerObj, "success", "error");

		QueryClient._executeQuery(handler, query.id);		
	},
	/*
	 * Run Saved query will call this new method as we can execute the Query using the query id itself.
	 */
	
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Runs the saved query with its structure specified by the paramter 'kwArgs.q'.
	 * Displays query results in a table.
	 */
  	runNewQuery: function(kwArgs) {
		this._isSubsequentView = true;
  		checkNotNull(kwArgs, "kwArgs");
  		checkNotNull(kwArgs.q, "kwArgs.q");
		if (kwArgs.preserveFilter == "true") {kwArgs.preserveFilter = true}
  		var query= QueryUtil.deserializeQuery(kwArgs.q);
		this._internalRunQuery(query, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, false, kwArgs.preserveFilter, undefined, undefined, this._lastColumnArgs);
  	},

	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Runs the find duplicates query
	 */	
	similarityQuery: function(kwArgs) {
		var currentWorkItem = this._queryEditorPageView.getCurrentWorkItem();
		this._lastColumnArgs=null;
		var query;
		var name= kwArgs.name || "Potential Duplicates";
		if (currentWorkItem && !this._queryEditorPageView.changesPending()) {
			
			query = QueryClient.createSimilarityQuery(this._queryEditorPageView.queryType, name, currentWorkItem.itemId);
			
		} else if (currentWorkItem && currentWorkItem.attributes) {
			var summary, description, item, comments = "";
			if(currentWorkItem.attributes.summary && currentWorkItem.attributes.summary.content) {
				summary = currentWorkItem.attributes.summary.content; 
			} else {
				summary =  kwArgs.summary || "";
			}
			if(currentWorkItem.attributes.description && currentWorkItem.attributes.description.content) {
				description = currentWorkItem.attributes.description.content; 
			} else {
				description =  kwArgs.content || "";
			}
			if(currentWorkItem.attributes.internalComments && currentWorkItem.attributes.internalComments.items) {
				for(var i=0;i<currentWorkItem.attributes.internalComments.items.length;i++) {
					item = currentWorkItem.attributes.internalComments.items[i];
					if(item.content) {
						comments += " " + item.content;
					}
				} 
			} 
			description += comments;

			query = QueryClient.createSimilarityQuery(name, currentWorkItem.itemId, summary, description);
		} else {
			if (kwArgs.summary || kwArgs.content) {
				query = QueryClient.createSimilarityQuery(this._queryEditorPageView.queryType, name, kwArgs.itemId, kwArgs.summary, kwArgs.content);	
			} else {
				query = QueryClient.createSimilarityQuery(this._queryEditorPageView.queryType, name, kwArgs.itemId);	
			}
		}

		if (kwArgs.preserveFilter == "true") {kwArgs.preserveFilter = true}

		this._internalRunQuery(query, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, false, kwArgs.preserveFilter);
	},
	
	// temporary fix for #35544
	runQuery: function(query, columnArgs) {
		this._lastQueryRun= query;
		this._lastColumnArgs = columnArgs;
		this._internalRunQuery(query, undefined, undefined, undefined, undefined, true, undefined, undefined, undefined, columnArgs);
	},

	/*
	 * Runs the query specified by the parameter and displays the results in a
	 * table.  This method must only be called by code within the
	 * com.ibm.epm.web.core bundle.
	 */
  	_internalRunQuery: function(query, index, orderByAttrib, orderDirection, itemsPerPage, keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs) {
		if (!this._currentOperation) {
			var message= query.name ? "Running Query '" + query.name + "'" : "Running Query";
			this._currentOperation= this.createOperation(message, query.itemId);
			this.startOperation(this._currentOperation);
		}
		this._lastQueryName = query.name ? query.name : "New Query";
		this._lastQueryURL = window.location.href;
		// If the last query url doesn't have an "action" in it, then it was the default query and we need to pick it up differently
		if(this._lastQueryURL.indexOf("action=") == -1) {
			this._lastQueryURL = QueryUtil.getQueryURI(query)
		}
		
		if (columnArgs) {
			var sortOrder= columnArgs.sortOrder;
			if (sortOrder.length > 0) {
				orderByAttrib= sortOrder[0].id;
				orderDirection= sortOrder[0].ascending ? "ascending" : "descending";
			}
		} else if (com.ibm.epm.web.core.client.QueryUtil.isStringDefined(orderByAttrib) == false) {
			// Get the sort order defaults and then run the query
			// Note: This is sub-optimal performance-wise:
			// (1) The editable query DTO should be cached
			// (2) The REST service call should be specialized only to return 
			//     sorting specific information 
			this._getSortOrderDefaults(query, index, itemsPerPage, keepEditor, preserveFilter, 
					filterAttribute, filterValue, columnArgs);
			return;				
		}
		
		// Run query
		this._internalGetResultsSet(query, index, orderByAttrib, orderDirection, itemsPerPage,
			keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs);
  	},
  	
	/*
 	 * Private Method
 	 * 
 	 * @param String query
 	 * @param Number index
 	 * @param String orderByAttrib
 	 * @param String orderDirection
 	 * @param String itemsPerPage
 	 * @param boolean keepEditor
 	 * @param boolean preserveFilter
 	 * @param String filterAttribute
 	 * @param String filterValue
 	 * @param Object columnArgs
 	 * @return void
 	 */	
	_internalGetResultsSet: function(query, index, orderByAttrib, orderDirection, itemsPerPage,
			keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs) {

   		var self= this;
		var handlerObj= {
			success: function(queryResultsSummaryPage) {
				
				// TODO : Remove this code once the server support returning headers for query with no result
				if(!queryResultsSummaryPage){
					
					queryResultsSummaryPage = {   
					"EPMHeaders": { "FieldDefinitions": [ {"attributeType": "smallString", "label": "Query" } ]},
					"QueryResult": [  { "Query": "No Resource matches the current filter criteria."  } ]
					};
					
				};
				self._querySuccessCallback(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor);
				if (!preserveFilter || !self._lastFilter || self._lastFilter == "") {
					self._queryEditorPageView.resetFilter();
				} else {
					if(self._lastFilter) {
						self._queryEditorPageView.setFilter(self._lastFilter)				
					}
				}
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				self._isReloading = false;
			},
			error: function(error) { 
				self._queryFailureCallback(error);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				self._isReloading = false;
			}
		};

		var handler= new ServiceResponseHandler(handlerObj, "success", "error");

		var isAscending= orderDirection === "ascending";

		var args= new Object();
		args.startIndex= index ? index - 1 : 0;
		if (itemsPerPage) {
			args.maxResults= itemsPerPage == "all" ? QueryUtil._getMaximumQueryPageSize() : itemsPerPage;
		}
		else {
			args.maxResults= QueryUtil._getDefaultQueryPageSize();
		}
		args.filterAttribute= filterAttribute;
		args.filterValue= filterValue;
		
		if (columnArgs) {
			args.sortOrder= columnArgs.sortOrder;
			args.resultColumns = columnArgs.columnIdentifiers ;
			
		} else if (orderByAttrib) {
			args.sortOrder= [ { id: orderByAttrib, ascending: isAscending ? true : false }];
		}
		
		//Set the has Soap Envelop value to false
		args._noSoapEnvelopProcessing = true; 
		
		QueryClient.getResultSet(this._queryEditorPageView.queryType, query, args, handler);		
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String query
 	 * @param Number index
 	 * @param boolean keepEditor
 	 * @param boolean preserveFilter
 	 * @param String filterAttribute
 	 * @param String filterValue
 	 * @param Object columnArgs
 	 * @return void
 	 */	
	_getSortOrderDefaults: function(query, index, itemsPerPage, keepEditor, 
			preserveFilter, filterAttribute, filterValue, columnArgs) {
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(dto) {
				self._processSortOrderDefaults(dto, query, index, itemsPerPage, keepEditor,
						preserveFilter, filterAttribute, filterValue, columnArgs);
			},
			// Service failure
			failure: function(errorObject) {
				self.handleError(errorObject, "[Error] Failed to retrieve sort order defaults");
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service		
		QueryClient.getEditableQuery(this._queryEditorPageView.queryType, handler, query.itemId);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object dto
 	 * @param String query
 	 * @param Number index
 	 * @param boolean keepEditor
 	 * @param boolean preserveFilter
 	 * @param String filterAttribute
 	 * @param String filterValue
 	 * @param Object columnArgs
 	 * @return void
 	 */	
	_processSortOrderDefaults: function(dto, query, index, itemsPerPage, keepEditor, 
			preserveFilter, filterAttribute, filterValue, columnArgs) {

		var orderByAttrib = null;
		var orderDirection = null;
		var id = null;
		var ascending = false;
		// Get sort order attributes if any are defined
		try {
			var id = dto.sorting[0].id;
			var ascending = dto.sorting[0].ascending;
		} catch(errorObject) {
			// Fail silently and fallback on defaults
			// this.handleError(errorObject, "[Error] Processing DTO for sort order defaults failed");
			// Continue processing
		}
		// Set sort order attribute
		if (com.ibm.epm.web.core.client.QueryUtil.isStringDefined(id)) {
			orderByAttrib = id;
		} else {
			orderByAttrib = "modified";
		}
		// Set sort order direction
		if (ascending) {
			orderDirection = "ascending";
		} else {
			orderDirection = "descending";
		}
		// Run query
		this._internalGetResultsSet(query, index, orderByAttrib, orderDirection, itemsPerPage,
			keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs);
	},		
  	/*
  	 * Called upon a successful query execution.  Displays the results in a
  	 * table.
  	 */
  	_querySuccessCallback: function(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor) {
			if(QueryUtil.serverTimeDelta) {
				return this._querySuccessCallbackWithTime(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor);
			}
		    var self = this;
			
			//TODO : Remove this call 
			// Call the result display funtion
			self._querySuccessCallbackWithTime(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor);
			
			//TODO : Get the time from the server and enable this client call
//			var serviceObj= {
//				success: function(timeNow) {
//					QueryUtil.setServerTimeDelta(timeNow)
//					self._querySuccessCallbackWithTime(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor);
//				},
//				failure: function(errorObj) {
//					console.debug("Error getting time: " + errorObj.msg);
//					self._querySuccessCallbackWithTime(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor);
//				}
//			};
//			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
			//QueryClient.getTime(srh, {  });
	},
	_querySuccessCallbackWithTime: function(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor) {
		
  		checkNotNull(queryResultsSummaryPage, "queryResultsSummaryPage");
  		checkNotNull(query, "query");
		// Fix to convert strings true/false to booleans
		var idRow = -1;
		
		// Get the headers
		var headers = QueryUtil.getQueryFieldsList(queryResultsSummaryPage);
		
		
		// Org COde 
//		for(var i=0;i<queryResultsSummaryPage.headers.length; i++) {
//				header = queryResultsSummaryPage.headers[i];
//				if(header.iconOnly) {
//					header.iconOnly = header.iconOnly == "true" || header.iconOnly === true ? true : false; 
//				};
//				if(header.isOrderable) {
//					header.isOrderable = header.isOrderable == "true" || header.isOrderable === true ? true : false; 
//				}
//		}
		
		for(var i=0;i<headers.length; i++) {
				header = headers[i];
				if(header.iconOnly) {
					header.iconOnly = header.iconOnly == "true" || header.iconOnly === true ? true : false; 
				};
				if(header.isOrderable) {
					header.isOrderable = header.isOrderable == "true" || header.isOrderable === true ? true : false; 
				}
		}
		
		this._lastQueryName = query.name ? query.name : "New Query";
		this._lastQueryItemId = query.itemId ? query.itemId : null;
		this._queryEditorPageView.showQueryResults(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage, keepEditor);

  		if (this._currentOperation) {
			this.finishOperation(this._currentOperation);
			this._currentOperation= null;
		}
  	},
	  
	/*
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Shows the query editor for a new query.
	 */
	newQuery: function(kwArgs) {
		this._isSubsequentView = true;
		queryKey = "New Query:" + kwArgs.tq;
		
		// if we're already caching the query editor editor, and the timestamp is the same, simply show the cached page
		if (queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastQueryItemsPerPage == kwArgs.itemsPerPage) {
			this._queryEditorPageView.showLastQueryResults();
			return;
		}
		
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastQueryItemsPerPage = kwArgs.itemsPerPage;

	// If its the same query key, but different other factors, rerun the query
		if(queryKey == this._lastQueryKey) {
			this._internalRunQuery(this._lastQueryRun, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, true,  undefined, undefined, undefined, this._lastColumnArgs);
			return;
		}

		this._lastQueryKey = queryKey;
		this._lastColumnArgs = null;
		
		this._queryEditorPageView.makeNewQueryLinkUnique(); // this can be removed when "Reset" works in the query editor
		// Otherwise show a fresh query
		this.editQuery({ id: "", type: this._queryEditorPageView.queryType});
	},
	
	/*
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Loads the query specified by 'kwArgs.id' in the query editor.
	 */
	editQuery: function(kwArgs) {
		this._isSubsequentView = true;
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs, "kwArgs.id");
		
		if(kwArgs.id == "") {
			this._queryEditorPageView.resetQueryEditor();
		}
		
		this._queryEditorPageView.queryType = kwArgs.type;
  		var self= this;
		var operation= this.createOperation("Opening Query", "editQuery");
		this.startOperation(operation);
		
		var serviceObject = {
			success: function(query) {
				if (!query) {
					self._handleQueryNotFound();
					return;
				}
				self.finishOperation(operation);
				query.queryType = self._queryEditorPageView.queryType;
				self._queryEditorPageView.showQueryInEditor(query);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			},
			error: function(error) { 
				self.finishOperation(operation);
				self.handleError(error, QueryEditorMessages.ERROR_WHILE_EDITING_QUERY);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			}
		};
		QueryClient.getEditableQuery(this._queryEditorPageView.queryType, new ServiceResponseHandler(serviceObject, "success", "error"), kwArgs.id);
	},
	
	_handleQueryNotFound: function() {
		this._showDefaultView();
		PlatformUI.getWorkbench().showMessage("Query does not exist");
	},
	  	
  	/*
  	 * Called if an error occurs while executing a query.
  	 */
  	_queryFailureCallback: function(error) {
		this.handleError(error, "Error running query");
		if (this._currentOperation) {
			this.finishOperation(this._currentOperation);
		}
  	},

	/*
	 * VIEW WORK ITEM-related methods.
	 */
	 
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Opens a work item editor for a new work item.
	 */
	newWorkItem: function(kwArgs) {
		this._isSubsequentView = true;
		this._setPageTitle("New Work Item");
		
		if(kwArgs.id) {
			kwArgs.ts = kwArgs.id;
		} else {
			if(!kwArgs.ts) kwArgs.ts = 1;
			kwArgs.ts = kwArgs.ts * -1;
		}
		// if we're already displaying a new work item editor, simply update the
		// currently displayed tab
		if (this._queryEditorPageView.isWorkItemEditorAvailable(kwArgs.ts) && !kwArgs.doReset) {
			this._queryEditorPageView.showEditorTab(kwArgs.ts, kwArgs.tab, kwArgs.doSave);
			return;
		}
	
		// if the page is loaded directly, wait until there is a default type (page fully loaded)
		if(this.defaultType) {
			this._newWorkItemPart2(kwArgs);	
		} else {
			var self=this;
			var then = new Date().getTime(); 
			var waitFunc = function() {
				if(self.defaultType) {
					self._newWorkItemPart2(kwArgs, interval);
				}
				var now = new Date().getTime();
				if((now - then) > 30000) { // time out after 30 seconds
					self._newWorkItemPart2(kwArgs, interval);
				}
			}
			var interval = setInterval(waitFunc, 200);
		}
	},
	_newWorkItemPart2: function(kwArgs, interval)	{
	
		if(interval) {
			clearInterval(interval);
		}
	
		if(this._createButton && this._createButton.makeUrlsUnique) {
			this._createButton.makeUrlsUnique();
		} 
		
		// otherwise fetch a template new work item object and display it
		var operation= this.createOperation("Opening New Work Item", "newWorkItem");
		this.startOperation(operation);
		var type = kwArgs.type || this.defaultType;
		var tempId =  kwArgs.ts || (new Date().getTime());
		this._showNewWorkItem(operation, type, /* tempId */ kwArgs.ts, kwArgs.tab);
		
		},
	login: function() {
		window.location.href = "https://" + window.location.hostname + ":" + com.ibm.team.repository.server.repourl.port.https + window.location.pathname + window.location.search + window.location.hash;
	},
	isGuest: function() {
		return com.ibm.team.repository.web.client.session.isGuest();
	},
	checkLogin: function() {
		if (com.ibm.team.repository.web.client.session.isGuest()) {
			if (confirm("You must be logged in to edit or create work items.\nPress OK to log in or Cancel to refresh.")) {
				this.login(); /* Will not return */
			}
			return false;
		}
		return true;
	},
	_showNewWorkItem: function(operation, type, tempId, tab) {
		if(!this.checkLogin()) {
			 this._showDefaultView();
			 return; // not reached
		}
		var self = this;
		this._newWorkItemArgs = { operation: operation };
			
		this._newWorkItemArgs.type = type;
		
		this._fetchWorkItem(this._newWorkItemArgs.operation, this._newWorkItemArgs.type, /* catid */ null, tempId, tab);
	},
	_findProperty: function(props, attributeName) {
		if(!props) return [];
		for(var i=0;i<props.allValues.length; i++) {
			if(props.allValues[i].attributeName == attributeName) {
				return props.allValues[i].uiItems;
			}
		}
		return [];
	},
	_fetchWorkItem: function(operation, typeId, categoryItemId, tempId, tab) {
		var self = this;
		var url = window.location.href;
		var handlerObj= {
			newWorkItemHandler: function(workItem) {
				self._promoteAttributes(workItem);
				if(workItem.id == -1) {
					workItem.id = tempId;
				}
				self._queryEditorPageView.showWorkItemEditor(workItem, /* tab */ tab, /* isRefresh */ false,  url);
				if (operation) {
					self.finishOperation(operation);
				}
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			},
			error: function(error) { 
				self.handleError(error, "Error creating new work item");
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			}
		};

		var responseHandler=
			new ServiceResponseHandler(handlerObj, "newWorkItemHandler", "error");

		typeId = typeId || this.defaultType;

		if(!typeId) {
			this._isReloading=false;
			var error = {
				status:"501", // dummy number
				message:"The default work item type is not defined. Select a type using the Create Work Item button."
			}
			this.handleError(error, "Error creating new work item.");
			// unreachable
			return;
		}
		var args = {
			typeId: typeId,
			newWorkItem: "true",
			includeHistory: false
		};
		if(categoryItemId) {
			args.categoryItemId = categoryItemId;
		}

		WorkItemClient.getWorkItem(responseHandler,args );
	},

	_populateCreateWorkItemButton: function(button) {
		// fetch the values required for the create work item drop down
		if(!button || !button.domNode) return;
		this._createButton = button; 
		var self=this;
		var serviceObj= {
			self: this,
			button: button,
			success: function(editProps) {
				var args = new Array();
				var defaultTypeMsg = "default work item";
				var defaultType = "defect";

				var options = editProps[0].uiItems;
				if(!options) {
					if(self._createButton) {
						self._createButton.domNode.style.display = "none";
					}
					self.defaultType = defaultType;
					return;
				}

				// make the default type the first in the drop down list
				var j = 0
				for (var i = 0; i < options.length; i++) {
					if (options[i].id == defaultType) {
						defaultTypeMsg = defaultType;
						args[j++] = {
							label: options[i].label,
							url: "#action=com.ibm.team.workitem.newWorkItem&type=" + encodeURIComponent(options[i].id),
							iconUrl: options[i].iconUrl,
							id: options[i].id
						}
					}
				}
				for (var i = 0; i < options.length; i++) {
					if (options[i].id != defaultType) {
						args[j++] = {
							label: options[i].label,
							url: "#action=com.ibm.team.workitem.newWorkItem&type=" + encodeURIComponent(options[i].id),
							iconUrl: options[i].iconUrl,
							id: options[i].id
						};
					}
				}

				self._queryEditorPageView.welcomePageWidget.initializeWorkItemLinks(args);
				
				self._createButton.domNode.title = "Click to create a " + defaultTypeMsg + " or use the drop down to select a type to create."
				self._createButton.populateDropDown(args);
				if(self._createButton.makeUrlsUnique) {
					self._createButton.makeUrlsUnique();
				}
				if(args.length == 0) {
					self.defaultType = defaultType;
				} else {
					self.defaultType = args[0].id;
				}
			},
			failure: function(errorObj) {
				this.self.handleError(errorObj, "Error loading properties for editing the work item");
			}
		};
		var srh= new ServiceResponseHandler(serviceObj, "success", "failure");

		var appArgs = {newWorkItem: true, ids: ["workItemType"]};
		WorkItemClient.getAllValues(srh, appArgs);
	},		
 	/*
 	 * Take some attributes passed in workitem.attributes[theAttribute:value] and store them as workitem.theAttribute
 	 * Rule of thumb: Promote well known immutable attributes 
 	 */
	_promoteAttributes: function(workItem) {
		if(!workItem) return;
		workItem.attributes = workItem.attributes || {};
    	if(workItem.attributes.id && workItem.attributes.id.label ) {
			workItem.id = workItem.attributes.id.label.replace(/\,/g,"");
		} else {
		    workItem.id = -1;  // will act like a new work item
		}
	},
 
	/*
	 * Sets the browser's URI so that the Jazz framework calls the
	 * 'viewWorkItem' method.  May be called by other code within the
	 * com.ibm.epm.web.core bundle.
	 *
	 * @param id work item id to open.
	 * @param tab (optional) indicates which tab ("Overview", etc.) the work
	 *  item editor should display initially.
	 */
	_internalViewWorkItem: function(id, tab) {
		checkNotNull(id, "id");
		var numId= parseInt(id, 10);
		if (!numId || numId <= 0) {
			this.handleError({message: "Error opening work item"}, "Work item ID must be positive int (saw " + id + ")");
		}
		//window.location.href= QueryUtil.getViewWorkItemURI(id, null, tab);
	},

	/**
	 * Remove the doSave parameter from the location because the save was cancelled
	 */
	_clearSaveParameter: function() {
		this._queryEditorPageView._updateSaveParameters();
		var loc = window.location.href.replace(/&doSave=\([0-9]+\)/, "");
		window.location.href= loc;
	} ,
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Opens the work item specified by 'kwArgs.id'.
	 *
	 * @param kwArgs map of arguments set by Jazz framework based on URI request
	 *  params.
	 * @param isRefresh (optional) specifies that the work item should be
	 *  reloaded even if it's currently open.
	 */
	viewWorkItem: function(kwArgs, isRefresh) {
		this._isSubsequentView = true;
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.id, "kwArgs.id");

		var id= parseInt(kwArgs.id, 10);
		if (!id || id <= 0) {
			this.handleError({message: "Error viewing work item"}, "Work item ID must be positive number (saw " + kwArgs.id + ")");
		}
		
		// if the work item editor is already open for the specified id, and
		// we're not doing an explicit refresh, then simply change the tab
		if (this._queryEditorPageView.isWorkItemEditorAvailable(id) && !isRefresh) {
			this._queryEditorPageView.showEditorTab(id, kwArgs.tab, kwArgs.doSave);
			return;
		}
		
		// otherwise try to fetch the work item specified by kwArgs.id
		var opMsg= (isRefresh ? "Refreshing" : "Opening") +  " Work Item " + id;
		var operation= this.createOperation(opMsg, "view work item");
		this.startOperation(operation);
		var url = window.location.href;
		var self= this;
		var serviceObject= {
			viewWorkItemHandler: function(workItem) {
				if (workItem) {
					self._promoteAttributes(workItem);
					self._queryEditorPageView.showWorkItemEditor(workItem, kwArgs.tab, isRefresh, url);
				} else {
					if (operation) {
						self.finishOperation(operation);
					}
					self._openDefaultView("Work Item " + id + " does not exist. Search for a different work item number.");
				}
				if (operation) {
					self.finishOperation(operation);
				}
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			},
			error: function(error) { 
				self.handleError(error, "Error viewing work item");
				if (operation) {
					self.finishOperation(operation);
				}
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			}
		};

		var responseHandler=
			new ServiceResponseHandler(serviceObject, "viewWorkItemHandler", "error");

		// Avoid console message wrt bad work item id's
		if(id && id < 0 ) id = -1;
		WorkItemClient.getWorkItem(responseHandler, {includeHistory: false, id: id});
	},
	removeEditorTab: function(id) {
		this._queryEditorPageView.removeEditorTab(id);
	},
	setIsDirty: function(workItemEditor, isDirty) {
		this._queryEditorPageView.setIsDirty(workItemEditor, isDirty);
	},
	getLastQueryInfo: function() {
		var currentWidgetId =this._queryEditorPageView.getCurrentWidgetId();
		var currentWorkItem =this._queryEditorPageView.getCurrentWorkItem();
		var isQuery = currentWidgetId ? currentWidgetId.indexOf("Query") != -1 : false; 
		var isWorkItem = !isQuery; 
		return { name: this._lastQueryName, itemId: this._lastQueryItemId, url: this._lastQueryURL, currentWidgetId: currentWidgetId, isQuery: isQuery, isWorkItem: isWorkItem, currentWorkItem: currentWorkItem };
	},
	isInResults: function(workItemId) {
		if(this._queryEditorPageView.queryResultsWidget) { 
			var results = this._queryEditorPageView.queryResultsWidget.getResults();
		}
//		console.debug(results);
		if (!results)
			return false;
		for (var i = 0; i < results.length; i++) {
			if (results[i] && results[i].indices && results[i].indices[0] == workItemId)
				return true;
		}
		return false;
	},
	/*
	 * Return the currently displayed work item
	 */
	getCurrentWorkItem: function() {
		try{
			return this._queryEditorPageView._multipaneContentWidget.currentWidget._workItem;
		} catch(e) {
			return null;
		}
	},
	getCachedWorkItemEditor: function(id) {
		try{
			return this._queryEditorPageView._multipaneContentWidget.getCachedWidget("__jazzWorkItemEditor" , id, /* simple */ true);
		} catch(e) {
			return null;
		}
	},
	
	removeEditor: function(id, /* show next tab */ showNext) {
		this._queryEditorPageView.removeEditorTab(id, showNext );
	},
	
	/*
	 * Get the list of "Recent work Items"
	 */
	getRecentWorkItemList: function() {
		var cachedItems = this._queryEditorPageView._multipaneContentWidget.getWidgetCache();
		if(!cachedItems || cachedItems.length == 0) {
			return [];
		}
		
		var workItemList = [];
		for(var i=0;i<cachedItems.length; i++) {
			if(cachedItems[i].widget && cachedItems[i].widget._workItem) {
				var item = cachedItems[i].widget._workItem;
				item.workItemItemId = item.itemId;
				workItemList.push(item);
			}
		}
		return workItemList;
	},
	_modifyCategoryValues: function( values, possibleValues, modifiers, isNew) {
		modifiers.allCategories = possibleValues;
		var categories = modifiers.allCategories;
		
		var currentValue = values[0];
		if(!currentValue) return;
		if(!possibleValues) return;
		
		var found = false;
		// Map the categories to fully qualified names and search for current value
		for(var i=0;i<modifiers.allCategories.length;i++) {
				var fullLabel = EditorUtil._getFullName(modifiers.allCategories, i, "label");
				categories[i].title = fullLabel;
				categories[i].shortLabel = categories[i].label;
				
				if (modifiers.allCategories[i].id == currentValue.id) {
					found = true;
					if (!dojo.isIE) {
						categories[i].label = this._getClippedCategoryLabel(categories[i].title.replace(/ \/ /g, "/"));
					}
				}

				modifiers.allCategories[i].title = fullLabel;
			}
		
		// If the current category is not in the allCategories list, then add it
		if(!found) {
			var argumentArray = [currentValue.label];
			if(currentValue.arguments) {
				argumentArray = currentValue.arguments.split("/");
			}
			if(argumentArray.length == 1) {
				currentValue.title = currentValue.label;
			} else { // currentValue.arguments cannot be null
				currentValue.title = currentValue.arguments.replace(/\//, " / ");
				currentValue.label = this._getClippedCategoryLabel(currentValue.arguments);
			}
			currentValue.shortLabel = currentValue.label;
			categories.unshift(currentValue);
		}
		
		if (isNew) {
			categories = this._filterCategories(modifiers.allCategories) 
			}
		return { possibleValues: categories };
	},
		_handleCategoryChange: function(widget, value, isInitialization) {
			if(isInitialization) return;
					
			// always expand the categories after something is selected.
			if (value.id != "showall") {
				widget._select.title = widget._select.options[widget._select.selectedIndex].title;
			}
			
			// update the label for the selected item
			for (var i = 0; i < widget._modifiers.allCategories.length; i++) {
				if(!dojo.isIE && widget._modifiers.allCategories[i].id == value.id) {
					widget._modifiers.allCategories[i].label = this._getClippedCategoryLabel(widget._modifiers.allCategories[i].title.replace(/ \/ /g, "/"));
				} else {
					widget._modifiers.allCategories[i].label = widget._modifiers.allCategories[i].shortLabel;
				}	
			}
			widget._possibleValues = widget._modifiers.allCategories;
			EditorUtil._bindValuesToSelect(widget._possibleValues, widget._select);
		},
		_getClippedCategoryLabel: function(longLabel) {
			if(dojo.isIE || longLabel.length < 24) return longLabel;
			longLabel = "..." + longLabel.substring((longLabel.length-21),longLabel.length);
			return longLabel;
		},
		/*
		 * Filter out all but top-level categories and add an extra "(Show All)"
		 * option which on selection will display all of the categories.
		 */
		_filterCategories: function(categoryArr) {
			var newArr= [];
			newArr.push({label: "(Show All)", id: "showall"});
			for (var i=0; i<categoryArr.length; i++) {
				if (EditorUtil._getIndent(categoryArr[i].label) === 0) {
					newArr.push(categoryArr[i]);
				}
			}
			return newArr;
		}
});

})();
