/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

//TODO : Change the date funtions to use dojo.date utilities rather doing the calculation all by ourself

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.util.WBSUtil");
            
    //Localization
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
    
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
	var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    /*
     * Date related  purpose utility methods.
     */
    com.ibm.epm.work.web.ui.internal.util.WBSUtil = {
		
		getWBSElementName: function(ElementType){
			
			if(ElementType.indexOf(WBSConstants.EPM_PROGRAM_TYPE)!= -1){
				return Messages.PROGRAM;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_PROJECT_TYPE)!= -1){
				return Messages.PROJECT;
			}
			else if(ElementType.indexOf(WBSConstants.WORKITEM_TYPE)!= -1){
				return Messages.WORKITEM;
			}
			else if(ElementType.indexOf(WBSConstants.ITERATION_TYPE)!= -1){
				return Messages.ITERATION;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_MILESTONE_TYPE)!= -1){
				return Messages.MILESTONE;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_TASK_TYPE)!= -1){
				return Messages.TASK;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_SUMMARY_TASK_TYPE)!= -1){
				return Messages.SUMMARYTASK;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_CHANGE_REQUEST_TYPE)!= -1){
				return Messages.CHANGEREQUEST;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_ISSUE_TYPE)!= -1){
				return Messages.ISSUE;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_DEFECT_TYPE)!= -1){
				return Messages.DEFECT;
			}
			else if(ElementType.indexOf(WBSConstants.EPM_RISK_TYPE)!= -1){
				return Messages.RISK;
			}
			
			
		},
		/**
		 * Return the project id 
		 * If its a new unsaved element 
		 * get it from the containingProject
		 * @param {Object} row
		 */
		_getProjectIdFromRow: function(row,parentRow){
			if(row)
			{ 
				/**
				 * For new unsaved elements
				 * If the parent of the unsaved is iteration case is not handled 
				 * currently
				 */
					if (row.id && row.id.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1)
					{
					  						   
						   if(!parentRow)
							{
							  return WBSConstants.EMPTY_STRING;	
							}
						   if(parentRow && parentRow.resourcetype && parentRow.resourcetype == WBSConstants.EPM_PROJECT_TYPE && parentRow.href)
							{
							   return this._getResourceIDFromURI(parentRow.href);
							}
							 
							 if(parentRow && parentRow.resourcetype && parentRow.resourcetype == WBSConstants.ITERATION_TYPE )
							{
							  return WBSConstants.EMPTY_STRING;
							}
							
							if(parentRow && parentRow.containingProject)
							{
								return this._getResourceIDFromURI(parentRow.containingProject);
							}
					
					}
				
					if(row.resourcetype && row.resourcetype == WBSConstants.EPM_PROJECT_TYPE)
						{
						return this._getResourceIDFromURI(row.href);
						}
						
					if(row.resourcetype && (row.resourcetype == WBSConstants.EPM_PROGRAM_TYPE || row.resourcetype == WBSConstants.ITERATION_TYPE))
					    {
							return WBSConstants.EMPTY_STRING;
						}
					if(row.loadedelement && row.loadedelement[0] && row.loadedelement[0].containingProject)
					{
						return this._getResourceIDFromURI(row.loadedelement[0].containingProject);
					}
			   
			}
		},
		
		/**
		 * Method to get the resourceID from URI. 
		 * @param {Object} relativeHref: HREF from which resourceID will be parsed
		 */
        _getResourceIDFromURI: function(relativeHref){
			if(relativeHref == null || relativeHref == WBSConstants.EMPTY_STRING)
			{
				return WBSConstants.EMPTY_STRING;
			}
            var index = relativeHref.lastIndexOf("/");
            if (index > 0) {
                return relativeHref.substring(index + 1, relativeHref.length);
            }
        },
		
		isfieldEditableScheduleGrid: function(row, fieldName){
			if(((row.hasChildren && row.resourcetype !== WBSConstants.ITERATION_TYPE)|| 
				// Element has a task/project assignment
				row.hasAssignments ||
				// Disable proposed schedule edit for Programs
				(row.resourcetype && row.resourcetype == WBSConstants.EPM_PROGRAM_TYPE)) &&
			   ((fieldName === WBSConstants.PROPOSED_START_FIELD_NAME) || 
				(fieldName === WBSConstants.PROPOSED_FINISH_FIELD_NAME) ||
				(fieldName === WBSConstants.PROPOSED_DURATION_FIELD_NAME))){
						
					return false;
			} 
			else if(((row.hasChildren && row.resourcetype !== WBSConstants.ITERATION_TYPE) ||
					  // Element does not have an assignment or is not type duration based then do not allow edit of Actual schedule
	  	 			  (row.assignmenttype != WBSConstants.ASSIGNMENT_TYPE_DURATION) ||
					  (row.hasAssignments === false) || (row.hasProfileAssignments === true) ||
  					  // Disable actual schedule edit for Projects
				      (row.resourcetype && row.resourcetype == WBSConstants.EPM_PROJECT_TYPE)) &&
					   ((fieldName === WBSConstants.ACTUAL_START_FIELD_NAME) ||
						(fieldName === WBSConstants.ACTUAL_FINISH_FIELD_NAME) ||
						(fieldName === WBSConstants.PERCENT_DC_FIELD_NAME))){
						
							return false;
			}
			return true; 
		},
		
		isfieldEditableOverviewGrid: function(row, cellField, unSavedRow){
			// The work item type cell is editable if its for an unsaved work item type or iteration type
			if ((row.resourcetype == WBSConstants.WORKITEM_TYPE || row.resourcetype == WBSConstants.ITERATION_TYPE) && 
			    cellField == "wiTypeName" && unSavedRow) {
			    return true;
			}
			
            if ((row.resourcetype == WBSConstants.WORKITEM_TYPE && row.workItemType == WBSConstants.EPM_TASK_TYPE) && 
               ((cellField == "constraintType") || (cellField == "constraintDate"))) {
                
                if (row.constraintType == WBSConstants.AS_SOON_AS_POSSIBLE && cellField == "constraintDate") {
                    return false;
                }
                else 
                    return true;
            }
            return false;
		}

    };
    
})();
