/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
        
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;    
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
	var EPMDomNodeConstants = com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    
    /*
     * Superclass for most Admin UI pages, to provide common functionality.
     *
     * Much of this class handles prompting the user to save an unsaved page when refreshing or
     * moving away to another page. To use:
     * - call _registerNavigation() during initialization of the page.
     * - call _unregisterNavigation() during the destroy() function of the page.
     * - your refresh code that gets run when user presses a refresh button should first call
     *   _confirmRefresh() to determine if the user agrees to the refresh of a dirty page.
     * - for any of this to work, your subclass must enable/disable the save button when appropriate
     *   so that the code can tell if the page is dirty, using disableSaveButton().
     */
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.AdminGrid", [dijit._Widget, dijit._Templated], {

        // The subclass must assign a Save text button to this variable
        // (by calling createSaveButton)
        saveButton: null,

        // The subclass must assign a Delete icon button to this variable
        deleteButton: null,

        /*
         * Any row selection change (selected, unselected) causes this to be called.
         * Determine if state of delete button needs to be changed based on whether any
         * rows are selected or not.
         */
        gridRowSelectionChanged: function(e){
            var selectedCount = this._grid._generalGrid.selection.getSelectedCount();
            this.disableDeleteButton(selectedCount == 0);
        },

        /*
         * Called when grid determines a cell has been edited. Will enable save button.
         */
        onApplyCellEdit: function(e){
            this.disableSaveButton(false);
        },

        /*
         * Disable save button if passed true, else enable it.
         */
        disableSaveButton: function(state){
			this.saveButton.disabled = state;
        },
        
        /*
         * Return true if page is dirty and requires saving.
         */
        isDirty: function() {
            // It can't be dirty if there's not even a save button yet
            if (!this.saveButton)
                return false;
                
            return !this.saveButton.disabled;
        },
        
        /*
         * When called, sets up the "navigate away" functionality to warn users when switching pages
         * without saving. NOTE: Only set this up once per workbench, otherwise every navigate attempt
         * will trigger multiple calls. So, set this up once for current page, and unregister it when
         * navigating to another page (which may then register it again if necessary).
         */
        _registerNavigation: function() {
            if (!this.navigateAwayHandle) {
                this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
                this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);
            }
        },
        
        /*
         * Reverses the registration done by _registerNavigation. Must be called before user is allowed
         * to navigate away. Also, always call during destroy() so that there is no chance of this
         * being left registered after the page has been destroyed.
         */
        _unregisterNavigation: function() {
            if (this.navigateAwayHandle) {
                PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
                this.navigateAwayHandle = null;
                PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
                this.onBeforeUnloadHandle = null;
            }
        },
              
        /*
         * Called when user tries to navigate away from this page. Puts up prompt for 
         * user to confirm, if there are unsaved changed. 
         * Returns true to prevent it, false to allow it.
         */
        _handleNavigate: function(urlObj) {
            /*
             * Clicking on same URL isn't really navigating away, so detect and ignore that.
             */
            var navigateAway = true;
            var winLoc = new dojo._Url(window.location.href);
            
            // Switching to different domain - allow onBeforeUnload to prompt
            if (urlObj.host && urlObj.scheme && !(urlObj.host === winLoc.host && urlObj.scheme === winLoc.scheme)) {
                navigateAway = true;
            }
            
            // Staying within same domain
            else {
                if (urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path))) {
                    // The link is to the current URL (not including the tab param), so ignore it
                    if (this._stripTabParam(urlObj.fragment) == this._stripTabParam(winLoc.fragment)) {
                        navigateAway = true;
                    }

                    // The link is to the same exact base URI (different fragment) - prompt for save/cancel
                    else {
                        navigateAway = this._handleNavigatePrompt();
                    }
                }
            }
            
            return !navigateAway;
        },

        // Not sure what a "tab param" is; code was pulled from QueryEditorPageView.js        
        _stripTabParam: function(string) {
            if (!string)
                return string;
            var index = string.indexOf("&tab=");
            if (index == -1)
                return string;
            var result = string.substring(0,index);
            var end = string.substring(index+1);
            index = end.indexOf("&");
            if (index == -1)
                return result;
            return result + end.substring(index);
        },
        
        /*
         * If this is called, user is navigating away from current page to another page.
         * If page is dirty, put up prompt for user to confirm.
         */
        _handleNavigatePrompt: function() {
            /*
             * Page is dirty; must prompt user.
             */
            if (this.isDirty()) {
                /*
                 * User agreed to navigate away; unregister the navigation events to prevent dups.
                 */
                if (confirm(AdminMessages.NAVIGATE_AWAY + "\n\n" + AdminMessages.UNSAVED_CHANGES + "\n\n" + AdminMessages.OK_OR_CANCEL)) {
                    this._unregisterNavigation();
                    return true;
                }
                
                /*
                 * User canceled navigation away.
                 */
                else {
                    return false;
                }
            }

            /*
             * Page not dirty; allow navigation w/o prompt, being sure to unregister the navigation events to prevent dups.
             */
            this._unregisterNavigation();
            return true;
        },

        /*
         * Call just before allowing a refresh to prompt user about refreshing a dirty page.
         * Returns true if refresh is OK, false if not.
         */
        _confirmRefresh: function() {
            if (this.isDirty() && !confirm(AdminMessages.REFRESH_WARNING))
                return false;
            return true;
        },
            
        // TODO what is this used for?
        navigateAwayEvent: function() {
            return this.dirty ? AdminMessages.UNSAVED_CHANGES : false;
        },

        /*
         * Disable delete button if passed true, else enable it.
         */
        disableDeleteButton: function(state){
            var oldState = this.deleteButton.attr('disabled');
            if (oldState != state) {
                this.deleteButton.attr('disabled',state);
                if (state)
                    this.deleteButton.attr('iconClass',IconClassConstants.EPM_DELETE_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
                else
                  this.deleteButton.attr('iconClass',IconClassConstants.EPM_DELETE_ICON_CLASS);
            }
        },
        
        /*
         * Create a generic save button.  When pressed, it'll call the function provided
         * by the callback arg.
         */
        createSaveButton: function(callback) {
			var saveButton = document.createElement(EPMDomNodeConstants.BUTTON);
            saveButton.innerHTML = AdminMessages.SAVE;
			saveButton.disabled = true;
            dojo.connect(saveButton, EPMDomNodeConstants.ON_CLICK, this, callback);
            return saveButton;
        }
        
    });
})();
