/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");
    
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");    
    dojo.require("com.ibm.epm.web.core.util.internal.Util");
    
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var Util = com.ibm.epm.web.core.util.internal.Util;
    
    /*
     * Superclass for most Admin UI grid views, to provide common functionality.
     */
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.AdminGridView", abstractView, {
    
        /*
         * After instantiation, call setContainingPage() to set this variable to the page
         * (subclass of AdminGrid) that created this.
         */
        _containingPage: null,
    
        /*
         * Will be set to an array containing the results of the dojo.subscribe calls,
         * for possible later unsubscribe.
         */
        _subscriptions: null,
            
        /*
         * The Admin UI grids need to be as high as possible, without being too high
         * that they cause a browser scrollbar to appear.  The only way to do this is
         * to detect resize events and then force the height to be the correct
         * height (which has to be calculated based on browser size).
         *
         * To use this code in a subclass, just set this.initializeResizeHandling to true in the function
         * that creates the grid (usually "build*Grid()"). Upon first row insertion
         * after grid creation, the code will then activate.
         *
         * There is also width resizing, which is much simpler. Just set one column width in the layout
         * to "auto", or use percentage to deal with widths, and the columns will resize 
         * correctly.
         */
         
        /*
         * Flag used to cause grid resizing code to only run once when the first
         * row (even the automatic blank row) is inserted. Necessary because this code
         * must run after the grid's DOM nodes are all created, and creation happens async,
         * so row insertion is the best place to ensure the grid is fully created.
         */
        initializeResizeHandling: false,
        
        /*
         * Called by page that creates this grid view, allowing this grid view to access
         * the containing page via this._containingPage.
         */
        setContainingPage: function(page) {
            this._containingPage = page;
        },
        
        /*
         * On notification of a row selection changing, notify containing page that
         * this happened. (This may change button states.)
         */
        _gridRowSelectionChanged: function(e){
            if (this._containingPage)
                this._containingPage.gridRowSelectionChanged(e);
        },
        
        /*
         * On notification of a cell's content changing, notify containing page that
         * the page is now dirty. (This will enable the save button.)
         */
        _onApplyCellEdit: function(e) {
            // Ignore any "edits" that didn't really change anything (but accept the edit if the row is a new row)
            if (e.value == this._cellOldValue) {
                var row = this._generalGrid.model.getRow(e.rowIndex);
                if (row.id.indexOf("_unsaved") < 0)
                    return;
            }
            
            // Enables save button
            if (this._containingPage)
                this._containingPage.onApplyCellEdit(e);
        },
        
        /*
         * Subscribe to events in order to control button states.
         */
        _subscribeToEvents: function(){
            this._subscriptions = [
			    dojo.subscribe(EPMGridPubSub.ON_ROW_SELECTION_CHANGED, this, "_gridRowSelectionChanged"),
			    dojo.subscribe(EPMGridPubSub.ON_APPLY_CELL_EDIT, this, "_onApplyCellEdit")
			];            
        },
        
        /*
         * Determine the height of the browser window.
         * Works for both IE and Firefox.
         */
        getWindowHeight: function() {
            var height = 0;
            
            // Non IE
            if (typeof( window.innerWidth ) == 'number')
                height = window.innerHeight;
            
            // IE 6+ in standards compliance mode
            else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight))
                height = document.documentElement.clientHeight;
                
            return height;
        },

        /*
         * The IBM "footer" at the bottom of every page causes a slight complication
         * when determining proper grid height. When the browser is high enough to not
         * require a scroll bar, the footer will be visible, else it'll be obscured
         * below the bottom of the browser window (and you'll have to scroll to it).
         * Whether the footer is visible or not affects how high we want the grid to be
         * -- need to subtract the height of the footer, but only when that footer is
         * visible.
         * 
         * TODO does it work in IE?
         */
        getFooterHeightCorrection: function() {
            var offset = document.documentElement.scrollHeight -
                document.documentElement.clientHeight;
            if (offset > 35)
                offset = 35;
            return offset;
        },
        
        /*
         * Called every time a row is inserted, including the special empty row
         * that is ALWAYS inserted (even for an empty data set). We use it to 
         * trigger the automatic grid height resizing after the grid has been created.
         */
        newRowInserted: function(row, rowID, rowIndex){
            this.inherited(arguments);
            
            if (this.initializeResizeHandling) {
                this.initializeResizeHandling = false;
                this.setupResizeHandling();
            }
        },
        
        /*
         * Determine the grid's real offset from the top of the browser window.
         * Necessary because the offsetTop variable is only relative to a DOM
         * node's parent.
         */
        getGridOffsetTop: function() {
            var offset = 0;
            
            var node = this._generalGrid.domNode;
            while (node) {
                offset += node.offsetTop;
                node = node.offsetParent;
            }
            
            return offset;
        },
        
        /*
         * Override if subclass may need to account for the space taken by the grid's
         * horizontal scroll bar.
         */
        hasGridHScroll: function() {
            return false;
        },
        
        /*
         * Call this upon creation of the first grid row after the grid has been
         * created. Sets up a handler to automatically change the grid height for
         * every resize event.  Also does the initial height change.
         */
        setupResizeHandling: function() {            
            /*
             * Create the callback function that will happen whenever window is
             * resized. This will calculate the new grid height and set it.
             * setInterval is here because the async behavior means that sometimes
             * the _generalGrid has not been created yet.
             */
            var self = this; 
            var resizeGrid = function() {
                var intervalID = setInterval(function(){
                    if (self._generalGrid) {
                        clearInterval(intervalID);
                        
                        var h = self.getWindowHeight() - self.getGridOffsetTop() - 45 + self.getFooterHeightCorrection();
                        if (self.hasGridHScroll())
                            h -= 18;
                        if (h >= 0) {
                            self._generalGrid.domNode.style.height = h + "px";
                            //Defect 12460 : The grids start overflowing on addition of new rows
                            //This is an IE specific fix to set the maximum height of the grid
                            if(dojo.isIE) {
                            	self._generalGrid.domNode.style.maxHeight = h + "px";
                            }
                            self._generalGrid.resize();
                            //When resizing the window, there is a lot of flicker and the window resizing is also not smooth
                            //We should not be updating the grid when the browser is being resized.
                            //self._generalGrid.update();
                        }
                    }
                }, 250);
            };
            dojo.connect(window, "onresize", resizeGrid);
            
            // Do the initial resize of the height after grid creation
            resizeGrid();
        },
		
        /**
		 * Select the first row of the grid by default
		 */
		_selectFirstRowByDefault: function() {
            if (this._generalGrid && this._generalGrid.selection.selectedIndex <= 0) {
                this._generalGrid.selection.select(0);
            }
        },

        /*
         * Listen to events for when cell edits start.
         */
        createView: function() {
            this.inherited('createView', arguments);
            
            dojo.connect(this._generalGrid, "onStartEdit", this, this._onStartEdit);
        },
        
        /*
         * Store info for when a cell edit starts, so that validation can be done when
         * the cell edit has completed.  (The later use of this info is completely
         * optional depending on the needs of the grid.)
         */
        _onStartEdit: function(inCell, inRowIndex) {
            this._cellEditing = inCell;
            var row = this._generalGrid.model.getRow(inRowIndex);
            if (row) {
                this._curCellField = inCell.field;
                this._cellOldValue = row[inCell.field];
            }
        },
        
        /*
         * Util that's called during onApplyCellEdit for fields that need to 
         * validate the numeric cell value change. If it was invalid, revert the 
         * cell back to its original (valid) value. Returns true if
         * validation passed, false if an error happened.
         *
         * Validation passes if value is actually a number (int or decimal)
         * and it is within the specified range (or >= 0 if range not specified).
         */
        _validateChangeInNumberField: function(oldValue, newValue, item, row, rowIndex, field, min, max) {
            var validated = true;
            var errorMessage = null;
            
            // Min value defaults to 0 if not provided
            if (min == undefined)
                min = 0;

            /*
             * Determine if validation fails.
             */
            validated = Util.isNumber(newValue);
            if (!validated)
                errorMessage = AdminMessages.CELL_NOT_A_NUMBER;
            if (validated) {
                validated = (newValue >= min);
                if (!validated)
                    errorMessage = dojo.string.substitute(AdminMessages.CELL_BELOW_MIN_NUMBER, [min]);
            }
            if (validated && (max != undefined)) {
                validated = (newValue <= max);
                if (!validated)
                    errorMessage = dojo.string.substitute(AdminMessages.CELL_ABOVE_MAX_NUMBER, [max]);
            }
            
            /*
             * If it failed, revert to original value.
             */
            if (!validated)
                this._popupTooltip(rowIndex, errorMessage);

            return validated;
        },

        /*
         * If cell fails validation, call this to put up a 5 second tooltip and
         * revert the cell.
         */
        _popupTooltip: function(rowIndex, errorMessage) {
            var node = this._cellEditing.getNode(rowIndex);
            if (node) {
                var self = this;
                    
                var showTooltipTimeout = setTimeout(dojo.hitch(this, function() {
                    self.tooltipCellNode = self._cellEditing.getNode(rowIndex);
                    dijit.showTooltip(errorMessage, self.tooltipCellNode);
                    clearTimeout(self.showTooltipTimeout);
                }), 5);
                var hideTooltipTimeout = setTimeout(dojo.hitch(this, function() {
                    dijit.hideTooltip(self.tooltipCellNode);
                    clearTimeout(self.hideTooltipTimeout);
                }), 5000);
            }
                
            this._generalGrid.edit.cancel();
        },
        
        /*
         * Util that's called during onApplyCellEdit for fields that need to 
         * validate the number of decimal places of a number field is correct.
         * Supports 0 or more places; if user enters more than the allowed nmber, revert the 
         * cell back to its original (valid) value. Returns true if
         * validation passed, false if an error happened.
         */
        _validateChangeInDecimalField: function(oldValue, newValue, item, row, rowIndex, field, places) {
            var validated = true;
            var errorMessage = null;
            
            /*
             * Determine if validation fails.
             */
            var parsedFull = parseFloat(newValue);
            var parsedTruncated = parsedFull.toFixed(places);
            if (parsedFull != parsedTruncated) {
                validated = false;
                if (places == 0)
                    errorMessage = AdminMessages.CELL_WHOLE_NUMBER_ONLY;
                else
                    errorMessage = dojo.string.substitute(AdminMessages.CELL_WRONG_DECIMAL_PLACES, [places]);
            }
            else {
                if (newValue != parsedTruncated)
                    this._generalGrid.model.store._setValueOrValues(item, field, parsedTruncated, true);
            }
            
            /*
             * If it failed, revert to original value.
             */
            if (!validated)
                this._popupTooltip(rowIndex, errorMessage);

            return validated;
        }
        
        
    });
})();
