/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.proficiency.ProficiencyGrid");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.proficiency.ProficiencyGridView");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
    
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.proficiency.ProficiencyGrid", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/proficiency/templates/ProficiencyGrid.html"),
        
        _grid: null,
        
        postCreate: function(){
        	
			this._createToolbar();
            
            if (this._grid == null) {
                this._createProficiencyGrid();
            }
            this._registerNavigation();            
        },
        
		 
        /**
         * Creates the Proficiency Grid
         */
        _createProficiencyGrid: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
        
            this._grid = new com.ibm.epm.admin.web.ui.internal.view.proficiency.ProficiencyGridView({
                _treeID: Constants.PROFICIENCY_URI
            });
			this._grid.setContainingPage(this);
            this.proficiencyGridAttachPoint.appendChild(this._grid.domNode);
            
            this._grid.buildProficiencyGrid();
        },
        
        /**
         * Adds new Proficiency
         */
        _addProficiency: function(){
            if (this._grid) {
                this._grid.addProficiency();
            }
        },
        
        /**
         * Saves the changes in the Proficiency Grid
         */
        _saveProficiency: function(){
            if (this._grid) {
                this.disableSaveButton(true);
                this._grid.saveProficiency();
            }
        },
        
        /**
         * Refreshes the Proficiency Grid
         */
        _refreshProficiency: function(){
            if (!this._confirmRefresh())
                return;

            //Delete the grid and prepare again
            if (this._grid) {
                this.proficiencyGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
            }
            
            this._createProficiencyGrid();
        },
        
        
        /**
         * Deletes the selected Proficiency
         */
        _deleteProficiency: function(){
        
            if (this._grid) {
                this._grid.deleteProficiency();
                
            }
        },
        
        /**
         * Creates the Toolbar
         */
        _createToolbar: function(){
            this.addButton = new dijit.form.Button({
                title: AdminMessages.ADD_PROFICIENCY,
				label : AdminMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addProficiency)
            });
            
            this.deleteButton = new dijit.form.Button({
                title: AdminMessages.DELETE_PROFICIENCY,
				label : AdminMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteProficiency)
            });
            
            this.refreshButton = new dijit.form.Button({
                title: AdminMessages.REFRESH,
				label : AdminMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshProficiency)
            });
            
            this.saveButton = this.createSaveButton(this._saveProficiency);

            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this._proficiencyGridToolBar = new EPMToolbar({
                rightContentList: rightContentList,
				title:AdminViewletMessages.PROFICIENCIES_ADMIN_PAGE_TITLE
            });
           
            this._toolbar.appendChild(this._proficiencyGridToolBar.domNode);
        },
        
        destroy: function(){
            if (this._grid && this._grid.destroy) {
                this.proficiencyGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
                delete this._grid;
            }
            this._unregisterNavigation();            
        }
    });
})();
