/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

 (function() { // module container
dojo.provide("com.ibm.epm.web.core.ui.internal.view.query.QueryResults");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.epm.web.core.client.QueryUtil");



var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var QueryUtil			= com.ibm.epm.web.core.client.QueryUtil;

var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;

var ROW_WORK_ITEM_ID_PROP	= "_jazzRowWorkItemId";
var ROW_QUERY_PROP			= "_jazzRowQuery";
var ROW_LINKS_PROP			= "_jazzRowLinks";

// constant for stripping hyperlinks
var STRIP_HYPERLINK_REGEX	= /(<a.*?>)(.*?)(<\/a>)/gi;

function getRow(e) {
	var element= e.target || e.srcElement;
	// walk up the DOM tree until we find a row element
	while (element && element.tagName !== "TR") {
		element= element.parentNode;
	}
	return element || null;
}

/**
 * Provides a UI to iterate through the work items in a query result.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.epm.web.core.ui.internal.view.query.QueryResults",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/view/query/templates/QueryResults.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.epm.web.core/ui/internal/view/query/templates/Query.css"),
		
		// object initializer
		constructor: function() {
			this._rowCache= [];
			this._filterInProgress= false;
			this._queuedFilter= null;
			
			// Initialize the configurations for known data types
			this._columnConfigs = {
				internalState: {
						showIcon: true	
					},
				
				internalSeverity: {
						titlePrefix: "Severity",
						titleSelector: "_getTypeTitle"	
					},
				internalPriority: {
						titlePrefix: "Priority",
						titleSelector: "_getTypeTitle"	
					},
				workItemType: {
						titlePrefix: "Work Item Type",
						titleSelector: "_getTypeTitle"	
					},
				id: {
						isLinked: true
					},
				summary: {
						isLinked: true,
						isHtml: true,
						formatterSelector: "_getSummaryLabel"
					},
				modified: {
						formatterSelector: "_getModifiedLabel" 
				},
				creationDate: {
						formatterSelector: "_getDateLabel" 
				},
				resolutionDate: {
						formatterSelector: "_getDateLabel" 
				},
				dueDate: {
						formatterSelector: "_getDateLabel" 
				}
					
			}
			
		},
		/*
		 *  Return the results of the last query
		 */ 
		getResults: function() {
			return this._rowCache;
		}, 
		/*
		 * Methods for table population and clearing.
		 */
		
		setQueryResultsTable: function(query, queryResultsSummaryPage, orderByAttribute, direction, itemsPerPage) {
			checkNotNull(query, "query");
			checkNotNull(queryResultsSummaryPage, "queryResultsSummaryPage");
			
			// if no matches, provide an empty arr for easier programming later
			if (!queryResultsSummaryPage.workItemSummaryDTOs) {
				queryResultsSummaryPage.workItemSummaryDTOs= [];
			}
			
			// reset the UI
			this._resetTableHeader();
			this._resetTableBody();
			
			// populate the UI
			this._populateTableHeader(queryResultsSummaryPage, orderByAttribute, direction);
			this._populateTableBody(query, queryResultsSummaryPage);
			
			// show the bottom page control
			dojox.data.dom.removeChildren(this._bottomPageControlSection);
			this.parentView.showQueryBottomPageControl(query,queryResultsSummaryPage, orderByAttribute, direction, itemsPerPage, this._bottomPageControlSection );
			
		},
		
		/*
		 * methods to do with resetting the UI
		 */
		
		// clear header rows
		_resetTableHeader: function() {
			dojox.data.dom.removeChildren(this._headerRowLine);
			dojox.data.dom.removeChildren(this._headerRow);
		},
		
		// clear body rows
		_resetTableBody: function() {
			dojox.data.dom.removeChildren(this._tableBody);
			for (var i=0; i<this._rowCache.length; i++) {
				this._rowCache[i]= null;
			}
			this._rowCache= [];
		},
		
		/*
		 * methods to do with populating the UI
		 */
		
		_populateTableHeader: function(queryResultsSummaryPage, orderByAttribute, direction) {
			
			// if there are zero or one query results, it makes no sense to
			// allow users to reorder them
			var preventOrdering= queryResultsSummaryPage.rows == null || queryResultsSummaryPage.rows.length < 2;
			
			if (!preventOrdering) {
				var uriInfo= this._getUriInfo();
			}

			var self= this;
			function addCol(typeId, displayText, isOrderable, alignment, width, color, isIconOnly, preventOrdering) {
				
				// If nothing specified, assume it is em
				if( width && 
					width.toLowerCase().indexOf("em") == -1 &&
					width.toLowerCase().indexOf("px") == -1) {
					// Making the Width based on the number of columns returned in the result
					// Org Code :- width = width + "em"
					width = 10 + "em";
				}
				//TODO : better way to specify the width of  columns
				width = 20 + "em";
				// Add top white line trim
				var headerColumnLine = document.createElement("th");
				dojo.addClass(headerColumnLine, "headerColumnLine");
				dojo.addClass(headerColumnLine, typeId);
				headerColumnLine.innerHTML = "&nbsp;";
				self._headerRowLine.appendChild(headerColumnLine);
				
				var th= document.createElement("th");
				self._headerRow.appendChild(th);
				th.className= typeId;
				if (isOrderable && !preventOrdering) {
					var link= document.createElement("a");
					link.appendChild(document.createTextNode(displayText));
					// IE Bug: Required because CSS whitespace nowrap does not work
					var nobr = document.createElement("nobr");
					nobr.appendChild(link);
					th.appendChild(nobr);
					var newDirection = "descending";					
					if (typeId === orderByAttribute) {
						// Add sorting direction
						var orderDirectionImage = document.createElement("img");
						var imagePath = "plugin/com.ibm.epm.web.core/resources/ui/internal/view/query/images/";
						dojo.addClass(orderDirectionImage, "sortDirection");
						if (direction == "descending") {
							orderDirectionImage.alt = " (descending)";
							orderDirectionImage.src = ImageClient.getImageUrl(imagePath + "q-result-down-arrow.gif");
							newDirection = "ascending";
						} else {
							orderDirectionImage.alt = " (ascending)";
							orderDirectionImage.src = ImageClient.getImageUrl(imagePath + "q-result-up-arrow.gif");
							newDirection = "descending";
						}
						link.appendChild(orderDirectionImage);
					}
					link.href= uriInfo.baseUri + (!uriInfo.isFirst ? "&" : "") + "orderBy=" + typeId + "&direction=" + newDirection;		
					link.title= "Order results by '" + displayText + "' (" + newDirection + ")";
					// Style link
					dojo.addClass(link, "headerColumnSortedLink");
					// Style column
					dojo.addClass(th, "headerColumnSorted");
				} else {
					th.appendChild(document.createTextNode(displayText));
					// Style column
					dojo.addClass(th, "headerColumnUnsorted");
				}
			}
		
			//Get the hearders list
			var headers = QueryUtil.getQueryFieldsList(queryResultsSummaryPage);
			console.log("headers");
			console.log(headers);
			//Org code :- dojo.forEach(queryResultsSummaryPage.headers,
			//	function(col) {
			
			dojo.forEach(headers,
				function(col) {		
					// After the new query format - we have only Label and atrributeType coming from the server
					// So removing the other options that are not available
					// Further making the table currently not sortable
					col.isOrderable = false;
			
					// Previous call :- addCol(col.attributeId, col.label, col.isOrderable, col.alignment, col.width, col.color, col.iconOnly, preventOrdering );
					addCol(col.attributeId, col.label, col.isOrderable, col.alignment, col.width, col.color, col.iconOnly, preventOrdering );
				});
					
		},
		
		// get the current URI fragment, sans 'orderBy' and 'direction' attributes
		_getUriInfo: function() {
			var uriInfo= {
				baseUri: "#",
				isFirst: true
			};
						
			var oldFrag= QueryUtil.getFragment(document.location.href);
			if (oldFrag) {
				var nvps= oldFrag.split("&");
				for (var i=0; i<nvps.length; i++) {
					var nvp= nvps[i].split("=");
					// strip off orderBy and direction (we'll deal with them
					// later); also strip off 'index' since we want to go back
					// to page 1 on a re-order
					if (nvp[0] != "orderBy" && nvp[0] != "direction" && nvp[0] != "index") {
						if (uriInfo.isFirst) {
							uriInfo.isFirst= false;
						} else {
							uriInfo.baseUri += "&";
						}
						uriInfo.baseUri += nvp[0] + "=" + nvp[1]; 
					}
				}
			}
			
			return uriInfo;
		},
		_getLabelAttributeFromRow: function(row, headers, attributeId) {
			for(var i=0;i<headers.length;i++) {
				if(headers[i].attributeId == attributeId) {
					return row.labels[i];
				}
			}
			return null;
		},
		_populateTableBody: function(query, queryResultsSummaryPage) {
			// if we don't have any work item summaries display the "no results" message
			
			//Get the result Data
			var queryResult = QueryUtil.getQueryResultSet(queryResultsSummaryPage);
			
			// Org Code :- if (queryResultsSummaryPage.rows == null || !queryResultsSummaryPage.rows.length || queryResultsSummaryPage.rows.length === 0) {
			
			if (queryResult == null || !queryResult.length || queryResult.length === 0) {
				this._setNoResultsVisibility(/* isVisible */ true);
				this._bottomPageControlSection.style.display = "none";
				return;
			} else {
				this._setNoResultsVisibility(/* isVisible */ false);
				this._bottomPageControlSection.style.display = "block";
				
				// Each row has two values... icons and labels
				// icons is an array for each column, what icon to display
				// labels is an array for each colum, what lable to display
				// if the column header says "icononly", then only show the icon
				// if the column header says "color", then use that color
				// use the width and alignment from the header
				
				// Org Code :- var rows= queryResultsSummaryPage.rows;
				var rows= queryResult;
				
				//Org code :- var headers= queryResultsSummaryPage.headers;
				var headers =  QueryUtil.getQueryFieldsList(queryResultsSummaryPage);
				
				for (var i=0; i<rows.length; i++) {
					var row= document.createElement("tr");
					this._tableBody.appendChild(row);
					this._populateRow(row, rows[i], headers, query);
					// find the owner and summary, if any
					var summary = this._getLabelAttributeFromRow(rows[i], headers, "summary");
					var ownerName = this._getLabelAttributeFromRow(rows[i], headers, "ownerName");
					this._rowCache[i]= {
						row: row,
						indices: [
							rows[i].id,
							summary == null ? "" : summary.toLowerCase(),
							ownerName == null ? "" : ownerName.toLowerCase(),
							summary == null ? "" : summary
							]
					};
				}

			}
		},
		
		_populateRow: function(tableRow, queryRow, headers, query) {
			checkNotNull(tableRow, "tableRow");
			checkNotNull(queryRow, "queryRow");
			var columnElements= [];
			var viewWorkItemURL= "";//QueryUtil.getViewWorkItemURI(queryRow.id, query);
			
			
			//Storing the Information, which wud be used for mouse click events
			// Org Code :- tableRow[ROW_WORK_ITEM_ID_PROP]= queryRow.id;
			// tableRow[ROW_QUERY_PROP]= query;
			
			var i = 0;
			var self = this;
			dojo.forEach(headers, function(header) {
				columnElements.push(self._doColumnElementHelper(
						self._createColumnElement(tableRow, queryRow, header, i++, viewWorkItemURL ),
						header.attributeId));
				}
			);
			
			for (var i=0; i<columnElements.length; i++) {
				var col= tableRow.insertCell(i);
				if(headers[i].width && headers[i].width > 0) { col.style.width = headers[i].width; }
				if(headers[i].alignment) { col.style.align = headers[i].alignment; }
				if(headers[i].color) { col.style.color = headers[i].color; }
				col.appendChild(columnElements[i].element);
				if (columnElements[i].className) {
					col.className= columnElements[i].className;
				}
			}
			return tableRow;
		},
		
		/*
		 * Modified Code
		 */
		
		_createColumnElement: function(tableRow, queryRow, header, i, workItemUrl) {
			
				var span = document.createElement("SPAN");

				var parentElement = span;
				
					var headerLabel = header.label;
					var label = queryRow[header.label];
					
					// IE Bug: Border is not rendered unless there is content
					if (label == null || label.length == 0) {
						label = "-";
						dojo.addClass(parentElement, "emptyContent");
					}
					parentElement.appendChild(document.createTextNode(label));						
				
				return span;

		},

		/*
		 * Origional Code
		 */
		_createColumnElement_org: function(tableRow, queryRow, header, i, workItemUrl) {
				// Header has:
				//  attributeId: "workItemType"
				//  attributeType: type, isOrderable
				//  label, width, alignment, color
				//  iconOnly
				var config = this._columnConfigs[header.attributeId];
				if(!config) { config = {}};
				var span = document.createElement("SPAN");
				if(config.className) {
					span.className = config.className;
				}
				
				if (!dojo.isIE || dojo.isIE > 6) {
					if (queryRow.iconUrls[i] && this._columnConfigs[header.attributeId] && this._columnConfigs[header.attributeId].showIcon) {
						var img = document.createElement("IMG");
						img.className = "queryResultIcon";
						img.src = queryRow.iconUrls[i];
						span.appendChild(img);
						dojo.addClass(span, "iconBackground");
					}
				}

				var parentElement = span;
				if(config.isLinked) {
					var link= document.createElement("a");
					link.href= workItemUrl;
					span.appendChild(link);
					parentElement = link;

					if(tableRow[ROW_LINKS_PROP] == null) {
						tableRow[ROW_LINKS_PROP] = [];
					}
					tableRow[ROW_LINKS_PROP].push(span);
				}
				// If there is an icon URL then add an icon
				if(header.iconOnly && queryRow.iconUrls[i]) {
					var titlePrefix = header.attributeType;
					if(config.titleSelector) {titlePrefix = this[config.titleSelector](config, header) };
					parentElement.appendChild(this._createAttributeImage(titlePrefix, queryRow.labels[i], queryRow.iconUrls[i]));				
				}
				// If it isn't icon only then add the text too
				if(!header.iconOnly) {
					var label = queryRow.labels[i];
					if(config.formatterSelector) {
						label = this[config.formatterSelector](label);
					}
					
					// we are currently not sending HTML from the server
//					if(config.isHtml) {
//						parentElement.innerHTML = label;	
//					} else {
					if (com.ibm.team.workitem.web.client.util.isStringDefined(label) == false) {
						// IE Bug: Border is not rendered unless there is content
						label = "-";
						dojo.addClass(parentElement, "emptyContent");
					}
					
					parentElement.appendChild(document.createTextNode(label));						
//					}
				}
				
				return span;

		},
		_getTypeTitle: function(config, header) {
			var prefix = config.titlePrefix == null ? header.label : config.titlePrefix;
			return prefix;
		},
		
		/*
		 * Looks for long unbroken strings ( > 40 chars ) and breaks them up so
		 * that the table doesn't get pushed beyond off the right side of the
		 * page.
		 *
		 * An example long unbroken string is:
		 * com.ibm.team.filesystem.tests.api.IFileSystemViewTest.test_deleteSubtreeLcom_ibm_team_scm_common_IChangeSetHandleLcom_ibm_team_scm_common_IComponentHandleLcom_ibm_team_repository_common_IVersionableHandle
		 */
		_getSummaryLabel: function(summary) {
			summary= this._removeHyperlinks(summary);
			return QueryUtil._breakupAllLongWords(summary);
		},
		
		// if any occurence of a hyperlink, strip out hyperlink tags
		// (nested links are illegal as per HTML 4, section 12.2.2)
		// http://www.w3.org/TR/html4/struct/links.html#idx-link-6
		_removeHyperlinks: function(summary) {
			return summary.indexOf("<a") === -1 ?
					summary :
					summary.replace(STRIP_HYPERLINK_REGEX, "$2");
		},
		
		_doColumnElementHelper: function(element, className) {
			return {
				element: element,
				className: className ? className : null
			};
		},
		
		_setNoResultsVisibility: function(isVisible) {
			this._noResultsMsg.className=
				isVisible ? "visible" : "hidden";
		},
		
		_createAttributeImage: function(attributeName, attributeTextValue, iconURL) {
			var img= document.createElement("img");
			img.title= attributeName + ": " + (attributeTextValue || "Not Set");
			img.alt= (attributeTextValue || "Not Set");
			img.src= iconURL ?
					ImageClient.getImageUrl(iconURL) :
					ImageClient.getImageUrl("/com.ibm.team.workitem.common/icons/obj16/type_none.gif");
			return img;
		},
		
		_getModifiedLabel: function(modified) {
			var date = new Date()
			date.setTime(modified);
			return QueryUtil.getModifiedLabel(date);
		},
		_getDateLabel: function(timeLong) {
			if(!timeLong || timeLong == "Unassigned") {
				return "None"
			}
			var date = new Date()
			date.setTime(timeLong);
			return QueryUtil.getDateLabel(date);
		},
	
		/*
		 * Methods related to filtering
		 */
		
		filter: function(filterStr) {
			if (!dojo.isString(filterStr)) {
				throw new Error("QueryResults#filter: input must be of type string");
			}

			// if a filter is in progress, queue the incoming request and return
			if (this._filterInProgress) {
				console.debug("queueing filter: " + filterStr);
				this._queuedFilter= filterStr;
				return;
			}
			
			//
			// BEGIN GUARDED SECTION
			//
			this.parentController._lastFilter=filterStr;
			this._filterInProgress= true;
			filterStr= dojo.trim(filterStr).toLowerCase();
			if (filterStr === "") {
				this._doUnfilterAll();
				this._setNoFilterMatchesVisibility(/* isVisible */ false);
			} else {
				this._doFilter(filterStr);
			}
			this._filterInProgress= false;
			//
			// END GUARDED SECTION
			//

			// if we queued a filter during processing, clear it and call
			// ourselves with the queued filter
			if (this._queuedFilter !== null) {
				console.debug("Applying queued filter: " + this._queuedFilter);
				var newStr= this._queuedFilter;
				this._queuedFilter= null;
				this.filter(newStr);
			}
		},
		
		_doFilter: function(filterStr) {
			var noMatches= true;
			for (var i=0; i<this._rowCache.length; i++) {
				var row= this._rowCache[i].row;
				var indices= this._rowCache[i].indices;
				if (this._isMatch(row, indices, filterStr)) {
					this._showRow(row);
					noMatches= false;
				} else {
					this._hideRow(row);
				}
			}
			this._setNoFilterMatchesVisibility(/* isVisible */ noMatches);
		},
		
		_doUnfilterAll: function() {
			for (var i=0; i<this._rowCache.length; i++) {
				this._showRow(this._rowCache[i].row);
			}
		},
		
		_isMatch: function(row, indices, filterStr) {
			for (var i=0; i<indices.length; i++) {
				//alert("indices[i]: " + indices[i] + ", filterStr: " + filterStr + ", index: " + indices[i].indexOf(filterStr));
				if ((indices[i]+ "").indexOf(filterStr) !== -1) {
					return true;
				}
			}
			return false;
		},
		
		_showRow: function(row) {
			row.className= "visibleRow";
		},
		
		_hideRow: function(row) {
			row.className= "hidden";
		},
		
		_setNoFilterMatchesVisibility: function(isVisible) {
			this._noFilterMatches.className=
				isVisible ? "visible" : "hidden";
		},
		
		/*
		 * Event handlers
		 */
		
		_handleTbodyMouseOver: function(e) {
			var row= getRow(e);
			if (!row) { return; }
			var rowLinks= row[ROW_LINKS_PROP];
			if (!rowLinks) { return; }
			for (var i=0; i<rowLinks.length; i++) {
				rowLinks[i].firstChild.className= "linkMouseover";
			}
		},
		
		_handleTbodyMouseOut: function(e) {
			var row= getRow(e);
			if (!row) { return; }
			this._removeRowLinkHighlighting(row);
		},
		
		_removeRowLinkHighlighting: function(row) {
			if(!row) { return; }
			var rowLinks= row[ROW_LINKS_PROP];
			if (!rowLinks) { return; }
			for (var i=0; i<rowLinks.length; i++) {
				rowLinks[i].firstChild.className= "";
			}
		},
		
		_handleTbodyMouseClick: function(e) {
			if(e.ctrlKey || e.shiftKey || e.altKey ) { // alt clicks open tabs or windows
				return;
			}
			var row= getRow(e);
			if (!row) { return; }
			
			// TODO : handle Mouse Click event
			// Org Code :- var id= row[ROW_WORK_ITEM_ID_PROP];
			// var query= row[ROW_QUERY_PROP];

			//window.location.href= QueryUtil.getViewWorkItemURI(id, query);
		},
		clearSelection: function() {
			if(!this._rowCache) { return; }
			for (var i = 0;i<this._rowCache.length; i++) {
				var value = this._rowCache[i];
				if(value && value.row) {
					this._removeRowLinkHighlighting(value.row);
				}
			};
		}
	}
);

})();