/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.rates.RatesGridView");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.rates.RatesGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.AdminUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog"); 
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
	var adminUtil = com.ibm.epm.admin.web.ui.internal.Util.AdminUtil;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGridView = com.ibm.epm.admin.web.ui.internal.view.AdminGridView;
    var DashboardUtil = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
    var PubSub = com.ibm.epm.web.core.grid.controller.PubSub;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.rates.RatesGridView", AdminGridView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/rates/templates/RatesGrid.html"),
        
        ratesController: null,
        
        // Provides way to turn off multiselect by passing in a param to the ctor called "_multiSelect"
        // (Used by the ResourceRateAssociation class in the profiles page)
        _multiSelect: true,
        
        _loadingCompletedSubscription: null,
        
        /** 
         * buildRatesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with TimecodesGrid.
         */
        buildRatesGrid: function(isRatesView, selectedRateURI){
            this.isRatesView = isRatesView;
            
            /*
             * Only show loading message if this grid view is being used in the profile page's rates dialog,
             * and also subscribe to event to tell when grid is done loading so message can be removed.
             */
            if (!this._multiSelect) {
                DashboardUtil.showMessage(AdminMessages.LOADING_RATES_DIALOG_IN_PROGRESS);
                this._loadingCompletedSubscription = dojo.subscribe(PubSub.LOADING_COMPLETED, this, "_onGridLoaded");
            }
            
            // Set flag to be used during the first row creation, to trigger grid
            // height change. But only do this for the rates view, not the rates
            // dialog in the profiles view.
            if (this.isRatesView)
                this.initializeResizeHandling = true;
        
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.admin.web.ui.internal.view.rates.RatesGridModel();
            }
            this._gridModel.selectedRateURI = selectedRateURI;
            this.ratesController = controllerUtil.getEPMGridController("RatesController");
            if (this.ratesController.isInitialized()) {
                this.ratesController.destroy();
                this.ratesController = controllerUtil.getEPMGridController("RatesController");
            }
            this.buildGrid(this, this._gridModel, this.ratesController);
            
            this._subscribeToEvents();
        },
        
        /*
         * Callback for when the grid finishes loading, so that "loading" message can be cleared
         * and the toolbar enabled. (Only used if this grid view is being used in the profile
         * page's rates dialog.)
         */
        _onGridLoaded: function() {
            DashboardUtil.hideMessageArea();
        },
        
		/*
         * Override the function defined in EPMGrid.
         * Make name and description of the Default rate uneditable.
         */
        isCellEditable: function(event) {			
			var rowIndex = parseInt(event.rowIndex, 10);
			if (isNaN(rowIndex)) {
			    return true;
			}

            // There's no controller when this widget is used by the user management
            // editor page, so always return false for uneditable.
			if (!this.ratesController) {
			    return false;
		    }
		    
            var row = this.ratesController.getRowByIndex(rowIndex);
            if (row.id === Constants.DEFAULT_RATE_URI) {
            	if (event.cell.field == Constants.NAME_FIELD_NAME || 
            	    event.cell.field == Constants.DESCRIPTION_FIELD_NAME) 
            	{
            		return false;
            	}
            }
	        return true;
        },
        
        /**
         * getGridTitle function - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return AdminMessages.RATE_GRID_TITLE;
        },
        
        /**
         * getGridInternalID function - Override the abstract class and returns the ID of the grid.
         *
         */
        getGridInternalID: function(){
            return Constants.RATE_GRID_TITLE;
        },
        
        /**
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.ratesGridAttachPoint;
        },
        
        /**
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var gridLayout = null;
            if (this.isRatesView) {
                gridLayout = [{
                    noscroll: false,
                    rows: [[{
                        name: AdminMessages.NAME,
                        width: 10,
                        field: Constants.COLUMN_NAME,
                        editor: dojox.grid.editors.Input
                    }, {
                        name: AdminMessages.DESCRIPTION,
                        width: 20,
                        field: Constants.COLUMN_DESCRIPTION,
                        editor: dojox.grid.editors.Input
                    }, {
                        name: AdminMessages.COST_RATE,
                        width: 10,
                        field: Constants.COLUMN_COSTRATE,
						styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
						editor: dojox.grid.editors.Input,
                        formatter: this.formatRate
                    }, {
                        name:AdminMessages.SELLING_RATE ,
                        width: "auto",
                        field: Constants.COLUMN_SELLINGRATE,
						styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
						editor: dojox.grid.editors.Input,
                        formatter: this.formatRate
                    }]]
                }];
            }
            else {
                gridLayout = [{
                    noscroll: false,
                    rows: [[{
                        name: AdminMessages.NAME,
                        width: 10,
                        field: Constants.COLUMN_NAME
                    }, {
                        name: AdminMessages.COST_RATE,
                        width: 8,
                        field: Constants.COLUMN_COSTRATE,
						styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
                        formatter: this.formatRate
                    }, {
                        name: AdminMessages.SELLING_RATE,
                        width: 8,
                        field: Constants.COLUMN_SELLINGRATE,
						styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
                        formatter: this.formatRate
                    }]]
                }];
            }
            return gridLayout;
        },
        
        /*
         * Catch the end of the cell edit and perform validation.  Will also
         * activate the save button if a valid change actually happened.
         */
        _onApplyCellEdit: function(e) {
            if (e && e.gridID == Constants.RATE_GRID_TITLE) {
                // By default, edit is good unless it fails validation
                var changed = true;
                
                /*
                 * Validate only the columns that require it (number values).
                 */
                var row = this._generalGrid.model.getRow(e.rowIndex);
                if (row) {
                    var item = this._generalGrid.model.store._getItemByIdentity(row.id);
                    if (item) {
                        if (this._curCellField == Constants.COLUMN_COSTRATE || 
                            this._curCellField == Constants.COLUMN_SELLINGRATE) 
                        {
                            changed = this._validateChangeInNumberField(this._cellOldValue, e.value, item, row, e.rowIndex, this._curCellField);
                            if (changed)
                                changed = this._validateChangeInDecimalField(this._cellOldValue, e.value, item, row, e.rowIndex, this._curCellField, 2);
                        }
                    }
                }
            
                // Notify save button to enable only if the cell changed
                if (changed)
                    this.inherited('_onApplyCellEdit', arguments);
            }
        },

        /*
         * Formats the rate value to conform to .xx precision
         */
        formatRate: function(inDatum){
            return isNaN(inDatum) ? inDatum : parseFloat(inDatum).toFixed(2);
        },
        
        /**
         * Saves rates
         */
        _saveRates: function(){
            this.persistGrid(true);
        },
        
        /*
		 * Handle the error 
		 */
        handleError: function(errorMessage){
            this.inherited("handleError", arguments);
            adminUtil.displayErrorMessage("errorMsgRates",errorMessage);
        },
		
        /**
         * Adds a grid row to the rate grid
         */
        _addRate: function(){
            var rowID = this._controller.getUniqueRowID(Constants.UNSAVED);
            var row = {
                "relativeHref": rowID
            };
            
            this._controller._insertElement(row, null, false);
            this.scrollToSelectedRow();
            this.startFirstCellEdit();
        },
        
        /**
         * Deletes selected rates
         */
        _deleteRate: function(){
            this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
               gridController: this._controller,
			   grid: this._generalGrid
            });
        },
		
        /*
         * Overridden to optionally set multiSelect=false after the grid is created.
         * This must be done here, even though it seems odd, because the grid
         * may not be created yet if we attempt to turn off multiselect 
         * earlier than this.
         */
        newRowInserted: function(row, rowID, rowIndex){
            this.inherited(arguments);
            
            // Only turn off multiselect if "_multiSelect: false" was passed into the ctor
            if (this._generalGrid && this._generalGrid.selection && !this._multiSelect)
                this._generalGrid.selection.multiSelect = false;
        },
        
        destroy: function() {
            if (this._loadingCompletedSubscription) {
                dojo.unsubscribe(this._loadingCompletedSubscription);
            }
            this.inherited(arguments);
        },
        
		/**
		 * @return Retrieves the sortable columns for this grid view
		 */
		getSortableColums:function(){
			var sortableFields ={};
			sortableFields[Constants.COLUMN_NAME] = Constants.COLUMN_NAME;
			sortableFields[Constants.COLUMN_DESCRIPTION] = Constants.COLUMN_DESCRIPTION;
			sortableFields[Constants.COLUMN_COSTRATE] = Constants.COLUMN_COSTRATE;
			sortableFields[Constants.COLUMN_SELLINGRATE] = Constants.COLUMN_SELLINGRATE;
			return sortableFields;
		}
		
    });
})();
