/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.rates.ResourceRateAssociation");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.rates.RatesGridView");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");    

    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
	var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.rates.ResourceRateAssociation", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/rates/templates/ResourceRateAssociation.html"),
        
        _grid: null,
        
		constructor: function(args) {
            this.okButtonLabel=AdminMessages.OK;
            this.cancelButtonLabel=AdminMessages.CANCEL;
            
            // This dialog is used in two places, each requiring different summary text
            if (args && args.summaryText && args.summaryText != "")
                this.dialogSummaryLabel=args.summaryText;
            else
                this.dialogSummaryLabel=AdminMessages.PROFILE_RATE_ASSOCIATION_DIALOG_SUMMARY;
                
            this.noRatesCheckBoxLabel=AdminMessages.RATES_UNSPECIFIED_LABEL;
		},
		
        postCreate: function(){
            this._selectRowSubscription = dojo.subscribe(EPMGridPubSub.ON_ROW_SELECTION_CHANGED, this, "_selectRow");
            this._selectRateNodeSubscription = dojo.subscribe("selectRateNode", this, "_selectRateNode");
        },
        
        _createRatesGrid: function(){
            // _multiSelect param causes this shared grid view to switch to single-select mode
            this._grid = new com.ibm.epm.admin.web.ui.internal.view.rates.RatesGridView({
                _treeID: Constants.RATE_URI,
                _multiSelect: false
            });
            this._content.appendChild(this._grid.domNode);
            this._grid.buildRatesGrid(false, this._selectedRateURI);
        },
        
        loadResourceRate: function(row, parent) {
            this._resourceData = row;
            this._container = parent;
            this._selectedRateURI = row.rateURI;
            this._createRatesGrid();
        },

        /*
         * Callback for whenever the "rates unspecified" check box is clicked.
         * If value is set to true, this will unselect any rate from the above
         * grid. (Likewise, if a row in the grid is selected, this check box
         * will automatically be unchecked by some other code.)
         */        
        _onNoRatesCheckBoxChanged: function(b) {
            var checked = this._noRatesCheckBox.checked;
            if (checked) {
                this._grid._generalGrid.selection.unselectAll();
            }
        },
        
        _onOK: function(){
            var selectedRowIndexes = this._grid._generalGrid.selection.getSelected();

            var changed = false;
            
            // A rate is specified
            if (selectedRowIndexes != null && selectedRowIndexes.length > 0) {
                var selectedRow = this._grid._generalGrid.model.getRow(selectedRowIndexes[0]);
                
                if (selectedRow) {
                    if (this._resourceData.rate != selectedRow.name) {
                        this._resourceData.rate = selectedRow.name;
                        this._resourceData.rateURI = selectedRow.id;
                        changed = true;
                    }
                }
            }
            
            // A rate is not specified
            else {
                if (this._resourceData.rate != "") {
                    this._resourceData.rate = "";
                    this._resourceData.rateURI = "";
                    changed = true;
                }
            }
            
            // Update the cell's data and publish notification of the change
            if (changed) {
                this._container.setRowData();
                this._publishGridChangeEvent();
            }
            
            this._container.closeDialog();
        },
		
		_onCancel: function(){
            this._container.closeDialog();
        },
        
		
		_publishGridChangeEvent: function(){
			//Inform the parent that the grid has been modified
			var _fieldName = Constants.COLUMN_RATE;
			var publishedObj = {
				controllerID: Constants.SYSTEMPROFILE_CONTROLLER,
				fieldName: _fieldName
			}
			dojo.publish(Constants.PROFILE_FIELD_CHANGE_EVENT, [publishedObj]);
		},
		
        destroy: function(){
            dojo.unsubscribe(this._selectRowSubscription);
            dojo.unsubscribe(this._selectRateNodeSubscription);
            if (this._grid && this._grid.destroy) {
                this._grid.destroy();
                delete this._grid;
            }
        },
        
        /*
         * Event happens when user selects a row in the dialog, or unselects a row via the
         * checkbox. Will unselect the check box if a row was just selected.
         */
        _selectRow: function(e){
            var self = this;
            var intervalID = setInterval(function(){
                if (self._grid) {
                    var selected = self._grid._generalGrid.selection.getSelected();
                    if (selected != "") {
                        self._noRatesCheckBox.checked = false;
                    }
                   
                    else {
                        self._noRatesCheckBox.checked = true;
                    }
                    
                    clearInterval(intervalID);
                }
            }, 50);
        },
        
        /*
         * Event happens only when the dialog is loading; used to communicate the currently selected
         * rate to the dialog so this can select the appropriate row.
         */
        _selectRateNode: function(rowIndex){
            var self = this;
            var intervalID = setInterval(function(){
                if (self._grid) {
                    self._grid._generalGrid.setGridRowSelection(rowIndex);
                    self._grid.scrollToRow(rowIndex);
                    clearInterval(intervalID);
                }
            }, 50);
        }
        
    });
})();
