/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGrid");

    dojo.require("com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGridView");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");    
	dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
    
	var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
	var EPMRiskCategoryGridView = com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGridView;    	
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGrid", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/risk/templates/EPMRiskCategoryGrid.html"),
        
        _grid: null,
        
        postCreate: function(){
        	this._createToolbar();
            
            if (!this._grid) {
                this._createEPMRiskCategoryGrid();
            }
            this._registerNavigation();            
        },
      
	   /**
         * Creates the EPMRisk category Grid
         */
        _createEPMRiskCategoryGrid: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
        
            this._grid = new EPMRiskCategoryGridView({
                _treeID: Constants.RISK_CATEGORY_URI
            });
			this._grid.setContainingPage(this);
            this._epmRiskCategoryGridDIV.appendChild(this._grid.domNode);
            
            this._grid.buildEPMRiskCategoryGrid();
        },
        
        /**
         * Adds new Skill
         */
        _addRiskCategory: function(){
            if (this._grid) {
                this._grid.addRiskCategory();
            }
        },
        
        /**
         * Saves the changes in the Skill Grid
         */
        _saveRiskCategory: function(){
            if (this._grid) {
                this.disableSaveButton(true);
                this._grid.saveRiskCategory();
            }
        },
        
        /**
         * Refreshes the Skill Grid
         */
        _refreshRiskCategory: function(){
            if (!this._confirmRefresh()) {
				return;
			}
            //Delete the grid and prepare again
            if (this._grid) {
                this._epmRiskCategoryGridDIV.removeChild(this._grid.domNode);
                this._grid.destroy();
            }
            
            this._createEPMRiskCategoryGrid();
        },
        
        
        /**
         * Deletes the selected Skills
         */
        _deleteRiskCategory: function(){
        
            if (this._grid) {
                this._grid.deleteRiskCategory();
                
            }
        },
        
        /**
         * Creates the Toolbar
         */
        _createToolbar: function(){
			
            this.addButton = new dijit.form.Button({
                title: AdminMessages.ADD_RISK_CATEGORY,
				label : AdminMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addRiskCategory)
            });
            
            this.deleteButton = new dijit.form.Button({
                title: AdminMessages.DELETE_RISK_CATEGORY,
				label : AdminMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteRiskCategory)
            });
            
            this.refreshButton = new dijit.form.Button({
                title: AdminMessages.REFRESH,
				label : AdminMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshRiskCategory)
            });
            
            this.saveButton = this.createSaveButton(this._saveRiskCategory);
            
            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this._toolBar = new EPMToolbar({
                rightContentList: rightContentList,
				title:AdminMessages.RISKCATEGORIES_ADMIN_PAGE_TITLE
            });
           
            this._toolbarDIV.appendChild(this._toolBar.domNode);
        },
        
        destroy: function(){
            if (this._grid && this._grid.destroy) {
                this._epmRiskCategoryGridDIV.removeChild(this._grid.domNode);
                this._grid.destroy();
                delete this._grid;
            }
            this._unregisterNavigation();            
        }
    });
})();
