/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGridView");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.AdminUtil");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog");        
    
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
	var EPMRiskCategoryGridModel = com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGridModel;
	var adminUtil = com.ibm.epm.admin.web.ui.internal.Util.AdminUtil;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGridView = com.ibm.epm.admin.web.ui.internal.view.AdminGridView;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.risk.EPMRiskCategoryGridView", AdminGridView, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/risk/templates/EPMRiskCategoryGridView.html"),
        
        epmRiskCategoryGridController: null,
        
        /* 
         * Builds the Risk category Grid
         */
        buildEPMRiskCategoryGrid: function(){
            // Set flag to be used during the first row creation, to trigger grid
            // height change.
            this.initializeResizeHandling = true;
                
            if (!this._gridModel) {
                this._gridModel = new EPMRiskCategoryGridModel();
            }
            this.epmRiskCategoryGridController = controllerUtil.getEPMGridController(Constants.RISK_CATEGORY_CONTROLLER);
            this.buildGrid(this, this._gridModel, this.epmRiskCategoryGridController);
            
            this._subscribeToEvents();
        },
        
        /*
         * getGridTitle function - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return AdminMessages.RISK_CATEGORY_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this._epmRatesCategorygridDIV;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        
        
            var epmRiskCategoryGridLayout = [{
                noscroll: false,
                defaultCell: {
                    width: 8,
                    editor: dojox.grid.editors.Input,
                    styles: 'text-align: left;'
                },
                rows: [[{
                    name: AdminMessages.NAME,
                    width: 15,
                    field: Constants.COLUMN_NAME
                }, {
                    name: AdminMessages.DESCRIPTION,
                    width: "auto",
                    field: Constants.COLUMN_DESCRIPTION
                }]]
            
            }];
            
            return epmRiskCategoryGridLayout;
        },
        
        /*
         * Adds a new risk category to the Grid
         * Incoming : None
         * Outgoing : None
         */
        addRiskCategory: function(){
            var rowID = this._controller.getUniqueRowID(Constants.UNSAVED);
            var row = {
                "relativeHref": rowID
            };
            
            this._controller._insertElement(row, null, false);
            this.scrollToSelectedRow();
            this.startFirstCellEdit();
        },
        
        /*
         * Saves the risk category Grid Infomation
         * Incoming : None
         * Outgoing : None
         */
        saveRiskCategory: function(){
        
            this.persistGrid(true);
        },
        
        /*
         * Deletes the skill
         */
        deleteRiskCategory: function(){
        this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
               gridController: this._controller,
			   grid: this._generalGrid
            });
        },
		
        /*
		 * Handle the error 
		 */
        handleError: function(errorMessage){
            this.inherited("handleError", arguments);
            adminUtil.displayErrorMessage("errorMsgRisk",errorMessage);
        },
		
		/*
         * Destroys the  Grid Widget
         * Incoming : None
         * Outgoing : None
         */
        _destroyResourceWidget: function(){
        
            this.destroy();
            
        },

		/**
		 * @return Retrieves the sortable columns for this grid view
		 */
		
		getSortableColums:function(){
			var sortableFields ={};
			sortableFields[Constants.COLUMN_NAME] = Constants.COLUMN_NAME;
			sortableFields[Constants.COLUMN_DESCRIPTION] = Constants.COLUMN_DESCRIPTION;
			return sortableFields;
		}
        
    });
})();
