/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.skills.SkillsGrid");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.skills.SkillsGridView");    
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
    
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.skills.SkillsGrid", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/skills/templates/SkillsGrid.html"),
        
        _grid: null,
        
        postCreate: function(){    
		    
			 this._createToolbar();
            
            if (this._grid == null) {
                this._createSkillGrid();
            }
            
            this._registerNavigation();
        },
		
		 
        /**
         * Creates the Skill Grid
         */
        _createSkillGrid: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
        
            this._grid = new com.ibm.epm.admin.web.ui.internal.view.skills.SkillsGridView({
                _treeID: Constants.SKILL_URI
            });
			this._grid.setContainingPage(this);
            this.skillsGridAttachPoint.appendChild(this._grid.domNode);
            
            this._grid.buildSkillGrid();
            
        },
        
        /**
         * Adds new Skill
         */
        _addSkill: function(){
            if (this._grid) {
                this._grid.addSkill();
            }
        },
        
        /**
         * Saves the changes in the Skill Grid
         */
        _saveSkill: function(){
            if (this._grid) {
                this.disableSaveButton(true);
                this._grid.saveSkill();
            }
        },
        
        /**
         * Refreshes the Skill Grid
         */
        _refreshSkill: function(){
            if (!this._confirmRefresh())
                return;

            //Delete the grid and prepare again
            if (this._grid) {
                this.skillsGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
            }
            
            this._createSkillGrid();
        },        
        
        /**
         * Deletes the selected Skills
         */
        _deleteSkill: function(){
        
            if (this._grid) {
                this._grid.deleteSkills();
                
            }
        },
        
        /**
         * Creates the Toolbar
         */
        _createToolbar: function(){
            this.addButton = new dijit.form.Button({
                title: AdminMessages.ADD_SKILL,
				label : AdminMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addSkill)
            });
            
            this.deleteButton = new dijit.form.Button({
                title: AdminMessages.DELETE_SKILL,
				label : AdminMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteSkill)
            });
            
            this.refreshButton = new dijit.form.Button({
                title: AdminMessages.REFRESH,
				label : AdminMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshSkill)
            });
            
            this.saveButton = this.createSaveButton(this._saveSkill);
            
            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this._skillsGridToolBar = new EPMToolbar({
                rightContentList: rightContentList,
				title:AdminMessages.SKILLS_ADMIN_PAGE_TITLE
            });
            this._toolbar.appendChild(this._skillsGridToolBar.domNode);
        },

        destroy: function(){
            if (this._grid && this._grid.destroy) {
                this.skillsGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
                delete this._grid;
            }
            this._unregisterNavigation();            
        }
    });
})();
