/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/**
 * @author ujjvala
 */
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.staticpooltype.StaticPoolTypeGrid");

    dojo.require("com.ibm.epm.admin.web.ui.internal.view.staticpooltype.StaticPoolTypeGridView");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
    
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.staticpooltype.StaticPoolTypeGrid", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/staticpooltype/templates/StaticPoolTypeGrid.html"),
        
        _grid: null,
        
        postCreate: function(){
        
            this._createToolbar();
            
            if (this._grid == null) {
                this._createStaticPoolTypeGrid();
            }
            this._registerNavigation();            
        },
        
        /**
         * Creates the StaticPool Type Grid
         */
        _createStaticPoolTypeGrid: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
        
            this._grid = new com.ibm.epm.admin.web.ui.internal.view.staticpooltype.StaticPoolTypeGridView({
                _treeID: Constants.STATICPOOL_TYPE_URI
            });
			this._grid.setContainingPage(this);
            this.staticPoolTypeGridAttachPoint.appendChild(this._grid.domNode);
            
            this._grid.buildStaticPoolTypeGrid();
        },
        
        /**
         * Adds new Static Pool Type
         */
        _addStaticPoolType: function(){
            if (this._grid) {
                this._grid.addStaticPoolType();
            }
        },
        
        /**
         * Saves the changes in the StaticPool Type Grid
         */
        _saveStaticPoolType: function(){
            if (this._grid) {
                this.disableSaveButton(true);
                this._grid.saveStaticPoolType();
            }
        },
        
        /**
         * Refreshes the StaticPool Type Grid
         */
        _refreshStaticPoolType: function(){
            if (!this._confirmRefresh())
                return;

            //Delete the grid and prepare again
            if (this._grid) {
                this.staticPoolTypeGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
            }
            
            this._createStaticPoolTypeGrid();
        },
        
        
        /**
         * Deletes the selected StaticPool Type
         */
        _deleteStaticPoolType: function(){
        
            if (this._grid) {
                this._grid.deleteStaticPoolType();
                
            }
        },
        
        /**
         * Creates the Toolbar
         */
        _createToolbar: function(){
            this.addButton = new dijit.form.Button({
                title: AdminMessages.ADD_POOL_TYPE,
				label : AdminMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addStaticPoolType)
            });
            
            this.deleteButton = new dijit.form.Button({
                title: AdminMessages.DELETE_POOL_TYPE,
				label : AdminMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteStaticPoolType)
            });
            
            this.refreshButton = new dijit.form.Button({
                title: AdminMessages.REFRESH,
				label : AdminMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshStaticPoolType)
            });

            this.saveButton = this.createSaveButton(this._saveStaticPoolType);
            
            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this._staticPoolTypeGridToolBar = new EPMToolbar({
                rightContentList: rightContentList,
                title: AdminMessages.POOLTYPES_ADMIN_PAGE_TITLE
            });
            
            
            this._toolbar.appendChild(this._staticPoolTypeGridToolBar.domNode);
        },
        
        destroy: function(){
            if (this._grid && this._grid.destroy) {
                this.staticPoolTypeGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
                delete this._grid;
            }
            this._unregisterNavigation();            
        }
    });
})();
