/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){

    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.systemprofile.EPMCompetenciesGridView");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.systemprofile.EPMCompetenciesGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridView");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserConstants");
	dojo.require("com.ibm.epm.admin.web.client.internal.proficiency.ProficiencyClient");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    
    //Localization
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    var AdminViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    var CompetenciesGridView = com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridView;
    var CompetenciesGridModel = com.ibm.epm.admin.web.ui.internal.view.systemprofile.EPMCompetenciesGridModel;
	var ProficiencyClient = com.ibm.epm.admin.web.client.internal.proficiency.ProficiencyClient;
    var ControllerUtil = com.ibm.epm.web.core.grid.controller.ControllersFactory;//com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var EPMUserConstants = com.ibm.epm.admin.web.ui.internal.view.user.EPMUserConstants;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var DashboardUtil = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
    var PubSub = com.ibm.epm.web.core.grid.controller.PubSub;
	
	var selectOptionsForProficiency = [];
	var selectOptionsForProficiencyId = [];
	
	var selectOptionsForCompetency = [];
	var selectOptionsForCompetencyId = [];
    
	dojo.declare("com.ibm.epm.admin.web.ui.internal.view.systemprofile.DynamicSelect", dojox.grid.editors.Select, {
    
        fieldName: null,
        
        _loadingCompletedSubscription: null,

        constructor: function(inCell){
            this.fieldName = inCell.field;
        },
        
        format: function(inDatum, inRowIndex){
            if (this.fieldName === "proficiencyLevel") {
				if (selectOptionsForProficiency && selectOptionsForProficiency.length > 0) {
					this.options = this.values = selectOptionsForProficiency;
				}
				else {
					this.options = this.values = [];
				}
            }
			if (this.fieldName === "name") {
				if (selectOptionsForCompetency && selectOptionsForCompetency.length > 0) {
					this.options = this.values = selectOptionsForCompetency;
				}
				else {
					this.options = this.values = [];
				}
            }
            return this.inherited("format", arguments);
        }
    });
    
    var DynamicSelect = com.ibm.epm.admin.web.ui.internal.view.systemprofile.DynamicSelect;
	
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.systemprofile.EPMCompetenciesGridView", CompetenciesGridView, {
        // Stores an interval ID so it can be turned off during destroy() as a last resort to prevent a runaway interval
        _intervalID: -1,
        
        // Each of these becomes true when a proficiency or competency dropdown's data has been fully loaded
        _selectOptionsForProficiencyLoaded: false,
        _selectOptionsForCompetencyLoaded: false,
    
        /* 
         * Builds the competency Grid
         */
        buildCompetencyGrid: function(_container){
            this._container = _container;

            DashboardUtil.showMessage(AdminViewletMessages.LOADING_COMPETENCIES_DIALOG_IN_PROGRESS);
            this._loadingCompletedSubscription = dojo.subscribe(PubSub.LOADING_COMPLETED, this, "_onGridLoaded");
            
            /*
             * Load the variables that store all possible competencies and proficiencies for the dropdowns.
             * Note that this is via asynchronous GETs and this block of code may be passed by before the
             * loading completes. When loading completes, each dropdown will set 
             * _selectOptionsForProficiencyLoaded and _selectOptionsForCompetencyLoaded to true.
             */
            var self = this;
            this.getoptions(self, self.setSelectOptionsForProficiency,EPMUserConstants.PROFICIENCY_URI, _container);
            this.getoptions(self, self.setSelectOptionsForCompetency,EPMUserConstants.COMPETENCY_URI, _container);
            
            /*
             * Wait for the option values to be fully retrieved before creating the rest of the grid and
             * enabling the toolbar. (Necessary to prevent the UI from being used before it is done being
             * loaded.)
             */
            var self = this;
            this._intervalID = setInterval(function(){
                if (self._selectOptionsForProficiencyLoaded === true && self._selectOptionsForCompetencyLoaded === true) {
                    clearInterval(self._intervalID);
                    self._intervalID = -1;
                    
                    if (!self._gridModel) {
                        self._gridModel = new CompetenciesGridModel();
                    }
                    self.userCompetencyGridController = ControllerUtil.getEPMProfileCompetencyController(EPMUserConstants.EPM_PROFILE_COMPETENCY_CONTROLLER_ID);
            
                    self.buildGrid(self, self._gridModel, self.userCompetencyGridController);
                }
            }, 50);
        },
		
        /*
         * Callback for when the grid finishes loading, so that "loading" message can be cleared
         * and the toolbar enabled.
         */
        _onGridLoaded: function() {
            DashboardUtil.hideMessageArea();
                    
            // Enable toolbar buttons now that the grid is usable
            this._container.disableAddButton(false);
            this._container.disableRemoveButton(false);
        },
        
		getoptions : function(self, callback, uri, _container){
			var self = this;
			var serviceObj = {
                        success: function(data){
                           var collectionElements1 = [];
						   var collectionElements2 = [];
						   for (var i = 0; i < data.length; i++) {							
								collectionElements1.push(data[i].title);
								collectionElements2.push(data[i].relativeHref);
							}
							
						   callback(self, collectionElements1,collectionElements2, _container); 
                        },
                        failure: function(errorObj){
                        }
                    };
                    var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
                    ProficiencyClient.getURI(srh, uri);
		},
		
		setSelectOptionsForProficiency: function(self, options, values, _container){
        	if (options && options.length > 0) {
				selectOptionsForProficiency = options;
				selectOptionsForProficiencyId = values;
			}
			else {
				if (_container) {
					_container.displayError(AdminViewletMessages.NO_PROFICIENCY_FOUND);
				}
			}
            self._selectOptionsForProficiencyLoaded = true;
        },
		
		getSelectOptionsForProficiency: function()
		{
			return selectOptionsForProficiency;
		},
		
		getSelectOptionsForProficiencyId: function()
		{
			return selectOptionsForProficiencyId;
		},
		
		setSelectOptionsForCompetency: function(self, options, values, _container){
        	if (options && options.length > 0) {
				selectOptionsForCompetency = options;
				selectOptionsForCompetencyId = values;
			}
			else {
				if (_container) {
					_container.displayError(AdminViewletMessages.NO_COMPETENCY_FOUND);
				}
			}
            self._selectOptionsForCompetencyLoaded = true;
        },
		
		getSelectOptionsForCompetency: function()
		{
			return selectOptionsForCompetency;
		},
		
		getSelectOptionsForCompetencyId: function()
		{
			return selectOptionsForCompetencyId;
		},
        
		addAssignment: function()
		{
			var rowID = this._controller.getUniqueRowID(EPMUserConstants.NEW_ELEMENT_UNSAVED_PREFIX);
			var rows = {
				"id": rowID,
                "relativeHref": rowID,
                "name": selectOptionsForCompetency ? selectOptionsForCompetency[0] : "",
                "proficiencyLevel": selectOptionsForProficiency ? selectOptionsForProficiency[0] : "",
				"competencyid":"",
				"pid":""
			};
			this._controller._insertElement(rows, null, false);
			this.scrollToSelectedRow();
            this.startFirstCellEdit();
		},
		
		deleteAssignment: function(row)
		{
			this._controller._deleteElement(row);
		},
		
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        
            var competencyGridLayout = [{
            
                rows: [[{
                    name: AdminViewletMessages.NAME,
                    width: 14,
                    field: "name",
					editor: DynamicSelect
                },{
                    name: AdminViewletMessages.PROFICIENCY_LEVEL,
                    width: 14,
                    field: "proficiencyLevel",
					editor: DynamicSelect
                }]]
            
            }];
            
            return competencyGridLayout;
        },
        
        /*
         * Override to also set multiSelect=false after the grid is created.
         * This must be done here, even though it seems odd, because the grid
         * may not be created yet if we attempt to turn off multiselect 
         * earlier than this.
         */
        newRowInserted: function(row, rowID, rowIndex){
            this.inherited(arguments);
            
            if (this._generalGrid && this._generalGrid.selection)
                this._generalGrid.selection.multiSelect = false;
        },
        
        destroy: function() {
            if (this._intervalID >= 0) {
                clearInterval(this._intervalID);
            }
            this.inherited(arguments);
        }
    });
})();
