/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGrid");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridView");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");    
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");

    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGrid", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/systemprofile/templates/SystemProfileGrid.html"),
        
        _grid: null,
        
        postCreate: function(){
        	this._createToolbar();
            
            if (this._grid == null) {
                this._createSystemProfileGrid();
            }
			this.rateAssociationEvent = dojo.subscribe(Constants.PROFILE_FIELD_CHANGE_EVENT, this, this._enableSaveButton);
            this.dialogChangeEvent = dojo.subscribe(Constants.DIALOG_CHANGE_EVENT, this, this._onDialogChangeEvent);
            this._registerNavigation();
        },
		
		 /**
         * Creates the profile Grid
         */
        _createSystemProfileGrid: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
        
            this._grid = new com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridView({
                _treeID: Constants.PROFILE_URI
            });
			this._grid.setContainingPage(this);
            this.systemProfileGridAttachPoint.appendChild(this._grid.domNode);
            
            this._grid.buildSystemProfileGrid();
        },
        
        /**
         * Adds new profile
         */
        _addProfile: function(){
            if (this._grid) {
                this._grid.addProfile();
            }
        },
        
        /**
         * Saves the changes in the Profile Grid
         */
        _saveProfile: function(){
            if (this._grid) {
                this.disableSaveButton(true);
                this._grid.saveProfile();
            }
        },
        
        /**
         * Refreshes the Profile Grid
         */
        _refreshProfile: function(){
            if (!this._confirmRefresh())
                return;

            //Delete the grid and prepare again
            if (this._grid) {
                this.systemProfileGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
            }
            
            this._createSystemProfileGrid();
        },
        
        
        /**
         * Deletes the selected Profile
         */
        _deleteProfile: function(){
        
            if (this._grid) {
                this._grid.deleteProfiles();
                
            }
        },
        
        /**
         * Creates the Toolbar
         */
        _createToolbar: function(){
            this.addButton = new dijit.form.Button({
                title: AdminMessages.ADD_PROFILE,
				label : AdminMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addProfile)
            });
            
            this.deleteButton = new dijit.form.Button({
                title: AdminMessages.DELETE_PROFILE,
				label : AdminMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteProfile)
            });
            
            this.refreshButton = new dijit.form.Button({
                title: AdminMessages.REFRESH,
				label : AdminMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshProfile)
            });
            
            this.saveButton = this.createSaveButton(this._saveProfile);

            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this._systemProfilerofileGridToolBar = new EPMToolbar({
                rightContentList: rightContentList,
				title:AdminMessages.PROFILES_ADMIN_PAGE_TITLE
            });
           	
            this._toolbar.appendChild(this._systemProfilerofileGridToolBar.domNode);
        },
        
		/**
         *  _enableSaveButton : Event handler when there is any updation in Grid
         *  Incoming : publishedObj
         *  Outgoing : none
         *
         */
        _enableSaveButton: function(publishedObj){
			if (!publishedObj) {
				return;
			}
			if (publishedObj.controllerID == Constants.SYSTEMPROFILE_CONTROLLER) {
				if (this._grid) {
					this.disableSaveButton(false);
				}
			}
		},

        /*
         * Override superclass to prevent the editing of a dialog grid from
         * affecting the Save button.
         */
        onApplyCellEdit: function(e) {
            // Change happened in an editor dialog, so ignore it
            if (e) {
                if (e.gridID == Constants.SKILL_GRID_TITLE || 
                    e.gridID == Constants.COMPETENCY_GRID_TITLE ||
                    e.gridID == Constants.RATE_GRID_TITLE)
                {
                    return;
                }
            }
            
            // Change happened in main profile grid, so enable Save
            this.disableSaveButton(false);
        },


        /*
         * Registered to an event for when the user presses OK in one of the editor dialogs
         * after making changes. Just need to enable Save button.
         */
        _onDialogChangeEvent: function(publishedObj) {
            this.disableSaveButton(false);
        },
        
        destroy: function(){
            if (this._grid && this._grid.destroy) {
                this.systemProfileGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
                delete this._grid;
            }
            dojo.unsubscribe(this.rateAssociationEvent);
            dojo.unsubscribe(this.dialogChangeEvent);
            this._unregisterNavigation();            
        }
    });
})();
