/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){

    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
    
    //TreeUtil
    dojo.require("com.ibm.epm.web.core.tree.TreeUtil");
    // Base class definition of EPMTree
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMFinancialTree");
    dojo.require("com.ibm.epm.web.core.grid.controller.TreeBasedGridController");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    // Constants
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.TreeWidget");
	
    var AdminConstants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    //TreeUtil
    var TreeUtil = com.ibm.epm.web.core.tree.TreeUtil;
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    // Base class definition of EPMTree
    var EPMTree = com.ibm.epm.web.core.ui.internal.EPMFinancialTree;
    var IconClass = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
	var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    var TreeWidget = com.ibm.epm.admin.web.ui.internal.view.TreeWidget;
	
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget", TreeWidget, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/timecodes/templates/TimecodesTree.html"),
        
        _projectID: null,
       // stores the various attributes of the 'Rename' menu item
		renameItem: {},
	    renameAllowed: true,
        _lastUniqueRowID: 0,
	    
		/*
         * Creates the tree label & initiates context menu
         */
        postCreate: function(){
        	// initialise the 'Rename' menu item
			this.renameItem = {
                "type": EPMConstants.MENU_TYPE,
                "id": EPMConstants.CODE_RENAME_ID,
                "parentID": null,
                "addSeparator": true,
                label: EPMConstants.CODE_RENAME_LABEL,
                iconClass: IconClass.RENAME_ICON_CLASS,
                _onClick: dojo.hitch(this, this._rename)
        	};
			
			// Context Menu			
            this._ctxtItems = null; // _TreeNode, the active tree node in right-clicking  for context menu
            this._initMenu();
            			
            // List of subscriptions for this Widget
            this._subscriptions = [];
            this.eventSubscribe();
            
        },
        
		/**
         *  setctxMenu : the method will be called when the _openmenu method is called
         *  in the EPMTree.js
         *  Incoming : event object
         *  Outgoing : none
         *
         */
		setctxMenu: function(nodeItem){
			console.log("set ctxt is called");
			// If element is new(unsaved) then resourceType passed is EPMConstants.NEWElEMENTSUFFIX
			if (this._timecodesTree._isNodeNew(nodeItem)) {
				this._buildNewElementMenu();
			}
			else{
				this._timecodesTree.createMenu(this._ctxtItems);
			}
		},
		
		/**
         * _rename: called when user clicks on 'Rename' menu item.
         * @param {Object} TreeNode
         * Outgoing: none.
         */
        _rename: function(e){
            this._timecodesTree.closeMenu();
            this._ctxNode = this._timecodesTree.getNodeContext();
            this._timecodesTree._renameNode(this._ctxNode);
        },
		
		/**
		 * _buildNewElementMenu: Method to build menu for a new element. <br>
		 */
		_buildNewElementMenu: function(){
			this._timecodesTree.createMenu([this.renameItem]);
		},
		
		/**
		* onNameChangeValidate function will be called when you enter
		* value in the tree node
		*/
		onNameChangeValidate: function(oldValue, newValue){
			if((newValue != null) && (newValue.length > 32)){
				EPMErrorHandler.setError(Messages.TIME_CODE_LENGTH_EXCEED);
				return false;				
			}
			return true;
		},
		
		/**
		* isRenameAllowed function will be called when you dblclick
		* on the tree node
		*/
		isRenameAllowed: function(){
			return this.renameAllowed;
		},
		
		/**
		 * The method eventSubscribe to subscribe the events from Grid, Tree, etc..
		 * This method will be called from postCreate
		 */
        eventSubscribe: function(){
            this._subscriptions = [            //subscribe to new node creation event
            dojo.subscribe("OnNewTimecode", this, "_createTimeCodeElement"), 
			dojo.subscribe(GridEvents.REPLACE_NODE, this, "_replaceTimeCodeNode"), 
			
			// EPMGrid events for mouse hover and select
			dojo.subscribe(GridEvents.ROW_DELETED, this, "_rowDeleted"),
			dojo.subscribe(EPMGridPubSub.ON_ROW_CLICK, this, "_selectNodeOnRowClick"),			
		    dojo.subscribe(EPMTreePubSub.ON_RENAME_NODE, this, "_onRenameNode"),
		    dojo.subscribe(EPMTreePubSub.ON_CREATE_NODE, this, "_onRenameNode")
			];
        },
        
        _getTreeBasedController: function(){
            if (!this.TreeBasedGridController) {
                this.TreeBasedGridController = controllerUtil.getTreeBasedGridController("TimecodesController");
            }
            return this.TreeBasedGridController;
        },
        
        /**
         *  _selectNodeOnRowClick : Event handler when there is a row click event on the Grid
         *  Incoming : publishedObject
         *  Outgoing : none
         *
         */
        _selectNodeOnRowClick: function(publishedItem){
            var row = this._validateAndGetRow(publishedItem);
            if (row && row.id) {
                this._timecodesTree._selectNodeById(row.id);
            }
        },
        
        /**
         * _validateAndGetRow: Method to validate the published data and verify the grid id.
         * Incoming: publishedItem by grid.
         * Outgoing: row selected/hovered upon.
         *
         */
        _validateAndGetRow: function(publishedItem){
            if (this._getTreeBasedController() && publishedItem.gridID == this._getTreeBasedController()._baseGridID) {
                if ((publishedItem) && (publishedItem.rowIndex > -1)) {
                    var row = this._getTreeBasedController().getRowByIndex(publishedItem.rowIndex);
                    return row;
                }
            }
            return null;
        },
        
				
		/**
         *  _rowDeleted : Event handler when there is a row deleted on the Grid
         *  Incoming : publishedObject
         *  Outgoing : none
         *
         */
		_rowDeleted:function(publishedItem){
			var publishedData = null;
            if (dojo.isArray(publishedItem)) {
                publishedData = publishedItem[0];
            }
            else {
                publishedData = publishedItem;
            }
            var localtreeId = publishedData.uniqueIdentifier.treeID;
            
            if (localtreeId == "TimecodesTree" && (this._timecodesTree)) {
                if (publishedData.rowid) {
                    var nodeID = publishedData.rowid;
                    if (nodeID) {
                        var treeItem = this._getItemNodeByIdentity(nodeID);
                        this._timecodesTree._deleteItemNode(treeItem);
												
						var publishObj = {
						 	treeID: localtreeId
						 };
						//Publishing this event so that the grid will be updated with latest tree visible 
						//elements. Take care of the ROW_DELETED event of the grid it shouldn't be published
						//on removal of row from the grid 
	                	dojo.publish(EPMTreePubSub.ON_DELETE_NODE, [publishObj]);
                    }
                }
            }
			
		},

        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
        },
        
        _buildTimecodesTree: function(renameAllowed){
            if ((typeof(renameAllowed) != "undefined") && (renameAllowed != null)) {
				this.renameAllowed = renameAllowed;
			}
			this._createTree();
            this._timecodesTreeContent.appendChild(this._timecodesTree.domNode);
        },
        
        /**
        *_createTree function is called to create a tree widget
        */
        _createTree: function(){
            // Call the tree util to create the store
            
            this._timecodesTree = dijit.byId(AdminConstants.TIME_CODE_TREE_ID);
			if(this._timecodesTree){
				this._timecodesTree.destroy();
				delete this._timecodesTree;
			}
            if (!this._timecodesTree) {
                var props = TreeUtil.getTreeStore(AdminConstants.TIME_CODE_TREE_ID, Messages.TIME_CODE_TREE_ROOT_LABEL);
				props._containingWidget = this;
                this._timecodesTree = new EPMTree(props);
            }
            this._timecodesTree.startup();
            
            // Initialization Arguments : topLevelURI, linkType, ID
            this._timecodesTree.initialize("timecodes", "ChildTimeCode", "TimecodesTree", [], this.menuImage, this._projectID);
            this._timecodesTree.createMenu(this._ctxtItems);
            
            if (this._projectID === null) {
                var treeLabel = document.createElement("label");
                treeLabel.innerHTML = Messages.TIME_CODE_TREE_LABEL;
                this._timecodesTreeLable.appendChild(treeLabel);
            }
        },
        
        /*
         * Defining menu items for timecodes tree<br>
         * item.type: indicates whether its a popup menu or a menuItem. <br>
         * item.id: uniqueID of the item to be added. <br>
         * item.parentID: ID of the menu under which the item has to be created. <br>
         * item.addSeparator:Boolean value to depict whether to add a separator before adding item or not. <br>
         * item.label: label of the item. <br>
         * item.iconClass: the icon to be placed next to the label. <br>
         * item._onClick: the function to be performed when the menuItem is clicked. <br>
         */
        _initMenu: function(){
            this._ctxtItems = [{"type": EPMConstants.POPUP_TYPE,
                "id": EPMConstants.CREATE_ID,
                "parentID": null,
                "addSeparator": false,
                label: Messages.CREATE_LABEL,
                iconClass: IconClass.CREATE_ICON_CLASS
            }, {
                "type": EPMConstants.MENU_TYPE,
                "id": EPMConstants.CREATE_TIMECODE_ID,
                "parentID": EPMConstants.CREATE_ID,
                "addSeparator": false,
                label: Messages.TIMECODE_LABEL,
                iconClass: IconClass.TIMECODES_ICON_CLASS,
                _onClick: dojo.hitch(this, this._createTimecode)
            },
			this.renameItem ];
        },
        
        // Generate incremented number
        _incrementNumber: function(){
            this._lastUniqueRowID++;
            return this._lastUniqueRowID;
        },
        
        _getItemNodeByIdentity: function(identifier){
        
            var node = this._timecodesTree.getNodeByItemIdentity(identifier);
            if (node) {
                return node.item;
            }
            else {
                return null;
            }
        },
        
        //Create new root level timecode 
        _createRootTimecode: function(){
        
            var tempID = this._incrementNumber();
            var tempTCname = "Timecode";
            
            var item = {
                "href": "/timecode/" + tempID + "_unsaved",
                "name": tempTCname,
                "identifier": "/timecode/" + tempID + "_unsaved",
                "detailLevel": false,
                "iconClass": IconClass.TIMECODES_ICON_CLASS,
                "resourcetype": AdminConstants.TIME_CODE_TYPE
            }
            
            this._timecodesTree._addNode(null, item, 0);
            dojo.publish("CreateNode", [item]);
            this._timecodesTree._selectNodeById(item.identifier);
            
        },
        
        _createTimecode: function(e){
            var domNode = e.target;
            var menuWidget = dijit.getEnclosingWidget(domNode);
            var menuLable = menuWidget.label;
            
            var tempID = this._incrementNumber();
            
            this._ctxNode = this._timecodesTree.getNodeContext();
            var timecodename = "New_TC";
            
            if ((this._ctxNode.isExpandable == true) && (this._ctxNode.isExpanded == false)) {
                this._timecodesTree._expandNode(this._ctxNode);
            }
            
            var parentID;
            if (dojo.isArray(this._ctxNode.item)) {
                if (dojo.isArray(this._ctxNode.item[0].identifier)) {
                    parentID = this._ctxNode.item[0].identifier[0];
                }
                else {
                    parentID = this._ctxNode.item[0].identifier;
                }
            }
            else {
                if (dojo.isArray(this._ctxNode.item.identifier)) {
                    parentID = this._ctxNode.item.identifier[0];
                }
                else {
                    parentID = this._ctxNode.item.identifier;
                }
            }
            console.log(parentID);
            var newnode = {
                "name": timecodename,
                "description": timecodename + " description",
                "parentIdentifier": parentID,
                "code": "",
                "resourcetype": AdminConstants.TIME_CODE_TYPE
            };
            console.log(newnode);
            // Close the popup menu
            this._timecodesTree.closeMenu();
            //			if (this._isMenuOpen) {
            //				this._closeMenu();
            //			}
            dojo.publish("OnNewTimecode", [newnode]);
            
        },
        
        //Create child elements.
        _createTimeCodeElement: function(row){
        
            if (this._timecodesTree) {
                var tempID = this._incrementNumber();
                var tempProjname = "Timecode";
                
                var item = {
                    "href": "/timecode/" + tempID + "_unsaved",
                    "name": tempProjname,
                    "identifier": "/timecode/" + tempID + "_unsaved",
                    "parentIdentifier": row.parentIdentifier,
                    "detailLevel": false,
                    "iconClass": IconClass.TIMECODES_ICON_CLASS,
                    "resourcetype": AdminConstants.TIME_CODE_TYPE
                };
                
                
                var node = this._getItemNodeByIdentity(row.parentIdentifier);
                
                // Since new element is added, Expand the current Tree Node to show the
                // new added child
                //var nodeWidget = this._wbsTree.lastFocused;
                if ((this._ctxNode.isExpandable == true) && (this._ctxNode.isExpanded == false)) {
                    this._timecodesTree._expandNode(this._ctxNode);
                }
                this._timecodesTree._addNode(node, item, 0);
                
                this._timecodesTree._selectNodeById(item.identifier);
                dojo.publish("CreateNode");
            }
        },
        
        
        // Replace the temp node in the tree with actual data from the server
        //Gets acitvated when Save operation is done by the WBSGrid
        
        _replaceTimeCodeNode: function(publishedItem){
        
            var publishedData = null;
            
            if (dojo.isArray(publishedItem)) {
                publishedData = publishedItem[0];
            }
            else {
                publishedData = publishedItem;
            }
            
            var localtreeId = publishedData.uniqueIdentifier.treeID;
            
            if (localtreeId == "TimecodesTree" && (this._timecodesTree)) {
            
                var nodeID = publishedData.oldRowID;
                var oldItem = this._getItemNodeByIdentity(nodeID);
                
                var newNodeData = publishedData.newObject;
                
                //_deleteNode
                //delete the old node and add the new node to the same parent node
                var parentID = oldItem.parentIdentifier;
                this._timecodesTree._deleteItemNode(oldItem);
                
                //Create the new item
                var resourceIconClass = newNodeData.resourcetype.substring(3) + "Icon";
                
                
                var newItem = {
                
                    "href": newNodeData.identifier,
                    "name": newNodeData.title,
                    "identifier": newNodeData.identifier, // using relatedHref itself as Identifier
                    "parentIdentifier": parentID,
                    "detailLevel": false,
                    "iconClass": IconClass.TIMECODES_ICON_CLASS,
                    "resourcetype": AdminConstants.TIME_CODE_TYPE
                
                };
                
                
                if (parentID) {
                    //Get the parent Item
                    var parentNode = this._getItemNodeByIdentity(parentID);
                    //Add the new node
                    this._timecodesTree._addNode(parentNode, newItem, 0);
					
					// If the new is added due to collapse donot expand the parent node
					if (!publishedData.isParentCollapsed) {
						// Expand the parent node for the saved node
						this._timecodesTree._expandNodeUsingID(parentID);
					}
                }
                else {
                    //Add the new node
                    this._timecodesTree._addNode(null, newItem, 0);
                }
                
                
                // select the newly saved element
				this._timecodesTree._selectNodeById(newNodeData.identifier);
            }
        },

        _refreshTree: function(){
            this._timecodesTree.refresh();
        },//refresh
        destroy: function(){ //Destroy datastore, treedata and tree
            if (this._timecodesTree) {
                this._timecodesTree.destroy();
            }
            TreeUtil.destroyTreeStore();
            delete this._timecodesTree;
            delete TreeUtil;
            this._unsubscribeToAllEvents();
        }//destroy
    });
})();
