/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSBaselineGridModel");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.grid.model.AbstractGridModel");
    
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    
    
    dojo.require("com.ibm.team.process.web.client.projectarea");
    
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    
    var abstractGridModel = com.ibm.epm.web.core.grid.model.AbstractGridModel;
    
    var Constants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSBaselineGridModel", abstractGridModel, {
    
        newElementPrefix: "_unsaved",
        constructor: function(){
        },
        
        
        /*
         * Set the baseline1 id that has been selected in model
         */
        
        setTheIdForBaseline1: function(selectedValue){
            this._baselineId1 = selectedValue;
        },
		
		 /*
         * Set the baseline2 id that has been selected in model
         */
        setTheIdForBaseline2: function(selectedValue){
            this._baselineId2 = selectedValue;
        },
        
        /*
         * Returns the WBS id 
         * by taking relative href
         */
        _getWBSID: function(relativeHref){
            if (!relativeHref) {
                return "";
            }
            var index = relativeHref.lastIndexOf("/");
            if (index > 0) {
                return relativeHref.substring(index + 1, relativeHref.length);
            }
        },
        
		/*
		 * Return the empty row with the required elements in it 
		 */
        getEmptyRow: function(rowID, treeElementHandle){
        
		    var now = new Date();
            var row = {
                "id": rowID,
                "href": rowID,
                "wbsid": this._getWBSID(rowID),
                "parentId": null,
                "workItemType": null,
                "name": treeElementHandle.title,
                "resourcetype": treeElementHandle.resourcetype,
                "loadedelement": null,
                "proposedstart": now,
                "proposedfinish": now,
                "proposedduration": 1,
                "proposedeffort": 8,
                "baseline1proposedstart": " - ",
                "baseline1proposedfinish": " - ",
                "baseline1duration": " - ",
                "baseline1effort": " - ",
			    "baseline2proposedstart": " - ",
                "baseline2proposedfinish": " - ",
                "baseline2duration": " - ",
                "baseline2effort": " - "
            
            };
            if (treeElementHandle.parentIdentifier) {
                row.parentId = treeElementHandle.parentIdentifier;
            }
            if (treeElementHandle.workItemType) {
                row.workItemType = treeElementHandle.workItemType;
            }
            return row;
        },
        
		/*
		 * Reset the Baseline1 values if they are unchecked
		 */
		resetTheBaseline1Values: function(value)
		{
			this.resetTheBaseline1ValuesFlag = value;
									
		},
		
		/*
		 * Reset the Baseline2 values if they are unchecked
		 */
		resetTheBaseline2Values: function(value)
		{
			this.resetTheBaseline2ValuesFlag = value;
									
		},
		/*
		 * Reset the values to "-" in Model
		 */
		resetTheBaseline1inModel : function(row)
		{
		  row.baseline1proposedstart = " - ";
		  row.baseline1proposedfinish = " - ";
		  row.baseline1duration = " - ";
		  row.baseline1effort = " - ";	
		},
		
		/*
		 * Reset the values to "-" in Model
		 */
		resetTheBaseline2inModel : function(row)
		{
			row.baseline2proposedstart = " - ";
			row.baseline2proposedfinish = " - ";
			row.baseline2duration = " - ";
			row.baseline2effort = " - ";
			
		},
		
		/*
		 * Check for the baseline links
		 * Populate the row accordingly
		 */
        updateRowData: function(row, loadedElement, linkName){
			if (linkName == null) {
            
                if (loadedElement.plannedSchedule) {
                    row.proposedstart = dateUtil.parseDate(loadedElement.plannedSchedule.startDate);
                    row.proposedfinish = dateUtil.parseDate(loadedElement.plannedSchedule.finishDate);
                    row.proposedduration = loadedElement.plannedSchedule.duration;
                    row.proposedeffort = loadedElement.plannedSchedule.effort;
                }
                
                
                if (loadedElement.resourceType) {
                    row.resourcetype = loadedElement.resourceType;
                }
                else 
                    if (row.resourcetype) {
                        loadedElement.resourceType = row.resourcetype;
                    }
                this._updateName(row, loadedElement);
                
                row.loadedelement = [loadedElement];
                
                this._updateParent(row, loadedElement);
                
            }
			
			/*
			 * Populating in model for Basline 2
			 */
			if (linkName == Constants.BASELINE2) 
			 {
			 	
				if (loadedElement.baselinePlannedSchedule && this._getResourceIDFromURI(loadedElement.relativeHref) == this._baselineId2) 
				{
					row.baseline2proposedstart = dateUtil.parseDate(loadedElement.baselinePlannedSchedule.startDate);
			 		row.baseline2proposedfinish = dateUtil.parseDate(loadedElement.baselinePlannedSchedule.finishDate);
			 		row.baseline2duration = loadedElement.baselinePlannedSchedule.duration;
			 		row.baseline2effort = loadedElement.baselinePlannedSchedule.effort;
			 	}
			   else 
			   {
			   	  this.resetTheBaseline2inModel(row);
			 	}
			 }	 
			 
			 /*
			  * Populating in model for Baseline1
			  */
			 if (linkName == Constants.BASELINE) 
			 {
			 	if (loadedElement.baselinePlannedSchedule && this._getResourceIDFromURI(loadedElement.relativeHref) == this._baselineId1) 
				{
			 		row.baseline1proposedstart = dateUtil.parseDate(loadedElement.baselinePlannedSchedule.startDate);
			 		row.baseline1proposedfinish = dateUtil.parseDate(loadedElement.baselinePlannedSchedule.finishDate);
			 		row.baseline1duration = loadedElement.baselinePlannedSchedule.duration;
			 		row.baseline1effort = loadedElement.baselinePlannedSchedule.effort;
			 	}
			   else 
			   { 
			        this.resetTheBaseline1inModel(row);

			 	}
				
				
			 }	 
			 
			 if (this.resetTheBaseline1ValuesFlag == true) {
			 	 this.resetTheBaseline1inModel(row);
		 	
			 } 
			 
			 if (this.resetTheBaseline2ValuesFlag == true) {
			 	 this.resetTheBaseline2inModel(row);

			 } 
            
        },
		
		
        
        _updateName: function(row, loadedElement){
            if (loadedElement.Name) {// for proxy elements.
                row.name = loadedElement.Name;
            }
            else 
                if (loadedElement.name) {//for Programs.
                    row.name = loadedElement.name;
                }
                else 
                    if (loadedElement.summary) {// for work items.
                        row.name = loadedElement.summary;
                    }
        },
        
        
        /*
         * Updates the parent
         */
        _updateParent: function(row, loadedElement){
            var parentLink = getLink(loadedElement, Constants.PARENT);
            if (!parentLink) {
                parentLink = getLink(loadedElement, Constants.PROGRAM_LINK);
            }
            if (parentLink) {
                row.parentId = parentLink.relativeHref;
            }
        },
        
        
        /*
         * Get resource id from relative href
         */
        _getResourceIDFromURI: function(relativeHref){
            var index = relativeHref.lastIndexOf("/");
            if (index > 0) {
                return relativeHref.substring(index + 1, relativeHref.length);
            }
        },
		
		/*
		 * Get resource from relative href
		 */
		_getResourceFromURI: function(relativeHref){
            var index = relativeHref.lastIndexOf("/");
            if (index > 0) {
                return relativeHref.substring(0, index);
            }
        },
		
		        
		/*
		 * The get calls for baseline will be made only if it is checked
		 * Also do not make a call if it is a Program 
		 * 
		 */
        getRowGetUri_Link: function(loadedElement, links, linkName){
				
            if (loadedElement && loadedElement.relativeHref) {
			    var resource = this._getResourceFromURI( loadedElement.relativeHref);
					if (resource != Constants.BASELINE && resource != Constants.EPM_PROGRAM_TYPE && this._baselineId1 != null && this._baselineId1!="" ) {
						if (linkName == Constants.BASELINE) {
							
							return loadedElement.relativeHref + Constants.SLASH + Constants.BASELINE + Constants.SLASH + this._baselineId1;
						}
					}
					if (resource != Constants.BASELINE && resource != Constants.EPM_PROGRAM_TYPE && this._baselineId2 != null && this._baselineId2!="" ) {
						if (linkName == Constants.BASELINE2) {
							
							return loadedElement.relativeHref + Constants.SLASH + Constants.BASELINE + Constants.SLASH + this._baselineId2;
						}
					}
            }
        },
		
		/**
		 * To avoid the saving of the baselines this method is over written.
		 * For loading the baselines the view registers the baseline links 
		 * hence on replacing the new row the call will come for the saving the links.
		 * As we are not saving the baseline from this model we are returning the null.
		 * @param {Object} row
		 * @param {Object} linkName
		 */
		prepareSaveParams_link: function(row, linkName){
            return null;
        },
        
		/**
		 * To avoid the saving of the baselines this method is over written.
		 * For loading the baselines the view registers the baseline links 
		 * hence on replacing the new row the call will come for the saving the links.
		 * As we are not saving the baseline from this model we are returning the null.
		 * @param {Object} row
		 * @param {Object} linkName
		 * @param {Object} newElementID
		 */
        getRowPostURI_link: function(row, linkName, newElementID){
            return null;
        }        
       
    });
})();
