/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGanttGridModel");
    
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    dojo.require("com.ibm.epm.work.web.ui.internal.wbsGrids.AbstractWBSGridModel");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    //To get the Project Area
    dojo.require("com.ibm.team.process.web.client.projectarea");
	    
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    var abstractWBSModel = com.ibm.epm.work.web.ui.internal.wbsGrids.AbstractWBSGridModel;
    
    var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;

    dojo.declare("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGanttGridModel", abstractWBSModel, {
        constructor: function(){
        },
        
		/**
		 * getEmptyRow: returns the initialized row for schedule grid. <br>
		 * @param {Object} rowID: ID fow which the empty row is requested. <br>
		 * @param {Object} treeElementHandle: Contains some values passed by tree with which we can initialize the grid row eg, name, resourceType. <br>
		 */
        getEmptyRow: function(rowID, treeElementHandle){
			var row = this.inherited(arguments);
			row.percentC = "-";
			row.percentEC = "-";
			row.percentDC = "-";
			row.slack = -1;
            row.earlyFinishDate = "";
			row.earlyStartDate ="";
			row.lateFinishDate = "";
			row.lateStartDate = "";
			row.predecessors = "";
			row.successors = "";
			row.hasChildren = false;
			row.loadedcriticalpathelement = "";
			row.actualstart =  "-";
            row.actualfinish = "-";
            row.actualeffort = "-";
			row.hasAssignments = false;
			row.hasProfileAssignments = false;
			row.loadedplannedscheduleelement = "";
			row.containingProject = "";

            return row;
        },

		/**
		 * Called after a get call to the server, used to update the row with the latest server data. <br>
		 * @param {Object} row: row to be updated with the latest information. <br>
		 * @param {Object} loadedElement: Element values fetched from server. <br>
		 * @param {Object} linkName: nullable field, if not null, used to update the link values of the row <br>
		 */
        updateRowData: function(row, loadedElement, linkName){
			this.inherited(arguments);
			
            if (linkName == null) {
				if(loadedElement.containingProject)
				{
					row.containingProject = loadedElement.containingProject;
				}
                var predecessorsLinks = getLink(loadedElement, WBSConstants.DEPENDENCY_LINK);
				var successorsLinks = getLink(loadedElement, WBSConstants.BLOCKS_LINK);
                if (predecessorsLinks) {
					if (dojo.isArray(predecessorsLinks)) {
						row.predecessors = [predecessorsLinks[0].relativeHref];
						for (var i = 1; i < predecessorsLinks.length; i++) {
							row.predecessors.push(predecessorsLinks[i].relativeHref);
						}
					}else{
						row.predecessors = [predecessorsLinks.relativeHref];
					}
                    row.loadedelement = [loadedElement];
                }else{
					row.predecessors = "";
				}

                if (successorsLinks) {
					if (dojo.isArray(successorsLinks)) {
						row.successors = [successorsLinks[0].relativeHref];
						for (var i = 1; i < successorsLinks.length; i++) {
							row.successors.push(successorsLinks[i].relativeHref);
						}
					}else{
						row.successors = [successorsLinks.relativeHref];
					}
                    row.loadedelement = [loadedElement];
                }else{
					row.successors = "";
				}

				var childrenLink = getLink(loadedElement, WBSConstants.CHILDREN_LINK);
				//Program has a link name "Projects" for the child projects.
				var projectChildrenLink = getLink(loadedElement, WBSConstants.PROJECTS_LINK);
				// Projects can have only iterations
				var iterationLink = null;
				if (loadedElement.resourceType == WBSConstants.EPM_PROJECT_TYPE) {
					iterationLink = getLink(loadedElement, WBSConstants.ITERATIONS_LINK);
				}
				
				if(childrenLink || projectChildrenLink || iterationLink){
					row.hasChildren = true;
				}
				
				this._updatePlannedSchedule(row, loadedElement);
				this._updateProjectAssignments(row, loadedElement);
				// Actuals need to be updated when link is null as the
				// task assignment also have actuals but specific to the assignment
				this._updateActualSchedule(row, loadedElement);
            }

            if ((linkName == null) ||
			    (linkName == WBSConstants.TASK_ASSIGNMENT)) {			
				this._updateTaskAssignments(row, loadedElement);
			}

			// Dont display the rolled up data for Program
			// TODO: Fix the xsd at server side not to return/calculate these values
			if (loadedElement.resourceType == WBSConstants.EPM_PROGRAM_TYPE) {
				row.percentC = "-";
				row.percentEC = "-";
				row.percentDC = "-";
			}
			else {
				if (loadedElement.percentEC >= 0) 
					row.percentEC = loadedElement.percentEC;
				if (loadedElement.percentDC >= 0) 
					row.percentDC = loadedElement.percentDC;
				
				if (loadedElement.percentC >= 0) {
					if ((loadedElement.percentEC != loadedElement.percentC && loadedElement.percentDC != loadedElement.percentC) ||(loadedElement.percentC == 0)) {
						// Only set percentC if it is not equal to a calculated value or percentC is reduced to zero(16115).
						row.percentC = loadedElement.percentC;
					}
				}else if(loadedElement.percentC == -1) {
					row.percentC = "-";
				}
			}

            if (linkName == WBSConstants.CRITICAL_PATH) {
                row.slack = loadedElement.slack;
                row.earlyFinishDate = loadedElement.earlyFinishDate;
                row.earlyStartDate = loadedElement.earlyStartDate;
                row.lateFinishDate = loadedElement.lateFinishDate;
                row.lateStartDate = loadedElement.lateStartDate;
                row.loadedcriticalpathelement = [loadedElement];
            }
        },

		/**
		 * if the planned schedule is part of the loaded element then updates the row with schedule values
		 * @param {Object} row : model row to which the schedule values needs to be updated.
		 * @param {Object} loadedElement: the wbs element that is fetched from server.
		 * @return {boolean}: true if the scheule is updated else false.
		 */
		_updatePlannedSchedule: function(row, loadedElement){
			var scheduleUpdated = false;
            if (loadedElement.plannedSchedule) {
                row.plannedstart = dateUtil.parseDate(loadedElement.plannedSchedule.startDate);
                row.plannedfinish = dateUtil.parseDate(loadedElement.plannedSchedule.finishDate);
                row.plannedduration = loadedElement.plannedSchedule.duration;
                row.plannedeffort = loadedElement.plannedSchedule.effort;
				scheduleUpdated = true;
            }
			return scheduleUpdated;
		},

		/**
		 * if the actual schedule is part of the loaded element then updates the row with schedule values
		 * @param {Object} row : model row to which the schedule values needs to be updated.
		 * @param {Object} loadedElement: the wbs element that is fetched from server.
		 * @return {boolean}: true if the scheule is updated else false.
		 */
		_updateActualSchedule: function(row, loadedElement){
			var scheduleUpdated = false;
            if (loadedElement.actualSchedule) {
				var leastDate = dateUtil.parseDate(WBSConstants.LEAST_DATE);
				var actualStartDate = dateUtil.parseDate(loadedElement.actualSchedule.startDate);
				if ((actualStartDate) && (dateUtil.compareDates(actualStartDate, leastDate) > 0)) {
					row.actualstart = actualStartDate;
				} else {
					row.actualstart = "-";
				}

				var actualFinishDate = dateUtil.parseDate(loadedElement.actualSchedule.finishDate);
				if ((actualFinishDate) && (dateUtil.compareDates(actualFinishDate, leastDate) > 0)) {
					row.actualfinish = actualFinishDate;
				} else {
					row.actualfinish = "-";
				}
				row.actualeffort = loadedElement.actualSchedule.effort?loadedElement.actualSchedule.effort:"-";
				scheduleUpdated = true;
            }
			return scheduleUpdated;
		},

		/**
		 * if the task assignments is part of the loaded element then updates the flag in the row to indicate assignments
		 * @param {Object} row : model row to which the schedule values needs to be updated.
		 * @param {Object} loadedElement: the wbs element that is fetched from server.
		 */
		_updateTaskAssignments: function(row, loadedElement){
			if (loadedElement.TaskAssignment) {
				 //Obtain the current active assignment
				 for (var i = 0; i < loadedElement.TaskAssignment.length; i++) {
                 	if (loadedElement.TaskAssignment[i].title == "true") {
                 		row.hasAssignments = true;
                    	row.taskAssignmentUri = loadedElement.TaskAssignment[i].relativeHref;
                    	row.loadedtaskassignment = loadedElement.TaskAssignment;
						//Set the profile assignment to true if the active assignment is for the profile.
						if(loadedElement.TaskAssignment[i].relativeHref.indexOf("profile/") > -1){
							row.hasProfileAssignments = true;
						}
                    	break;
                    }
                 }
			}
		},

		/**
		 * if the project assignments is part of the loaded element then updates the flag in the row to indicate assignments
		 * @param {Object} row : model row to which the schedule values needs to be updated.
		 * @param {Object} loadedElement: the wbs element that is fetched from server.
		 */		
		_updateProjectAssignments: function(row, loadedElement){
			if (loadedElement.ProjectAssignment) {
				row.hasAssignments = true;
			}
		},
				      
		/**
		 * Method called to update the actual schedule in the save params <br>
		 * @param {Object} row: row from which the parameters to be saved are extracted <br>
		 * @param {Object} saveParams <br>
		 */	    
		prepareSaveParamsActualSchedule: function(row) {
			var saveParams = {
                actualSchedule: ""
            };
			
			var startDate = null;
			var finishDate = null;

			if (row.actualstart != "-") {
	        	startDate = dateUtil.formatDateForSave(row.actualstart);			
	            if (startDate != null) {
					saveParams.actualSchedule = WBSConstants.STARTDATE + WBSConstants.EQUALTO + startDate + WBSConstants.TIMESTAMP;
				}
			} else {
				saveParams.actualSchedule = WBSConstants.STARTDATE + WBSConstants.EQUALTO + WBSConstants.LEAST_DATE + WBSConstants.TIMESTAMP;
			}
			if (row.actualfinish != "-") {
				finishDate = dateUtil.formatDateForSave(row.actualfinish);
				if (finishDate != null) {
					saveParams.actualSchedule = saveParams.actualSchedule +
								WBSConstants.COMMA + WBSConstants.FINISHDATE + WBSConstants.EQUALTO + finishDate +	WBSConstants.TIMESTAMP;
				}
			} else {
				saveParams.actualSchedule = saveParams.actualSchedule +
								WBSConstants.COMMA + WBSConstants.FINISHDATE + WBSConstants.EQUALTO + WBSConstants.LEAST_DATE + WBSConstants.TIMESTAMP;
			}
			if (row.percentDC != "-") {
				saveParams.percentageDC = row.percentDC;
			}
			
			return saveParams;
		},

		/**
		 * returns the parameters to be saved for given link.<br>
		 * @param {Object} row: row for which the saveParams need to be fetched. <br>
		 * @param {Object} linkName: Link for which the params need to be fetched. <br>
		 */
		prepareSaveParams_link: function(row, linkName){
			if (linkName == WBSConstants.TASK_ASSIGNMENT) {
				var saveParams = this.prepareSaveParamsActualSchedule(row);
				return saveParams;
			}
			return null;
		},
		
		/**
		 * Method called on schedule grid save call. <br>
		 * @param {Object} row: row from which the parameters to be saved are extracted <br>
		 * returns: All the data which needs to be saved for that row(modified as well as unmodified). <br>
		 */
		prepareSaveParams: function(row){
			var saveParams = this.inherited(arguments);
			if (row.loadedelement && row.loadedelement[0]) {
				var prevPredecessors;
				var predecessorsLinks = getLink(row.loadedelement[0], WBSConstants.DEPENDENCY_LINK);
                if (predecessorsLinks) {
					if (dojo.isArray(predecessorsLinks)) {
						prevPredecessors = [predecessorsLinks[0].relativeHref];
						for (var i = 1; i < predecessorsLinks.length; i++) {
							prevPredecessors.push(predecessorsLinks[i].relativeHref);
						}
					}
					else {
						prevPredecessors= [predecessorsLinks.relativeHref];
					}
				}				
				var newPredecessors = row.predecessors;
				var isNewPredecessor = true;
				var isPrevPredecessorDeleted = true;
				saveParams.DependsOn = new Array();
				saveParams.removeDependency = new Array();
				
				if (prevPredecessors || newPredecessors) {
					
					//Compare every value of the new predlist with the prev list, if it was not there, add that.
					if (!prevPredecessors || (!prevPredecessors[0])) {
						saveParams.DependsOn = newPredecessors;
					}
					else 
						if(!newPredecessors || (!newPredecessors[0])){
							saveParams.removeDependency = prevPredecessors;
						}
						else {
							for (var i in newPredecessors) {
								isNewPredecessor = true;
								for (var j in prevPredecessors) {
									if (prevPredecessors[i] == newPredecessors[j]) {
										isNewPredecessor = false;
									}
								}
								if (isNewPredecessor == true) {
									saveParams.DependsOn.push(newPredecessors[i]);
								}
							}
							//Now compare every value of the prev predlist with the new list, if it is not there, delete that.
							for (var i in prevPredecessors) {
								isPrevPredecessorDeleted = true;
								for (var j in newPredecessors) {
									if (prevPredecessors[i] == newPredecessors[j]) {
										isPrevPredecessorDeleted = false;
									}
								}
								if (isPrevPredecessorDeleted == true) {
									saveParams.removeDependency.push(prevPredecessors[i]);
								}
							}

						}
				}
					saveParams.linkType = WBSConstants.DEPENDENCY_LINKTYPE;
					saveParams.BLOCKS = "";
			}
            return saveParams;
        },
		
		/**
		 * Over written function from the abstract model to compare the dependency and actual schedule.
		 * @param {Object} saveParams : the save params prepared using the model row.
		 * @param {Object} tempSaveParams : The save params prepared using the loaded element.
		 * @param {String} field : The Current field which is checked if it is modified.
		 * @param {Object} row : The Model Row that is mapped to grid and is updated from ui.
		 * @param {Object} loadedelement : The backup data returned by the server.
		 * @return {boolean} if field is dependency link then 
		 * 						if any new dependency is added or removed then true.
		 * 						else false.
		 * 					if field is one of the actual schedule params then 
		 * 						if any filed in the actual schedule is updated then true.
		 * 						else false.
		 * 					else returns the result from the inherited method.
		 */
		_isFieldModified: function(saveParams, tempSaveParams, field, row, loadedelement){
			if(field == WBSConstants.DEPENDENCY_LINK){
				var dependancyModified = this._isDependancyModified(saveParams, tempSaveParams);
				if(dependancyModified){
					console.log("dependency modified");
				}
				return dependancyModified;
			} else if(field == WBSConstants.ACTUAL_SCHEDULE_PARAM){
				var actualsModified = false;
				var row_orig = {	};
				if(this._updateActualSchedule(row_orig, loadedelement) == true){
					var actualScheduleParam = this._getModifiedActualScheduleParam(row, row_orig);
					if(actualScheduleParam && actualScheduleParam != ""){
						saveParams[WBSConstants.ACTUAL_SCHEDULE_PARAM] = actualScheduleParam;
						actualsModified = true;
					}
				}
				return actualsModified;
			}
			else{
				return this.inherited(arguments);
			}
		},

		/**
		 * compares the actual schedule fields from row and row_orig and adds only the modified fields
		 * @param {Object} row : The Model Row that is mapped to grid and is updated from ui.
		 * @param {Object} row_orig : The temporary row that contains only the schedule fields updated.
		 * @return {String} : the comma separated name value pairs of modified schedule parameters.
		 * 						Empty string in none of the schedule fields are modified.
		 */
		_getModifiedActualScheduleParam: function(row, row_orig){
			var modifiedActualScheduleParam = "";
			var startDate = dateUtil.formatDateForSave(row.actualstart);
			var startDate_org = dateUtil.formatDateForSave(row_orig.actualstart);
			if(this.compareFields(startDate, startDate_org, "actualstart")){
				modifiedActualScheduleParam =  modifiedActualScheduleParam + WBSConstants.STARTDATE + WBSConstants.EQUALTO + startDate + WBSConstants.TIMESTAMP;
			}
			var finishDate = dateUtil.formatDateForSave(row.actualfinish);
			var finishDate_org = dateUtil.formatDateForSave(row_orig.actualfinish);			
			if(this.compareFields(finishDate, finishDate_org, "actualfinish")){
				if(modifiedActualScheduleParam != ""){
					modifiedActualScheduleParam = modifiedActualScheduleParam + WBSConstants.COMMA;
				}
				modifiedActualScheduleParam =  modifiedActualScheduleParam +
                WBSConstants.FINISHDATE + WBSConstants.EQUALTO + finishDate + WBSConstants.TIMESTAMP;
			}
			return modifiedActualScheduleParam;
		},
				
		/**
		 * Checks if the dependency for the WBS element is modified.
		 * @param {Object} saveParams : the save params prepared using the model row.
		 * @param {Object} tempSaveParams : The save params prepared using the loaded element.
		 */
		_isDependancyModified: function(saveParams, tempSaveParams){
			var dependancyValue = saveParams[WBSConstants.DEPENDENCY_LINK];
			var dependancyValue_orig = tempSaveParams[WBSConstants.DEPENDENCY_LINK];
//			if(!dependancyValue_orig){
//				return true;
//			}
			
			if(dojo.isArray(dependancyValue)){
				if (dojo.isArray(dependancyValue_orig)) {
					return this._areArraysSame(dependancyValue, dependancyValue_orig);
				}else{
					return this._areArraysSame(dependancyValue, [dependancyValue_orig]);
				}
			} else {
				if (dojo.isArray(dependancyValue_orig)) {
					return this._areArraysSame(dependancyValue_orig, [dependancyValue]);
				}else{
					return this.compareFields(dependancyValue, dependancyValue_orig, WBSConstants.DEPENDENCY_LINKTYPE);
				}
			}
		},
		/**
		 * compares 2 arrays and returns true if both the arrays have same elements.
		 * else false.
		 * @param {Array} array1: first array to be compared
		 * @param {Array} array2: second array too be compares.
		 */
		_areArraysSame: function(array1, array2){
			//If 2 arrays have different length then the array is modified.
			if(array1.length != array2.length){
				return true;
			}
			
			for(var i in array1){
				var field1 = array1[i];
				var fieldPresent = false;
				for (var j in array2){
					var field2 = array2[j];
					if(field1 == field2){
						fieldPresent = true;
						break;
					}
				}
				//If a field is not preset in array then array is modified.
				if(fieldPresent == false){
					return true;
				}
			}
			return false;
		},

		/**
		 * returns the latest server side data preserved against the gridRow for the requested link name <br>
		 * @param {Object} gridRow: Row for which the values are requested. <br> 
		 * @param {Object} linkName: Corresponding link name for which we need to pass the latest client side information. <br>
		 */
        getLoadedelement_link: function(gridRow, linkName){
            if (linkName == WBSConstants.CRITICAL_PATH) {
                return gridRow.loadedcriticalpathelement[0];
            } else if (linkName == WBSConstants.TASK_ASSIGNMENT) {
				if ((gridRow.hasAssignments) &&
					 (gridRow.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_DURATION) &&
					 gridRow.loadedtaskassignment) {
					return gridRow.loadedtaskassignment[0];
				}
			}
			return null;
        },
		
		getRowPostURI_link: function(row, linkName, newElementID){
            if (linkName == WBSConstants.TASK_ASSIGNMENT) {
				if ((row.hasAssignments) &&
					(row.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_DURATION)) {
					var rowURIObj = {
		                rowURI: row.taskAssignmentUri
            		};
					return rowURIObj;
				}
			}
			
			return null;	
        }
    });
})();
