/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridView");
    
    dojo.require("dijit.Toolbar");
    dojo.require("dijit.form.Button");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGrid");
    dojo.require("com.ibm.epm.web.core.ui.internal.DojoData");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.TreeBasedGridController1");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
    dojo.require("com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants");
    dojo.require("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridModel");
    dojo.require("com.ibm.epm.web.core.client.internal.WorkItemClient");
    dojo.require("dojo._base.event");
    // Editors
    dojo.require("com.ibm.epm.work.web.ui.internal.ProjectCalendarEditor");
    
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    dojo.require("dojox.grid.compat._data.dijitEditors");
	
	// Needed for dojo.keys
    dojo.require("dojo._base.event");
	
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    
    // EPM Core Constants
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
	
	dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
	
	dojo.require("com.ibm.epm.work.web.ui.internal.util.WBSUtil");
    
    var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
	
	var ControllersFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    
    var wbsElementURI = "/wbsElement/";
	var mandatoryGetErrorCodes = [
		"CRIPM3687"
	];
		
    var scheduleControllerID = com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants.ScheduleGridController;
    
    var controllerID = com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants.WBSGeneralGridController;
    var TreeBasedGridController = com.ibm.epm.web.core.grid.controller.TreeBasedGridController1;
    var getController = com.ibm.epm.web.core.grid.controller.TreeBasedGridController.getTreeBasedGridController;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var EPMWorkItemClient = com.ibm.epm.web.core.client.internal.WorkItemClient;
    
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
	var WBSUtil = com.ibm.epm.work.web.ui.internal.util.WBSUtil;
    
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridView", abstractView, //	[dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/wbsGrids/templates/WBSGeneralGrid.html"),
        _isBuzy: false,
        newDlg: null,
        
        constructor: function(treeID){
            //			this.inherited(arguments);
        },
        
        _cleanup: function(){
            if (this._wbsGeneralGrid) {
                this.wbsGenGridAttachPoint.removeChild(this._wbsGeneralGrid.domNode);
                this._wbsGeneralGrid.destroy();
            }
        },
        
        /**
         * postCreate: Method called after a widget's dom has been setup, all initialization and event subscriptions are done here <br>
         * @param none
         */
        postCreate: function(){
            this._subscriptions = [
				// Tree event on rename of node
				dojo.subscribe(EPMTreePubSub.ON_RENAME_NODE, this, "_renameNode"),
                dojo.subscribe(EPMTreePubSub.ON_PARENT_CHILD_CHANGE_SYNC_GRID, this, "_onParentChildChangeSyncGrid")
            ];
            this._createToolbar();
            this.buildWBSGeneralGrid();
            
            if (this._controller) {
                if(!this._controller.workItemTypes) {
                    this._getWorkItemTypes(); 
                }
            }
            
            // Initialize the keyboard save event handler for saves from grid.
            // The keyboard save event is fired twice.
            // This boolean allows only one of them to persist the grid. 
            this.eventHandled = false;
            
            // Initialize the view to listen to the keyboard/special events
            this.subscribeToKeyboardEvents();
            
            // Objects to indicate if there is a pending request from the tree to sync the grid after
            // a drag-n-drop operation
            this.pendingSync = false;
            this.pendingSyncPublishObj = null;
        },
        
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
        },
        
        /**
         * Destroy function calls destroy of AbstractGridView and unsubscribe to other events.
         */
        destroy: function(){
            this.inherited(arguments);
            this._unsubscribeToAllEvents();
        },
        
        /**
         * _renameNode: Event handler when the tree publishes the EPMTreePubSub.ON_RENAME_NODE event.<br>
         *
         * @param {Object} publishObj - The published object contains the tree id, the node.item, old and new values of the name field.
         */
        _renameNode: function(publishObj){
            if ((publishObj) && (publishObj.treeID == WBSConstants.WBS_TREE_ID)) {
                var id = publishObj.item.identifier[0];
                var item = this._generalGrid.model.store._getItemByIdentity(id);
                if (item) {
                    this._generalGrid.model.store._setValueOrValues(item, WBSConstants.NAME_FIELD_NAME, publishObj.newValue, true);
                }
            }
        },
        
        buildWBSGeneralGrid: function(){
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridModel();
            }
            var wbsGeneralGridController = controllerUtil.getTreeBasedGridController(controllerID);
			wbsGeneralGridController.markDirtyOnHidden = true;
            this.buildGrid(this, this._gridModel, wbsGeneralGridController, [WBSConstants.CONSTRAINT]);
        },
        
        createView: function(){
        
            this.inherited('createView', arguments);
            
            this._generalGrid.autoHeight = true;
            this._generalGrid.update();
            dojo.connect(this._generalGrid, "onApplyCellEdit", this, this.onApplyCellEdit);
            dojo.connect(this._generalGrid, "onStartEdit", this, this.onStartEdit);
            dojo.connect(this._generalGrid, "onCellClick", this, this.onCellClick);
        },
        
        /**
         * over written to set the flag for making
         * the save calls sequential
         */
        persistGrid: function(){
            if (this._generalGrid) {
                this._generalGrid.edit.apply();
                if (this._controller && this._controller.persistGrid) {
                    this._controller.persistGrid(true);
                }
            }
        },
        
        /**
         * getGridTitle: Returns the title of the grid.
         * Incoming:
         *	none
         * Outgoing:
         *  none
         */
        getGridTitle: function(){
            return WorkMessages.WBS_OVERVIEW_GRID_ID;
        },
        
         /**
         * getGridInternalID: Returns the ID of the grid.
         * Incoming:
         *	none
         * Outgoing:
         *  none
         */
        getGridInternalID: function(){
            return WBSConstants.WBS_OVERVIEW_GRID_ID;
        },

		/*
         * Override the function defined in EPMGrid.
         */
        isCellEditable: function(event){
			var editable = this._isCellEditable(event);
			if ((editable == false) || (!event.cell.editor)) {
				// if the cell is not editable, reset the cellEditing flag
				this.cellEditing = false;
			}
					
			return editable;
        },
        
		/*
         * Internal method to do the necessary validations and return true if the cell is not editable, false if it editable
         */	
        _isCellEditable: function(event){
            var rowIndex = parseInt(event.rowIndex, 10);
            if (isNaN(rowIndex)) {
                return true;
            }
            var row = this._getTreeBasedController().getRowByIndex(rowIndex);
			//Constraint date and type are editable only for tasks.
			var cellField = event.cell.field;
			
			// The work item type cell is editable if its for an unsaved work item type or iteration type
			if (WBSUtil.isfieldEditableOverviewGrid(row, cellField, row.id.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1)){
                return true;
            }
            else {
                return false;
            }
        },
        
        onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
			this.cellEditing = false;
            var row = this._generalGrid.model.getRow(inRowIndex);
            if (row) {
                var item = this._generalGrid.model.store._getItemByIdentity(row.id);
                if (item) {
                    var cellFieldName = this.curCellField;
                    switch (cellFieldName) {
                        case "constraintDate":
                            this._handleChangeInConstraintDate(row, item, inValue);
                            break;
                        case "constraintType":
                            if (inValue == WBSConstants.AS_SOON_AS_POSSIBLE) {
                                row.constraintDate = " - ";
                            }
                            else { 
                                if (inValue == WBSConstants.FINISH_NO_LATER_THAN) {
                                    // Choose the finish date of the WBS item.
                                    // In the absence of that being available it 
                                    // should be the current date. The latter possibility 
                                    // should never arrive but is just the mechanism to 
                                    // prevent having an FNLT constraint without a date.
                                    if (row.proposedfinish) {
                                        row.constraintDate = row.proposedfinish;
                                    } else {
                                        row.constraintDate = new Date();
                                    }    
                                }
                            }
                            break;
                        case "wiTypeName" :
                           if (inValue != this.oldWiTypeName) {
                               /*var workItemType = this._getWITypeID(inValue, row.parentId);
							   if(!workItemType){
							   		workItemType = inValue;
							   }
							   var oldWorkItemType = this._getWITypeID(this.oldWiTypeName, row.parentId);*/
							   var workItemType = inValue;
                               var oldWorkItemType = this.oldWiTypeName;
                               
                               if (workItemType == WBSConstants.ITERATION_TYPE ||
							   		workItemType == WBSConstants.EPM_ITERATION_TYPE_NAME ) {
									// Setting it to "iteration" if it is "Iteration";
									workItemType = WBSConstants.ITERATION_TYPE;
                                   this._generalGrid.model.store._setValueOrValues(item, "resourcetype", workItemType, true);
                               } else {
                                   this._generalGrid.model.store._setValueOrValues(item, "resourcetype", WBSConstants.WORKITEM_TYPE, true);
								   var wiTypeLabel = this._getWITypeLabel(workItemType, row.parentId);
								   this._generalGrid.model.store._setValueOrValues(item, "wiTypeName", wiTypeLabel, true);
                                   this._generalGrid.model.store._setValueOrValues(item, "workItemType", workItemType, true);
                               }
							   
							   //Change the constraint type if the new type is not task
							   if(workItemType !== WBSConstants.EPM_TASK_TYPE){
							   		this._generalGrid.model.store.setValue(item, "constraintType", WBSConstants.DEFAULT_CONSTRAINT_VALUE);
									this._generalGrid.model.store.setValue(item, "constraintDate", WBSConstants.DEFAULT_CONSTRAINT_VALUE);
							   }else{
							   		//set the constraint type to default value.
							   		this._generalGrid.model.store.setValue(item, "constraintType", WBSConstants.AS_SOON_AS_POSSIBLE);
							   }
                               var publishObj = {
                                    workItemType: workItemType,
                                    rowIndex: inRowIndex,
                                    gridID: this._generalGrid._gridID,
                                    oldWorkItemType: oldWorkItemType
                               };
                    
                               // Publish an event for the tree to change the icon of the object
                               dojo.publish(EPMTreePubSub.ON_WI_TYPE_CHANGE, [publishObj]); 
                           }
                           break;        
                    }
                }
            }
        },
        
        _handleChangeInConstraintDate: function(row, item, newValue){
            if (newValue == null) {
				/* A valid date must be sent to the server.  Reset to the 
				 * previously valid date if the user entered nothing.
				 */
				if (this.oldConstraintDate != null) {
					row.constraintDate = this.oldConstraintDate;
				} else {
					/* oldConstraintDate should never equal null, this is a 
					 * last resort to prevent the server from receiving a 
					 * null date.
					 */
					row.constraintDate = new Date();
				}
            }
            else {
                if (row.loadedelement) {
                    var finishDate = row.loadedelement[0].proposedSchedule.finishDate;
                    if (finishDate) {
                        finishDate = dateUtil.parseDate(finishDate);
                        if (!isNaN(finishDate)) {
                            if (newValue < finishDate) {
                                var buttonlist = [{
                                    label: WorkMessages.OK
                                }];
                                
                                var dialogConfirmation = dojo.string.substitute(WorkMessages.INVALID_CONSTRAINT_DATE_ALERT, [WorkMessages.CONSTRAINT_DATE, WorkMessages.FINISH_DATE]);
								var dialogHeader = dojo.string.substitute(WorkMessages.INVALID_CONSTRAINT_DATE_DIALOG, [WorkMessages.CONSTRAINT_DATE, WorkMessages.FINISH_DATE]);
                                
                                var labelList = [{
                                    label: dialogHeader
                                }, {
                                    label: dialogConfirmation 
                                }];
                                
                                // The naming convention is mandatory :- for list of buttons, use buttonlist 
                                // for list of messages:- labelList
                                // buttonlist, labellist are defined in the EPMConfirmDialog
                                var titleMessage = dojo.string.substitute(WorkMessages.INVALID_CONSTRAINT_DATE_HEADER, [WorkMessages.CONSTRAINT_DATE]);
								// Canceling the Cell edit as it is having the invalid constraint Date
                                this._generalGrid.edit.cancel();  
                                this.scheduleSaveDialog = new EPMConfirmDialog({
                                    labelList: labelList,
                                    title: titleMessage,
                                    buttonlist: buttonlist,
									onClose:dojo.hitch(this, this._closedButtonNodeHandler, item)
                                });
                            }
                        }
                    }
                }
            }
        },
		
		/**
		 * _closedButtonNodeHandler: reset the constraint date.
		 * @param {Object} item: store item which needs to be updated.
		 */
		_closedButtonNodeHandler: function(item){
			this._generalGrid.model.store.setValue(item, "constraintDate", this.oldConstraintDate);
		},
        
        /**
         * _getTreeBasedController: To create/return the controller.
         * 							Added extra check for TreeBasedGridController._gridData. In case not found, recreating the controller again.
         * 							This extra check is because some times, the controller is partailly available without gridData.
         * Incoming: none.
         * Outgoing: TreeBasedGridController.
         */
        _getTreeBasedController: function(){
            if (!this.TreeBasedGridController) {
                this.TreeBasedGridController = controllerUtil.getTreeBasedGridController(controllerID);
            }
            if ((this.TreeBasedGridController) && (!this.TreeBasedGridController._gridData)) {
                delete this.TreeBasedGridController;
                this.TreeBasedGridController = controllerUtil.getTreeBasedGridController(controllerID);
            }
            return this.TreeBasedGridController;
        },
        
        getAttachpoint: function(){
            return this.wbsGenGridAttachPoint;
        },

        /**
         *  handleError : Inherited method which is called to handle any error messages
         */        
        handleError: function(error, row, rowID){
			if(rowID && this.mandateError(error)){
				this._markRowAsInvalid(error, rowID);
				 this._displayError(error);
				return;
			}
            this.inherited("handleError", arguments);
            this._displayError(error);
			// since the save failed the save button is enabled		
			this._viewlet.toggleSaveButton(false);
        },
		
		mandateError: function(error){
			var mandatoryError = false;
			for(var i in mandatoryGetErrorCodes){
				if(error.errorCode.indexOf(mandatoryGetErrorCodes[i]) != -1){
					mandatoryError = true;
				}
			}
			return mandatoryError;
		},
		
		formateWIType: function(inText, rowIndex){
			return inText;
		},
		/**
		 * Format the Actual schedule And Percentage DC
		 * @param {Object} inData
		 * @param {Object} inRowIndex
		 */
		 _columnFormatter: function(inData, inRowIndex){
		 	
            var row = null;
            if (this.grid && this.grid.model) {
                row = this.grid.model.getRow(inRowIndex);
            }
			if (row) {
				
				if (this.field == "constraintDate") {
					inData = dateUtil.formatDate(inData);
				}
				if(!WBSUtil.isfieldEditableOverviewGrid(row, this.field, row.id.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1)){
					var color = "color : #999999"
					this.customStyles.push(color);
				}
			}
			
			return inData;
		 },
		
        /**
         *  getGridLayout : Method defines the grid layout, these attributes are static.<br>
         */
        getGridLayout: function(){
            var gridLayout = [{
                noscroll: false,
                //                defaultCell: {styles: 'text-align: left;'},
                rows: [[{
                    name: WorkMessages.WBS_OVERVIEW_TYPE, //9
                    width: 10,
                    field: "wiTypeName",
					formatter: this._columnFormatter,
                    editor: dojox.grid.editors.Select,
                    options: this._populateWorkItemDropDown(true),
                    values: this._populateWorkItemDropDown(true)
                }, {
                    name: WorkMessages.WBS_OVERVIEW_STATE, //3
                    width: 4,
                    field: "state"
                    //                    editor: dojox.grid.editors.Input
                }, {
                    name: WorkMessages.WBS_OVERVIEW_ASSIGNMENTTYPE, //4
                    width: 10,
                    field: "assignmenttype"
                }, {
                    name: WorkMessages.WBS_OVERVIEW_CALENDAR, //5
                    width: 6,
                    field: "calendarName",
                    editor: com.ibm.epm.work.web.ui.internal.ProjectCalendarEditor
                }, {
                    name: WorkMessages.WBS_OVERVIEW_CONSTRAINTTYPE, //6
                    width: 10,
                    field: "constraintType",
                    editor: dojox.grid.editors.Select,
                    //editorClass: "dijit.form.ComboBox",
                    options: [WBSConstants.AS_SOON_AS_POSSIBLE, WBSConstants.FINISH_NO_LATER_THAN],
                    values: [WBSConstants.AS_SOON_AS_POSSIBLE, WBSConstants.FINISH_NO_LATER_THAN]
                }, {
                    name: WorkMessages.WBS_OVERVIEW_CONSTRAINTDATE, //7
                    width: 10,
                    field: "constraintDate",
                    formatter: this._columnFormatter,
                    editor: dojox.grid.editors.DateTextBox,
                    editorClass: "dijit.form.DateTextBox",
                    constraint: {
                        formatLength: 'short',
                        selector: "date"
                    }
                }, {
                    name: WorkMessages.WBS_OVERVIEW_WBSID, //1
                    width: 5,
                    field: "wbsid"
                }]]
            }];
            
            return gridLayout;
        },
        
        onStartEdit: function(inCell, inRowIndex){
			this.cellEditing = true;
            var row = this._generalGrid.model.getRow(inRowIndex);
            if (row) {
                this.curCellField = inCell.field;
                
                if (inCell.field === "constraintDate") {
                    this.oldConstraintDate = row.constraintDate;
                }
                
                if (inCell.field === "wiTypeName") {
                    this.oldWiTypeName = row.wiTypeName;
                    
                    // Remove the Iteration type as a selection option if the 
                    // parent is a work item type. Also clean a possibly dirty
                    // editor that has the iteration type removed from it.
                    // NOTE: The latter is possible as the editor is shared across
                    // all the grid cells. The editor will be dirty in the following use case:
                    // 1. Create a child WBS item of a work item type item in the tree (say Story).
                    // 2. The code in the following block's condition will be true (as the parent is a story).
                    // 3. The new option set created will *not* contain the iteration type.
                    // 4. A new editor will be created that does not contain the iteration type.
                    // 5. Now Create a new child WBS item of a project.
                    // 6. The shared editor created in step 4 now needs to be changed to contain the
                    //    iteration type.
                    var parentRowIndex = this._controller.getRowIndex(row.parentId);
                    var parentRow = this._controller.getRowByIndex(parentRowIndex);
                    if (parentRow) {                       
                        // Get a default one which contains the iteration type
                        var addIteration = true;
                        // Check if the parent's type is a work item type
                        // Only projects and iteration parent types can contain items
                        // that are of iteration type. Any other type should strip it
                        // it out
                        if (parentRow.resourcetype == WBSConstants.WORKITEM_TYPE) {
                            // Get a list of options without the iteration type
							addIteration = false;
                        }
						
                        var options = this._getWorkItemTypeDropDownOptions(row, parentRow, addIteration);
						if (options) {
							var optionIDs = [];
							var optionLabels = [];
							if(addIteration){
								optionIDs.push(WBSConstants.EPM_ITERATION_TYPE_NAME);
								optionLabels.push(WBSConstants.EPM_ITERATION_TYPE_NAME);
							}
							for(var i in options){
								optionIDs.push(options[i].id)
								optionLabels.push(options[i].label);
							}
                            // Reset the editor
                            inCell.options = optionLabels;
                            inCell.values = optionIDs;
                            var newEditor = new dojox.grid.editors.Select(inCell);
                            inCell.editor = newEditor;
                        }                        
                    }    
                }
            }
        },
		
		_getWorkItemTypeDropDownOptions: function(row, parentRow, addIteration){
			var options = [];
			
			var treeNode = {
				resourcetype: parentRow.resourcetype,
				identifier: parentRow.id,
				parentIdentifier: parentRow.parentId
			};
			if(this._controller._tree && this._controller._tree._getContainingProjectID){

				var containingProjectID = this._controller._tree._getContainingProjectID(treeNode);
				if(containingProjectID == null){
					return options;
				}
				var allWIConfig = ControllersFactory.getWIConfigHandler().getAllWIConfig(containingProjectID);
				if(allWIConfig == EPMConstants.LOADING_CONFIG){
					return options;
				}else{
					for(var i in allWIConfig){
						var wiConfig = allWIConfig[i];
						if(wiConfig){
							if (wiConfig.id in this.objectConverter([	WBSConstants.EPM_RISK_TYPE, 
																		WBSConstants.EPM_DOCUMENT_TYPE, 
																		WBSConstants.EPM_PROJECT_TYPE])) {
								continue;
							}
							options.push(wiConfig);
						}
					}
				}
			}
			return options;
		},
        
        _createToolbar: function(){
        },
        
        saveInProgress: function() {
            if (this._viewlet) {
                this._viewlet.saveInProgress();
            }   
        },
        
        saveComplete: function() {
            
            if (this.pendingSync) {
                if (this.pendingSyncPublishObj) {
                    this._requestGridToSync(this.pendingSyncPublishObj);
                    this.pendingSyncPublishObj = null; 
                }
                this.pendingSync = false;
            }
            
            if (this._viewlet) {
            	this._viewlet.saveComplete();
            } 	
        },
        
        /*
         * Method used to handle keyboard events from the tree
         * Currently, this method is used to respond to only the 
         * TAB key (to move from the tree to the grid) and the save
         * keyboard action of CTRL+S.
         */
        _onKeyDownFromTree: function(data) {
            if(data.nodeWidgetItem) {
                var rowIndex = this._validateAndGetRow(data.nodeWidgetItem);
                // Set the focus on the correct row
                if (this._generalGrid) {
                    if (rowIndex >= 0) {
                        var keys = dojo.keys;
                        // Check if the tab key was pressed
                        if (data.keySet.keyCode == keys.TAB) {
                            // Set the focus to the first cell on the selected row
                            this._generalGrid.selection.select(rowIndex);
                            this._generalGrid.focus.setFocusIndex(rowIndex, 0);
                            return;
                        }else if ((data.keySet.shiftKey) && (!data.keySet.altKey) && 
						   (!data.keySet.ctrlKey) && ((data.keySet.keyCode === 70)) &&
						   (this._viewlet)) {
						   	
							// Refresh viewlet: Shift+f/Shift+F
							this._viewlet.confirmAndRefreshViewlet();
						}	 
						else {
                            // Check if CTRL+S was pressed to save the grid
                            if (data.keySet.ctrlKey && data.keySet.keyCode === 83) {
                                
                                // Check if the grid is already being saved
                                // Ignore this call to save if that was the case
                                if (this._viewlet) {
                                    if (!this._viewlet.isSaveInProgress()) {
                                        // Save the grid
                                        this._viewlet.persistViewlet();
                                    }    
                                }    
                                
                                // This is to override the browser's standard
                                // save page dialog from popping up.
                                data.event.preventDefault();
                            }
                        }    
                    }        
                }
            }
        },
        
        /*
         * Method used to handle *some* keyboard events from the whole document.
         * Currently, this method is used to respond to
         *  - the save keyboard action of CTRL+S,
         *  - a new node action on Insert key,
         *  - refresh of viewlet on Shift+F/Shift+f
         */
        _onKeyDownFromGrid: function(event) {
     		if (this._controller) {
				// Check is the view is active
				if (!this._controller.isViewActive()) {
					return;
				}
				// Process the short cut key events only if the tree/grid are not in edit state
				var isEditing =  this._controller.isEditInProgress();
				if (isEditing) {
					return;
				}
			}
           
            // Check if CTRL+S was pressed in the grid
            if (event.ctrlKeyPressed && event.keyCode === 83) {
                
               // This is to override the browser's standard
               // save page dialog from popping up.
               event.preventDefault();
                
               if (this.eventHandled === false) {
                   this.eventHandled = true;
                   
                   // Check if the grid is already being saved
                   // Ignore this call to save if that was the case
                   if (this._viewlet) {
                      if (!this._viewlet.isSaveInProgress()) {
                         // Save the grid
                         this._viewlet.persistViewlet();
                      }    
                   }
               }
               
               // Reset the handler boolean
               this.eventHandled = false;
            } else {
				// Check if Insert was pressed in the grid
                if ((!event.altKey) && (!event.ctrlKey) && (!event.shiftKey)) {
					if (event.keyCode === dojo.keys.INSERT) {
	                    // This is to override the browser's default (if any)
	                    event.preventDefault();
	                    
	                    var keySetObj = {
	                        altKey: event.altKeyPressed,
	                        keyCode: event.keyCode
	                    };
	                    
	                    var data = {
	                        keySet: keySetObj,
	                        event: event
	                    };
	                    
	                    dojo.publish(EPMTreePubSub.ON_REQUEST_NEW_NODE, [data]);
					}
				} else if ((event.shiftKey) && (!event.altKey) && 
						   (!event.ctrlKey) && ((event.keyCode === 70)) &&
						   (this._viewlet)) {
						   	
					// Refresh viewlet: Shift+F/Shift+F
					this._viewlet.confirmAndRefreshViewlet();
				}					
            }
        },
        
        /*
         * Populates the drop down list with work item types
         * as defined on the server. These types are stored
         * in the controller object by the tree in
         * WBSTreeWidget._buildWorkItemMenu().
         * 
         * The types have the following attributes:
         * id: ID of the type
         * label: Display label
         * iconUrl: URL of the associated icon
         */
        _populateWorkItemDropDown: function(needIterationType) {
            var typeLabelArray = new Array();
            
            if (needIterationType) {
                typeLabelArray.push(WBSConstants.EPM_ITERATION_TYPE_NAME);
            }
            
            if (this._controller) {
                if (this._controller.workItemTypes) {
                    
                    // Iterate over all the workitem types
                    for (var i in this._controller.workItemTypes) {
                        var workItemType = this._controller.workItemTypes[i];
                        if (workItemType) {
                            
                            // Get the label reference and store
                            // in the array to be returned
                            var label = workItemType.label;
                            if (label) {
                                typeLabelArray.push(label);
                            }    
                        }
                    }
                } else {
                    var message = dojo.string.substitute(WorkMessages.ERROR_FINDING_WBS_TYPES, ["_populateWorkItemDropDown"]);
                    var errorMessage = {
                        errorMessage: message
                    };
                    this._displayError(errorMessage);
                }
            }
            
            return typeLabelArray;
        },
        
        /*
         * Returns the display name associated with a work item type
         * This method is used to display a human friendly name of
         * a stored work item type when retrieved from the server.
         */
        _getWorkItemDisplayName: function (workItemType, parentID) {
            var displayName = null;
            
            if (workItemType === WBSConstants.EPM_PROGRAM_TYPE) {
				
                displayName = WBSConstants.EPM_PROGRAM_TYPE_NAME;
				
            }else if (workItemType === WBSConstants.EPM_PROJECT_TYPE) {
				
                displayName = WBSConstants.EPM_PROJECT_TYPE_NAME;
				
            }else if (workItemType === WBSConstants.ITERATION_TYPE || 
                workItemType === WBSConstants.EPM_ITERATION_TYPE_NAME) {
					
                displayName = WBSConstants.EPM_ITERATION_TYPE_NAME;
				
            }else if (workItemType === WBSConstants.EPM_RISK_TYPE) {
				
                displayName = WBSConstants.EPM_RISK_TYPE_NAME;
				
            }else {
				
				displayName = this._getWITypeLabel(workItemType, parentID);
				
            }
            return displayName;
        },
		
		_getWITypeLabel: function(workItemTypeID, parentID){
			if (!this._controller) {
				return null;
			}
			if(!parentID){
				return null;
			}
			var parentRowIndex = this._controller.getRowIndex(parentID);
			var parentRow = this._controller.getRowByIndex(parentRowIndex);
			var wiConfigs = this._getWorkItemTypeDropDownOptions(null, parentRow, false);
			if(!wiConfigs){
				return;
			}
			for(var i in wiConfigs){
				if(wiConfigs[i].id == workItemTypeID){
					return wiConfigs[i].label;
				}
			}
			
		},
		
		_getWITypeID: function(workItemTypeLabel, parentID){
			if (!this._controller) {
				return null;
			}
			if(!parentID){
				return null;
			}
			var parentRowIndex = this._controller.getRowIndex(parentID);
			var parentRow = this._controller.getRowByIndex(parentRowIndex);
			var wiConfigs = this._getWorkItemTypeDropDownOptions(null, parentRow, false);
			if(!wiConfigs){
				return;
			}
			for(var i in wiConfigs){
				if(wiConfigs[i].label == workItemTypeLabel){
					return wiConfigs[i].id;
				}
			}
			return null;
			
		},
        
        /*
         * Adds the friendly work item type name to the object
         * fetched from the server. This method is called before
         * the controller loads the element into the grid.
         */
        massageData: function(loadedElement) {
            var workItemType = loadedElement.workItemType;
            
            if (!workItemType) {
                workItemType = loadedElement.resourcetype;
            }
            loadedElement.wiTypeName = this._getWorkItemDisplayName(workItemType, loadedElement.containingProject);
        },
        
        /*
         * Adds the friendly work item type name to an empty
         * row object.
         */
        massageEmptyRow: function(collectionItemHandle, emptyRow) {
            if (collectionItemHandle.workItemType) {
                var workItemType = collectionItemHandle.workItemType;                
                if (dojo.isArray(workItemType)) {
                    workItemType = workItemType[0];
                }
                
                if (!workItemType) {
                    workItemType = collectionItemHandle.resourcetype;
                    if (dojo.isArray(workItemType)) {
                        workItemType = workItemType[0];
                    }
                }
                emptyRow.wiTypeName = this._getWorkItemDisplayName(workItemType, emptyRow.parentId);
            } else {
                emptyRow.wiTypeName = "";
            }
        },
        
        onCellClick: function(event) {
            var cellField = event.cell.field;
            if (cellField != "wiTypeName" && cellField != "constraintType") {
                if (this._generalGrid.edit.isEditing()) {
                    this._generalGrid.edit.apply();
                }    
            }
        },
        
        /*
         * Fetches the work item types if the controller is missing them.
         */
        _getWorkItemTypes: function() {
            var workItemTypes = null;
            var self = this;
            var message = dojo.string.substitute(WorkMessages.ERROR_FINDING_WBS_TYPES, ["_getWorkItemTypes"]);
            var errorMessage = {
                errorMessage: message
            };
            
            var editProps = EPMWorkItemClient.getHardcodedWorkItemTypes();
            if (editProps) {
                var options = editProps[0].uiItems;
                if (options) {
                    var length = options.length;
                    for (var i = 0; i < length; i++) {
                        
                        var option = options[i];
                        
                        if (option) {
                            if (!(option.id in self.objectConverter([WBSConstants.EPM_PROJECT_TYPE, 
                                WBSConstants.EPM_RISK_TYPE, 
                                WBSConstants.EPM_DOCUMENT_TYPE]))) {
                                    
                                var workItemType = {
                                    id: options[i].id,
                                    label: options[i].label,
                                    iconUrl: options[i].iconUrl
                                };
                                
                                if (workItemTypes == null) {
                                    workItemTypes = new Array();
                                }
                                
                                workItemTypes.push(workItemType);
                            }
                        }    
                    }
                } else {
                    this._displayError(errorMessage);
                }        
            } else {
                this._displayError(errorMessage);
            }
            
            if (workItemTypes) {
                if (this._controller) {
                    this._controller.workItemTypes = workItemTypes;
                }
            }
        },
        
        /*
         * Converts an array of arguments into objects
         */
        objectConverter: function(arrayOrArgs) {
            var o = {};
            var a = (dojo.isArray(arrayOrArgs)) ? arrayOrArgs : arguments;
            var length = a.length;
            for (var i = 0; i < length; i++) {
                o[a[i]] = '';
            }
            return o;
        },
        
        /**
         * Method to handle the parent-child change initiated from the tree
         * 
         * @param {Object} publishObj: {
         *      treeID: ID of the tree.
         *      collectionItemHandle: information about the items which need
         *          to be synchronized.
         * }
         */
        _onParentChildChangeSyncGrid: function(publishObj) {
            if (publishObj) {
                
                var unsavedChanges = false;
                
                // Check if the viewlet is active or not
                if (this._viewlet) {
                    if (this._viewlet.getSite){
                        var site = this._viewlet.getSite();
                        if (site) {
                            if (site.isActive) {
                                var active = site.isActive();
                                if (active) {
                                    // Since the viewlet is active, 
                                    // check if its save button is enabled or not
                                    if (this._viewlet.isButtonEnabled) {
                                        unsavedChanges = this._viewlet.isButtonEnabled();
                                    }
                                }
                            }
                        }                    
                    }
                }
                
                if (unsavedChanges) {
                    // Prompt the user if they would like to save the changes
                    this._discardUnsavedChanges(publishObj);
                }
                else {
                    
                    // Nothing needs to be saved. Go ahead and sync the grid.
                    this._requestGridToSync(publishObj);
                }    
            }
        },
        
        /**
         * Prompts the user to indicate if they would like to save/discard
         * unsaved changes in the grid.
         *  
         * @param {Object} publishObj {
         *      treeID: ID of the tree.
         *      collectionItemHandle: information about the items which need
         *          to be synchronized.
         * }
         */
        _discardUnsavedChanges: function(publishObj){
            var buttonlist = [{
                label: WorkMessages.YES,
                action: dojo.hitch(this, this._saveChanges, publishObj)
            }, {
                label: WorkMessages.NO,
                action: dojo.hitch(this, this._cancelOperation, publishObj)
            }];
            
            var labelList = [
                {label: WorkMessages.CONFIRM_TO_SAVE_UNSAVED_CHANGES},
                {label: WorkMessages.CONFIRM_TO_SAVE_UNSAVED_CHANGES_2}
            ];
            
            this.discardUnsavedDialog = new EPMConfirmDialog({
                labelList: labelList,
                title: WorkMessages.DISCARD_CHANGES_TITLE,
                buttonlist: buttonlist
            });
            
            // Make sure that the cancel operation is handled even when the user clicks
            // on the 'X' button to close the dialog or hits ESC on the keyboard
            dojo.connect(this.discardUnsavedDialog.dialog, "onCancel", this, 
                this._handleCancelOperation, publishObj);
      
            this.discardUnsavedDialog.show();
        },
        
        /**
         * Handles the saving of the unsaved changes in the grid.
         * 
         * @param {Object} publishObj {
         *      treeID: ID of the tree.
         *      collectionItemHandle: information about the items which need
         *          to be synchronized.
         * }
         */
        _saveChanges: function(publishObj){
            this.discardUnsavedDialog.destroy();
            
            // Save the grid first
            this.persistGrid();
            
            // Save the request to sync the structure in a variable
            this.pendingSync = true;
            this.pendingSyncPublishObj = publishObj;
        },
        
        /**
         * Handles the discarding of the unsaved changes in the grid.
         * 
         * @param {Object} publishObj {
         *      treeID: ID of the tree.
         *      collectionItemHandle: information about the items which need
         *          to be synchronized.
         * }
         */
        _cancelOperation: function(publishObj){
            this.discardUnsavedDialog.destroy();
            this._handleCancelOperation(publishObj);
        },
        
        /**
         * Discards the unsaved changes and refreshes all rows of the grid.
         * 
         * @param {Object} publishObj {
         *      treeID: ID of the tree.
         *      collectionItemHandle: information about the items which need
         *          to be synchronized.
         * }
         */
        _handleCancelOperation: function(publishObj) {    
            // User indicated nothing needs to be saved. Go ahead and sync the grid.
            if (this._viewlet) {
                if (this._viewlet.setDeleteAndFetch && this._viewlet.refresh) {
                    this._viewlet.setDeleteAndFetch();
                    this._viewlet.refresh();
                } else {
                    // We should never hit this case but if we ever do,
                    // then make sure the grid matches the tree. Also
                    // logging a message that we hit this block.
                    this._requestGridToSync(publishObj);
                    console.log("WBSGeneralGrid._cancelOperation: Viewlet missing refresh method");
                }
            } else {
                // We should never hit this case but if we ever do,
                // then make sure the grid matches the tree. Also
                // logging a message that we hit this block.
                this._requestGridToSync(publishObj);
                console.log("WBSGeneralGrid._cancelOperation: Viewlet reference missing");
            }
        },
        
        /**
         * Makes a request to the grid's controller to update itself.
         * 
         * @param {Object} publishObj {
         *      treeID: ID of the tree.
         *      collectionItemHandle: information about the items which need
         *          to be synchronized.
         */
        _requestGridToSync: function(publishObj) {
            if (publishObj && this._controller && this._controller._onParentChildChangeSyncGrid) {
                this._controller._onParentChildChangeSyncGrid(publishObj);
            }
        }
    });
})();
