/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.resource.assignment.web.ui.internal2.RateAssignmentGridView");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGrid");
    dojo.require("dojo.data.ItemFileWriteStore");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.resource.assignment.web.ui.internal2.RateAssignmentGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog");
    dojo.require("com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewletConstants");
    dojo.requireLocalization("com.ibm.epm.resource.assignment.web.ui", "Messages");

    var ResourceAssignmentViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.resource.assignment.web.ui", "Messages");
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
   	var controllerFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
   	var constants =  com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewletConstants;
    var DateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    
    dojo.declare("com.ibm.epm.resource.assignment.web.ui.internal2.RateAssignmentGridView", abstractView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.resource.assignment.web", "ui/internal2/templates/RateAssignmentGrid.html"),
        
        constructor: function(){
            this._pconnects = [];
        },
        
        /** 
         * buildRatesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with TimecodesGrid.
         */
        buildRatesGrid: function(rates)
        {
            if (!this._gridModel)
            {
                this._gridModel = new com.ibm.epm.resource.assignment.web.ui.internal2.RateAssignmentGridModel();
            }
            var ratesController = controllerFactory.getEPMRateAssignmentGridController(constants.RATE_ASSIGNMENT);
			ratesController.rates = rates;
            this.buildGrid(this, this._gridModel, ratesController);
        },
        
        /**
         * Overridden function to connect to on cell click event
         */
		createView: function(){
            this.inherited('createView', arguments);
            this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_DBL_CLICK, this, this.onCellClick));
            this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_CLICK, this, this.onCellClick));
        },
        
        
        /**
         * getGridTitle funtion - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return constants.RATE_GRID_TITLE;
        },
        
        /**
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.ratesGridAttachPoint;
        },
        
        /**
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var gridLayout = [{
                    noscroll: false,
                    rows: [[{
                        name: ResourceAssignmentViewletMessages.FROM_DATE,
                        width: 7,
                        field: "fromDate",
                        formatter: DateUtil.formatDate,
	                    editor: dojox.grid.editors.DateTextBox,
    	                editorClass: "dijit.form.DateTextBox",
        	            constraint: {
            	            formatLength: 'short',
                	        selector: "date"
        	           	}
                    }, {
                        name: ResourceAssignmentViewletMessages.TO_DATE,
                        width: 7,
                        field: "toDate",
                        formatter: DateUtil.formatDate,
	                    editor: dojox.grid.editors.DateTextBox,
    	                editorClass: "dijit.form.DateTextBox",
        	            constraint: {
            	            formatLength: 'short',
                	        selector: "date"
        	           	}
                    }, {
                        name: ResourceAssignmentViewletMessages.COST_RATE,
                        width: 6,
                        field: "costRate",
						styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
                        editor: dojox.grid.editors.Dijit,
                        editorClass: "dijit.form.NumberTextBox",
                        constraint: {
                            min: 0
                        },
                        formatter: this.formatRate
                    }, {
                        name: ResourceAssignmentViewletMessages.SELLING_RATE,
                        width: 6,
                        field: "sellingRate",
						styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
                        editor: dojox.grid.editors.Dijit,
                        editorClass: "dijit.form.NumberTextBox",
                        constraint: {
                            min: 0
                        },
                        formatter: this.formatRate
                    }]]
                }];
            return gridLayout;
        },
        
        /**
         * Formats the value entered for cost/selling rate
         */
        formatRate: function(inDatum){
            return isNaN(inDatum) ? inDatum : parseFloat(inDatum).toFixed(2);
        },
        
        /**
         * Save all the changes
         */
        _saveRates: function()
        {
            if (this._generalGrid)
            {
				if ( this._generalGrid.edit) {
					this._generalGrid.edit.apply();
				}
                if (this._controller && this._controller.persistGrid)
                {
                    return this._controller.persistGrid();
                }
            }
        },
        
        /**
         * Add a new row to the rates
         */
        _addRate: function()
        {
            var rowID = this._controller.getUniqueRowID("rate_unsaved");
            var row = {
                "relativeHref": rowID
            };
            
            this._controller._insertElement(row, null, false);
			this.scrollToSelectedRow();
        },
        
        /**
         * Deletes selected rates
         */
        _deleteRate: function(){
         this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
               gridController: this._controller,
			   grid: this._generalGrid
            });
        },
        
        /**
         *  Handles the cell click event to validate fromDate and toDate
         */
        onCellClick: function(event){
            if (event) {
                if ((event.cell.field == 'fromDate') || (event.cell.field == 'toDate')) {
                    var row;
                    row = this._generalGrid.model.getRow(event.rowIndex);
                    if (row) {
                        var dateTextEditor = event.cell.editor.editor;
                        if (dateTextEditor) {
                            // Get the instance of the date picker associated with this cell
                            var datePicker = dateTextEditor._picker;
                            if (datePicker) {
                                // change the constraints based the current values of the start and end dates
                                if ((event.cell.field === 'fromDate') && (row.fromDate !== "")) {
                                    // disable all dates beyond end date
                                    dojo.mixin(datePicker.constraints, {
                                        max: row.toDate
                                    });
                                }
                                
                                if ((event.cell.field === 'toDate') && (row.fromDate !== "")) {
                                    // disable all dates before start date
                                    dojo.mixin(datePicker.constraints, {
                                        min: row.fromDate
                                    });
                                }
                            }
                        }
                        else {
                            // date picker hasn't been instantiated set the constraints in the cell
                            if ((event.cell.field === 'fromDate') && (row.toDate !== "")) {
                                // disable all dates beyond end date
                                dojo.mixin(event.cell.constraint, {
                                    max: row.toDate
                                });
                            }
                            
                            if ((event.cell.field === 'toDate') && (row.fromDate !== "")) {
                                // disable all dates before start date
                                dojo.mixin(event.cell.constraint, {
                                    min: row.fromDate
                                });
                            }
                        }
                    }
                }
            }
        },
        
		/**
         * OnBlur funtion - Overriding this funtion to avoid deselection of Rows
		 * @param {Object} data
		 */
		_onBlur: function(data){
			//donothing, since the abstract grid view is applying the editor's value
			return;
		},

		
        /**
         * Destroys the widget
         */
        destroy: function(){
            dojo.forEach(this._pconnects, dojo.disconnect);
            this._pconnects = [];
            this.inherited("destroy", arguments);
        }
    });
})();
