/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

if (!dojo._hasResource["com.ibm.epm.resource.assignment.web.ui.internal2.RateEditor"]) {
    dojo._hasResource["com.ibm.epm.resource.assignment.web.ui.internal2.RateEditor"] = true;
    dojo.provide("com.ibm.epm.resource.assignment.web.ui.internal2.RateEditor");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridEditor");
    dojo.require("com.ibm.epm.resource.assignment.web.ui.internal2.RateAssignmentAssociation");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
	dojo.require("com.ibm.epm.web.core.util.internal.Constants");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
	dojo.require("com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewletConstants");
	
    dojo.requireLocalization("com.ibm.epm.resource.assignment.web.ui", "Messages");
    
    var ResourceAssignmentViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.resource.assignment.web.ui", "Messages");
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    var Constants = com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewletConstants;
    var EPMConstants = com.ibm.epm.web.core.util.internal.Constants;
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
	var EPMGridEditor = com.ibm.epm.web.core.ui.internal.EPMGridEditor;
	
    dojo.declare("com.ibm.epm.resource.assignment.web.ui.internal2.RateEditor", EPMGridEditor, {
    
        /**
         *  constructor : This is the constructor function
         *  Incoming Arguments : inCell
         *  Outgoing Argument :
         *
         */
        constructor: function(inCell){
            this.rateDialog = null;
        },
        
        /**
         *  getController : This function is to get the controller
         *  Incoming Arguments :
         *  Outgoing Argument :
         *
         */
        getController: function(){
            if (!this._gridController) {
                this._gridController = controllerUtil.getEPMAssignmentGridController(Constants.ASSIGNED_RESOURCES_GRID_ID);
            }
            return this._gridController;
        },
        
        /**
         *  dodblclick : This function is to handle the double click event
         *  Incoming Arguments : e
         *  Outgoing Argument :
         *
         */
        dodblclick: function(e){
            this._onClick(e);
        },
        
        /**
         *  doclick : This function is to handle the click event
         *  Incoming Arguments : e
         *  Outgoing Argument :
         *
         */
        _onClick: function(e){
            var self = this;
            this._rowIndex = e.rowIndex;
            this.row = this.getController().getRowByIndex(this._rowIndex);
            if(this.row.href.indexOf("_unsaved") >=0)
            {
            	this._displayAndClearErrorMessage(ResourceAssignmentViewletMessages.CANNOT_MODIFY_RATES_OF_UNSAVED_ASSIGNMENT);
            	return;
            }
			if(!this.row.active){
				return;
			}
            
            if (self.rateDialog) {
                self.rateDialog = null;
            }
			this.rateInfo = new com.ibm.epm.resource.assignment.web.ui.internal2.RateAssignmentAssociation({Ok: ResourceAssignmentViewletMessages.OK});
            this.rateDialog = new jazz.ui.Dialog({
                contentNode : this.rateInfo.domNode,
                primaryTitle: ResourceAssignmentViewletMessages.RATE_TITLE,
                width: "400px",
                onOpen: dojo.hitch(this, this._onOpen),
                onClose: dojo.hitch(this, this._onClose)
            });
        },

        /**
         *  doclick : This function is to handle the click event
         *  Incoming Arguments : e
         *  Outgoing Argument :
         *
         */
        doclick: function(e){
			//Conditions in which the pop up is opened
			//1. When the current row is being edited and this cell is clicked
			if( (e && e.grid && e.grid.edit && e.grid.edit.isEditing() && e.grid.edit.info) &&
				(e.rowIndex == e.grid.edit.info.rowIndex)) {
				this._onClick(e);
			}
		},
        
        _closedialog: function(){
           this.rateInfo.destroy();
           if (this.rateDialog) {
                this.rateDialog.close();
                this.rateDialog.destroy();
                this.rateDialog = null;
            }
        },
        
        setRowData: function(){
            this.getController().setRowData(this.row, this.row.id, this._rowIndex);
			var rows = this.getController().getModifiedRows();
			if (rows && rows.length > 1) {
				var publishedObj = {
					controllerID: Constants.ASSIGNED_RESOURCES_GRID_ID,
					fieldName: "costRate"
				}
				dojo.publish(Constants.ASSIGNED_RESOURCES_FIELD_CHANGE_EVENT, [publishedObj]);
			}
        },
        
        /*
         * Displays the Error Message
         * Incoming : Error message
         * Outgoing : None
         */
        _displayErrorMessage: function(errorMessage){
	      this._displayAndClearErrorMessage(errorMessage);
        },
        
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
        /*
         * Display and clear the error message 
         * incoming : Error Message Object
         * OutGoing : None
         */
        _displayAndClearErrorMessage: function(errorMessage) {
            var self = this;
            EPMErrorHandler.setError(errorMessage);
            
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);  
        },
        
		/*
         * This method is called no matter how the dialog manages to close.
         * Destroys the assignment info object.
         */
        _onClose: function() {
            if (this.rateInfo) {
                this.rateInfo.destroy();
            }
            return true;    
        },
        /*
         * This method is called when the dialog opens.
         * Loads the rates into the grid.
         */
        _onOpen: function(){
            this.rateInfo.load(this.row, this);
        },

		/* Enter KeyDown Event on grid bring control here */		
		onEnterKeyDown: function(e){	
			dojo.stopEvent(e);
			this._onClick(e);   
		}
		        
    });
}
