/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){

    dojo.provide("com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewlet");
    
    // Viewlet
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.resource.assignment.web.ui", "Messages");
    
    
    dojo.require("com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentTabManager");
    dojo.require("com.ibm.epm.web.core.ui.internal.TreeEventListener");
    
    //Constants
    dojo.provide("com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewletConstants");
    
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    
    var ResourceAssignmentViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.resource.assignment.web.ui", "Messages");
    
    var Constants = com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewletConstants;
    var TreeEventListener = com.ibm.epm.web.core.ui.internal.TreeEventListener;
    
    dojo.declare("com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentViewlet", [Viewlet, TreeEventListener], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.resource.assignment.web", "ui/internal2/templates/ResourceAssignmentViewlet.html"),
        
        contextHelpId: Constants.RESOURCE_ASSIGNMENT_VIEWLET_CONTEXT_HELP_ID,
        noWBSTree: false,
        
        init: function(){
			this.inherited(arguments);
            
			this._createInformationLabel();
        
            // Set the title of the viewlet
            this._setTitle();
            
            // Disable viewlet if no WBS tree
            if (this._disableViewletIfNoWBSTree())
                return;
            
            // Create the Tabs
            this.ResourceAssignmentTabManager = new com.ibm.epm.resource.assignment.web.ui.internal2.ResourceAssignmentTabManager({
                SearchAndAssign: ResourceAssignmentViewletMessages.SEARCH_AND_ASSIGN_TAB,
                ProjectProfiles: ResourceAssignmentViewletMessages.PROJECT_PROFILES_TAB,
                Assign: ResourceAssignmentViewletMessages.ASSIGN_BUTTON,
                Unassign: ResourceAssignmentViewletMessages.UNASSIGN_BUTTON,
                Save: ResourceAssignmentViewletMessages.SAVE_BUTTON,
                From: ResourceAssignmentViewletMessages.FROM_DATE,
                To: ResourceAssignmentViewletMessages.TO_DATE,
                ContainingViewlet: this
            });
            this._content.appendChild(this.ResourceAssignmentTabManager.domNode);
        },
        
        /*
         * Create information label text. Do this just one time for the life of the
         * viewlet.
         */
        _createInformationLabel: function() {
            this.informationLabel = document.createElement("label");
            this.informationLabel.innerHTML = "";
			this.setCSHelpIdInToolbar(this.contextHelpId, this._informationLabel);
            this._informationLabel.appendChild(this.informationLabel);
        },
               
        /*
         * Disable viewlet and display an appropriate message if there is no WBS Tree
         * for the viewlet to make use of. Returns true if viewlet was disabled, signifying
         * to caller that further setup is not necessary.
         */
        _disableViewletIfNoWBSTree: function() {
            var tree = dijit.byId(Constants.WBS_TREE_ID);
            if (tree) {
                this._hideInformationLabel();
                return false;
            }

            this.noWBSTree = true;
            this._displayInformationLabel(ResourceAssignmentViewletMessages.WBS_TREE_MISSING);
            return true;
        },
        
        /*
         * Displays information label and hides the rest of the viewlet.
         */
        _displayInformationLabel: function(text) {
            dojo.addClass(this._content, "HiddenClass");
            this.informationLabel.innerHTML = text;
            dojo.removeClass(this._informationLabel, "HiddenClass");
        },
        
        /*
         * Hides information label and shows the rest of the viewlet.
         */
        _hideInformationLabel: function() {
            dojo.addClass(this._informationLabel, "HiddenClass");
            this.informationLabel.innerHTML = "";
            dojo.removeClass(this._content, "HiddenClass");
        },
        
        // Set the title of the viewlet
        _setTitle: function() {
            this.getSite().setTitle(ResourceAssignmentViewletMessages.VIEWLET_TITLE);
        },
        
        refresh: function() {
            if (!this.noWBSTree) {
				//Defect 14260: User quickly tranverses from Search tab to Query tab as soon as he enter Resource View
				//By the time refresh is invoked for this viewlet, the focus will be with Query or other viewlet
				//We check the state of the viewlet before actually refreshing. In a normal case when user enters the viewlet refresh gets invoked prpoerly
				if (this.getSite().isActive()) {
					this.ResourceAssignmentTabManager.refresh();
				}
				else {
					this.unSubscribeFromTreeEvents();
				}
            }
        },
        
        /**
         * Destroys the Viewlet
         */
        destroy: function(){
            if (this.ResourceAssignmentTabManager) {
                this.ResourceAssignmentTabManager.destroy();
                delete this.ResourceAssignmentTabManager;
            }
            
            if (this.informationLabel != null)
                this._informationLabel.removeChild(this.informationLabel);
            
            this.inherited("destroy", arguments);
        },
        
        /**
         *  afterTreeModified : Overridden function from TreeEventListener
         *  Incoming : none
         *  Outgoing : none
         *
         */
        afterTreeModified: function(){
            this.update();
        },
        
        subscribeToTreeEvents: function(){
            if (this.ResourceAssignmentTabManager) {
                this.ResourceAssignmentTabManager.subscribeToTreeEvents();
            }
        },
        
        unSubscribeFromTreeEvents: function(){
            if (this.ResourceAssignmentTabManager) {
                this.ResourceAssignmentTabManager.unSubscribeFromTreeEvents();
            }
        },
        
        
        setActive: function(active){
            if (!this.noWBSTree) {        
                if (active) {
					this.subscribeToTreeEvents();
					this.ResourceAssignmentTabManager.renderAssignmentGrid();
				}
				else {
					this.unSubscribeFromTreeEvents();
				}
            }
            
            this.inherited(arguments);
        }
        
    });
    
})();
