/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.util.internal.Util");
    
    /*
     * General purpose utility methods.
     */
    com.ibm.epm.web.core.util.internal.Util = {
        /*
         * Converts an associative array into an array of entries (reverse of toObject),
         * i.e. { id1: value1, ... } -> [ {id: id1, value: value1}, ... ]
         */
        /* object[] */
        toEntryArray: function(/* object */obj){
            var entries = new Array();
            if (obj) {
                for (var i in obj) {
                    entries.push({
                        id: i,
                        value: obj[i]
                    });
                }
            }
            return entries;
        },
        
        /*
         * Converts an array of entries into an associative array (reverse of toEntryArray),
         * i.e. [ {id: id1, value: value1}, ... ] -> { id1: value1, ... }
         */
        /* object */
        toObject: function(/* object[] */array){
            var obj = new Object();
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var entry = array[i];
                    obj[entry.id] = entry.value;
                }
            }
            return obj;
        },
        
        sort: function(items){
            var names = [];
            for (var i = 0; i < items.length; i++) {
                names[i] = items[i].name;
            }
            names = names.sort();
            var result = [];
            for (i = 0; i < names.length; i++) {
                result[i] = this._find(names[i], items);
            }
            return result;
        },
        
        _find: function(name, items){
            for (var i = 0; i < items.length; i++) {
                if (items[i].name == name) {
                    return items[i];
                }
            }
            return null;
        },
        
        removeChildren: function(element){
            while (element.firstChild) {
                element.removeChild(element.firstChild);
            }
        },
		
		addSpanforWhiteSpace: function(inDatum){
			return '<span class= "com-ibm-epm-web-core-ui-internal-templates-EPMGrid-textWrap">' + inDatum + "</span>";
		},
		
        /*
         * REALLY returns true ONLY if is a number, otherwise returns false.
         * Number can be positive or negative, have optional decimal point.
         * Values with letters, blank, or just spaces will return false.
         */
        isNumber: function(value) {
            return (value != undefined && value != null && !isNaN(value) && dojo.trim(value)!="");
        },
		
		addCellClass:function(item){
			dojo.forEach(item, function(cell){
				if (!cell.editor) {
					if (cell.cellClasses) {
						cell.cellClasses = 'cellNonEditable ' + cell.cellClasses;
					}else{
						cell.cellClasses = 'cellNonEditable';
					}
				}
			});
		}
        
		
    };
    
})();
